<?php
/**
 * CodeIgniter Controller
 *
 * PHP version 5
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package controllers
 * @version $Id: start.php 16153 2018-07-11 09:58:46Z christian $
*/

require_once APPPATH.'us_libraries/vendor/autoload'.EXT;
require_once APPPATH.'us_libraries/UsGenericProvider'.EXT;

/**
 * The oauth controller for untermStrich
 */
class Oauth extends My_Controller
{
    public function __construct()
    {
        $this->in_simple_mode = true;
        parent::__construct();
        
        $this->load->library('Oauth_tools', '', 'oauth_tools');
    }
    
    /**
     * Get config
     * 
     * @return  array
     */
    private function _get_config()
    {
        $use_account = $this->uri->segment(2);
        $oauth2account = $this->session->flashdata('oauth2account');
        
        if (!empty($use_account)) {
            $config = $this->oauth_tools->get_config(intval($use_account));
        } elseif (!empty($oauth2account)) {
            $config = $this->oauth_tools->get_config(intval($oauth2account));
        }
        
        if (!isset($config)) {
            echo '<h2>Error: Missing account information</h2>';
            die();
        }
        
        $this->session->set_flashdata('oauth2account', $config['usAccountId']);
        
        return $config;
    }
    
    /**
     * Oauth controller
     * 
     * @return void
     */
    public function index()
    {
        $this->load->model('Oauth_clients_model', 'oauth_clients');
        $this->load->helper('html');
        $this->lang->load('basic', $this->web_settings->getLanguage());
        
        $use_mode = $this->uri->segment(3);
        $params = $this->_get_query_param();
        
        $config = $this->_get_config();
        $account_id = $config['usAccountId'];
        $oauth2state = $this->session->flashdata('oauth2state');
        
        if ($use_mode==='logout') {
            return $this->_logout($config, $account_id);
        }
        
        if (isset($params['error'])) {
            echo '<h2>Error: '. htmlentities(strip_tags($params['error'])).'</h2>';
            echo '<p>Message:'. htmlentities(strip_tags($params['error_description'])).'</p>';
            echo '<p>Please retry: <a href="' . $config['redirectUri'] . '">' . $config['redirectUri'] . '</a></p>';

            $this->oauth_tools->delete($account_id);

            return;
        } elseif (!isset($params['code'])) {
            $provider = new \League\OAuth2\Client\Provider\UsGenericProvider($config);
            $authorizationUrl = $provider->getAuthorizationUrl();

            $this->session->set_flashdata('oauth2state', $provider->getState());

            $data = array(
                'url'           => $authorizationUrl,
                'us_version_ue' => urlencode($this->config->item('us_version')),
                'css_path'      => $this->config->item('us_css_path'),
                'js_path'       => $this->config->item('us_js_path'),
                'title'         => $this->lang->line('auth_oauth_login1')
            );
            $this->load->view('Basic/startpage_oauth', $data);
            return;
        } elseif (empty($params['state']) || (isset($oauth2state) && $params['state'] !== $oauth2state)) {
            if (isset($oauth2state)) {
                $this->session->unset_flashdata('oauth2state');
            }

            echo '<h2>Error - Invalid state</h2>';
            echo '<p>Please retry: <a href="' . $config['redirectUri'] . '">' . $config['redirectUri'] . '</a></p>';
            return;
        } else {
            $step = 'Init';
            try {
                $provider = new \League\OAuth2\Client\Provider\UsGenericProvider($config);
                
                $step = 'Call getAccessToken';
                $accessTokenObject = $provider->getAccessToken('authorization_code', [
                    'code' => $params['code']
                ]);
                
                $step = 'After getAccessToken';
                $this->oauth_tools->set_token($account_id, $accessTokenObject);
                
                $expired = $accessTokenObject->hasExpired();
                $access_expires = $accessTokenObject->getExpires();
                $expires_date = null;
                if (is_numeric($access_expires)) {
                    $expires_date = new UsDateTime();
                    $expires_date->setTimestamp($access_expires);
                }
                
                $resource_owner = '';
                try {
                    $access_token = $accessTokenObject->getToken();
                    $tks = explode('.', $access_token);
                    if (count($tks) === 3) {
                        list($headb64, $bodyb64, $cryptob64) = $tks;
                        $payload = json_decode(base64_decode($bodyb64));
                        if ($payload !== null) {
                            if (is_object($payload)) {
                                $resource_owner .= self::_print_item($payload, 'aud');
                                $resource_owner .= self::_print_item($payload, 'iss');
                                $resource_owner .= self::_print_item($payload, 'app_displayname', 'Application Name');
                                $resource_owner .= self::_print_item($payload, 'appid');
                                $resource_owner .= self::_print_item($payload, 'family_name', 'User Family Name');
                                $resource_owner .= self::_print_item($payload, 'given_name', 'User Given Name');
                                $resource_owner .= self::_print_item($payload, 'name', 'User Name');
                                $resource_owner .= self::_print_item($payload, 'scp', 'Scopes');
                                $resource_owner .= self::_print_item($payload, 'unique_name', 'User Unique Name');
                                $resource_owner .= self::_print_item($payload, 'upn');
                            }
                        }
                    }
                } catch (Exception $e) {}
                
                $data = array(
                    'action'        => 'window.opener.webApp.oauth.onLoginDo();',
                    'wait'          => 500,
                    'us_version_ue' => urlencode($this->config->item('us_version')),
                    'css_path'      => $this->config->item('us_css_path'),
                    'js_path'       => $this->config->item('us_js_path'),
                    'title'         => $this->lang->line('auth_oauth_login_ok'),
                    'account_id'    => $account_id,
                    'expires'       => $expires_date,
                    'expired'       => $expired,
                    'resource_owner'    => $resource_owner,
                );
                $this->load->view('Basic/startpage_oauth_ok', $data);
                
                return;
            } catch (UnexpectedValueException $e) {
                echo '<h2>'.$this->lang->line('auth_oauth_login_fail').'</h2>';
                echo '<p>Error UnexpectedValueException: '.$e->getMessage().'</p>';
                
                echo '<p>Details:<br><small>';
                    echo '<b>Step:</b> '.$step.'<br>';
                    echo '<b>Within:</b>'.$e->getFile().' - '.$e->getLine().'<br>';
                    echo '<pre style="padding-left: 20px;">';                            
                    echo $e->getTraceAsString();
                    echo '</pre>';
                echo '</small></p>';

                echo '<p>Please retry: <a href="' . $config['redirectUri'] . '">' . $config['redirectUri'] . '</a></p>';
                return;
            } catch (\League\OAuth2\Client\Provider\Exception\IdentityProviderException $e) {
                echo '<h2>'.$this->lang->line('auth_oauth_login_fail').'</h2>';
                echo '<p>Error IdentityProviderException: '.$e->getMessage().'</p>';
                
                $response = $e->getResponseBody();
                
                echo '<p>Details:<br><small>';
                    echo '<b>Step:</b> '.$step.'<br>';
                    echo '<b>Response:</b><br>';
                    echo '<pre style="padding-left: 20px;">';                            
                    var_dump($response);
                    echo '</pre>';
                echo '</small></p>';

                echo '<p>Please retry: <a href="' . $config['redirectUri'] . '">' . $config['redirectUri'] . '</a></p>';
                return;
            }
        }
    }
    
    /**
     * Print item from object
     * 
     * @param   object  $obj        Object
     * @param   string  $field      Field name to print
     * @param   mixed   $title      Optional title
     * @return  string
     */
    private static function _print_item($obj, $field, $title=false)
    {
        if (isset($obj->$field)) {
            $title = ($title!==false) ? $title : $field;
            return '<em>'.$title.':</em> '.$obj->$field . '<br>';
        }
        return '';
    }
    
    /**
     * Logout user
     * 
     * @param   array       $config         Config info
     * @param   integer     $account_id     Account ID
     */
    protected function _logout(array $config, $account_id)
    {
        $this->oauth_tools->delete($account_id);
        
        switch ($config['usAccountClientType']) {
            case Oauth_clients_model::TYPE_dropbox:
                $logoutUrl = $config['usClientAuthority'] . '/logout'; //Dropbox does not support redirect after logout
                break;
            default:
                $post_logout_redirect_uri = site_url('/simple/oauth_logout_done/'.$account_id); // The logout destination after the user is logged out from their account.
                $logoutUrl = $config['usClientAuthority'] . '/oauth2/logout?post_logout_redirect_uri='.urlencode($post_logout_redirect_uri);
                break;
        }
        
        $data = array(
            'url'           => $logoutUrl,
            'wait'          => 500,
            'us_version_ue' => urlencode($this->config->item('us_version')),
            'css_path'      => $this->config->item('us_css_path'),
            'js_path'       => $this->config->item('us_js_path'),
            'title'         => $this->lang->line('auth_oauth_logout1')
        );
        $this->load->view('Basic/startpage_oauth', $data);
    }
    
    /**
     * Get the current query parameters instead of from $_GET, from REQUEST_URI
     * 
     * @return  array       Query parameters
     */
    private function _get_query_param()
    {
        if (!isset($_SERVER['REQUEST_URI']) OR empty($_SERVER['REQUEST_URI'])) {
            return array();
        }
        
        $query = parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY);
        if (!isset($query) OR empty($query)) {
            return array();
        }
        
        parse_str($query, $reply);
        return $reply;
    }
    
}