<?php
/**
 * CodeIgniter Controller
 *
 * PHP version 5
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package pages
 * @version $Id: update.php 24183 2022-08-03 06:14:25Z christian.w $
*/

require_once APPPATH.'us_libraries/Page'.EXT;

/**
 * The update page of untermStrich
 */
class Update extends Page
{
    protected $ok_msg = 'Ok';

    /**
     * Constructor
     * 
     * @param   object  $CI The CI main object
     */
    public function __construct($CI)
    {
        parent::__construct($CI);
        
        $this->CI->login->restrict_right(Login_model::R_MASTERDATAMANAGEMENT, Login_model::L_ADMINISTRATION, __CLASS__);

        $this->ok_msg = '<p style="padding-top:10px;">Aktualisierung erfolgreich! / Update Successful!</p><p style="padding-top:10px;"><a href="'.site_url().'">Anmelden / Login &gt; &gt;</a></p>';
    }

    /**
     * Process the update DIRECT
     *
     * @param   integer $mode   One of the My_controller::m_... modes
     * @return  mixed   Ignore
     */
    public function index($mode)
    {
        if ($mode!==My_Controller::M_DIRECT) {
            basic_error('You may not call Update using xajax.', $mode, __CLASS__.'::'.__FUNCTION__);
        }

        $update_id = $this->CI->uri->segment(6);
        if ($update_id===false) {
            basic_error('Missing value', My_controller::M_DIRECT, __CLASS__.'::'.__METHOD__);
        }

        $this->CI->lang->load('masterdatamanagement', $this->CI->web_settings->getLanguage());
        $this->init_smarty($mode);
        $this->init_for_main();
        $this->init_for_agent();
        $this->init_ussettings();
        $this->CI->load->helper('process');
        $this->CI->load->helper('system');
        $this->CI->load->helper('us_html');
        $this->CI->load->library('US_Update', '', 'update');

        process_start_flush();

        ignore_user_abort(true);
        set_us_memory_limit();
        restart_timout_counter(true);
        
        echo $this->CI->smarty->view('Masterdatamanagement/setup/update_process', true);
        sleep(1);
        process_update_progressbar(5);

        $this->_message($this->CI->lang->linef('masterdatamanagement_download_action', 'untermStrich'));
        sleep(1);
        if (!$this->CI->update->get_update($update_id)) {
            return $this->_fail('Download failed');
        }
        process_update_progressbar(50);
        
        restart_timout_counter(true);

        $this->_message($this->CI->lang->linef('masterdatamanagement_install_action', 'untermStrich'));
        sleep(1);
        $ok = $this->CI->update->install_update(true);
        if ($ok===false) {
            return $this->_fail('Install failed - No file downloaded');
        } elseif ($ok!==true) {
            return $this->_fail($ok);
        }
        process_update_progressbar(100);

        $this->_message($this->ok_msg, true);
    }

    /**
     * Process a db update only DIRECT
     *
     * @param   integer $mode   One of the My_controller::m_... modes
     * @return  mixed   Ignore
     */
    public function dbupdate($mode)
    {
        if ($mode!==My_Controller::M_DIRECT) {
            basic_error('You may not call Update using xajax.', $mode, __CLASS__.'::'.__FUNCTION__);
        }

        $this->CI->lang->load('masterdatamanagement', $this->CI->web_settings->getLanguage());
        $this->init_smarty($mode);
        $this->init_for_main();
        $this->init_for_agent();
        $this->init_ussettings();
        $this->CI->load->helper('process');
        $this->CI->load->helper('system');
        $this->CI->load->library('US_Nightly', '', 'nightly');
        $this->CI->load->library('US_Db_update', '', 'db_update');

        process_start_flush();

        ignore_user_abort(true);
        set_us_memory_limit();
        restart_timout_counter(true);
        
        echo $this->CI->smarty->view('Masterdatamanagement/setup/update_process', true);
        sleep(1);
        process_update_progressbar(5);
        
        $this->CI->session->force_logoff_all();
        $this->CI->session->sess_create();
        
        $this->CI->nightly->run();
        process_update_progressbar(20);

        $path_to_from_current = INSTALLPATH.'data/from_current.sql';
        $ok = $this->CI->db_update->update($path_to_from_current, null, function($msg) {
            $this->_message('Action: Updating database - '.$msg);
        });
        if ($ok!==true) {
            log_message('error', 'DB UPDATE error: '.$ok);
            return $this->_fail($ok);
        }

        process_update_progressbar(100);

        $this->_message($this->ok_msg, true);
    }

    /**
     * Change the message
     *
     * @param   string  $message    Message
     * @param   boolean $hide       Hide progressbar
     * @return  void
     */
    private function _message($message, $hide=false)
    {
        echo '<script type="text/javascript">';
        if ($hide) echo '$("#progressbar").hide();';
        echo '$("#progressmessage").html("'.prepare_string($message).'");';
        echo '</script>'."\n";
        usleep(500);
    }

    /**
     * Shows an error message
     *
     * @param   string  $message    Message
     * @return  void
     */
    private function _fail($message)
    {
        echo '<script type="text/javascript">';
        echo '$("#progressbar").hide();';
        echo '$("#progressmessage").html("'.prepare_string($message).'").attr("class", "showerror ui-corner-all");';
        echo '</script>'."\n";
        return;
    }
    
    /*
     * Remove an additional installed portlet
     * 
     * @param   integer $mode   One of the My_controller::m_... modes
     * @return  void
     */    
    public function remove_additional($mode)
    {
        $this->init_xajax_response();
        
        $this->CI->load->library('US_Portlet_install', '', 'portlet_install');
        $this->CI->load->model('Portlets_model', 'portlets_model', true);
        $this->CI->lang->load('masterdatamanagement', $this->CI->web_settings->getLanguage());
        
        $remove_what = $this->CI->input->post('val1');
        $force_remove = ($this->CI->input->post('val2') === 'true') ? true : false;
        if ($remove_what===false) {
            basic_error('Missing value', $mode, __CLASS__.'::'.__METHOD__);
        }    
        
        if ($force_remove !== true) {
            $user_using_portlet = $this->CI->portlets_model->get_users_using_portlet($remove_what);
            
            
            $msg = $this->CI->lang->line('masterdatamanagement_update_widgets_ask_delete');
            if (count($user_using_portlet) > 0) {
                $msg.= '<br /><br />';
                $msg.= $this->CI->lang->line('masterdatamanagement_update_widgets_is_used_by').':';
                foreach ($user_using_portlet as $staff) {
                    $msg.= '<br />'.$staff->staff_name;
                }
            }
            $cld = 'dialogMessage.dialogs[\'MSGDIALOGID\'].dialog(\'close\');';//Snippet for closing the dialog
            $this->CI->xajax->message(
                    $msg,
                    US_Xajax_response::MSGM_QUESTION,
                    $cld . "usComm.loadShowWait();openx('update', 'remove_additional', '$remove_what', true);",
                    $cld,
                    true
            );            
        } else {
            $ok = $this->CI->portlet_install->uninstall($remove_what);
            if ($ok!==false) {
                log_message('error', 'PORTLET uninstall: '.$ok);
                basic_error($ok, $mode, __CLASS__.'::'.__METHOD__);
            }            
            
            $this->add_run_script("openx('masterdatamanagement_update_widgets', 'index');");
        }
        
        $this->assign_run_scripts_xajax();
        $this->CI->xajax->write();        
    }  
    
    /**
     * Download additional packages
     *
     * @param   integer $mode   One of the My_controller::m_... modes
     * @return  void
     */
    public function download_additional($mode)
    {
        if ($mode!==My_Controller::M_DIRECT) {
            basic_error('You may not call Update using xajax.', $mode, __CLASS__.'::'.__FUNCTION__);
        }

        $install_what = $this->CI->uri->segment(6);
        if ($install_what===false) {
            basic_error('Missing value', My_controller::M_DIRECT, __CLASS__.'::'.__METHOD__);
        }
        if ($install_what==='tika') {
            $install_what='org_apache_tika';
        }

        $this->CI->load->helper('url_get_contents');
        $this->CI->load->helper('us_html');
        $this->CI->load->helper('process');
        $this->CI->load->helper('system');
        $this->CI->load->library('US_Portlet_install', '', 'portlet_install');
        
        $this->CI->load->library('US_Portlet_update', '', 'us_portlet_update');

        try {
            $update_widgets = $this->CI->us_portlet_update->list_update_widgets();
        } catch (Exception $exc) {
            basic_error($exc->getMessage(), $mode, __CLASS__.'::'.__FUNCTION__);
        }
        
        if (isset($update_widgets[$install_what])) {
            $link = $update_widgets[$install_what]['link'];
            $title = (isset($update_widgets[$install_what]['title'])) ? $update_widgets[$install_what]['title'] : ucfirst(str_replace('_', ' ', str_replace('com_untermstrich_', '', $install_what)));
            $to_file = $this->CI->config->item('us_updatesdir').pathinfo($link, PATHINFO_BASENAME);
            $from_file = $link;
            $file_length = $update_widgets[$install_what]['size'];
        } else {
            basic_error('Unknown package to install', My_controller::M_DIRECT, __CLASS__.'::'.__METHOD__);                    
        }
        
        $this->CI->lang->load('masterdatamanagement', $this->CI->web_settings->getLanguage());
        $this->init_smarty($mode);
        $this->init_for_main();
        $this->init_for_agent();
        $this->init_ussettings();
        
        process_start_flush();
        
        ignore_user_abort(true);
        set_us_memory_limit();
        restart_timout_counter(true);
        
        $this->CI->smarty->assign('title', $title);
        echo $this->CI->smarty->view('Masterdatamanagement/setup/download_process', true);
        sleep(1);
        process_update_progressbar(5);
        
        $this->_message($this->CI->lang->linef('masterdatamanagement_download_action', $title));
        if (file_exists($to_file)) @unlink($to_file); //Remove old
        
        $info = url_get_contents($from_file, $to_file, $file_length*2);
        if ($info===false) {
            basic_error('The zip file "'.$from_file.'" cannot be read.', $mode, __CLASS__.'::'.__FUNCTION__);
        }
        process_update_progressbar(50);

        restart_timout_counter(true);
        
        $this->_message($this->CI->lang->linef('masterdatamanagement_install_action', $title));
        $ok = $this->CI->portlet_install->install($to_file);
        if ($ok!==false) {
            log_message('error', 'PORTLET install: '.$ok);
            return $this->_fail($ok);
        }
        
        process_update_progressbar(100);

        $this->_message($this->CI->lang->linef('masterdatamanagement_install_done', $title), true);
    }
    
    /**
     * Download node.js server
     *
     * @param   integer $mode   One of the My_controller::m_... modes
     * @return  void
     */
    public function download_nodejs($mode)
    {
        if ($mode!==My_Controller::M_DIRECT) {
            basic_error('You may not call Update using xajax.', $mode, __CLASS__.'::'.__FUNCTION__);
        }
        
        $this->CI->load->helper('url_get_contents');
        $this->CI->load->helper('us_html');
        $this->CI->load->helper('process');
        $this->CI->load->helper('system');
        $this->CI->load->library('US_Portlet_install', '', 'portlet_install');         
        
        $this->CI->lang->load('masterdatamanagement', $this->CI->web_settings->getLanguage());
        $this->init_smarty($mode);
        $this->init_for_main();
        $this->init_for_agent();
        $this->init_ussettings();        
        
        $stat = '';
        $this->CI->load->portlet('com_untermstrich_nodejs', null, $stat, true);
        if ($stat!=='') {
            basic_error('Butler Com_untermstrich_nodejs does not exist', $mode, __CLASS__.'::'.__FUNCTION__);
        } else {
            Com_untermstrich_nodejs::install_server();
        }
    }
    
    /**
     * Prepare upgrade view
     * 
     * @param   integer $mode   One of the My_controller::m_... modes
     * @return  mixed   Ignore
     */
    public function upgrade_prepare($mode)
    {
        if ($mode!==My_Controller::M_DIRECT) {
            basic_error('You may not call Update using xajax.', $mode, __CLASS__.'::'.__FUNCTION__);
        }

        $update_id = $this->CI->uri->segment(6);
        if ($update_id===false) {
            basic_error('Missing value', My_controller::M_DIRECT, __CLASS__.'::'.__METHOD__);
        }

        $this->CI->load->model('System_settings_model', 'system_settings');
        $this->CI->load->model('Email_accounts_model', 'email_accounts');
        $this->CI->load->library('US_Portlet_update', '', 'us_portlet_update');
        $this->CI->lang->load('masterdatamanagement', $this->CI->web_settings->getLanguage());
        $this->init_smarty($mode);
        $this->init_for_index($mode);
        
        $this->CI->smarty->assign('os_is_win', (strtoupper(substr(PHP_OS, 0, 3))==='WIN'));
        $this->CI->smarty->assign('os_is_mac', (strtoupper(substr(PHP_OS, 0, 6))==='DARWIN'));

        $php_version_3 = floatval(substr(phpversion(),0,3));
#$php_version_3 = 5.6;
        $this->CI->smarty->assign('php_version_3', $php_version_3);

        $result = $this->CI->db->query('SELECT @@version AS version');
        $sql_reply_info = $result->row();
        if (!$sql_reply_info) {
            basic_error('Failed to get SQL version from DB', My_controller::M_DIRECT, __CLASS__.'::'.__METHOD__);
        }
#$sql_reply_info->version='5.6.8';
        $mysql_version = explode('.', $sql_reply_info->version);
        $this->CI->smarty->assign('mysql_version', $mysql_version);

        $result_outsync = $this->CI->db->query('SELECT count(*) AS countoutsync FROM outsync where LfdNr_User>=1');
        $sql_outsync_info = $result_outsync->row();
        $count_outsync = 0;
        if ($sql_outsync_info && isset($sql_outsync_info->countoutsync)) {
            $count_outsync = intval($sql_outsync_info->countoutsync);
        }
#$count_outsync = 2;
        $this->CI->smarty->assign('count_outsync', $count_outsync);
        
        $java_version = null;
        $us_java_path = $this->CI->config->item('us_java_path');
        if (!empty($us_java_path)) {
            $bin = FCPATH.'java/test.jar';
            $exec_java = @shell_exec($us_java_path.' -jar '.escapeshellarg($bin).' 2>&1');
            if (stripos($exec_java, 'ACTIVE')!==false) {
                $java_parts = explode(' ', $exec_java);   
                if (count($java_parts) >= 3) {
                    $java_version = array($java_parts[1]);
                }         
            }
        }
        $this->CI->smarty->assign('java_version', $java_version);
        
        $o365_mail_no_oauth = array();
        if (datetime_diff(new UsDate('2022-08-01'), new UsDate('now')) >= 0) {
            $o365_mail_no_oauth = $this->CI->email_accounts->get_o365_email_accounts_without_oauth();
        }
#$a = new stdClass(); $a->email = 'Fake@example.com'; $o365_mail_no_oauth[] = $a;
        $this->CI->smarty->assign('o365_mail_no_oauth', $o365_mail_no_oauth);

        $has_fulltext_search_activated = $this->CI->system_settings->getFulltext_search_activate();
        $this->CI->smarty->assign('has_fulltext_search_activated', $has_fulltext_search_activated);
        
        $update_widgets = $this->CI->us_portlet_update->list_update_widgets();
        $all_widgets = $this->CI->load->list_all_widgets_portlet($mode);
        $faulty_widgets = $this->CI->load->list_faulty_widgets($mode);

        $widgets = array();
        foreach ($update_widgets as $name => $update_widget) {
            if (isset($all_widgets[$name]) && isset($all_widgets[$name]['last_update_date']) ) {
                if (datetime_diff($all_widgets[$name]['last_update_date'], $update_widget['date']) > 0) {
                    $widgets[] = $update_widget['title'];
                }
            }
        }
        foreach ($faulty_widgets as $faulty_widget) {
            if (!isset($widgets[$faulty_widget['name']])) {
                $widgets[] = $faulty_widget['name'];
            }
        }
        $this->CI->smarty->assign('widgets', $widgets);

        $this->CI->smarty->assign('update_id', $update_id);
        $this->CI->smarty->assign('xjxcookie', $this->CI->login->get_xsrf());
        
        $this->reply(Page::P_MIDDLE, $this->CI->smarty->view('Masterdatamanagement/setup/x4index', true));
        $this->reply(Page::P_RIGHT, $this->CI->smarty->view('Masterdatamanagement/setup/x4info', true));
        
        $this->write($mode);
    }
    
} // END class Update