-- USC Wenn der nächste Schritt fehlschlägt, importieren Sie bitte die Datenbank neu, ohne Stored Procedures und Funktionen.
DROP PROCEDURE IF EXISTS us_get_parent;
-- EOC

-- USC Wenn der nächste Schritt fehlschlägt, importieren Sie bitte die Datenbank neu, ohne Stored Procedures und Funktionen.
DROP FUNCTION IF EXISTS us_get_parent_package;
-- EOC

-- USC Erstelle alle Stored-Procedures.
-- --------------------------------------------------------------------------------------------------
-- Get the parent node
-- --------------------------------------------------------------------------------------------------
DELIMITER $$
CREATE FUNCTION us_get_parent_package(from_project_id INT(11), from_id BIGINT(20)) 
    RETURNS BIGINT(20)
    NOT DETERMINISTIC
    BEGIN
        DECLARE the_id BIGINT(20);
        SELECT id INTO the_id 
            FROM work_packages WHERE 
            lft < (SELECT lft FROM work_packages WHERE id = from_id) 
                AND 
            rgt > (SELECT rgt FROM work_packages WHERE id = from_id)
                AND 
            project_id =  from_project_id 
            ORDER BY lft DESC LIMIT 1;
        RETURN the_id;
END $$
DELIMITER ;
-- EOC

DROP FUNCTION IF EXISTS us_get_parent_package_preset;
-- EOC
DELIMITER $$
CREATE FUNCTION us_get_parent_package_preset(from_tree_id INT(11), from_id BIGINT(20)) 
    RETURNS BIGINT(20)
    NOT DETERMINISTIC
    BEGIN
        DECLARE the_id BIGINT(20);
        SELECT id INTO the_id 
            FROM work_package_presets WHERE 
            lft < (SELECT lft FROM work_package_presets WHERE id = from_id) 
                AND 
            rgt > (SELECT rgt FROM work_package_presets WHERE id = from_id)
                AND 
            tree_id =  from_tree_id 
            ORDER BY lft DESC LIMIT 1;
        RETURN the_id;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Add index to table, if not exists
--
-- tablename        Table name
-- indexname        Name of the index
-- indexinfo        Fields list
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_add_index_to_table;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_add_index_to_table(IN tablename VARCHAR(255), IN indexname VARCHAR(255), IN indexinfo VARCHAR(255))
BEGIN
    IF ((SELECT COUNT(*) AS index_exists FROM information_schema.statistics WHERE TABLE_SCHEMA = DATABASE() and table_name = tablename AND index_name = indexname) <= 0) THEN
        -- Simple alter table
        SET @stmt_alter = CONCAT('ALTER TABLE ', tablename, ' ADD  INDEX ', indexname, ' (', indexinfo, ')');
        PREPARE stmt_alter FROM @stmt_alter;
        EXECUTE stmt_alter;		
    END IF;
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS us_add_fulltext_index_to_table;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_add_fulltext_index_to_table(IN tablename VARCHAR(255), IN indexname VARCHAR(255), IN indexinfo VARCHAR(255))
BEGIN
    IF NOT EXISTS ( (SELECT * FROM information_schema.statistics WHERE TABLE_SCHEMA=DATABASE() and table_name=tablename AND index_type='FULLTEXT' AND index_name=indexname) ) THEN
        SET @stmt_create = CONCAT('CREATE FULLTEXT INDEX ', indexname, ' ON ', tablename, ' (', indexinfo, ')');
        PREPARE stmt_create FROM @stmt_create;
        EXECUTE stmt_create;		
    END IF;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Drop index of table, if exists
--
-- tablename        Table name
-- indexname        Name of the index
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_drop_index_if_exists;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_drop_index_if_exists(IN tablename VARCHAR(255), IN indexname VARCHAR(255) )
BEGIN
    IF ((SELECT COUNT(*) AS index_exists FROM information_schema.statistics WHERE TABLE_SCHEMA = DATABASE() and table_name = tablename AND index_name = indexname) > 0) THEN
        SET @s = CONCAT('DROP INDEX ' , indexname , ' ON ' , tablename);
        PREPARE stmt FROM @s;
        EXECUTE stmt;
    END IF;
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS us_drop_all_fulltext_indexes_if_exists;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_drop_all_fulltext_indexes_if_exists(IN tablename VARCHAR(255))
BEGIN
    DECLARE isDone INT;
    DECLARE indexname VARCHAR(255);

    DECLARE curs CURSOR FOR SELECT INDEX_NAME FROM information_schema.STATISTICS WHERE table_schema=DATABASE() AND TABLE_NAME=tablename AND INDEX_TYPE='FULLTEXT';
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET isDone = 1;

    OPEN curs;

    SET isDone = 0;
    REPEAT
        FETCH curs INTO indexname;

        CALL us_drop_index_if_exists(tablename, indexname);
    UNTIL isDone END REPEAT;

    CLOSE curs;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Add field to table, if not exists
--
-- tablename    Table name
-- fieldname    Field name
-- fieldtype    Field type including charset and collate
-- --------------------------------------------------------------------------------------------------
-- Remove that first, just in case
DROP PROCEDURE IF EXISTS us_add_to_table;
-- EOC
DROP PROCEDURE IF EXISTS us_add_to_table2;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_add_to_table2(IN tablename VARCHAR(255), IN fieldname VARCHAR(255), IN fieldtype VARCHAR(255))
BEGIN
    IF NOT EXISTS( (SELECT * FROM information_schema.COLUMNS WHERE TABLE_SCHEMA=DATABASE() AND COLUMN_NAME=fieldname AND TABLE_NAME=tablename) ) THEN
        IF EXISTS( (SELECT * FROM information_schema.STATISTICS WHERE table_schema=DATABASE() AND TABLE_NAME=tablename AND INDEX_TYPE='FULLTEXT') ) THEN
            CALL us_drop_all_fulltext_indexes_if_exists(tablename);
        END IF;

        SET @s = CONCAT('ALTER TABLE ', tablename, ' ADD ', fieldname, ' ', fieldtype);
        PREPARE stmt2 FROM @s;
        EXECUTE stmt2;
    END IF;

END $$
DELIMITER ;
-- EOC


-- --------------------------------------------------------------------------------------------------
-- Remove duplicate entries from table
--
-- tablename            Table name
-- pk                   Name of primary key field
-- matching_fields      Fields to match (group by)
-- validation_string    Validation string to check if all done
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_remove_duplicates_from_table;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_remove_duplicates_from_table(IN tablename VARCHAR(255), IN pk VARCHAR(255), IN matching_fields VARCHAR(255), OUT validation_string VARCHAR(4))
    BEGIN
        DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
        BEGIN
            ROLLBACK;
            SET autocommit=1;
        END;

        SET autocommit=0;
        START TRANSACTION;

        -- Trim spaces from matching_fields
        SET @matching_fields = replace(matching_fields, ' ', '');

        -- Build on_string
        SET @on_string = '';
        SET @strLen := 0; 
        SET @i := 1;
        WHILE @strLen < LENGTH(@matching_fields)  DO
                SET @str := SUBSTRING_INDEX(@matching_fields, ',', @i);
                SET @strLen := LENGTH(@str);
                SET @i := @i + 1;
                IF @i > 2 THEN
                        SET @on_string = CONCAT(@on_string, ' AND ');
                END IF;
                SET @on_string = CONCAT(@on_string, tablename, '.',SUBSTRING_INDEX(@str, ',', -1), '=temp.', SUBSTRING_INDEX(@str, ',', -1));
        END WHILE;

        -- Create query to get duplicate records to delete
        SET @sql1 = CONCAT('SELECT ',tablename,'.',pk,' FROM ', tablename, ' INNER JOIN(SELECT * FROM ', tablename, ' GROUP BY ', matching_fields, ' HAVING COUNT(*) >1) temp ON ', @on_string, ' where ', tablename,'.', pk, ' not in (SELECT ', pk, ' FROM ', tablename, ' GROUP BY ', matching_fields, ' HAVING  COUNT(*) > 1)');
        
        -- Create temp table for saving ids
        SET @unique_table_name = replace(CONCAT('us_remove_duplicates_from_table_',UUID()), '-', '');
        SET @sql2 = CONCAT('CREATE TEMPORARY TABLE ',@unique_table_name,' AS(',@sql1,')');
        PREPARE stmt2 FROM @sql2;
        EXECUTE stmt2;

        -- Delete from table
        SET @sql3 = CONCAT('DELETE FROM ',tablename, ' WHERE ', pk, ' IN(SELECT ',pk ,' FROM ',@unique_table_name,')');
        PREPARE stmt3 FROM @sql3;	
		  EXECUTE stmt3;	
    
        -- Delete temp table
        SET @sql4 = CONCAT('DROP TABLE ',@unique_table_name);
        PREPARE stmt4 FROM @sql4;
        EXECUTE stmt4;	

        COMMIT;
        SET autocommit=1;

        SET validation_string = 'true';	
    END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS us_remove_duplicates_from_table_without_pk;
-- EOC

-- Example:
-- CALL us_remove_duplicates_from_table_without_pk(
-- 	'completion_level', 
--      'id, project_id, work_area_id, staff_id, date, MAX(completion_level) AS completion_level ,completion_level_history_id', 
-- 	'project_id, work_area_id, completion_level_history_id',
-- 	@validate_eadfsefds);
-- CALL us_remove_duplicates_from_table_without_pk(
-- 	'budget_plan', 
--      'project_id, area_id, staff_id, budget_plan_history_id, planned_costs, planned_hours, start_date, end_date, MAX(last_changed_date) AS last_changed_date, proportion, threshold', 
-- 	'project_id, area_id, budget_plan_history_id',
-- 	@validate_eadfsefds);
-- DonauConsult:
-- CALL us_add_constraint_to_table('schaetzungfertig', 'schaetzungfertig_unique_1', 'UNIQUE', '`LfdNr_Projekt` , `LfdNr_AB` , `History_LfdNr`' );
-- CALL us_add_constraint_to_table('sollab', 'sollab_unique_1', 'UNIQUE', '`LfdNr_Projekt` , `LfdNr_AB` , `History_LfdNr`' );

DELIMITER $$
CREATE PROCEDURE us_remove_duplicates_from_table_without_pk(IN xtable VARCHAR(255), IN my_fields VARCHAR(255), IN matching_fields VARCHAR(255), OUT validation_string VARCHAR(4))
    BEGIN
	
	DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
	BEGIN
	    ROLLBACK;
	    SET autocommit=1;
	END;

	SET autocommit=0;
	START TRANSACTION;
		
	SET @sql1 = 'DROP TABLE IF EXISTS us_remove_duplicates_from_table_without_pk__tmp;';
	PREPARE stmt1 FROM @sql1;
	EXECUTE stmt1;
	-- create cleaned temporary table
	SET @sql1 = CONCAT('CREATE TEMPORARY TABLE us_remove_duplicates_from_table_without_pk__tmp AS(
		SELECT ',my_fields,' FROM ',xtable,'
		GROUP BY ',matching_fields,')');
	PREPARE stmt1 FROM @sql1;
	EXECUTE stmt1;
    
 	-- Delete from orig table - use delete, not truncate cause delete supports transactions
 	SET @sql1 = CONCAT('DELETE FROM ',xtable);
	PREPARE stmt1 FROM @sql1;
	EXECUTE stmt1;
	
	-- insert from tmp to orig table
	SET @sql1 = CONCAT('INSERT INTO ',xtable,' (SELECT * FROM us_remove_duplicates_from_table_without_pk__tmp)');
	PREPARE stmt1 FROM @sql1;
        EXECUTE stmt1;

        COMMIT;
        SET autocommit=1;

        SET validation_string = 'true';	
    END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Add constraint to table, if not exists
--
-- tablename        Table name
-- constraintname   Name of the constraint
-- constrainttype   Type of constraint (PRIMARY, UNIQUE, FULLTEXT, SPATIAL, FOREIGN)
-- constraintinfo   Fields list
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_add_constraint_to_table;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_add_constraint_to_table(IN tablename VARCHAR(255), IN constraintname VARCHAR(255), IN constrainttype VARCHAR(20), IN constraintinfo VARCHAR(255))
BEGIN
    IF NOT EXISTS( (SELECT * FROM information_schema.TABLE_CONSTRAINTS WHERE TABLE_SCHEMA=DATABASE() AND (CONSTRAINT_NAME=constraintname OR CONCAT('us_', CONSTRAINT_NAME)=constraintname) AND TABLE_NAME=tablename) ) THEN
        -- Alter table for PRIMARY or UNIQUE constraint
        IF UPPER(constrainttype) = 'PRIMARY' OR UPPER(constrainttype) = 'UNIQUE' THEN
            SET @new_table = CONCAT(tablename, '__', UUID_SHORT());

            -- Drop tmp table if exists
            SET @stmt_drop = CONCAT('DROP TABLE IF EXISTS ', @new_table);
            PREPARE stmt_drop FROM @stmt_drop;
            EXECUTE stmt_drop;

            -- Create new tmp table like orig. table
            SET @stmt_create = CONCAT('CREATE TABLE ', @new_table,' LIKE ', tablename);
            PREPARE stmt_create FROM @stmt_create;
            EXECUTE stmt_create;

            -- Alter created tmp table
            SET @stmt_alter = CONCAT('ALTER TABLE ', @new_table, ' ADD ', constrainttype, ' KEY ', constraintname, ' (', constraintinfo, ')');
            PREPARE stmt_alter FROM @stmt_alter;
            EXECUTE stmt_alter;	

            -- Insert IGNORE data form orig. to tmp table
            SET @stmt_insert = CONCAT('INSERT IGNORE INTO ', @new_table, ' SELECT * FROM ', tablename);
            PREPARE stmt_insert FROM @stmt_insert;
            EXECUTE stmt_insert;	

            -- Drop *__bak table if exists
            SET @stmt_drop_bak = CONCAT('DROP TABLE IF EXISTS ', tablename, '__bak');
            PREPARE stmt_drop_bak FROM @stmt_drop_bak;
            EXECUTE stmt_drop_bak;

            -- Rename orig. table to *__bak
            SET @stmt_bak = CONCAT('RENAME TABLE ', tablename, ' TO ', tablename, '__bak');
            PREPARE stmt_bak FROM @stmt_bak;
            EXECUTE stmt_bak;

            -- Rename tmp to orig. table
            SET @stmt_rename = CONCAT('RENAME TABLE ', @new_table, ' TO ', tablename);
            PREPARE stmt_rename FROM @stmt_rename;
            EXECUTE stmt_rename;		   

        ELSE
            -- Simple alter table
            SET @stmt_alter = CONCAT('ALTER TABLE ', tablename, ' ADD ', constrainttype, ' KEY ', constraintname, ' (', constraintinfo, ')');
            PREPARE stmt_alter FROM @stmt_alter;
            EXECUTE stmt_alter;		
        END IF;
    END IF;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Update extras views
-- --------------------------------------------------------------------------------------------------
-- @todo Remove the DROP command below, after some time
DROP PROCEDURE IF EXISTS update_extras_views;
-- EOC

DROP PROCEDURE IF EXISTS update_extras_views_do;
-- EOC

DELIMITER $$
CREATE PROCEDURE update_extras_views_do()
BEGIN
        -- Views to update
        CREATE OR REPLACE
         VIEW `externalexpenses_addi_merged`  AS
         SELECT *, 1+0 AS is_invoiced
         FROM zusatz_extkosten
         UNION
         SELECT *, 0 AS is_invoiced
         FROM zusatz_geextkosten;

        CREATE OR REPLACE
         VIEW `fees_addi_merged`  AS
         SELECT *, 1+0 AS is_invoiced
         FROM zusatz_honverw
         UNION
         SELECT *, 0 AS is_invoiced
         FROM zusatz_gehonverw;

END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Fill calender table for per day selects
--
-- start_date       Start date to add
-- end_date         End date to add
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_add_to_calendar;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_add_to_calendar(start_date DATE, end_date DATE)
BEGIN

    DECLARE curr_date DATE;
    SET curr_date=start_date;
    WHILE curr_date <= end_date DO
        INSERT IGNORE INTO `u10_calendar` VALUES(curr_date, CONCAT(MONTH(curr_date),'_',YEAR(curr_date)));
        SET curr_date = ADDDATE(curr_date, INTERVAL 1 DAY);
    END WHILE;

END $$
DELIMITER ;
-- EOC


-- --------------------------------------------------------------------------------------------------
-- Helper procedures for maintenance tools (Clean lookup tables)
-- 
-- Validate lut_fee_areas_proj
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_update_lost_lut_fee_areas_proj;
-- EOC

DELIMITER $$
CREATE PROCEDURE us_update_lost_lut_fee_areas_proj(offsett INTEGER, step INTEGER, projects_cnt INTEGER, OUT validation_string VARCHAR(4))
BEGIN
	DECLARE xproject_id INTEGER;
	DECLARE xwork_area_category_id INTEGER;
	DECLARE l_last_row_fetched_projects INTEGER;
	DECLARE cur_projects CURSOR FOR SELECT id, work_area_category_id FROM tmp_cur_projects;
	DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched_projects=1;
		
	IF projects_cnt = 0 THEN
		SET @s = CONCAT('SELECT COUNT(id) AS cnt FROM projects');
		PREPARE stmt FROM @s;
		EXECUTE stmt;	
	ELSE		
		-- create TEMPORARY TABLE
		DROP TABLE IF EXISTS tmp_cur_projects;
		SET @query = CONCAT('CREATE TEMPORARY TABLE tmp_cur_projects as select id, work_area_category_id from projects ORDER BY id ASC LIMIT ',offsett,', ',step);
		select @query;
		PREPARE stmt from @query;
		EXECUTE stmt;
		DEALLOCATE PREPARE stmt;	
	
		SET l_last_row_fetched_projects=0;
		OPEN cur_projects;
		cursor_loop: LOOP -- -------------------------------------
			FETCH cur_projects INTO xproject_id, xwork_area_category_id;
			IF l_last_row_fetched_projects = 1 THEN
				LEAVE cursor_loop;
			END IF;
				/*Do something with the row fetched*/
				CALL _us_update_lost_lut_fee_areas_proj1(xproject_id, xwork_area_category_id);
		END LOOP cursor_loop; -- -------------------------------------
		CLOSE  cur_projects;
		SET l_last_row_fetched_projects=0;	

		-- remove TEMPORARY TABLE
		DROP TABLE tmp_cur_projects;

	END IF;	
        SET validation_string = 'true';			
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS _us_update_lost_lut_fee_areas_proj1;
-- EOC

DELIMITER $$
CREATE PROCEDURE _us_update_lost_lut_fee_areas_proj1(IN xproject_id INTEGER, IN xwork_area_category_id INTEGER)
BEGIN
	DECLARE xfee_area_id INTEGER;
	DECLARE l_last_row_fetched INTEGER;
	DECLARE cur_work_areas CURSOR FOR 
		SELECT wa.fee_area_id AS fee_area_id FROM work_areas AS wa WHERE wa.id IN(SELECT work_area_id FROM lut_work_areas_work_areas_categories WHERE work_area_categories_id=xwork_area_category_id);
	DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

	SET l_last_row_fetched=0;
	OPEN cur_work_areas;
	cursor_loop: LOOP -- -------------------------------------
		FETCH cur_work_areas INTO xfee_area_id;
		IF l_last_row_fetched=1 THEN
			LEAVE cursor_loop;
		END IF;
			/*Do something with the row fetched*/
			CALL _us_update_lost_lut_fee_areas_proj2(xproject_id, xfee_area_id);
	END LOOP cursor_loop; -- -------------------------------------
	CLOSE  cur_work_areas;
	SET l_last_row_fetched=0;	
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS _us_update_lost_lut_fee_areas_proj2;
-- EOC

DELIMITER $$
CREATE PROCEDURE _us_update_lost_lut_fee_areas_proj2(IN xproject_id INTEGER, IN xfee_area_id INTEGER)
BEGIN
	DECLARE xcnt_fee_areas INTEGER;
	DECLARE l_last_row_fetched INTEGER;
	DECLARE cur_fee_areas CURSOR FOR 
		SELECT COUNT(fee_area_id) AS cnt_fee_areas FROM lut_fee_areas_proj WHERE project_id=xproject_id AND fee_area_id=xfee_area_id; 
	DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

	SET l_last_row_fetched=0;
	OPEN cur_fee_areas;
	cursor_loop: LOOP -- -------------------------------------
		FETCH cur_fee_areas INTO xcnt_fee_areas;
		IF l_last_row_fetched=1 THEN
			LEAVE cursor_loop;
		END IF;
			IF  xcnt_fee_areas=0 THEN
				/*Do insert*/
				INSERT  INTO lut_fee_areas_proj(`project_id`, `fee_area_id`, `costs`, `hours`, `status`)
				VALUES  (xproject_id, xfee_area_id, 0, 0, 20);				
			END IF;
	END LOOP cursor_loop; -- -------------------------------------
	CLOSE  cur_fee_areas;
	SET l_last_row_fetched=0;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Helper procedures for maintenance tools (Clean lookup tables)
-- 
-- Validate budget plan and completion levels
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_update_lost_budget_plan_completion_levels;
-- EOC

DELIMITER $$
CREATE PROCEDURE us_update_lost_budget_plan_completion_levels(offsett INTEGER, step INTEGER, projects_cnt INTEGER, OUT validation_string VARCHAR(4))
BEGIN
	DECLARE xproject_id INTEGER;
	DECLARE xwork_area_category_id INTEGER;
	DECLARE l_last_row_fetched_projects INTEGER;
	DECLARE cur_projects CURSOR FOR SELECT id, work_area_category_id FROM tmp_cur_projects;
	DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched_projects=1;
		
	IF projects_cnt = 0 THEN
		SET @s = CONCAT('SELECT COUNT(id) AS cnt FROM projects');
		PREPARE stmt FROM @s;
		EXECUTE stmt;	
	ELSE		
		-- create TEMPORARY TABLE
		DROP TABLE IF EXISTS tmp_cur_projects;
		SET @query = CONCAT('CREATE TEMPORARY TABLE tmp_cur_projects as select id, work_area_category_id  from projects ORDER BY id ASC LIMIT ',offsett,', ',step);
		select @query;
		PREPARE stmt from @query;
		EXECUTE stmt;
		DEALLOCATE PREPARE stmt;	
	
		SET l_last_row_fetched_projects=0;
		OPEN cur_projects;
		cursor_loop: LOOP -- -------------------------------------
			FETCH cur_projects INTO xproject_id, xwork_area_category_id;
			IF l_last_row_fetched_projects = 1 THEN
				LEAVE cursor_loop;
			END IF;
				/*Do something with the row fetched*/
				CALL _us_update_lost_budget_plan_completion_levels1(xproject_id, xwork_area_category_id);
				CALL _us_update_lost_budget_plan_completion_levels2_bp(xproject_id, xwork_area_category_id);
				CALL _us_update_lost_budget_plan_completion_levels2_cl(xproject_id, xwork_area_category_id);
		END LOOP cursor_loop; -- -------------------------------------
		CLOSE  cur_projects;
		SET l_last_row_fetched_projects=0;	

		-- remove TEMPORARY TABLE
		DROP TABLE tmp_cur_projects;

	END IF;	

        SET validation_string = 'true';			
END $$
DELIMITER ;
-- EOC

-- ----------------------------------------
-- Ensure we have values for each work area 
-- ----------------------------------------
DROP PROCEDURE IF EXISTS _us_update_lost_budget_plan_completion_levels1;
-- EOC

DELIMITER $$
CREATE PROCEDURE _us_update_lost_budget_plan_completion_levels1(IN xproject_id INTEGER, IN xwork_area_category_id INTEGER)
BEGIN
	DECLARE xwork_area_id INTEGER;
	DECLARE l_last_row_fetched INTEGER;
	DECLARE cur_work_areas CURSOR FOR 
		SELECT wa.id AS work_area_id FROM work_areas AS wa WHERE wa.id IN(SELECT work_area_id FROM lut_work_areas_work_areas_categories WHERE work_area_categories_id=xwork_area_category_id);
	DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

	SET l_last_row_fetched=0;
	OPEN cur_work_areas;
	cursor_loop: LOOP -- -------------------------------------
		FETCH cur_work_areas INTO xwork_area_id;
		IF l_last_row_fetched=1 THEN
			LEAVE cursor_loop;
		END IF;
			/*Do something with the row fetched */
			CALL _us_update_lost_budget_plan_completion_levels1_bp(xproject_id, xwork_area_id);
			CALL _us_update_lost_budget_plan_completion_levels1_cl(xproject_id, xwork_area_id);
	END LOOP cursor_loop; -- -------------------------------------
	CLOSE  cur_work_areas;
	SET l_last_row_fetched=0;	
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS _us_update_lost_budget_plan_completion_levels1_bp;
-- EOC

DELIMITER $$
CREATE PROCEDURE _us_update_lost_budget_plan_completion_levels1_bp(IN xproject_id INTEGER, IN xwork_area_id INTEGER)
BEGIN
	DECLARE xcnt_projects INTEGER;
	DECLARE l_last_row_fetched INTEGER;
	DECLARE cur_fee_areas CURSOR FOR 
		SELECT COUNT(project_id) AS cnt_projects FROM budget_plan WHERE project_id=xproject_id AND area_id=xwork_area_id; 
	DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

	SET l_last_row_fetched=0;
	OPEN cur_fee_areas;
	cursor_loop: LOOP -- -------------------------------------
		FETCH cur_fee_areas INTO xcnt_projects;
		IF l_last_row_fetched=1 THEN
			LEAVE cursor_loop;
		END IF;
			IF  xcnt_projects=0 THEN
				/*Do insert*/
				INSERT  INTO budget_plan(`project_id`, `area_id`, `staff_id`, `last_changed_date`, `planned_costs`, `planned_hours`,
				`start_date`, `end_date`, `proportion`, `threshold`, `budget_plan_history_id`)
				VALUES  (xproject_id, xwork_area_id, 0, null, 0, 0, null, null, 0, 0, null);				
			END IF;
	END LOOP cursor_loop; -- -------------------------------------
	CLOSE  cur_fee_areas;
	SET l_last_row_fetched=0;
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS _us_update_lost_budget_plan_completion_levels1_cl;
-- EOC

DELIMITER $$
CREATE PROCEDURE _us_update_lost_budget_plan_completion_levels1_cl(IN xproject_id INTEGER, IN xwork_area_id INTEGER)
BEGIN
	DECLARE xcnt_projects INTEGER;
	DECLARE l_last_row_fetched INTEGER;
	DECLARE cur_fee_areas CURSOR FOR 
		SELECT COUNT(project_id) AS cnt_projects FROM completion_level WHERE project_id=xproject_id AND work_area_id=xwork_area_id; 
	DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

	SET l_last_row_fetched=0;
	OPEN cur_fee_areas;
	cursor_loop: LOOP -- -------------------------------------
		FETCH cur_fee_areas INTO xcnt_projects;
		IF l_last_row_fetched=1 THEN
			LEAVE cursor_loop;
		END IF;
			IF  xcnt_projects=0 THEN
				/*Do insert*/
				INSERT  INTO completion_level(`project_id`, `work_area_id`, `staff_id`, `date`, `completion_level`, `completion_level_history_id`)
				VALUES  (xproject_id, xwork_area_id, 0, null, 0, null);				
			END IF;
	END LOOP cursor_loop; -- -------------------------------------
	CLOSE  cur_fee_areas;
	SET l_last_row_fetched=0;
END $$
DELIMITER ;
-- EOC

-- -------------------------------------------------------------
-- Ensure we remove work areas which are not part of the project
-- -------------------------------------------------------------
DROP PROCEDURE IF EXISTS _us_update_lost_budget_plan_completion_levels2_bp;
-- EOC

DELIMITER $$
CREATE PROCEDURE _us_update_lost_budget_plan_completion_levels2_bp(IN xproject_id INTEGER, IN xwork_area_category_id INTEGER)
BEGIN
        DELETE FROM budget_plan
        WHERE project_id=xproject_id AND
        area_id NOT IN (
            SELECT wa.id AS work_area_id FROM work_areas AS wa WHERE wa.id IN(SELECT work_area_id FROM lut_work_areas_work_areas_categories WHERE work_area_categories_id=xwork_area_category_id)
        );	
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS _us_update_lost_budget_plan_completion_levels2_cl;
-- EOC

DELIMITER $$
CREATE PROCEDURE _us_update_lost_budget_plan_completion_levels2_cl(IN xproject_id INTEGER, IN xwork_area_category_id INTEGER)
BEGIN
        DELETE FROM completion_level
        WHERE project_id=xproject_id AND
        work_area_id NOT IN (
            SELECT wa.id AS work_area_id FROM work_areas AS wa WHERE wa.id IN(SELECT work_area_id FROM lut_work_areas_work_areas_categories WHERE work_area_categories_id=xwork_area_category_id)
        );	
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Clean chars
--
-- tablename    Table name
-- ismultiline  Is a multi line field
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_clean_chars_column;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_clean_chars_column(IN tablename VARCHAR(255), IN colname VARCHAR(100) CHARSET utf8, IN ismultiline BOOLEAN)
BEGIN
    DECLARE char_i INTEGER;
    
    SET char_i = 0;
    char_loop: LOOP
        SET char_i = char_i + 1;

        IF char_i > 31 THEN
            LEAVE char_loop;
        END IF;

        IF char_i = 9 THEN -- TAB
            ITERATE char_loop;
        END IF;
        IF ismultiline AND char_i = 10 THEN -- LF
            ITERATE char_loop;
        END IF;
        IF ismultiline AND char_i = 13 THEN -- CR = ENTER
            ITERATE char_loop;
        END IF;

        -- CLEAN THA SHIT
        SET @s = CONCAT('UPDATE ', tablename, ' SET ', colname, ' = replace( ', colname, ', char(', char_i,'), "");');
        PREPARE stmt2 FROM @s;
        EXECUTE stmt2;

    END LOOP char_loop;

    
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS us_clean_chars;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_clean_chars(IN tablename VARCHAR(255), IN ismultiline BOOLEAN, OUT validation_string VARCHAR(4))
BEGIN
    DECLARE num_rows INTEGER;
    DECLARE i INTEGER;
    DECLARE col_name VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE dt_type VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;

    DECLARE col_names CURSOR FOR
        SELECT column_name, data_type
        FROM INFORMATION_SCHEMA.COLUMNS
        WHERE table_name = tablename
        AND table_schema = SCHEMA();
    
    OPEN col_names;

    SELECT FOUND_ROWS() INTO num_rows;

    SET i = 1;
    col_name_loop: LOOP
        IF i > num_rows THEN
            CLOSE col_names;
            LEAVE col_name_loop;
        END IF;
        
        FETCH col_names INTO col_name, dt_type;
        
        IF (dt_type='varchar' OR dt_type='longtext' OR dt_type='text') AND col_name NOT IN('Index') THEN

            CALL us_clean_chars_column(tablename, col_name, ismultiline);

        END IF;

        SET i = i + 1;
    END LOOP col_name_loop;
    SET validation_string = 'true';		
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS us_update_correspondence_attachment_number;
-- EOC

DELIMITER $$
CREATE PROCEDURE us_update_correspondence_attachment_number(IN corr_id INTEGER, OUT validation_string VARCHAR(4))
BEGIN
	DECLARE num_rows INTEGER;
	DECLARE thequery CURSOR FOR
	     SELECT id
	     FROM correspondence_attachments
	     WHERE correspondence_id = corr_id;
	OPEN thequery;     
	SELECT FOUND_ROWS() INTO num_rows;
	CLOSE thequery;
	UPDATE correspondence SET attachments=num_rows WHERE id = corr_id;
	SELECT num_rows AS `DEBUG`; 
        SET validation_string = 'true';
END $$
DELIMITER ;
-- EOC

DROP FUNCTION IF EXISTS `us_replace`;
-- EOC

-- If the creation of us_replace fails,
-- activate the log_bin_trust_function_creators within you MySQL config.
DELIMITER $$
CREATE FUNCTION `us_replace` (str TEXT CHARSET utf8, search_str TEXT,replace_str TEXT,ci BOOL)
    RETURNS TEXT CHARSET utf8
    DETERMINISTIC
    READS SQL DATA
    BEGIN
        DECLARE return_str TEXT CHARSET utf8 COLLATE utf8_general_ci DEFAULT '';
        DECLARE lower_str TEXT CHARSET utf8 COLLATE utf8_general_ci;
        DECLARE lower_search_str TEXT CHARSET utf8 COLLATE utf8_general_ci;
        DECLARE pos INT DEFAULT 1;
        DECLARE old_pos INT DEFAULT 1;
        
        IF ci = false THEN
            return replace(str, search_str, replace_str);	                     
        END IF;        
        
        SELECT lower(str) INTO lower_str;
        SELECT lower(search_str) INTO lower_search_str;
        SELECT locate(lower_search_str, lower_str, pos) INTO pos;
        WHILE pos > 0 DO
            SELECT concat(return_str, substr(str, old_pos, pos-old_pos), replace_str) INTO return_str;
            SELECT pos + char_length(search_str) INTO pos;
            SELECT pos INTO old_pos;
            SELECT locate(lower_search_str, lower_str, pos) INTO pos;
        END WHILE;
        SELECT concat(return_str, substr(str, old_pos, char_length(str))) INTO return_str;
        RETURN return_str;
END$$
DELIMITER ;
-- EOC

DROP FUNCTION IF EXISTS `us_replace_if_max_length`;
-- EOC

DELIMITER $$
CREATE FUNCTION `us_replace_if_max_length` ( str TEXT,search_str TEXT,replace_str TEXT,ci BOOL, max_length INT)
    RETURNS TEXT CHARSET utf8
    DETERMINISTIC
    READS SQL DATA
    BEGIN
        DECLARE return_str TEXT CHARSET utf8 COLLATE utf8_general_ci DEFAULT '';

        SELECT us_replace(str, search_str, replace_str, ci) INTO return_str;
        if char_length(return_str) > max_length THEN 
            SELECT str INTO return_str;
        END IF;

        RETURN return_str;
END$$
DELIMITER ;
-- EOC


DROP FUNCTION IF EXISTS us_delete_spaces;
-- EOC
DELIMITER $$
CREATE FUNCTION us_delete_spaces(str VARCHAR(255)) 
    RETURNS VARCHAR(255) CHARSET utf8
    DETERMINISTIC
    NO SQL
    BEGIN
        while instr(str, '  ') > 0 do
        set str = replace(str, '  ', ' ');
        end while;
        return trim(str);
END $$
DELIMITER ;
-- EOC


DROP FUNCTION IF EXISTS `us_wrap`;
-- EOC

DELIMITER $$
CREATE FUNCTION `us_wrap` (str TEXT CHARSET utf8, search_str TEXT,wrap_left TEXT,wrap_right TEXT,ci BOOL)
    RETURNS TEXT CHARSET utf8
    DETERMINISTIC
    READS SQL DATA
    BEGIN
        DECLARE val TEXT CHARSET utf8 COLLATE utf8_general_ci DEFAULT '';
        DECLARE return_str TEXT CHARSET utf8 COLLATE utf8_general_ci DEFAULT '';
        DECLARE lower_str TEXT CHARSET utf8 COLLATE utf8_general_ci;
        DECLARE lower_search_str TEXT CHARSET utf8 COLLATE utf8_general_ci;
        DECLARE pos INT DEFAULT 1;
        DECLARE old_pos INT DEFAULT 1;
        
        IF ci = true THEN
            SELECT lower(str) INTO lower_str;
            SELECT lower(search_str) INTO lower_search_str;	   
            SELECT locate(lower_search_str, lower_str, pos) INTO pos;	             
        ELSE
            SELECT str INTO lower_str;
            SELECT search_str INTO lower_search_str;	      
            SELECT locate(BINARY lower_search_str, BINARY lower_str, pos) INTO pos;	        
        END IF;
	      
        WHILE pos > 0 DO
            SELECT substr(str, pos, old_pos+char_length(search_str)-old_pos) INTO val;
            SELECT concat(return_str, substr(str, old_pos, pos-old_pos), wrap_left, val, wrap_right) INTO return_str;
            SELECT pos + char_length(search_str) INTO pos;
            SELECT pos INTO old_pos;
            -- SELECT locate(lower_search_str, lower_str, pos) INTO pos;
            IF ci = true THEN
                    SELECT locate(lower_search_str, lower_str, pos) INTO pos;	             
            ELSE
                    SELECT locate(BINARY lower_search_str, BINARY lower_str, pos) INTO pos;        
            END IF;            
        END WHILE;
        SELECT concat(return_str, substr(str, old_pos, char_length(str))) INTO return_str;
        RETURN return_str;
END$$
DELIMITER ;
-- EOC


-- --------------------------------------------------------------------------------------------------
-- Create move_form_types_log_archive, if not exists
--
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_create_move_form_types_log_archive;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_create_move_form_types_log_archive()
BEGIN

    DECLARE sql1 TEXT DEFAULT 'CREATE TABLE IF NOT EXISTS `move_form_types_log_archive` (
    `id` INT( 11 ) NOT NULL ,
    `moving_session_id` VARCHAR( 50 ) NOT NULL,
    `project_id` INT( 11 ) NOT NULL ,
    `correspondence_id` INT( 11 ) NOT NULL ,
    `correspondence_type_id` INT( 11 ) NOT NULL ,
    `project_path_correspondence` longtext NOT NULL,
    `correspondence_structure_path` VARCHAR( 254 ) NOT NULL,
    `filename` VARCHAR( 254 ) NOT NULL,
    `path_from` longtext NOT NULL,
    `path_to` longtext NOT NULL,
    `created_date` DATETIME NOT NULL,
    `created_by_id` INT( 11 ) NOT NULL
) ENGINE=';

    DECLARE sql2 TEXT DEFAULT '  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;';

    IF EXISTS( (SELECT * FROM information_schema.ENGINES WHERE ENGINE='ARCHIVE' AND SUPPORT='YES') ) THEN
        SET @s = CONCAT(sql1, 'ARCHIVE', sql2);
    ELSE
        SET @s = CONCAT(sql1, 'MyISAM', sql2);
    END IF;
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Create system_modifications_log, if not exists
--
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_create_system_modifications_log;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_create_system_modifications_log()
BEGIN
    DECLARE sql1 TEXT DEFAULT 'CREATE TABLE IF NOT EXISTS `system_modifications_log` (
    `datetime` DATETIME NOT NULL,

    `us_serial` VARCHAR( 255 ) DEFAULT NULL ,
    `us_instinfo_hash` VARCHAR( 50 ) DEFAULT NULL ,
    `us_version` VARCHAR( 50 ) DEFAULT NULL ,
    `us_last_update` VARCHAR( 20 ) DEFAULT NULL ,
    `us_datapath` VARCHAR( 255 ) DEFAULT NULL ,
    `us_datapath_writeable` VARCHAR( 10 ) DEFAULT NULL ,
    `us_advanced_is_readable` VARCHAR( 10 ) DEFAULT NULL ,
    `us_base_url` VARCHAR( 255 ) DEFAULT NULL ,

    `apache_os` VARCHAR( 50 ) DEFAULT NULL ,
    `apache_on_host` VARCHAR( 100 ) DEFAULT NULL ,
    `apache_on_port` VARCHAR( 10 ) DEFAULT NULL ,
    `apache_version` VARCHAR( 50 ) DEFAULT NULL ,

    `php_version` VARCHAR( 50 ) DEFAULT NULL ,

    `mysql_version` VARCHAR( 50 ) DEFAULT NULL ,
    `mysql_sql_mode` VARCHAR( 255 ) DEFAULT NULL ,
    `mysql_on_host` VARCHAR( 100 ) DEFAULT NULL ,
    `mysql_on_port` VARCHAR( 10 ) DEFAULT NULL ,
    `mysql_database` VARCHAR( 50 ) DEFAULT NULL ,
    `tempmysql_on_host` VARCHAR( 100 ) DEFAULT NULL ,
    `tempmysql_on_port` VARCHAR( 10 ) DEFAULT NULL ,
    `tempmysql_database` VARCHAR( 50 ) DEFAULT NULL ,
    
    `java_path` VARCHAR( 255 ) DEFAULT NULL ,
    `java_executable` VARCHAR( 10 ) DEFAULT NULL ,
    `java_version` VARCHAR( 100 ) DEFAULT NULL ,

    `libre_path` VARCHAR( 255 ) DEFAULT NULL ,
    `libre_temppath` VARCHAR( 10 ) DEFAULT NULL ,
    `libre_executable` VARCHAR( 10 ) DEFAULT NULL ,

    `proxy_host` VARCHAR( 100 ) DEFAULT NULL ,
    `proxy_port` VARCHAR( 10 ) DEFAULT NULL ,
    `proxy_reachable` VARCHAR( 10 ) DEFAULT NULL ,

    `mof_installed` VARCHAR( 10 ) DEFAULT NULL ,
    `mof_version` VARCHAR( 50 ) DEFAULT NULL ,
    `mof_as_server` VARCHAR( 10 ) DEFAULT NULL ,
    `mof_server_version` VARCHAR( 255 ) DEFAULT NULL ,
    `mof_server_reachable` VARCHAR( 10 ) DEFAULT NULL ,

    `enough_diskspace` VARCHAR( 10 ) DEFAULT NULL
) ENGINE=';
    DECLARE sql2 TEXT DEFAULT '  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;';

    IF EXISTS( (SELECT * FROM information_schema.ENGINES WHERE ENGINE='ARCHIVE' AND SUPPORT='YES') ) THEN
        SET @s = CONCAT(sql1, 'ARCHIVE', sql2);
    ELSE
        SET @s = CONCAT(sql1, 'MyISAM', sql2);
    END IF;
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    CALL us_add_to_table2('system_modifications_log', 'apache_user', 'VARCHAR( 50 ) DEFAULT NULL AFTER `apache_os`' );

    SET @s2 = 'ALTER TABLE `system_modifications_log` CHANGE COLUMN `apache_version` `apache_version` VARCHAR( 150 ) DEFAULT NULL;';
    PREPARE stmt2 FROM @s2;
    EXECUTE stmt2;

END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Migrate projekte to lut_measurements
--
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_projekte_to_lut_measurements;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_projekte_to_lut_measurements()
BEGIN

    IF NOT EXISTS( (SELECT * FROM lut_measurements) ) THEN
        PREPARE stmt1 FROM '
            INSERT INTO lut_measurements (project_id, measurement_id, value) 
            SELECT LfdNr, MessSize_LfdNr, MessSize_Value
            FROM projekte WHERE MessSize_LfdNr>0 AND MessSize_Value IS NOT NULL
        ';
        EXECUTE stmt1;
    END IF;

END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Cleans project paths - UNUSED AT THE MOMENT BUT MAY BE USEFUL SOMEDAY
--
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_clean_project_paths;
-- EOC

DELIMITER $$
CREATE PROCEDURE us_clean_project_paths()
BEGIN
    DECLARE x_fields TEXT DEFAULT 'path_correspondence,path_correspondence_default,path_inbox,path_inbox_default,path_outbox,path_outbox_default,path_drawings,path_drawings_default,orig_structure_tree_root_node,orig_translated_structure_tree_root_node';
    DECLARE current_field TEXT CHARSET utf8 COLLATE utf8_general_ci DEFAULT '';
    
    SET x_fields = concat(x_fields, ',');
    WHILE (LOCATE(',', x_fields) > 0)
    DO
        SET current_field = LEFT(x_fields, LOCATE(',',x_fields) - 1);    
        SET x_fields = SUBSTRING(x_fields, LOCATE(',',x_fields) + 1);

        SET @sql = CONCAT('UPDATE projects SET ',current_field,' = REPLACE(',current_field,', "\\\\", "/")
                WHERE (',current_field,' LIKE "./%" OR ',current_field,' LIKE "/%") AND ',current_field,' NOT LIKE "//%";');
        PREPARE stmt FROM @sql;
        EXECUTE stmt;
        DEALLOCATE PREPARE stmt;					

        SET @sql = CONCAT('UPDATE projects SET ',current_field,' = CONCAT(TRIM(TRAILING "/" FROM ',current_field,'), "/")
                WHERE (',current_field,' LIKE "./%" OR ',current_field,' LIKE "/%") AND ',current_field,' NOT LIKE "//%";');
        PREPARE stmt FROM @sql;
        EXECUTE stmt;
        DEALLOCATE PREPARE stmt;

        SET @sql = CONCAT('UPDATE projects SET ',current_field,' = REPLACE(',current_field,', "/", "\\\\")
                WHERE ',current_field,' LIKE "%\\\\\%" OR ',current_field,' REGEXP "^[a-zA-Z]" OR ',current_field,' LIKE "//%";');
        PREPARE stmt FROM @sql;
        EXECUTE stmt;
        DEALLOCATE PREPARE stmt;

        SET @sql = CONCAT('UPDATE projects SET ',current_field,' = CONCAT(TRIM(TRAILING "\\\\" FROM ',current_field,'), "\\\\")
                WHERE ',current_field,' LIKE "%\\\\\%";');
        PREPARE stmt FROM @sql;
        EXECUTE stmt;
        DEALLOCATE PREPARE stmt;
    END WHILE;
END $$
DELIMITER ;
-- EOC


-- --------------------------------------------------------------------------------------------------
-- Create cache_project_measurement
--
-- Set either filter_project_id_from and filter_project_id_to to a project ID     OR
-- filter_project_id_from=1 AND filter_project_id_from=~0 >> 33
-- ~0 >> 33 equeals max value for signed integer
-- always add parameter @validate_outinfo
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_create_cache_project_measurement;
-- EOC

DELIMITER $$
CREATE PROCEDURE us_create_cache_project_measurement(IN filter_project_id_from INTEGER, IN filter_project_id_to INTEGER, OUT validation_string VARCHAR(4))
BEGIN

    DECLARE xproject_id INTEGER;
    DECLARE xvalue DOUBLE;
    DECLARE xname VARCHAR(50) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE xname1 VARCHAR(50) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE xname2 VARCHAR(50) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE xname3 VARCHAR(50) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE xname4 VARCHAR(50) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE xname5 VARCHAR(50) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE l_last_row_fetched INTEGER;

    DECLARE measurements CURSOR FOR
        SELECT 
		lm.project_id,
		lm.value,
		m.name,
		lam1.name AS trans_name1,
		lam2.name AS trans_name2,
		lam3.name AS trans_name3,
		lam4.name AS trans_name4,
		lam5.name AS trans_name5

	FROM
		lut_measurements as lm
			LEFT JOIN
		measurements AS m ON lm.measurement_id = m.id
			LEFT JOIN
		langs_measurements AS lam1 ON lm.measurement_id = lam1.measurements_id AND lam1.language='german'
			LEFT JOIN
		langs_measurements AS lam2 ON lm.measurement_id = lam2.measurements_id AND lam2.language='germanarch'
			LEFT JOIN
		langs_measurements AS lam3 ON lm.measurement_id = lam3.measurements_id AND lam3.language='germanch'
			LEFT JOIN
		langs_measurements AS lam4 ON lm.measurement_id = lam4.measurements_id AND lam4.language='english'
			LEFT JOIN
		langs_measurements AS lam5 ON lm.measurement_id = lam5.measurements_id AND lam5.language='french'
        WHERE lm.project_id BETWEEN filter_project_id_from AND filter_project_id_to;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    SET @query_rem = CONCAT('DELETE FROM cache_project_measurement WHERE project_id BETWEEN ',filter_project_id_from,' AND ',filter_project_id_to);
    PREPARE stmt_rem FROM @query_rem;
    EXECUTE stmt_rem;
    DEALLOCATE PREPARE stmt_rem;
    
    SET @query = '
        INSERT INTO cache_project_measurement SET project_id=?, 
            name_german=?,
            name_germanarch=?,
            name_germanch=?,
            name_english=?,
            name_french=? 
        ON DUPLICATE KEY UPDATE 
            name_german=LEFT( CONCAT(name_german, ", ", ?), 480),
            name_germanarch=LEFT( CONCAT(name_germanarch, ", ", ?), 480),
            name_germanch=LEFT( CONCAT(name_germanch, ", ", ?), 480),
            name_english=LEFT( CONCAT(name_english, ", ", ?), 480),
            name_french=LEFT( CONCAT(name_french, ", ", ?), 480)
    ';
    PREPARE stmt FROM @query;

    SET l_last_row_fetched=0;
    OPEN measurements;
    cursor_loop: LOOP -- -------------------------------------
        FETCH measurements INTO xproject_id, xvalue, xname, xname1, xname2, xname3, xname4, xname5;

        -- Catch error, when a measurement was deleted. Should only happen on internal demo test server
        SET xname = IFNULL(xname, ' ');

        IF l_last_row_fetched=1 THEN
                LEAVE cursor_loop;
        END IF;
        
        SET @project_id=xproject_id;
        SET @name1=CONCAT(  ROUND(xvalue),  ' ',    LEFT(IFNULL(xname1, xname), 15));
        SET @name2=CONCAT(  ROUND(xvalue),  ' ',    LEFT(IFNULL(xname2, xname), 15));
        SET @name3=CONCAT(  ROUND(xvalue),  ' ',    LEFT(IFNULL(xname3, xname), 15));
        SET @name4=CONCAT(  ROUND(xvalue),  ' ',    LEFT(IFNULL(xname4, xname), 15));
        SET @name5=CONCAT(  ROUND(xvalue),  ' ',    LEFT(IFNULL(xname5, xname), 15));
        EXECUTE stmt USING @project_id, 
            @name1, @name2, @name3, @name4, @name5,
            @name1, @name2, @name3, @name4, @name5;

    END LOOP cursor_loop; -- -------------------------------------
    CLOSE measurements;
    SET l_last_row_fetched=0;

    DEALLOCATE PREPARE stmt;

    SET validation_string = 'true';
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Add PRIMARY and AUTO_INCREMENT to table, if not exists
--
-- tablename    Table name
-- fieldname    Field name
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_add_pk_auto_increment;
-- EOC

DELIMITER $$
CREATE PROCEDURE us_add_pk_auto_increment(IN tablename VARCHAR(255), IN fieldname VARCHAR(255))
BEGIN
IF NOT EXISTS (
        SELECT NULL FROM information_schema.TABLE_CONSTRAINTS WHERE
        CONSTRAINT_SCHEMA = DATABASE() AND
        CONSTRAINT_TYPE = 'PRIMARY KEY' AND
        TABLE_NAME = tablename
    ) THEN
    -- recreate id column with PK if PK is not set
    
    SET @s = CONCAT('ALTER TABLE ', tablename, ' DROP ', fieldname);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    SET @s = CONCAT('ALTER TABLE ', tablename, ' ADD COLUMN ', fieldname, ' INT(11) NOT NULL PRIMARY KEY AUTO_INCREMENT FIRST');
    PREPARE stmt2 FROM @s;
    EXECUTE stmt2;
ELSE
    -- just add AUTO_INCREMENT if not set
    SET @s = CONCAT('ALTER TABLE ', tablename, ' CHANGE COLUMN ', fieldname, ' ', fieldname, ' INT(11) NOT NULL AUTO_INCREMENT FIRST');
    PREPARE stmt3 FROM @s;
    EXECUTE stmt3;
END IF;
END $$
DELIMITER ;
-- EOC


-- --------------------------------------------------------------------------------------------------
-- Add AUTO_INCREMENT to table, if not exists
--
-- tablename    Table name
-- fieldname    Field name
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_add_auto_increment;
-- EOC

DELIMITER $$
CREATE PROCEDURE us_add_auto_increment(IN tablename VARCHAR(255), IN fieldname VARCHAR(255))
BEGIN
IF NOT EXISTS (
        SELECT NULL FROM information_schema.TABLE_CONSTRAINTS WHERE
        CONSTRAINT_SCHEMA = DATABASE() AND
        CONSTRAINT_TYPE = 'PRIMARY KEY' AND
        TABLE_NAME = tablename
    ) THEN
    -- no recreate id column with PK if PK is not set
    CALL raise_error__Missing_PRIMARY_to_set_AUTO_INCREMENT;
ELSE
    -- just add AUTO_INCREMENT if not set
    SET @s = CONCAT('ALTER TABLE ', tablename, ' CHANGE COLUMN ', fieldname, ' ', fieldname, ' INT(11) NOT NULL AUTO_INCREMENT FIRST');
    PREPARE stmt3 FROM @s;
    EXECUTE stmt3;
END IF;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Fill the crm_data_table with values from outbox, inbox and address_desc
-- --------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS us_fill_crm_table;
-- EOC
DROP TABLE IF EXISTS crm_data_table;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Change name of column
-- --------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS check_if_column_exists;
-- EOC
DROP PROCEDURE IF EXISTS check_rename_column_exists;
-- EOC
DELIMITER $$
CREATE PROCEDURE check_rename_column_exists(tablename VARCHAR(255), column_name_old VARCHAR (255), column_name_new VARCHAR (255), column_type VARCHAR (255))
begin
    IF EXISTS((SELECT * FROM information_schema.COLUMNS WHERE TABLE_SCHEMA=DATABASE() and table_name = tablename and COLUMN_NAME LIKE column_name_old)) THEN
        IF EXISTS( (SELECT * FROM information_schema.STATISTICS WHERE table_schema = DATABASE() AND TABLE_NAME=tablename AND INDEX_TYPE='FULLTEXT') ) THEN
            CALL us_drop_all_fulltext_indexes_if_exists(tablename);
        END IF;

        SET @s = CONCAT('ALTER TABLE `', tablename, '` CHANGE COLUMN `', column_name_old, '` `', column_name_new, '` ', column_type);
        PREPARE stmt2 FROM @s;
        EXECUTE stmt2;
    END IF;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Change column
--
-- @todo If the field already exists and there is a fulltext index, this method will always remove the fulltest index. 
-- Either always catch such calls or improve this procedure to not always remove the fulltext index.
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS change_column;
-- EOC
DELIMITER $$
CREATE PROCEDURE change_column(IN tablename VARCHAR(255), IN fieldname VARCHAR(255), IN fieldtype VARCHAR(255))
BEGIN
    IF EXISTS( (SELECT * FROM information_schema.COLUMNS WHERE TABLE_SCHEMA = DATABASE() and table_name=tablename and COLUMN_NAME LIKE fieldname) ) THEN
        IF EXISTS( (SELECT * FROM information_schema.STATISTICS WHERE table_schema = DATABASE() AND TABLE_NAME=tablename AND INDEX_TYPE='FULLTEXT') ) THEN
            CALL us_drop_all_fulltext_indexes_if_exists(tablename);
        END IF;

        SET @s = CONCAT('ALTER TABLE `', tablename, '` MODIFY COLUMN `', fieldname, '` ', fieldtype);
        PREPARE stmt FROM @s;
        EXECUTE stmt;
    END IF;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Remove column
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS remove_column;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_column(tablename VARCHAR(255), column_name_old VARCHAR (255))
begin
    IF EXISTS((SELECT * FROM information_schema.COLUMNS WHERE TABLE_SCHEMA=DATABASE() and table_name = tablename and COLUMN_NAME LIKE column_name_old)) THEN
        SET @s = CONCAT('ALTER TABLE `', tablename, '` DROP COLUMN `', column_name_old, '`');
        PREPARE stmt2 FROM @s;
        EXECUTE stmt2;
    END IF;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Remove column which is a PRIMARY KEY
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS remove_pk_column;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_pk_column(tablename VARCHAR(255), column_name_old VARCHAR (255))
begin
    IF EXISTS((SELECT * FROM information_schema.COLUMNS WHERE TABLE_SCHEMA=DATABASE() and table_name = tablename and COLUMN_NAME LIKE column_name_old)) THEN
        SET @s = CONCAT('ALTER TABLE `', tablename, '` DROP COLUMN `', column_name_old, '`, DROP PRIMARY KEY');
        PREPARE stmt2 FROM @s;
        EXECUTE stmt2;
    END IF;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- PROCEDURES for the nested sets
-- --------------------------------------------------------------------------------------------------

-- INSERT new node

DROP PROCEDURE IF EXISTS us_insert_work_package_presets;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_insert_work_package_presets(IN parent_id VARCHAR(100), IN new_name VARCHAR(100), IN is_purpose BOOLEAN, IN is_special INTEGER, IN is_special_phase INTEGER, IN is_fee_area BOOLEAN, IN threshold DOUBLE, IN is_extern BOOLEAN, IN cost_centre_id INTEGER, IN tag_id INTEGER, OUT insert_id VARCHAR(100))
BEGIN
--   DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
--     BEGIN
--         ROLLBACK;
--         SET autocommit=1;
--     END;

        /* SET by SELECT */
        (SELECT rgt, tree_id 
            INTO @myRight, @myTreeId
            FROM work_package_presets WHERE id = parent_id);

--     SET autocommit=0;
--     START TRANSACTION;
        SET @unique_wp_table_name = replace(CONCAT('work_packages_tmp_', UUID()), '-', '');
        SET @s = CONCAT('CREATE TEMPORARY TABLE IF NOT EXISTS ', @unique_wp_table_name, ' AS (SELECT id AS wp_id, lft, rgt, tree_id FROM work_package_presets WHERE ((rgt >=',@myRight,') 
                        OR (lft >=', @myRight,')) AND tree_id = ', @myTreeId, ')');
        PREPARE stmt1 FROM @s;
        EXECUTE stmt1;

        UPDATE work_package_presets SET lft=lft+1000000 WHERE lft >= @myRight AND tree_id = @myTreeId;
        UPDATE work_package_presets SET rgt=rgt+1000000 WHERE rgt >= @myRight AND tree_id = @myTreeId;

        SET @s = CONCAT('UPDATE ', @unique_wp_table_name, ' SET rgt = rgt + 2 WHERE rgt >=', @myRight, ' AND tree_id = ', @myTreeId);
        PREPARE stmt2 FROM @s;
        EXECUTE stmt2;

        SET @s = CONCAT('UPDATE ', @unique_wp_table_name, ' SET lft = lft + 2 WHERE lft >=', @myRight, ' AND tree_id = ', @myTreeId);
        PREPARE stmt3 FROM @s;
        EXECUTE stmt3;

        INSERT INTO work_package_presets(name, lft, rgt, tree_id, is_purpose_required, is_special, is_special_phase, is_fee_area, threshold, is_extern, cost_centre_id, tag_id) VALUES(new_name, @myRight, @myRight + 1, @myTreeId, is_purpose, is_special, is_special_phase, is_fee_area, threshold, is_extern, cost_centre_id, tag_id);

        SET insert_id = LAST_INSERT_ID();

        SET @s = CONCAT('UPDATE work_package_presets wp INNER JOIN ', @unique_wp_table_name, ' wp_tmp ON wp.id = wp_tmp.wp_id SET wp.lft = wp_tmp.lft, wp.rgt = wp_tmp.rgt');
        PREPARE stmt4 FROM @s;
        EXECUTE stmt4;

        SET @s = CONCAT('DROP TABLE ', @unique_wp_table_name);
        PREPARE stmt5 FROM @s;
        EXECUTE stmt5;
--     COMMIT;
--     SET autocommit=1;
END $$
DELIMITER ;
-- EOC

-- DELETE  node

DROP PROCEDURE IF EXISTS us_delete_work_package;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_delete_work_package(IN tableName VARCHAR(100), IN tree_project VARCHAR(100), IN wp_id VARCHAR(100), OUT validation_string VARCHAR(4))
us_delete_work_packagelbl:BEGIN
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;

    -- Check of we have data to delete
    SET @s = CONCAT('SELECT count(*) INTO @rowsCount FROM ', tableName, ' WHERE id = ', wp_id);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    IF (@rowsCount < 1) THEN
        LEAVE us_delete_work_packagelbl;
    END IF;

    -- Get data to delete
    SET @s = CONCAT('SELECT lft, rgt, rgt - lft + 1, ', tree_project, ' INTO @myLeft, @myRight, @myWidth, @myTreeId FROM ', tableName, ' WHERE id = ', wp_id);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    SET autocommit=0;
    START TRANSACTION;

        SET @s = CONCAT('DELETE FROM ', tableName, ' WHERE lft = ', @myLeft, ' AND ', tree_project, ' = ', @myTreeId);
        PREPARE stmt2 FROM @s;
        EXECUTE stmt2;

        SET @unique_wp_table_name = replace(CONCAT('work_packages_tmp_', UUID()), '-', '');
        SET @s = CONCAT('CREATE TEMPORARY TABLE IF NOT EXISTS ', @unique_wp_table_name, ' AS (SELECT id AS wp_id, lft, rgt, ', tree_project, ' FROM ', tableName, ' WHERE ((rgt >=',@myRight,') 
                        OR (lft >=', @myRight,')) AND ', tree_project, '  = ', @myTreeId, ')');
        PREPARE stmt3 FROM @s;
        EXECUTE stmt3;

        SET @s = CONCAT('UPDATE ', tableName, ' SET rgt=rgt+1000000 WHERE rgt >', @myRight, ' AND ', tree_project, '  = ', @myTreeId);
        PREPARE stmt4 FROM @s;
        EXECUTE stmt4;

        SET @s = CONCAT('UPDATE ', tableName, ' SET lft=lft+1000000 WHERE rgt >', @myRight, ' AND ', tree_project, '  = ', @myTreeId);
        PREPARE stmt5 FROM @s;
        EXECUTE stmt5;

        SET @s = CONCAT('UPDATE ', @unique_wp_table_name, ' SET rgt = rgt - 2 WHERE rgt >', @myRight, ' AND ', tree_project, '  = ', @myTreeId);
        PREPARE stmt6 FROM @s;
        EXECUTE stmt6;

        SET @s = CONCAT('UPDATE ', @unique_wp_table_name, ' SET lft = lft - 2 WHERE lft >', @myRight, ' AND ', tree_project, '  = ', @myTreeId);
        PREPARE stmt7 FROM @s;
        EXECUTE stmt7;

        SET @s = CONCAT('UPDATE ', tableName, ' wp INNER JOIN ', @unique_wp_table_name, ' wp_tmp ON wp.id = wp_tmp.wp_id SET wp.lft = wp_tmp.lft, wp.rgt = wp_tmp.rgt');
        PREPARE stmt8 FROM @s;
        EXECUTE stmt8;

        SET @s = CONCAT('DROP TABLE ', @unique_wp_table_name);
        PREPARE stmt9 FROM @s;
        EXECUTE stmt9;

    COMMIT;
    SET autocommit=1;
    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC


-- DELETE parent including all children
DROP PROCEDURE IF EXISTS us_delete_work_package_branch;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_delete_work_package_branch(IN tableName VARCHAR(100), IN tree_project VARCHAR(100), IN parent_id VARCHAR(100), OUT validation_string VARCHAR(4))
us_delete_work_package_branchlbl:BEGIN
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;
    SET autocommit=0;
    START TRANSACTION;

    -- Check of we have data to delete
    SET @s = CONCAT('SELECT count(*) INTO @rowsCount FROM ', tableName, ' WHERE id = ', wp_id);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    IF (@rowsCount < 1) THEN
        LEAVE us_delete_work_package_branchlbl;
    END IF;

    -- Get data to delete
    SET @s = CONCAT('SELECT  lft, rgt, rgt - lft + 1, ', tree_project, 'INTO @myLeft, @myRight, @myWidth,  @myTreeId FROM ', tableName, ' WHERE id = ', parent_id);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    SET @s = CONCAT('DELETE FROM ', tableName, ' WHERE ', tree_project, ' = ', @myTreeId, ' AND lft BETWEEN ', @myLeft, ' AND ', @myRight);
    PREPARE stmt2 FROM @s;
    EXECUTE stmt2;

    SET @s = CONCAT('UPDATE ', tableName, ' SET rgt = rgt - ', @myWidth, ' WHERE rgt > ', @myRight, ' AND ', tree_project, ' = ', @myTreeId);
    PREPARE stmt3 FROM @s;
    EXECUTE stmt3;

    SET @s = CONCAT('UPDATE ', tableName, ' SET lft = lft - ', @myWidth, ' WHERE lft > ', @myRight, ' AND ', tree_project, ' = ', @myTreeId);
    PREPARE stmt4 FROM @s;
    EXECUTE stmt4;

    COMMIT;

    SET autocommit=1;
    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC


-- DELETE whole tree

DROP PROCEDURE IF EXISTS us_delete_whole_work_package;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_delete_whole_work_package(IN tableName VARCHAR(100), IN tree_project VARCHAR(100), IN del_tree_id VARCHAR(100))
BEGIN
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;

    SET autocommit=0;
    START TRANSACTION;

    SET @s = CONCAT('DELETE FROM ', tableName, ' WHERE ', tree_project, ' = ', del_tree_id);
    PREPARE stmt FROM @s;
    EXECUTE stmt;

    COMMIT;
    SET autocommit=1;
END $$
DELIMITER ;
-- EOC

-- MOVE a branch or node inside the same level
DROP PROCEDURE IF EXISTS us_move_work_package;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_move_work_package(IN tableName VARCHAR(100), IN tree_project VARCHAR(100), IN item_id VARCHAR(100), IN neighbour_id VARCHAR(100), IN project_id VARCHAR(100))
BEGIN

    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;

    SET autocommit=0;
    START TRANSACTION;

    SET @s = CONCAT('SELECT lft, rgt, ', tree_project, ' INTO  @myLeft,  @myRight,  @myTreeId  FROM ', tableName, ' WHERE id = ', item_id, ' AND ', tree_project, ' = ', project_id);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
    
    SET @s = CONCAT('SELECT lft, rgt INTO  @myLeft2,  @myRight2 FROM ', tableName,' WHERE id = ', neighbour_id, ' AND ', tree_project, ' = ', project_id);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;


    IF tableName = 'work_package_presets' THEN
        SET @s = CONCAT('SELECT lft, rgt INTO @parentLft, @parentRgt FROM ', tableName, ' WHERE id = us_get_parent_package_preset(', @myTreeId, ', ', item_id, ') AND tree_id = ', project_id);
    ELSE
        SET @s = CONCAT('SELECT lft, rgt INTO @parentLft, @parentRgt FROM ', tableName, ' WHERE id = us_get_parent_package(', @myTreeId, ', ', item_id, ') AND project_id = ', project_id);
    END IF;

    
    PREPARE stmt4 FROM @s;
    EXECUTE stmt4;

    SET @unique_wp_table_name = replace(CONCAT('work_packages_tmp_', UUID()), '-', '');
    SET @s = CONCAT('CREATE TEMPORARY TABLE IF NOT EXISTS ', @unique_wp_table_name, ' AS (SELECT id AS wp_id, lft, rgt, ', tree_project, ' FROM  ', tableName, '  WHERE ((lft BETWEEN ', @myLeft, ' AND ', @myRight, ' OR id = ', item_id, ') 
                    OR (lft BETWEEN ', @myLeft2, ' AND ', @myRight2, ' OR id = ', neighbour_id, ')) AND ', tree_project, ' = ', @myTreeId, ')');
    PREPARE stmt5 FROM @s;
    EXECUTE stmt5;

    SET @s = CONCAT('UPDATE ', tableName, ' SET lft=lft+1000000, rgt=rgt+1000000 WHERE ', tree_project, ' = ', project_id, ' AND (lft BETWEEN ', @myLeft, ' AND ', @myRight, ' AND ', tree_project, ' = ', @myTreeId, ') OR id = ', item_id);
    PREPARE stmt6 FROM @s;
    EXECUTE stmt6;

    SET @s = CONCAT('UPDATE ', tableName, ' SET lft=lft+1000000, rgt=rgt+1000000 WHERE ', tree_project, ' = ', project_id, ' AND (lft BETWEEN ', @myLeft2, ' AND ', @myRight2, ' AND ', tree_project, ' = ', @myTreeId, ') OR id = ', neighbour_id);
    PREPARE stmt7 FROM @s;
    EXECUTE stmt7;

    IF ((@myLeft2 -1 > @parentLft) AND (@myRight + 1 < @parentRgt) AND ((@myLeft NOT BETWEEN @myLeft2 AND @myRight2) AND (@myLeft2 NOT BETWEEN @myLeft AND @myRight))) THEN
	SET @s = CONCAT('UPDATE ', @unique_wp_table_name, ' SET 
        lft=CASE
            WHEN lft = ', @myLeft2, ' OR (lft BETWEEN ', @myLeft2, ' AND ', @myRight2, ')
                THEN lft-(', @myLeft2, '-', @myLeft, ')
            WHEN lft = ', @myLeft, ' OR (lft BETWEEN ', @myLeft, ' AND ', @myRight, ')
                THEN lft+(', @myRight2, '-', @myRight, ')
            ELSE
                lft
            END,
        rgt=CASE
            WHEN rgt = ', @myRight2, ' OR (rgt BETWEEN ', @myLeft2, ' AND ', @myRight2, ')
                THEN rgt-(', @myLeft2, '-', @myLeft, ')
            WHEN rgt = ', @myRight, ' OR (rgt BETWEEN ', @myLeft, ' AND ', @myRight, ')
                THEN rgt+(', @myRight2, '-', @myRight, ')
            ELSE
                rgt
            END
        WHERE ', tree_project, '=', @myTreeId, ' AND lft BETWEEN ', @myLeft, ' AND ', @myRight2);
    END IF;
    PREPARE stmt8 FROM @s;
    EXECUTE stmt8;

    SET @s = CONCAT('UPDATE ', tableName, ' wp INNER JOIN ', @unique_wp_table_name, ' wp_tmp ON wp.id = wp_tmp.wp_id SET wp.lft = wp_tmp.lft, wp.rgt = wp_tmp.rgt');
    PREPARE stmt9 FROM @s;
    EXECUTE stmt9;

    SET @s = CONCAT('DROP TABLE ', @unique_wp_table_name);
    PREPARE stmt10 FROM @s;
    EXECUTE stmt10;

    COMMIT;
    SET autocommit=1;
END $$
DELIMITER ;
-- EOC

-- INSERT into the work package table

-- INSERT new node

DROP PROCEDURE IF EXISTS us_insert_work_package;
-- EOC

-- If you update this PROCEDURE also update us_migrate_11_12_copy_projects_fee_areas_insert
DELIMITER $$
CREATE PROCEDURE us_insert_work_package(IN parent_id VARCHAR(100), IN new_name VARCHAR(100), IN tag_id INTEGER, IN is_purpose BOOLEAN, IN is_special INTEGER, IN is_special_phase INTEGER, IN is_fee_area BOOLEAN, IN threshold DOUBLE, IN is_extern BOOLEAN, IN cost_centre_id INTEGER, OUT insert_id VARCHAR(100))
BEGIN
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;

    /* SET by SELECT */
    (SELECT rgt, project_id 
        INTO @myRight, @myProjectId
        FROM work_packages WHERE id = parent_id);

    SET autocommit=0;
    START TRANSACTION;
        SET @unique_wp_table_name = replace(CONCAT('work_packages_tmp_', UUID()), '-', '');
        SET @s = CONCAT('CREATE TEMPORARY TABLE IF NOT EXISTS ', @unique_wp_table_name, ' AS (SELECT id AS wp_id, lft, rgt, project_id FROM work_packages WHERE ((rgt >=',@myRight,') 
                        OR (lft >=', @myRight,')) AND project_id = ', @myProjectId, ')');
        PREPARE stmt1 FROM @s;
        EXECUTE stmt1;
        
        UPDATE work_packages SET rgt=rgt+1000000 WHERE rgt >= @myRight AND project_id = @myProjectId;
        UPDATE work_packages SET lft=lft+1000000 WHERE lft >= @myRight AND project_id = @myProjectId;

        SET @s = CONCAT('UPDATE ', @unique_wp_table_name, ' SET rgt = rgt + 2 WHERE rgt >=', @myRight, ' AND project_id = ', @myProjectId);
        PREPARE stmt2 FROM @s;
        EXECUTE stmt2;

        SET @s = CONCAT('UPDATE ', @unique_wp_table_name, ' SET lft = lft + 2 WHERE lft >=', @myRight, ' AND project_id = ', @myProjectId);
        PREPARE stmt3 FROM @s;
        EXECUTE stmt3;

        INSERT INTO work_packages(name, tag_id, lft, rgt, project_id, is_purpose_required, is_special, is_special_phase, is_fee_area, budget_threshold, is_extern, cost_centre_id) VALUES(new_name, tag_id, @myRight, @myRight + 1, @myProjectId, is_purpose, is_special, is_special_phase, is_fee_area, threshold, is_extern, cost_centre_id);

        SET insert_id = LAST_INSERT_ID();

        SET @s = CONCAT('UPDATE work_packages wp INNER JOIN ', @unique_wp_table_name, ' wp_tmp ON wp.id = wp_tmp.wp_id SET wp.lft = wp_tmp.lft, wp.rgt = wp_tmp.rgt');
        PREPARE stmt4 FROM @s;
        EXECUTE stmt4;

        SET @s = CONCAT('DROP TABLE ', @unique_wp_table_name);
        PREPARE stmt5 FROM @s;
        EXECUTE stmt5;
    COMMIT;
    SET autocommit=1;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- PROCEDURES for the work packages history table
-- --------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS us_update_work_packages_history;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_update_work_packages_history(IN old_wp_id BIGINT(20), IN new_wp_id BIGINT(20), IN fee_area_id BIGINT(20), OUT validation_string VARCHAR(4))
us_update_work_packages_historylbl:BEGIN    
    DECLARE history_date DATETIME;
    DECLARE l_last_row_fetched INTEGER;
    DECLARE cur_history_date CURSOR FOR SELECT DATE_FORMAT(wph.date, '%Y-%m-%d') AS `date` FROM work_packages_history AS wph WHERE wph.wp_id = old_wp_id GROUP BY wph.date;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    SELECT count(*), lft INTO @cntNewLft, @newLft FROM work_packages WHERE id = new_wp_id;
    SELECT count(*), lft INTO @cntOldLft, @oldLft FROM work_packages WHERE id = old_wp_id;
    SELECT count(*), lft INTO @cntFeeLft, @feeLft FROM work_packages WHERE id = fee_area_id;

    -- Catch no rows
    if @cntNewLft<1 || @cntOldLft<1 || @cntFeeLft<1 THEN
        LEAVE us_update_work_packages_historylbl;
    END IF;

    SET l_last_row_fetched=0;
    OPEN cur_history_date;
    cursor_loop: LOOP -- -------------------------------------
            FETCH cur_history_date INTO history_date;
            IF l_last_row_fetched=1 THEN
                    LEAVE cursor_loop;
            END IF;
            call us_update_wp_work_packages_history(old_wp_id, new_wp_id, fee_area_id, history_date, @newLft, @oldLft, @feeLft);
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  cur_history_date;


    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS us_update_wp_work_packages_history;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_update_wp_work_packages_history(IN old_wp_id BIGINT(20), IN new_wp_id BIGINT(20), IN fee_area_id BIGINT(20), IN history_date DATETIME, IN newLft BIGINT(100), IN oldLft BIGINT(20), IN feeLft BIGINT(20))
BEGIN    
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;
    SET @myID = NULL;

    SET @s = CONCAT('SELECT IF(count(id) = 0, NULL, id) INTO @myID FROM work_packages_history WHERE wp_id = ', new_wp_id, ' AND `date` = "', history_date, '"');
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    SET @myProjectID = null; 
    SET @myLOC = null; 
    SET @myBudgetProp = null; 
    SET @myBudgetCosts = null; 
    SET @myBudgetHours = null; 
    SET @myBudgetThreshold = null; 
    SET @myFeeCosts = null; 
    SET @myFeeHours = null; 
    SET @myFeeProp = null; 
    IF (SELECT count(*) FROM work_packages_history WHERE wp_id = old_wp_id AND `date` = history_date) > 0 THEN
        SET @s = CONCAT('SELECT project_id, completion_level, budget_proportion, budget_planned_costs, budget_planned_hours, budget_threshold, fee_costs, fee_hours, fee_proportion 
                         INTO @myProjectID, @myLOC, @myBudgetProp, @myBudgetCosts, @myBudgetHours, @myBudgetThreshold, @myFeeCosts, @myFeeHours, @myFeeProp  FROM work_packages_history 
                         WHERE wp_id = ', old_wp_id, ' AND `date` = "', history_date, '"');
        PREPARE stmt2 FROM @s;
        EXECUTE stmt2;
    END IF;

    SET autocommit=0;
    START TRANSACTION;
        IF (@myID IS NOT NULL) THEN 
            SET @s = CONCAT('UPDATE work_packages_history SET completion_level = completion_level+', @myLOC, ', budget_proportion = budget_proportion+', @myBudgetProp, ', budget_planned_costs = budget_planned_costs+', @myBudgetCosts,
                ', budget_planned_hours = budget_planned_hours+', @myBudgetHours, ', budget_threshold = budget_threshold+', @myBudgetThreshold,
                ' WHERE project_id = ', @myProjectID, ' AND `date` = "', history_date, '" AND (wp_id IN (SELECT id FROM work_packages WHERE (',newLft,' > lft AND ',newLft,' < rgt) AND project_id = ', @myProjectID, ') OR wp_id =', new_wp_id, ')');
            PREPARE stmt3 FROM @s;
            EXECUTE stmt3;

            SET @s = CONCAT('UPDATE work_packages_history SET completion_level = completion_level-', @myLOC, ', budget_proportion = budget_proportion-', @myBudgetProp, ', budget_planned_costs = budget_planned_costs-', @myBudgetCosts,
                ', budget_planned_hours = budget_planned_hours-', @myBudgetHours, ', budget_threshold = budget_threshold-', @myBudgetThreshold,
                ' WHERE project_id = ', @myProjectID, ' AND `date` = "', history_date, '" AND wp_id IN (SELECT id FROM work_packages WHERE (',oldLft,' > lft AND ',oldLft,'< rgt) AND project_id = ', @myProjectID, ')');
            PREPARE stmt4 FROM @s;
            EXECUTE stmt4;

            SET @s = CONCAT('DELETE FROM work_packages_history WHERE wp_id = ', old_wp_id, ' AND `date` = "', history_date, '"');
            PREPARE stmt5 FROM @s;
            EXECUTE stmt5;
        END IF;
        IF (@myID IS NULL) THEN
            SET @s = CONCAT('UPDATE work_packages_history SET budget_planned_costs = 0, budget_planned_hours = 0, fee_costs = 0, fee_hours = 0, fee_proportion = 0, wp_id = ', new_wp_id, ' WHERE project_id = ', @myProjectID, ' AND `date` = "', history_date, '" AND wp_id = ', old_wp_id);
            PREPARE stmt6 FROM @s;
            EXECUTE stmt6;

            SET @s = CONCAT('UPDATE work_packages_history SET completion_level = completion_level+', @myLOC, ', budget_proportion = budget_proportion+', @myBudgetProp, ', budget_planned_costs = budget_planned_costs+', @myBudgetCosts,
                ', budget_planned_hours = budget_planned_hours+', @myBudgetHours, ', budget_threshold = budget_threshold+', @myBudgetThreshold,
                ' WHERE project_id = ', @myProjectID, ' AND `date` = "', history_date, '" AND wp_id IN (SELECT id FROM work_packages WHERE (',newLft,' BETWEEN lft AND rgt) AND project_id = ', @myProjectID, ')');
            PREPARE stmt7 FROM @s;
            EXECUTE stmt7;

            SET @s = CONCAT('UPDATE work_packages_history SET completion_level = completion_level-', @myLOC, ', budget_proportion = budget_proportion-', @myBudgetProp, ', budget_planned_costs = budget_planned_costs-', @myBudgetCosts,
                ', budget_planned_hours = budget_planned_hours-', @myBudgetHours, ', budget_threshold = budget_threshold-', @myBudgetThreshold,
                ' WHERE project_id = ', @myProjectID, ' AND `date` = "', history_date, '" AND wp_id IN (SELECT id FROM work_packages WHERE (',oldLft,' > lft AND ',oldLft,'< rgt) AND project_id = ', @myProjectID, ')');
            PREPARE stmt8 FROM @s;
            EXECUTE stmt8;
        END IF;
	COMMIT;
    SET autocommit=1;
    call us_update_fee_wp_work_packages_history(old_wp_id, fee_area_id, history_date, @myFeeCosts, @myFeeHours, @myFeeProp, @myProjectID, oldLft, feeLft);
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS us_update_fee_wp_work_packages_history;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_update_fee_wp_work_packages_history(IN old_wp_id BIGINT(20), IN fee_area_id BIGINT(20), IN history_date DATETIME, IN new_fee_costs DOUBLE, IN new_fee_hours DOUBLE, IN fee_prop DOUBLE, IN my_project_id DOUBLE, IN oldLft BIGINT(20), IN feeLft BIGINT(20))
BEGIN    
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;

    SET @myID = NULL;

    SET @s = CONCAT('SELECT IF(count(id) = 0, NULL, id) INTO @myID FROM work_packages_history WHERE wp_id = ', fee_area_id, ' AND `date` = "', history_date, '"');
    PREPARE stmt8 FROM @s;
    EXECUTE stmt8;

    SET autocommit=0;
    START TRANSACTION;
        IF (@myID IS NOT NULL) THEN 
            SET @s = CONCAT('UPDATE work_packages_history SET fee_costs = fee_costs+IFNULL(', new_fee_costs,', 0), fee_hours = fee_hours+IFNULL(', new_fee_hours,', 0), fee_proportion = fee_proportion+IFNULL(', fee_prop,', 0)
                 WHERE project_id = ', my_project_id, ' AND `date` = "', history_date, '" AND (wp_id IN (SELECT id FROM work_packages WHERE (',feeLft,' > lft AND ',feeLft,' < rgt) AND project_id = ', my_project_id, ') OR wp_id =', fee_area_id, ')');
            PREPARE stmt9 FROM @s;
            EXECUTE stmt9;

            SET @s = CONCAT('UPDATE work_packages_history SET fee_costs = fee_costs-IFNULL(', new_fee_costs,', 0), fee_hours = fee_hours-IFNULL(', new_fee_hours,', 0), fee_proportion = fee_proportion-IFNULL(', fee_prop,', 0)
                 WHERE project_id = ', my_project_id, ' AND `date` = "', history_date, '" AND wp_id IN (SELECT id FROM work_packages WHERE (',oldLft,' > lft AND ',oldLft,'< rgt) AND project_id = ', my_project_id, ')');
            PREPARE stmt10 FROM @s;
            EXECUTE stmt10;
        END IF;
        IF (@myID IS NULL) THEN
            SET @s = CONCAT('INSERT INTO work_packages_history (project_id, fee_costs, fee_hours, fee_proportion, wp_id, `date`) VALUES (', my_project_id, ', ', new_fee_costs,', ', new_fee_hours,', ', fee_prop,', ', fee_area_id, ', "', history_date, '")');
            PREPARE stmt11 FROM @s;
            EXECUTE stmt11;

            SET @s = CONCAT('UPDATE work_packages_history SET fee_costs = fee_costs+IFNULL(', new_fee_costs,', 0), fee_hours = fee_hours+IFNULL(', new_fee_hours,', 0), fee_proportion = fee_proportion+IFNULL(', fee_prop,', 0)
                 WHERE project_id = ', my_project_id, ' AND `date` = "', history_date, '" AND wp_id IN (SELECT id FROM work_packages WHERE (',feeLft,' > lft AND ',feeLft,' < rgt) AND project_id = ', my_project_id, ')');
            PREPARE stmt12 FROM @s;
            EXECUTE stmt12;

            SET @s = CONCAT('UPDATE work_packages_history SET fee_costs = fee_costs-IFNULL(', new_fee_costs,', 0), fee_hours = fee_hours-IFNULL(', new_fee_hours,', 0), fee_proportion = fee_proportion-IFNULL(', fee_prop,', 0)
                 WHERE project_id = ', my_project_id, ' AND `date` = "', history_date, '" AND wp_id IN (SELECT id FROM work_packages WHERE (',oldLft,' > lft AND ',oldLft,'< rgt) AND project_id = ', my_project_id, ')');
            PREPARE stmt13 FROM @s;
            EXECUTE stmt13;
        END IF;
    COMMIT;
    SET autocommit=1;
END $$
DELIMITER ;
-- EOC

-- Update parent nodes after reassign
DROP PROCEDURE IF EXISTS us_update_wp_budget_after_reassign;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_update_wp_budget_after_reassign(IN new_lft BIGINT(20), IN budget_costs DOUBLE, IN budget_hours DOUBLE, IN budget_prop DOUBLE, IN old_lft BIGINT(20), IN act_project_id BIGINT(20), OUT validation_string VARCHAR(4))
BEGIN
    SET @s = CONCAT('UPDATE work_packages SET budget_proportion=budget_proportion+IFNULL(', budget_prop,', 0), budget_planned_costs=budget_planned_costs+IFNULL(', budget_costs,', 0), budget_planned_hours=budget_planned_hours+IFNULL(',budget_hours,', 0)
     WHERE lft !=',new_lft,' AND (',new_lft,' BETWEEN lft AND rgt) AND project_id = ',act_project_id);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    SET @s1 = CONCAT('UPDATE work_packages SET budget_proportion=budget_proportion-IFNULL(', budget_prop,', 0), budget_planned_costs=budget_planned_costs-IFNULL(', budget_costs,', 0),budget_planned_hours=budget_planned_hours-IFNULL(',budget_hours,', 0) 
     WHERE (',old_lft,' BETWEEN lft AND rgt) AND project_id = ',act_project_id);
    PREPARE stmt2 FROM @s1;
    EXECUTE stmt2;

    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS us_update_wp_fee_after_reassign;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_update_wp_fee_after_reassign(IN new_lft BIGINT(20), IN fee_costs DOUBLE, IN fee_hours DOUBLE, IN fee_prop DOUBLE, IN old_lft BIGINT(20), IN act_project_id BIGINT(20), OUT validation_string VARCHAR(4))
BEGIN

    SET @s = CONCAT('UPDATE work_packages SET fee_proportion=fee_proportion+IFNULL(', fee_prop,', 0), fee_costs=fee_costs+IFNULL(', fee_costs,', 0), fee_hours=fee_hours+IFNULL(',fee_hours,', 0) 
     WHERE lft !=',new_lft,' AND (',new_lft,' BETWEEN lft AND rgt) AND project_id = ',act_project_id);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    SET @s1 = CONCAT('UPDATE work_packages SET fee_proportion=fee_proportion -IFNULL(', fee_prop,', 0), fee_costs=fee_costs-IFNULL(', fee_costs,', 0),fee_hours=fee_hours-IFNULL(',fee_hours,', 0)
     WHERE (',old_lft,' BETWEEN lft AND rgt) AND project_id = ',act_project_id);
    PREPARE stmt2 FROM @s1;
    EXECUTE stmt2;

    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC


-- --------------------------------------------------------------------------------------------------
-- DELETE row and update the costs_sum in lut_fee_areas_offer table
-- --------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS us_update_delete_lut_fee_offer;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_update_delete_lut_fee_offer(IN wp_id VARCHAR(100), OUT validation_string VARCHAR(4))
BEGIN    
    DECLARE cur_offer_id INTEGER;
    DECLARE l_last_row_fetched INTEGER;
    DECLARE cur_lut_fee_offers CURSOR FOR SELECT offer_id FROM lut_fee_areas_offer WHERE work_package_id = wp_id;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;
    SET autocommit=0;
    START TRANSACTION;

    SET l_last_row_fetched=0;
    OPEN cur_lut_fee_offers;
    cursor_loop: LOOP -- -------------------------------------
            FETCH cur_lut_fee_offers INTO cur_offer_id;
            IF l_last_row_fetched=1 THEN
                    LEAVE cursor_loop;
            END IF;
            /* SET by SELECT */
            (SELECT (costs_sum-costs) 
                INTO @newSumCosts
                FROM lut_fee_areas_offer WHERE offer_id = cur_offer_id AND work_package_id = wp_id);
            UPDATE lut_fee_areas_offer SET costs_sum = @newSumCosts WHERE offer_id = cur_offer_id;
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  cur_lut_fee_offers;

    DELETE FROM lut_fee_areas_offer WHERE work_package_id=wp_id;

    COMMIT;
    SET autocommit=1;
    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Work package maintenance 
-- --------------------------------------------------------------------------------------------------

-- Get the wholes between work packages and delete them
DROP PROCEDURE IF EXISTS repair_wps;
-- EOC
DELIMITER $$
CREATE PROCEDURE repair_wps(IN tableName VARCHAR(100), IN tree_project VARCHAR(100), myTreeId VARCHAR(100), OUT validation_string VARCHAR(4))
repair_wpslbl:BEGIN

DECLARE i INTEGER DEFAULT 1;

    SET @s = CONCAT('SET @myCount = (SELECT count(id)*2 FROM ', tableName, ' WHERE ',tree_project,' = ', myTreeId, ')');
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    cursor_loop: LOOP -- -------------------------------------
    IF i > @myCount THEN
        LEAVE cursor_loop;
    END IF;
    
    SET i = i+1;

    SET @s = CONCAT('SET @wp_exists = (SELECT count(*) FROM ', tableName, ' WHERE ',tree_project,' = ', myTreeId, ' AND (lft = ', i, ' OR rgt = ', i, '))');
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    IF @wp_exists = 0 THEN    
        SET @s = CONCAT('SET @curr_lft = (SELECT MIN(lft) FROM ', tableName, ' WHERE ',tree_project,' = ', myTreeId, ' AND lft > ', i, ')');
        PREPARE stmt1 FROM @s;
        EXECUTE stmt1;

        SET @s = CONCAT('SET @curr_rgt = (SELECT MIN(rgt) FROM ', tableName, ' WHERE ',tree_project,' = ', myTreeId, ' AND rgt > ', i, ')');
        PREPARE stmt1 FROM @s;
        EXECUTE stmt1;
        
    	IF @curr_lft < @curr_rgt THEN
            SET @to_del = @curr_lft;
    	ELSE
            SET @to_del = @curr_rgt;
    	END IF;
    	
    	-- PROCEDURE
    	CALL us_delete_not_existing_work_package_branch(tableName, tree_project, i, @to_del-1, myTreeId); 
    END IF;     
    END LOOP cursor_loop; -- -------------------------------------
    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC

-- DELETE not existing work packages (simulate branch deletion)
DROP PROCEDURE IF EXISTS us_delete_not_existing_work_package_branch;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_delete_not_existing_work_package_branch(IN tableName VARCHAR(100), IN tree_project VARCHAR(100), IN myLft VARCHAR(100), IN myRgt VARCHAR(100), IN myTreeId VARCHAR(100))
BEGIN
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;
    
    SET @myWidth = myRgt-myLft+1;
    
    SET autocommit=0;
    START TRANSACTION;

    SET @s = CONCAT('UPDATE ', tableName, ' SET rgt = rgt + 1000, lft = lft+1000 WHERE ', tree_project, ' = ', myTreeId);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    SET @s = CONCAT('UPDATE ', tableName, ' SET rgt = (rgt - ', @myWidth, ') - 1000 WHERE rgt > (', myRgt, '+1000) AND ', tree_project, ' = ', myTreeId);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    SET @s = CONCAT('UPDATE ', tableName, ' SET lft = (lft - ', @myWidth, ') - 1000 WHERE lft > (', myRgt, '+1000) AND ', tree_project, ' = ', myTreeId);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
    
    SET @s = CONCAT('UPDATE ', tableName, ' SET lft = lft - 1000 WHERE lft > 1000 AND ', tree_project, ' = ', myTreeId);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
    
    SET @s = CONCAT('UPDATE ', tableName, ' SET rgt = rgt - 1000 WHERE rgt > 1000 AND ', tree_project, ' = ', myTreeId);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    COMMIT;

    SET autocommit=1;
END $$
DELIMITER ;
-- EOC

-- UPDATE nodes with no children where rgt-lft > 1
DROP PROCEDURE IF EXISTS us_update_lft_rgt;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_update_lft_rgt(IN tableName VARCHAR(100), IN tree_project VARCHAR(100), OUT validation_string VARCHAR(4))
BEGIN
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;
    
    SET autocommit=0;
    START TRANSACTION;

    SET @s = CONCAT('UPDATE ',tableName,' SET rgt = lft + 1
        WHERE
        id IN (SELECT 
            myTable.id
        FROM
            (SELECT 
                wp.id
            FROM
                ',tableName, ' wp
            LEFT JOIN ',tableName,' wp2 ON wp2.',tree_project,' = wp.',tree_project,'
                AND wp2.lft > wp.lft
                AND wp2.rgt < wp.rgt
            WHERE
                wp.rgt - wp.lft > 1
            GROUP BY wp.id
            HAVING COUNT(wp2.id) = 0
            ORDER BY wp.',tree_project,') AS myTable)');
    PREPARE stmt3 FROM @s;
    EXECUTE stmt3;

    COMMIT;

    SET autocommit=1;
    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Work package parent calculation
-- --------------------------------------------------------------------------------------------------

-- POSSIBLE FIELDS TO UPDATE WITH THIS: budget_proportion, budget_planned_costs, budget_planned_hours
DROP PROCEDURE IF EXISTS recalc_budget_parent;
-- EOC
DELIMITER $$
CREATE PROCEDURE recalc_budget_parent(IN field_name VARCHAR(100), OUT validation_string VARCHAR(4))
BEGIN
    SET @s = CONCAT('UPDATE work_packages AS top 
        SET 
    ',field_name,' = (SELECT 
            SUM(chld.',field_name,')
        FROM
            (SELECT 
                ',field_name,', project_id, lft, rgt, is_fee_area
            FROM
                work_packages) AS chld
        WHERE
            top.project_id = chld.project_id
            AND (chld.lft > top.lft
            AND chld.rgt < top.rgt)
            AND chld.rgt - chld.lft = 1)
        WHERE
            top.rgt - top.lft > 1;');
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC
    
-- POSSIBLE FIELDS TO UPDATE WITH THIS: fee_proportion, fee_costs, fee_hours
DROP PROCEDURE IF EXISTS recalc_fee_parent;
-- EOC
DELIMITER $$
CREATE PROCEDURE recalc_fee_parent(IN field_name VARCHAR(100), OUT validation_string VARCHAR(4))
BEGIN
    SET @s = CONCAT('UPDATE work_packages top 
        SET 
        ',field_name,' = (SELECT 
            SUM(chld.',field_name,')
        FROM
            (SELECT 
                ',field_name,',
                project_id,
                lft,
                rgt,
                is_fee_area
            FROM
                work_packages) AS chld
        WHERE
            top.project_id = chld.project_id
            AND (chld.lft > top.lft
            AND chld.rgt < top.rgt) AND chld.is_fee_area = 1)
            WHERE
            top.rgt - top.lft > 1 AND top.is_fee_area = 0;');
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Work package check if fee areas are wrong at the projects
-- --------------------------------------------------------------------------------------------------

-- Check if there are some fee areas missing in a branch
DROP PROCEDURE IF EXISTS create_repair_tempTable;
-- EOC
DELIMITER $$
CREATE PROCEDURE create_repair_tempTable(IN temptableName VARCHAR (100))
create_repair_tempTablelbl:BEGIN

	DROP TEMPORARY TABLE IF EXISTS wp_repair_table;

	CREATE TEMPORARY TABLE IF NOT EXISTS wp_repair_table AS (SELECT 
	wp.project_id,
        wp.id,
        wp.lft,
        wp.rgt,
        (wp.rgt-wp.lft) AS diff,
        wp.is_fee_area,
        (COUNT(parent.id) - 1) AS depth,    
        (SELECT 
                id
            FROM
                work_packages wp2
            WHERE
                wp2.lft < wp.lft AND wp2.rgt > wp.rgt
                AND wp2.project_id = wp.project_id
            ORDER BY wp2.rgt - wp.rgt ASC
            LIMIT 1) AS parent_id
            FROM
                work_packages AS wp,
                work_packages AS parent
            WHERE
                wp.lft BETWEEN parent.lft AND parent.rgt
                AND wp.lft > 1
                AND wp.project_id = parent.project_id
            GROUP BY wp.id
            ORDER BY wp.project_id , wp.lft);

    CALL check_fee_area_some_missing(temptableName);
    
    DROP TEMPORARY TABLE IF EXISTS wrong_wps;
END $$
DELIMITER ;
-- EOC

-- Check if there are some fee areas missing in a branch
DROP PROCEDURE IF EXISTS check_fee_area_some_missing;
-- EOC
DELIMITER $$
CREATE PROCEDURE check_fee_area_some_missing(IN temptableName VARCHAR (100))
check_fee_area_some_missinglbl:BEGIN
BLOCK1: begin
    DECLARE curr_wp_id INTEGER;
    DECLARE curr_depth INTEGER;
    DECLARE curr_parent INTEGER;
    DECLARE l_last_row_fetched INTEGER;
    DECLARE curr_wps CURSOR FOR SELECT id, depth, parent_id FROM wp_repair_table WHERE is_fee_area = 1 GROUP BY project_id, depth, parent_id;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    SET l_last_row_fetched=0;
    OPEN curr_wps;
    cursor_loop: LOOP -- -------------------------------------
        FETCH curr_wps INTO curr_wp_id, curr_depth, curr_parent;
        IF l_last_row_fetched=1 THEN
            LEAVE cursor_loop;
        END IF;

        SET @myParent = curr_parent;

        SET @myDepth = curr_depth;
        
        BLOCK2: BEGIN
            DECLARE curr_wp_id2 INTEGER;
            DECLARE l_last_row_fetched2 INTEGER;
            DECLARE curr_wps2 CURSOR FOR SELECT id FROM wp_repair_table WHERE parent_id = @myParent AND depth=@myDepth AND is_fee_area = 0;
            DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched2=1;

            SET l_last_row_fetched2=0;
            OPEN curr_wps2;
            cursor_loop2: LOOP -- -------------------------------------
                FETCH curr_wps2 INTO curr_wp_id2;
                IF l_last_row_fetched2=1 THEN
                                LEAVE cursor_loop2;
                END IF;

                /* SET by SELECT */
                (SELECT (rgt-lft) 
                    INTO @myDistance
                    FROM wp_repair_table WHERE id =  curr_wp_id2);

                IF @myDistance = 1 THEN  
                    /* SET by SELECT */
                    (SELECT project_id 
                        INTO @project
                        FROM wp_repair_table WHERE id = curr_wp_id2);

                    SET @s = CONCAT('INSERT INTO ',temptableName,' VALUES(',@project,')');
                    PREPARE stmt1 FROM @s;
                    EXECUTE stmt1;


                    LEAVE cursor_loop2;
                END IF;
            END LOOP cursor_loop2; -- -------------------------------------
        END BLOCK2;
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  curr_wps;
END BLOCK1;
END $$
DELIMITER ;
-- EOC

-- Check if there are too much fee areas in one project
DROP PROCEDURE IF EXISTS check_too_much_fee_areas;
-- EOC
DELIMITER $$
CREATE PROCEDURE check_too_much_fee_areas(IN temptableName VARCHAR (100))
check_too_much_fee_areaslbl:BEGIN
    SET @s = CONCAT('INSERT INTO ',temptableName,' (project_id) SELECT  
        project_id
    FROM
        work_packages wp
    WHERE
        (SELECT 
                COUNT(*)
            FROM
                work_packages wp2
            WHERE
                wp.project_id = wp2.project_id
                AND ((wp2.lft > wp.lft AND wp2.rgt < wp.rgt)
                OR (wp2.lft < wp.lft AND wp2.rgt > wp.rgt))
                AND wp2.is_fee_area = 1) > 0
                AND wp.is_fee_area = 1
                AND wp.rgt - wp.lft > 1
    GROUP BY project_id
    ORDER BY project_id , lft');

    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
END $$
DELIMITER ;
-- EOC

-- Check if there are fee areas missing in a branch or in the whole project
DROP PROCEDURE IF EXISTS check_missing_fee_areas;
-- EOC
DELIMITER $$
CREATE PROCEDURE check_missing_fee_areas(IN temptableName VARCHAR (100))
check_missing_fee_areaslbl:BEGIN
    SET @s = CONCAT('INSERT INTO ',temptableName,' (project_id) SELECT 
    project_id
    FROM
        work_packages wp
    WHERE
        (SELECT 
                COUNT(*)
            FROM
                work_packages wp2
            WHERE
                wp.project_id = wp2.project_id
                AND ((wp2.lft > wp.lft AND wp2.rgt < wp.rgt)
                OR (wp2.lft < wp.lft AND wp2.rgt > wp.rgt))
                AND wp2.is_fee_area = 1) = 0
                AND wp.is_fee_area = 0
                AND wp.rgt - wp.lft > 1
    GROUP BY project_id
    ORDER BY project_id , lft');

    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- ADD FULLTEXT INDEXES
-- --------------------------------------------------------------------------------------------------
-- Projekte
DROP PROCEDURE IF EXISTS add_projects_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_projects_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('projekte', 'projects_fulltext', 'projektnummer, projekt, beschreibung, ort');
END $$
DELIMITER ;
-- EOC

-- Adressen
DROP PROCEDURE IF EXISTS add_addresses_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_addresses_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('adressen', 'addresses_fulltext', 'name, firma, anrede, titel, master_titel, vorname, ansprechpartner, strasse, strasse2, pltz, ort, land, email, url, comment_main');
END $$
DELIMITER ;
-- EOC

-- Address descriptions
DROP PROCEDURE IF EXISTS add_address_descriptions_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_address_descriptions_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('adrbeschr', 'address_description_fulltext', 'beschreibung');
END $$
DELIMITER ;
-- EOC

-- Todo
DROP PROCEDURE IF EXISTS add_opentasks_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_opentasks_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('todo', 'opentasks_fulltext', 'beschreibung, bemerkung, ErlVermerk');
END $$
DELIMITER ;
-- EOC

-- Externalexpenses
DROP PROCEDURE IF EXISTS add_externalexpenses_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_externalexpenses_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('externalexpenses', 'externalexpenses_fulltext', 'purpose, description');
END $$
DELIMITER ;
-- EOC

-- Fees
DROP PROCEDURE IF EXISTS add_fees_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_fees_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('fees', 'fees_fulltext', 'purpose, fee_no, remit_purpose');
END $$
DELIMITER ;
-- EOC

-- Personalappointments
DROP PROCEDURE IF EXISTS add_personalappointments_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_personalappointments_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('perstermin', 'personalappointments_fulltext', 'bemerkung, ort');
END $$
DELIMITER ;
-- EOC

-- Drawings
DROP PROCEDURE IF EXISTS add_drawings_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_drawings_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('drawings', 'drawings_fulltext', 'plan_no, plan_content, plan_index, sub_plan, redline_remarks, cad_file, plot_file, purpose, clipboard');
END $$
DELIMITER ;
-- EOC

-- Drawings Outbox
DROP PROCEDURE IF EXISTS add_drawings_outbox_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_drawings_outbox_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('planausgabe', 'drawings_outbox_fulltext', 'bemerkung');
END $$
DELIMITER ;
-- EOC

-- Outbox
DROP PROCEDURE IF EXISTS add_outbox_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_outbox_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('outbox', 'outbox_fulltext', 'no, content, outbox_index, mail_out_file, attached_file');
END $$
DELIMITER ;
-- EOC

-- Inbox
DROP PROCEDURE IF EXISTS add_inbox_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_inbox_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('inbox', 'inbox_fulltext', 'no, content, inbox_index, msg_file, attached_file');
END $$
DELIMITER ;
-- EOC

-- Formular
DROP PROCEDURE IF EXISTS add_correspondence_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_correspondence_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('formular', 'correspondence_fulltext', 'betrifft, nurtext, rechnungsnummer');
END $$
DELIMITER ;
-- EOC

-- Dateien
DROP PROCEDURE IF EXISTS add_files_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_files_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('dateien', 'files_fulltext', 'pfad, beschreibung');
END $$
DELIMITER ;
-- EOC

-- Adress-Emails
DROP PROCEDURE IF EXISTS add_address_emails_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_address_emails_default_fulltext_indexes()
BEGIN
    CALL us_add_fulltext_index_to_table('address_emails', 'addresses_emails_fulltext', 'email');
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- REMOVE FULLTEXT INDEXES
-- --------------------------------------------------------------------------------------------------
-- Projekte
DROP PROCEDURE IF EXISTS remove_projects_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_projects_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('projekte', 'projects_fulltext');
END $$
DELIMITER ;
-- EOC

-- Adressen
DROP PROCEDURE IF EXISTS remove_addresses_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_addresses_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('adressen', 'addresses_fulltext');
END $$
DELIMITER ;
-- EOC

-- Address Descriptions
DROP PROCEDURE IF EXISTS remove_address_descriptions_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_address_descriptions_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('adrbeschr', 'address_description_fulltext');
END $$
DELIMITER ;
-- EOC

-- Todo
DROP PROCEDURE IF EXISTS remove_opentasks_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_opentasks_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('todo', 'opentasks_fulltext');
END $$
DELIMITER ;
-- EOC

-- Externalexpenses
DROP PROCEDURE IF EXISTS remove_externalexpenses_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_externalexpenses_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('externalexpenses', 'externalexpenses_fulltext');
END $$
DELIMITER ;
-- EOC

-- Fees
DROP PROCEDURE IF EXISTS remove_fees_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_fees_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('fees', 'fees_fulltext');
END $$
DELIMITER ;
-- EOC

-- Personalappointments
DROP PROCEDURE IF EXISTS remove_personalappointments_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_personalappointments_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('perstermin', 'personalappointments_fulltext');
END $$
DELIMITER ;
-- EOC

-- Drawings
DROP PROCEDURE IF EXISTS remove_drawings_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_drawings_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('drawings', 'drawings_fulltext');
END $$
DELIMITER ;
-- EOC

-- Drawings Outbox
DROP PROCEDURE IF EXISTS remove_drawings_outbox_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_drawings_outbox_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('planausgabe', 'drawings_outbox_fulltext');
END $$
DELIMITER ;
-- EOC

-- Outbox
DROP PROCEDURE IF EXISTS remove_outbox_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_outbox_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('outbox', 'outbox_fulltext');
END $$
DELIMITER ;
-- EOC

-- Inbox
DROP PROCEDURE IF EXISTS remove_inbox_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_inbox_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('inbox', 'inbox_fulltext');
END $$
DELIMITER ;
-- EOC

-- Formular
DROP PROCEDURE IF EXISTS remove_correspondence_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_correspondence_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('formular', 'correspondence_fulltext');
END $$
DELIMITER ;
-- EOC

-- Dateien
DROP PROCEDURE IF EXISTS remove_files_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_files_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('dateien', 'files_fulltext');
END $$
DELIMITER ;
-- EOC

-- Adress-Emails
DROP PROCEDURE IF EXISTS remove_address_emails_default_fulltext_indexes;
-- EOC
DELIMITER $$
CREATE PROCEDURE remove_address_emails_default_fulltext_indexes()
BEGIN
    CALL us_drop_index_if_exists('address_emails', 'addresses_emails_fulltext');
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- FULLTEXT SEARCH PROCEDURES
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS add_all_fulltext_indexes_if_fulltext_search_is_active;
-- EOC
DELIMITER $$
CREATE PROCEDURE add_all_fulltext_indexes_if_fulltext_search_is_active()
BEGIN
    IF EXISTS ( (SELECT * FROM system_settings WHERE name='fulltext_search_activate' AND value=1) ) THEN
        CALL add_projects_default_fulltext_indexes;
        CALL add_addresses_default_fulltext_indexes;
        CALL add_address_descriptions_default_fulltext_indexes;
        CALL add_opentasks_default_fulltext_indexes;
        CALL add_externalexpenses_default_fulltext_indexes;
        CALL add_fees_default_fulltext_indexes;
        CALL add_personalappointments_default_fulltext_indexes;
        CALL add_drawings_default_fulltext_indexes;
        CALL add_drawings_outbox_default_fulltext_indexes;
        CALL add_outbox_default_fulltext_indexes;
        CALL add_inbox_default_fulltext_indexes;
        CALL add_correspondence_default_fulltext_indexes;
        CALL add_files_default_fulltext_indexes;
    END IF;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Check if every project has a top node with lft=1 if not -> insert for such project a top node
-- --------------------------------------------------------------------------------------------------
-- Check if there are too much fee areas in one project
DROP PROCEDURE IF EXISTS check_projects_for_top_node;
-- EOC
DELIMITER $$
CREATE PROCEDURE check_projects_for_top_node(IN temptableName VARCHAR (100))
check_projects_for_top_nodelbl:BEGIN
    DECLARE project_ids INTEGER;
    DECLARE project_names VARCHAR(300);
    DECLARE rgt_values INTEGER;
    DECLARE l_last_row_fetched INTEGER;
    DECLARE project_with_no_topnode CURSOR FOR SELECT 
    projects.id,
    projects.project_name,
    CASE WHEN max(work_packages.rgt) IS NULL THEN 0 ELSE max(work_packages.rgt) END
        FROM
            projects
                LEFT JOIN
            work_packages on work_packages.project_id = projects.id
        WHERE
            projects.id NOT IN (SELECT 
            project_id
        FROM
            work_packages
        GROUP BY project_id)
        OR (projects.id IN (SELECT 
            project_id
        FROM
            work_packages wp
        WHERE
            1 != (SELECT 
                MIN(lft)
            FROM
                work_packages wp2
            WHERE
                wp2.project_id = wp.project_id))) GROUP BY projects.id;
                
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;
    DECLARE EXIT HANDLER FOR SQLEXCEPTION, SQLWARNING
    BEGIN
        ROLLBACK;
        SET autocommit=1;
    END;
    SET autocommit=0;
    START TRANSACTION;

    SET l_last_row_fetched=0;
    OPEN project_with_no_topnode;
    cursor_loop: LOOP -- -------------------------------------
            FETCH project_with_no_topnode INTO project_ids, project_names, rgt_values;
            IF l_last_row_fetched=1 THEN
                    LEAVE cursor_loop;
            END IF;
            IF rgt_values = 0 THEN
                INSERT INTO work_packages (`name`, `lft`, `rgt`, `project_id`) VALUES (project_names, '1', '4', project_ids);
                INSERT INTO work_packages (`name`, `lft`, `rgt`, `project_id`, `is_fee_area`) VALUES ('Standard', '2', '3', project_ids, 1);
            ELSE
                INSERT INTO work_packages (`name`, `lft`, `rgt`, `project_id`) VALUES (project_names, '1', rgt_values+1, project_ids);
            END IF;

    SET @s = CONCAT('INSERT INTO ',temptableName,' (project_id) VALUES (',project_ids,');');
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  project_with_no_topnode;

    COMMIT;
    SET autocommit=1;
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Times/Travels/Extras check if wp id belongs to the project, if not update the wrong entries with the first valied wp id (set auto_assigned_wp true for this entries)
-- --------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS update_wrong_entries;
-- EOC
DELIMITER $$
CREATE PROCEDURE update_wrong_entries(IN tableName VARCHAR(100), OUT validation_string VARCHAR(4))
BEGIN
    DECLARE wrong_ids INTEGER;
    DECLARE project_ids INTEGER;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;    
    DECLARE wrong_vals CURSOR FOR SELECT * FROM my_view; 
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    DROP VIEW IF EXISTS my_view;
    
    IF tableName != 'times' THEN
        SET @mywhere =  CONCAT(' myTable.is_deducted = 0');
    ELSE
        SET @mywhere =  CONCAT(' myTable.is_deducted = 0 AND myTable.is_deducted_staff = 0 AND myTable.is_deducted_woa = 0');
    END IF;


    SET @s = CONCAT('CREATE VIEW my_view as 
                    SELECT id, project_id FROM ', tableName, ' myTable
                            WHERE
                    work_package_id NOT IN (SELECT id
                            FROM
                    work_packages wp
                            WHERE
                    myTable.project_id = wp.project_id) AND myTable.project_id IS NOT NULL AND myTable.staff_id IS NOT NULL
                            AND
                    ',@mywhere);
                    
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

    OPEN wrong_vals;
    cursor_loop: LOOP -- -------------------------------------
        FETCH wrong_vals INTO wrong_ids, project_ids;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;            
            /*Do something with the row fetched*/
            /* SET by SELECT */
            (SELECT id 
                INTO @myWPID
                FROM work_packages WHERE project_id = project_ids AND rgt-lft = 1 LIMIT 1);

            IF @myWPID IS NOT NULL THEN
                SET @s = CONCAT('UPDATE ',tableName,' SET work_package_id = ', @myWPID,', auto_assigned_wp = 2 WHERE id = ', wrong_ids);
                PREPARE stmt2 FROM @s;
                EXECUTE stmt2;
            END IF;
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  wrong_vals;
    
    DROP VIEW IF EXISTS my_view;

    SET validation_string = 'true';	
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS get_wrong_deducted_entries;
-- EOC
DELIMITER $$
CREATE PROCEDURE get_wrong_deducted_entries(IN tableName VARCHAR(100), IN temptableName VARCHAR (100))
BEGIN    
    IF tableName != 'times' THEN
        SET @mywhere =  CONCAT(' myTable.is_deducted != 0');
    ELSE
        SET @mywhere =  CONCAT(' (myTable.is_deducted != 0 OR myTable.is_deducted_staff != 0 OR myTable.is_deducted_woa != 0)');
    END IF;

    SET @s = CONCAT('INSERT INTO ',temptableName,' (project_id, staff_id, date) SELECT project_id, staff_id, date FROM ', tableName, ' myTable
                        WHERE
                    work_package_id NOT IN (SELECT id
                        FROM
                    work_packages wp
                        WHERE
                    myTable.project_id = wp.project_id) AND myTable.project_id IS NOT NULL AND myTable.staff_id IS NOT NULL
                        AND
                    ',@mywhere);
                    
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
END $$
DELIMITER ;
-- EOC-- USC Spalten fuer bestehenden Tabellen werden angelegt.

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Projects
CALL us_add_to_table2('projektangebot', 'title', 'varchar(80) DEFAULT \'\' AFTER `datei`');
-- EOC
CALL us_add_to_table2('projektangebot', 'GueltigBis', 'DATETIME DEFAULT NULL AFTER `Bezeichnung`');
-- EOC
CALL us_add_to_table2('projektangebot', 'BeauftragNr', 'varchar(40) DEFAULT NULL AFTER `GueltigBis`' );
-- EOC
CALL us_add_to_table2('projektangebot', 'BeauftragDatum', 'DATETIME DEFAULT NULL AFTER `BeauftragNr`');
-- EOC
CALL us_add_to_table2('projektangebot', 'BeauftragWahrscheinl', 'INT( 11 ) NULL AFTER `BeauftragDatum`');
-- EOC
CALL us_add_to_table2('projektangebot', 'Person', 'INT( 11 ) NULL AFTER `BeauftragWahrscheinl`');
-- EOC
CALL us_add_to_table2('projektangebot', 'IsCalcexp', 'BOOLEAN DEFAULT 0 AFTER `Person`');
-- EOC
CALL us_add_to_table2('projektangebot', 'IsEstimation', 'BOOLEAN DEFAULT 0 AFTER `IsCalcexp`');
-- EOC
CALL us_add_to_table2('projektangebot', 'portlet_desc', 'LONGTEXT NULL DEFAULT NULL AFTER `IsEstimation`');
-- EOC
CALL us_add_to_table2('projektangebot', 'portlet_data', 'LONGTEXT NULL DEFAULT NULL AFTER `IsEstimation`');
-- EOC
CALL us_add_to_table2('projektangebot', 'vat_rate', 'DOUBLE NULL DEFAULT NULL AFTER `BeauftragWahrscheinl`');
-- EOC
CALL change_column('projektangebot', 'vat_rate', 'DOUBLE NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('projektangebot', 'currency_id', 'INT( 11 ) NULL DEFAULT NULL');
-- EOC
CALL change_column('projektangebot', 'Bezeichnung', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `title`');
-- EOC
CALL us_add_to_table2('projektangebot', 'retention_type', 'INT( 11 ) NOT NULL DEFAULT 0');
-- EOC
CALL us_add_to_table2('projektangebot', 'retention_years', 'INT( 11 ) NOT NULL DEFAULT 0');
-- EOC
CALL us_add_to_table2('projektangebot', 'retention', 'DOUBLE NOT NULL DEFAULT 0');
-- EOC

CALL us_add_to_table2('bereichetab', 'aktiv', 'BOOLEAN DEFAULT 1 AFTER `indi`' );
-- EOC
CALL us_add_to_table2('bereichetab', 'special_id', 'INT( 11 ) NULL AFTER `aktiv`' );
-- EOC
CALL us_add_to_table2('bereichetab', 'was_special_id', 'INT( 11 ) NULL AFTER `aktiv`' );
-- EOC

CALL us_add_to_table2('honorarbereiche', 'is_special', 'BOOLEAN DEFAULT 0 AFTER `LfdNr`' );
-- EOC
CALL us_add_to_table2('honorarbereiche', 'is_project_only', 'BOOLEAN DEFAULT 0 AFTER `is_special`' );
-- EOC

CALL us_add_to_table2('arbeitsbereiche', 'is_special', 'BOOLEAN DEFAULT 0 AFTER `LfdNr`' );
-- EOC
CALL us_add_to_table2('arbeitsbereiche', 'is_project_only', 'BOOLEAN DEFAULT 0 AFTER `is_special`' );
-- EOC
CALL us_add_to_table2('arbeitsbereiche', 'is_purpose_required', 'BOOLEAN DEFAULT 0 AFTER `LfdNr`' );
-- EOC

CALL us_add_to_table2('projekte', 'company_id', 'INT( 11 ) NULL COMMENT "Project company ID rules over staff company ID" AFTER `AendVon`' );
-- EOC
CALL us_add_to_table2('projekte', 'currency_id', 'INT( 11 ) NULL COMMENT "Is derived on creation from company" AFTER `company_id`' );
-- EOC
CALL us_add_to_table2('projekte', 'ignore_required_purpose', 'BOOLEAN DEFAULT 0 AFTER `LfdNr`' );
-- EOC
CALL us_add_to_table2('projekte', 'work_package_preset_id', 'BIGINT DEFAULT NULL AFTER `ignore_required_purpose`' );
-- EOC
-- SV_Pfad AS path_correspondence
CALL us_add_to_table2('projekte', 'path_correspondence_default', 'varchar(255) COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `SV_Pfad`' );
-- EOC
-- EM_Pfad AS path_inbox
CALL us_add_to_table2('projekte', 'path_inbox_default', 'varchar(255) COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `EM_Pfad`' );
-- EOC
-- EM_OUT_Pfad AS path_outbox
CALL us_add_to_table2('projekte', 'path_outbox_default', 'varchar(255) COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `EM_OUT_Pfad`' );
-- EOC
-- path_drawings
CALL us_add_to_table2('projekte', 'planpfad', 'varchar(255) COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `path_outbox_default`' );
-- EOC
CALL us_add_to_table2('projekte', 'path_drawings_default', 'varchar(255) COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `planpfad`' );
-- EOC
-- original structure tree nodes (prepareation for future)
CALL us_add_to_table2('projekte', 'orig_structure_id', 'int(11) DEFAULT NULL AFTER `path_drawings_default`' );
-- EOC
CALL us_add_to_table2('projekte', 'orig_structure_tree_root_node', 'varchar(255) COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `orig_structure_id`' );
-- EOC
CALL us_add_to_table2('projekte', 'orig_translated_structure_tree_root_node', 'varchar(255) COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `orig_structure_tree_root_node`' );
-- EOC
CALL us_add_to_table2('projekte', 'auto_calc', 'INT( 2 ) DEFAULT 1' );
-- EOC
CALL us_add_to_table2('projekte', 'auto_calc_fee', 'INT( 2 ) DEFAULT 0' );
-- EOC
CALL us_add_to_table2('projekte', 'show_tags', 'INT( 2 ) DEFAULT 0' );
-- EOC
CALL us_add_to_table2('projekte', 'use_budget_top_prop', 'INT( 2 ) DEFAULT 0' );
-- EOC

-- X2 Migration helper fields
CALL us_add_to_table2('lut_hon_proj_history', 'migrate_valid_from_date', 'DATETIME NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('sollab_history', 'migrate_valid_from_date', 'DATETIME NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('schaetzungfertig_history', 'migrate_valid_from_date', 'DATETIME NULL DEFAULT NULL');
-- EOC

-- X2 Related IDs in formular/correspondence needed by update_alloc_with_payments_fees
CALL us_add_to_table2('formular', 'related_id', 'INT( 11 ) DEFAULT NULL COMMENT \'Related item ID\'' );
-- EOC
CALL us_add_to_table2('formular', 'related_type', 'VARCHAR(30) DEFAULT NULL COMMENT \'Related item Type\'' );
-- EOC

-- Add more options to fee_multipliers
CALL us_add_to_table2('aufschlag', 'is_standard', 'BOOLEAN DEFAULT 0 AFTER SatzNr' );
-- EOC
CALL us_add_to_table2('aufschlag', 'is_r2', 'BOOLEAN DEFAULT 0 AFTER is_standard' );
-- EOC
CALL us_add_to_table2('aufschlag', 'is_r3', 'BOOLEAN DEFAULT 0 AFTER is_r2' );
-- EOC
CALL us_add_to_table2('aufschlag', 'is_r4', 'BOOLEAN DEFAULT 0 AFTER is_r3' );
-- EOC
-- USC Stelle sicher, dass Views angelegt werden koennen.

-- Cleans corrupt backup recovery - just in case

DROP TABLE IF EXISTS `address_categories`;
-- EOC
DROP TABLE IF EXISTS `address_descriptions`;
-- EOC
DROP TABLE IF EXISTS `address_descriptions_comp`;
-- EOC
DROP TABLE IF EXISTS `address_files`;
-- EOC
DROP TABLE IF EXISTS `address_groups`;
-- EOC
DROP TABLE IF EXISTS `address_relations_both`;
-- EOC
DROP TABLE IF EXISTS `address_relations_comp`;
-- EOC
DROP TABLE IF EXISTS `addresses`;
-- EOC
DROP TABLE IF EXISTS `addresses_addi_def`;
-- EOC
DROP TABLE IF EXISTS `addresses_fullname`;
-- EOC
DROP TABLE IF EXISTS `address_groups_fullname`;
-- EOC
DROP TABLE IF EXISTS `budget_plan`;
-- EOC
DROP TABLE IF EXISTS `budget_plan_extended_data`;
-- EOC
DROP TABLE IF EXISTS `budget_plan_history`;
-- EOC
DROP TABLE IF EXISTS `calls`;
-- EOC
DROP TABLE IF EXISTS `companies`;
-- EOC
DROP TABLE IF EXISTS `company_installation_merged`;
-- EOC
DROP TABLE IF EXISTS `completion_level`;
-- EOC
DROP TABLE IF EXISTS `completion_level_extended_data`;
-- EOC
DROP TABLE IF EXISTS `completion_level_history`;
-- EOC
DROP TABLE IF EXISTS `correspondence`;
-- EOC
DROP TABLE IF EXISTS `correspondence_addi_def`;
-- EOC
DROP TABLE IF EXISTS `correspondence_attachments`;
-- EOC
DROP TABLE IF EXISTS `correspondence_locking`;
-- EOC
DROP TABLE IF EXISTS `correspondence_readonly`;
-- EOC
DROP TABLE IF EXISTS `correspondence_structure`;
-- EOC
DROP TABLE IF EXISTS `correspondence_types`;
-- EOC
DROP TABLE IF EXISTS `correspondence_types_structure`;
-- EOC
DROP TABLE IF EXISTS `cost_centres`;
-- EOC
DROP TABLE IF EXISTS `countries`;
-- EOC
DROP TABLE IF EXISTS `deletions`;
-- EOC
DROP TABLE IF EXISTS `dispatch_modes`;
-- EOC
DROP TABLE IF EXISTS `drawing_sizes`;
-- EOC
DROP TABLE IF EXISTS `drawing_types`;
-- EOC
DROP TABLE IF EXISTS `drawings_addi_def`;
-- EOC
DROP TABLE IF EXISTS `drawings_outbox`;
-- EOC
DROP TABLE IF EXISTS `expenses_rates`;
-- EOC
DROP TABLE IF EXISTS `externalexpenses_addi_invoiced_def`;
-- EOC
DROP TABLE IF EXISTS `externalexpenses_addi_merged`;
-- EOC
DROP TABLE IF EXISTS `externalexpenses_addi_merged_def`;
-- EOC
DROP TABLE IF EXISTS `externalexpenses_addi_planned_def`;
-- EOC
DROP TABLE IF EXISTS `externalexpenses_calc`;
-- EOC
DROP TABLE IF EXISTS `externalexpenses_merged`;
-- EOC
DROP TABLE IF EXISTS `externalexpenses_planned_calc`;
-- EOC
DROP TABLE IF EXISTS `extras_rates`;
-- EOC
DROP TABLE IF EXISTS `extras_rates_history`;
-- EOC
DROP TABLE IF EXISTS `extras_types`;
-- EOC
DROP TABLE IF EXISTS `fee_accounts`;
-- EOC
DROP TABLE IF EXISTS `fee_areas`;
-- EOC
DROP TABLE IF EXISTS `fee_multipliers`;
-- EOC
DROP TABLE IF EXISTS `fees_addi_invoiced_def`;
-- EOC
DROP TABLE IF EXISTS `fees_addi_merged`;
-- EOC
DROP TABLE IF EXISTS `fees_addi_merged_def`;
-- EOC
DROP TABLE IF EXISTS `fees_addi_planned_def`;
-- EOC
DROP TABLE IF EXISTS `fees_calc`;
-- EOC
DROP TABLE IF EXISTS `fees_merged`;
-- EOC
DROP TABLE IF EXISTS `fees_planned_calc`;
-- EOC
DROP TABLE IF EXISTS `fees_reminder_level_logging`;
-- EOC
DROP TABLE IF EXISTS `files`;
-- EOC
DROP TABLE IF EXISTS `holiday_rates`;
-- EOC
DROP TABLE IF EXISTS `holidays`;
-- EOC
DROP TABLE IF EXISTS `inbox_addi_def`;
-- EOC
DROP TABLE IF EXISTS `inbox_path`;
-- EOC
DROP TABLE IF EXISTS `locking`;
-- EOC
DROP TABLE IF EXISTS `locking_info`;
-- EOC
DROP TABLE IF EXISTS `lut_addresses_address_categories`;
-- EOC
DROP TABLE IF EXISTS `lut_addresses_address_group`;
-- EOC
DROP TABLE IF EXISTS `lut_addresses_projects`;
-- EOC
DROP TABLE IF EXISTS `lut_correspondence_copies`;
-- EOC
DROP TABLE IF EXISTS `lut_correspondence_structure_type`;
-- EOC
DROP TABLE IF EXISTS `lut_drawings_addresses`;
-- EOC
DROP TABLE IF EXISTS `lut_fee_areas_proj`;
-- EOC
DROP TABLE IF EXISTS `lut_fee_areas_proj_history`;
-- EOC
DROP TABLE IF EXISTS `lut_outbox_attachments`;
-- EOC
DROP TABLE IF EXISTS `lut_outbox_bcc_addresses`;
-- EOC
DROP TABLE IF EXISTS `lut_personalapp_address`;
-- EOC
DROP TABLE IF EXISTS `lut_proj_category_rates`;
-- EOC
DROP TABLE IF EXISTS `lut_proj_proj_grps`;
-- EOC
DROP TABLE IF EXISTS `lut_route_proj`;
-- EOC
DROP TABLE IF EXISTS `lut_staff_occ_group`;
-- EOC
DROP TABLE IF EXISTS `lut_task_desc_proj`;
-- EOC
DROP TABLE IF EXISTS `lut_transmittal_form_addresses`;
-- EOC
DROP TABLE IF EXISTS `lut_wiw_items_projects`;
-- EOC
DROP TABLE IF EXISTS `lut_wiw_items_themes`;
-- EOC
DROP TABLE IF EXISTS `lut_work_areas_work_areas_categories`;
-- EOC
DROP TABLE IF EXISTS `lut_working_user`;
-- EOC
DROP TABLE IF EXISTS `measurements`;
-- EOC
DROP TABLE IF EXISTS `numbergeneration`;
-- EOC
DROP TABLE IF EXISTS `numbergeneration_log`;
-- EOC
DROP TABLE IF EXISTS `occupation_groups`;
-- EOC
DROP TABLE IF EXISTS `occupation_groups_fee_multipliers`;
-- EOC
DROP TABLE IF EXISTS `opentasks`;
-- EOC
DROP TABLE IF EXISTS `opentasks_addi_def`;
-- EOC
DROP TABLE IF EXISTS `outbox_addi_def`;
-- EOC
DROP TABLE IF EXISTS `outbox_path`;
-- EOC
DROP TABLE IF EXISTS `overtime`;
-- EOC
DROP TABLE IF EXISTS `overtime_rates`;
-- EOC
DROP TABLE IF EXISTS `payments_fees_values`;
-- EOC
DROP TABLE IF EXISTS `personalappointments`;
-- EOC
DROP TABLE IF EXISTS `project_archives`;
-- EOC
DROP TABLE IF EXISTS `project_categories`;
-- EOC
DROP TABLE IF EXISTS `project_offers`;
-- EOC
DROP TABLE IF EXISTS `project_participants_addi_def`;
-- EOC
DROP TABLE IF EXISTS `project_phases`;
-- EOC
DROP TABLE IF EXISTS `projects`;
-- EOC
DROP TABLE IF EXISTS `projects_addi_def`;
-- EOC
DROP TABLE IF EXISTS `projects_groups`;
-- EOC
DROP TABLE IF EXISTS `report_budget_plan_history`;
-- EOC
DROP TABLE IF EXISTS `report_completion_level`;
-- EOC
DROP TABLE IF EXISTS `report_completion_level_bbl1`;
-- EOC
DROP TABLE IF EXISTS `report_completion_level_bbl2`;
-- EOC
DROP TABLE IF EXISTS `report_completion_level_current`;
-- EOC
DROP TABLE IF EXISTS `report_completion_level_fee`;
-- EOC
DROP TABLE IF EXISTS `report_completion_level_history`;
-- EOC
DROP TABLE IF EXISTS `report_cost_centres_info`;
-- EOC
DROP TABLE IF EXISTS `report_externalexpenses`;
-- EOC
DROP TABLE IF EXISTS `report_externalexpenses_planned`;
-- EOC
DROP TABLE IF EXISTS `report_extras`;
-- EOC
DROP TABLE IF EXISTS `report_extras_logging`;
-- EOC
DROP TABLE IF EXISTS `report_fee_areas_project_allocation`;
-- EOC
DROP TABLE IF EXISTS `report_fee_areas_project_allocation_history`;
-- EOC
DROP TABLE IF EXISTS `report_measurements`;
-- EOC
DROP TABLE IF EXISTS `report_overtime_datemonth`;
-- EOC
DROP TABLE IF EXISTS `report_projects_fee_areas`;
-- EOC
DROP TABLE IF EXISTS `report_start_end_times`;
-- EOC
DROP TABLE IF EXISTS `report_times_from_to`;
-- EOC
DROP TABLE IF EXISTS `report_times`;
-- EOC
DROP TABLE IF EXISTS `report_times_changed_gross`;
-- EOC
DROP TABLE IF EXISTS `report_times_datemonth`;
-- EOC
DROP TABLE IF EXISTS `report_times_logging`;
-- EOC
DROP TABLE IF EXISTS `report_travels`;
-- EOC
DROP TABLE IF EXISTS `report_travels_logging`;
-- EOC
DROP TABLE IF EXISTS `routes`;
-- EOC
DROP TABLE IF EXISTS `staff`;
-- EOC
DROP TABLE IF EXISTS `staff_additional_def`;
-- EOC
DROP TABLE IF EXISTS `staff_favourite_proj`;
-- EOC
DROP TABLE IF EXISTS `start_end_times`;
-- EOC
DROP TABLE IF EXISTS `sys_installation`;
-- EOC
DROP TABLE IF EXISTS `system_settings`;
-- EOC
DROP TABLE IF EXISTS `target_hours_daily`;
-- EOC
DROP TABLE IF EXISTS `task_descriptions`;
-- EOC
DROP TABLE IF EXISTS `times_rates`;
-- EOC
DROP TABLE IF EXISTS `times_rates_history`;
-- EOC
DROP TABLE IF EXISTS `travels_rates`;
-- EOC
DROP TABLE IF EXISTS `travels_rates_history`;
-- EOC
DROP TABLE IF EXISTS `travels_types`;
-- EOC
DROP TABLE IF EXISTS `user_settings`;
-- EOC
DROP TABLE IF EXISTS `vacations`;
-- EOC
DROP TABLE IF EXISTS `whatiswhere_addi_def`;
-- EOC
DROP TABLE IF EXISTS `wiw_articles`;
-- EOC
DROP TABLE IF EXISTS `wiw_filings`;
-- EOC
DROP TABLE IF EXISTS `wiw_items`;
-- EOC
DROP TABLE IF EXISTS `wiw_merged`;
-- EOC
DROP TABLE IF EXISTS `wiw_racks`;
-- EOC
DROP TABLE IF EXISTS `wiw_rooms`;
-- EOC
DROP TABLE IF EXISTS `wiw_themes`;
-- EOC
DROP TABLE IF EXISTS `wiw_trays`;
-- EOC
DROP TABLE IF EXISTS `work_areas`;
-- EOC
DROP TABLE IF EXISTS `work_areas_categories`;
-- EOC
DROP TABLE IF EXISTS `workinput`;
-- EOC
DROP TABLE IF EXISTS `zip_to_name_merged`;
-- EOC

-- USC X2 Views fuer bestehende Tabellen werden angelegt.

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Projects
CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `project_offers`
 AS SELECT LfdNr AS id, ProjLfdNr AS project_id, AdrLfdNr AS address_id, AngebotsNummer AS offer_no, 
 AngebotsDatum AS date, AngebotsStatus AS status, Datei AS file, title, Bezeichnung AS name,
 GueltigBis AS valid_to_date, BeauftragNr AS order_no, BeauftragDatum AS order_date, BeauftragWahrscheinl AS order_probability, vat_rate, Person AS staff_id,
 IsCalcexp AS is_calcexp, IsEstimation AS is_estimation, portlet_desc, portlet_data, currency_id,
 retention_type, retention_years, retention
 FROM projektangebot;
-- EOC


CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `work_areas_categories`
 AS SELECT LfdNr AS id, indi AS one_project_only, BereichName AS name, aktiv AS is_active, special_id, was_special_id FROM bereichetab;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `fee_areas`
 AS SELECT LfdNr AS id, HonorarName AS name, Feld AS sort, extern AS external, is_special, is_project_only FROM honorarbereiche;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `work_areas`
 AS SELECT LfdNr AS id, ArbName AS name, Feld AS sort, HonBer AS fee_area_id, is_special, is_purpose_required, is_project_only FROM arbeitsbereiche;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_work_areas_work_areas_categories`
 AS SELECT ArbNr AS work_area_id, SatzNr AS work_area_categories_id, schwelle AS threshold FROM lut_arb_hon;
-- EOC


CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `projects`
 AS SELECT LfdNr AS id, Projekt AS project_name, ProjektNummer AS project_number, Beginn AS start_date, Ende AS end_date, Beschreibung AS purpose,
 Ort AS location, BerSatz AS work_area_category_id, Bauherr AS client_id, Planer1 AS project_manager_id, Planer2 AS office_in_charge_id,
 company_id, currency_id, LfdNr_Phase AS project_phase_id, Kategorie as project_category_id,
 SV_Pfad AS path_correspondence, path_correspondence_default, 
 EM_Pfad AS path_inbox, path_inbox_default, EM_OUT_Pfad AS path_outbox, path_outbox_default, Planpfad AS path_drawings, path_drawings_default,
 orig_structure_id, orig_structure_tree_root_node, orig_translated_structure_tree_root_node,
 LfdNr_ProjektArchiv AS project_archive_id,
 PrivatProjekt AS is_private_project, Aufwand AS is_calc_from_expenses, AbrArt AS is_add_to_indirect_costs, StdAbr AS is_included_in_time_sheet,
 ErstVon AS created_by_id, AendVon AS changed_by_id, ErstAm AS created_date, AendAm AS change_date, AufbewahrungsFrist AS safekeeping_period,
 Bilddatei AS photo_file, MessSize_LfdNr As measurement_id, MessSize_Value AS measurement_value, isSchwelle AS is_threshold,
 BudgetNotBound as is_budget_not_bound, SyncroDok AS sync_correspondence, SyncroEmailIn AS sync_inbox, SyncroEmailOut AS sync_outbox, ignore_required_purpose, work_package_preset_id, auto_calc, auto_calc_fee, show_tags, use_budget_top_prop
 FROM projekte;
-- EOC


CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_fee_areas_proj`
 AS SELECT LfdNr AS id, Hon_LfdNr AS fee_area_id, Proj_LfdNr AS project_id, HonStatus AS status, History_LfdNr AS lut_fee_areas_proj_history_id,
 Stunden AS hours, Kosten AS costs, KostenProzent AS costs_percent, KostenGesamt AS costs_sum, KostenGesamtEingabe AS is_costs_sum
 FROM lut_hon_proj;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_fee_areas_proj_history`
 AS SELECT LfdNr AS id, Proj_LfdNr AS project_id, GueltigBis AS valid_to_date, HistBem AS comment, migrate_valid_from_date
 FROM lut_hon_proj_history;
-- EOC


CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `budget_plan`
 AS SELECT LfdNr_Projekt AS project_id, LfdNr_AB AS area_id, LfdNr_MA AS staff_id, History_LfdNr AS budget_plan_history_id,
 SollKosten AS planned_costs, SollStunden AS planned_hours, Beginn AS start_date, Ende AS end_date,
 Datum AS last_changed_date, Anteil AS proportion, schwelle AS threshold FROM sollab;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `budget_plan_history`
 AS SELECT LfdNr AS id, Proj_LfdNr AS project_id, GueltigBis AS valid_to_date, HistBem AS comment, migrate_valid_from_date 
 FROM sollab_history;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `completion_level`
 AS SELECT LfdNr AS id, LfdNr_Projekt AS project_id, LfdNr_AB AS work_area_id, LfdNr_MA AS staff_id, Datum AS date, Grad AS completion_level,
 History_LfdNr AS completion_level_history_id FROM schaetzungfertig;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `completion_level_history`
 AS SELECT LfdNr AS id, Proj_LfdNr AS project_id, GueltigBis AS valid_to_date, HistBem AS comment, migrate_valid_from_date 
 FROM schaetzungfertig_history;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `fee_multipliers` AS
 SELECT LfdNr AS id, OrdNr AS sort, Aufschlag AS name, Beschreibung AS purpose, SatzNr AS rate_id, 
 is_standard, is_r2, is_r3, is_r4, Arbeitskosten AS is_work, MAuswert AS is_ignore_mgmt
 FROM aufschlag;
-- EOC

-- USC Procedures fuer das X2 Update werden angelegt.

-- -----------------------------------------
-- Calculates the sum of the fields
-- -----------------------------------------

DROP PROCEDURE IF EXISTS calc_sum;
-- EOC
DELIMITER $$
CREATE PROCEDURE calc_sum(curr_project_id VARCHAR(255))
BEGIN
	
    DECLARE done INT DEFAULT FALSE;
    DECLARE left_val INT;
    DECLARE right_val INT;    
    DECLARE id_val INT;       
    DECLARE wp CURSOR FOR SELECT id, lft, rgt FROM work_packages WHERE rgt-lft > 1 AND project_id = curr_project_id;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

    OPEN wp;

    REPEAT
            FETCH wp INTO id_val, left_val, right_val;

            IF NOT done THEN
                -- Fee area related values 
                /* SET by SELECT */
                (SELECT SUM(fee_costs), SUM(fee_hours) 
                    INTO @fee_costs_sum, @fee_hours_sum
                    FROM work_packages WHERE lft >= left_val AND rgt <=  right_val AND project_id = curr_project_id AND (rgt-lft=1 OR is_fee_area IS TRUE));
                -- Work area related values
                /* SET by SELECT */
                (SELECT SUM(budget_planned_costs), SUM(budget_planned_hours), SUM(budget_proportion)
                    INTO @budget_costs_sum, @budget_hours_sum, @budget_proportion
                    FROM work_packages WHERE lft > left_val AND rgt <  right_val AND project_id = curr_project_id AND rgt-lft = 1);
                 
                UPDATE work_packages SET fee_costs = @fee_costs_sum where id = id_val;
                UPDATE work_packages SET fee_hours = @fee_hours_sum where id = id_val;
                UPDATE work_packages SET budget_planned_costs = @budget_costs_sum where id = id_val;
                UPDATE work_packages SET budget_planned_hours = @budget_hours_sum where id = id_val;
                UPDATE work_packages SET budget_proportion = @budget_proportion where id = id_val;                   
            END IF;
      UNTIL done END REPEAT;

    CLOSE wp; 
END $$
DELIMITER ;
-- EOC


-- -----------------------------------------
-- Get the earliest date for the start and the latest date for the end
-- -----------------------------------------

DROP PROCEDURE IF EXISTS get_date;
-- EOC
DELIMITER $$
CREATE PROCEDURE get_date(curr_project_id VARCHAR(255))
BEGIN
	
    DECLARE done INT DEFAULT FALSE;
    DECLARE left_val INT;
    DECLARE right_val INT;    
    DECLARE id_val INT;       
    DECLARE wp CURSOR FOR SELECT id, lft, rgt FROM work_packages WHERE rgt-lft > 1 AND project_id = curr_project_id;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

    OPEN wp;
        REPEAT
            FETCH wp INTO id_val, left_val, right_val;

            IF NOT done THEN
                /* SET by SELECT */
                (SELECT MIN(budget_start), MAX(budget_end) 
                    INTO @start_date, @end_date
                    FROM work_packages WHERE lft > left_val AND rgt <  right_val AND project_id = curr_project_id);

                UPDATE work_packages SET budget_start = @start_date where id = id_val;
                UPDATE work_packages SET budget_end = @end_date where id = id_val;        
            END IF;
        UNTIL done END REPEAT;
    CLOSE wp; 
END $$
DELIMITER ;
-- EOC

-- ---------------------------------------
-- Calculates the completion level depending on the proportion
-- ---------------------------------------

DROP PROCEDURE IF EXISTS calc_completion_level;
-- EOC
DELIMITER $$
CREATE PROCEDURE calc_completion_level(curr_project_id VARCHAR(255))
BEGIN

    DECLARE done INT DEFAULT FALSE;
    DECLARE left_val INT;
    DECLARE right_val INT;    
    DECLARE id_val INT;   
    DECLARE proportion_val FLOAT;         
    DECLARE wp CURSOR FOR SELECT id, lft, rgt, budget_proportion FROM work_packages WHERE rgt-lft > 1 AND project_id = curr_project_id AND budget_proportion > 0;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

    OPEN wp;
        REPEAT
            FETCH wp INTO id_val, left_val, right_val, proportion_val;
            IF NOT done THEN
                /* SET by SELECT */
                (SELECT (SUM((budget_proportion * completion_level)/(proportion_val*100))*100)
                    INTO @completion_level
                    FROM work_packages WHERE lft > left_val AND rgt <  right_val AND project_id = curr_project_id);
                UPDATE work_packages SET completion_level = @completion_level where id = id_val;
            END IF;
	UNTIL done END REPEAT;
    CLOSE wp; 
END $$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Create 01migrate_11_12, if not exists
-- --------------------------------------------------------------------------------------------------

DROP PROCEDURE IF EXISTS us_create_01migrate_11_12;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_create_01migrate_11_12()
BEGIN

    DECLARE sql1 TEXT DEFAULT 'CREATE TABLE IF NOT EXISTS `01migrate_11_12` (
  `elem` varchar(100) NOT NULL,
  `info` varchar(500) NOT NULL,
  `date` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
) ENGINE=';

    DECLARE sql2 TEXT DEFAULT '  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;';

    IF EXISTS( (SELECT * FROM information_schema.ENGINES WHERE ENGINE='ARCHIVE' AND SUPPORT='YES') ) THEN
        SET @s = CONCAT(sql1, 'ARCHIVE', sql2);
    ELSE
        SET @s = CONCAT(sql1, 'MyISAM', sql2);
    END IF;
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

END $$
DELIMITER ;
-- EOC

-- ---------------------------------------
-- Add one day to archive dates (Because they are now 'from', instead of 'to' values)
-- ---------------------------------------

DROP PROCEDURE IF EXISTS update_add_one_day_projects_history_dates;
-- EOC
DELIMITER $$
CREATE PROCEDURE update_add_one_day_projects_history_dates(curr_project_id VARCHAR(255))
BEGIN

    DECLARE valid_from_dates datetime;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    -- --------------------------- Get current dates ---------------------------
    DECLARE project_dates CURSOR FOR 	SELECT migrate_valid_from_date
            FROM lut_hon_proj_history AS lh WHERE lh.Proj_LfdNr=curr_project_id AND lh.migrate_valid_from_date > '1901-01-31'
        UNION DISTINCT
            SELECT migrate_valid_from_date
            FROM sollab_history AS bh WHERE bh.Proj_LfdNr=curr_project_id AND bh.migrate_valid_from_date > '1901-01-31'
        UNION DISTINCT
            SELECT migrate_valid_from_date
            FROM schaetzungfertig_history AS cl WHERE cl.Proj_LfdNr=curr_project_id AND cl.migrate_valid_from_date > '1901-01-31'
        ORDER BY migrate_valid_from_date DESC;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    -- --------------------------- Add one day to dates ---------------------------
    OPEN project_dates;
    cursor_loop: LOOP -- -------------------------------------
        FETCH project_dates INTO valid_from_dates;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
        
        -- Do not update, if the new date already exists in work_packages_history
        IF ( (SELECT COUNT(*) FROM work_packages_history AS wph WHERE wph.project_id=curr_project_id AND wph.date=ADDDATE(valid_from_dates, INTERVAL 1 DAY))>0 ) THEN 
            LEAVE cursor_loop;
        END IF;
        
        UPDATE work_packages_history AS wph SET wph.date = ADDDATE(valid_from_dates, INTERVAL 1 DAY) WHERE wph.project_id=curr_project_id AND wph.date=valid_from_dates;

        UPDATE work_packages_history_descriptions AS wph SET wph.archive_date = ADDDATE(valid_from_dates, INTERVAL 1 DAY) WHERE wph.project_id=curr_project_id AND wph.archive_date=valid_from_dates;
        
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  project_dates;
END $$
DELIMITER ;
-- EOC

-- -----------------------------------------
-- Extract a single string from a json like format
-- -----------------------------------------

DROP FUNCTION IF EXISTS `us_json_extract`;
-- EOC
DELIMITER $$
CREATE FUNCTION `us_json_extract`(
    json_doc TEXT,
    key_name VARCHAR (20)
) RETURNS TEXT
DETERMINISTIC
NO SQL
BEGIN
    DECLARE found_val TEXT;
    SET key_name = CONCAT('"', key_name, '"');
    
    -- Do we have the key in the json_doc
    IF INSTR(json_doc, key_name) <= 0 THEN
        RETURN null;
    END IF;
    
    -- Get value
    SET found_val = SUBSTRING_INDEX(
		SUBSTRING_INDEX(
			SUBSTRING_INDEX(
				json_doc,
				key_name,
				- 1
			),
			',"',
			1
		),
		':',
		-1
	);
    
    -- Return trimmed value
    RETURN TRIM(BOTH '"' FROM TRIM(TRAILING '}' FROM TRIM(LEADING '{' FROM found_val)));
END $$
DELIMITER ;
-- EOC

-- -----------------------------------------
-- Extract a single array from a json like format
-- -----------------------------------------

DROP FUNCTION IF EXISTS `us_json_extract_arr`;
-- EOC
DELIMITER $$
CREATE FUNCTION `us_json_extract_arr`(
    json_doc TEXT,
    key_name VARCHAR (20)
) RETURNS TEXT
DETERMINISTIC
NO SQL
BEGIN
    DECLARE found_val TEXT;
    SET key_name = CONCAT('"', key_name, '":[');
    
    -- Do we have the key in the json_doc
    IF INSTR(json_doc, key_name) <= 0 THEN
        RETURN null;
    END IF;
    
    -- Get value
    SET found_val = SUBSTRING_INDEX(
		SUBSTRING_INDEX(
			SUBSTRING_INDEX(
				json_doc,
				key_name,
				- 1
			),
			'],',
			1
		),
		':[',
		-1
	);
    
    -- Return trimmed value
    RETURN TRIM(TRAILING '}' FROM TRIM(LEADING '{' FROM found_val));
END $$
DELIMITER ;
-- EOC

-- -----------------------------------------
-- Get a value from a string set/list
-- -----------------------------------------

DROP FUNCTION IF EXISTS `us_get_from_set`;
-- EOC
DELIMITER $$
CREATE FUNCTION `us_get_from_set`(
    key_index INTEGER,
    strlist TEXT
) RETURNS TEXT
DETERMINISTIC
NO SQL
BEGIN
    RETURN TRIM(BOTH '"' FROM SUBSTRING_INDEX(SUBSTRING_INDEX(strlist, ',', key_index), ',', -1));
END $$
DELIMITER ;
-- EOC



-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- USC Views fuer bestehende Tabellen werden angelegt.
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS us_migrate_11_12_prepare_old_views;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_prepare_old_views()
us_migrate_11_12_prepare_old_viewslbl:BEGIN
    -- Prevent execution, if externekosten is missing
    IF ( (SELECT COUNT(*) FROM information_schema.TABLES WHERE table_schema = DATABASE() AND table_name='externekosten')<=0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_11_12_prepare_old_views';
        LEAVE us_migrate_11_12_prepare_old_viewslbl;
    END IF;

    -- Allow calcexp for externalexpenses
    CALL us_add_to_table2('externekosten', 'is_deducted', 'tinyint(1) unsigned NOT NULL DEFAULT 0' );
    CALL us_add_to_table2('externekosten', 'deducted_day', 'datetime DEFAULT NULL AFTER `is_deducted`' );
    CALL us_add_to_table2('externekosten', 'is_ignore', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `deducted_day`' );

    CALL us_add_to_table2('kopien', 'company_id', 'INT( 11 ) NULL AFTER `KostenStelle`' );
    CALL us_add_to_table2('aenderungkopien', 'company_id', 'INT( 11 ) NULL AFTER `Kostenstelle`' );
    CALL us_add_to_table2('zeiten', 'company_id', 'INT( 11 ) NULL AFTER `Bemerkung_Intern`' );
    CALL us_add_to_table2('zeiten', 'skill_id', 'INT( 11 ) NULL' );
    CALL us_add_to_table2('aenderungzeiten', 'company_id', 'INT( 11 ) NULL AFTER `LfdNr`' );
    CALL us_add_to_table2('aenderungzeiten', 'skill_id', 'INT( 11 ) NULL' );
    CALL us_add_to_table2('km', 'company_id', 'INT( 11 ) NULL AFTER `ErstAm`' );
    CALL us_add_to_table2('aenderungkm', 'company_id', 'INT( 11 ) NULL AFTER `LfdNr`' );

    CALL us_add_to_table2('aenderungzeiten', 'Bemerkung_Intern', 'longtext' );
    CALL us_add_to_table2('aenderungzeiten', 'is_client_changed', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `Bemerkung_Intern`' );
    CALL us_add_to_table2('aenderungzeiten', 'addi_val1', 'double NOT NULL DEFAULT 0' );
    CALL us_add_to_table2('aenderungzeiten', 'addi_val2', 'double NOT NULL DEFAULT 0' );
    CALL us_add_to_table2('aenderungkopien', 'is_client_changed', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `Bemerkung`' );
    CALL us_add_to_table2('aenderungkopien', 'addi_val1', 'double NOT NULL DEFAULT 0' );
    CALL us_add_to_table2('aenderungkopien', 'addi_val2', 'double NOT NULL DEFAULT 0' );
    CALL us_add_to_table2('aenderungkm', 'is_client_changed', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `Bemerkung`' );
    CALL us_add_to_table2('aenderungkm', 'addi_val1', 'double NOT NULL DEFAULT 0' );
    CALL us_add_to_table2('aenderungkm', 'addi_val2', 'double NOT NULL DEFAULT 0' );

    CALL us_add_to_table2('planverz', 'is_autopath', 'BOOLEAN DEFAULT FALSE' );

    CALL us_add_to_table2('planverz', 'Arbeitsbereich', 'INT( 11 ) DEFAULT NULL' );
    CALL us_add_to_table2('planlieferschein', 'Arbeitsbereich', 'INT( 11 ) DEFAULT NULL' );
    CALL us_add_to_table2('planeingang', 'Arbeitsbereich', 'INT( 11 ) DEFAULT NULL' );

    -- Add currencies support
    CALL us_add_to_table2('kopien', 'company_cost_per_unit', 'DOUBLE NULL AFTER `Ep`' );
    CALL us_add_to_table2('kopien', 'company_cost_per_unit_ex', 'DOUBLE NOT NULL DEFAULT 1 AFTER `company_cost_per_unit`' );
    CALL us_add_to_table2('kopien', 'company_cost_per_unit_id', 'INT( 11 ) NULL AFTER `company_cost_per_unit_ex`' );

    CALL us_add_to_table2('kopien', 'company_price_per_unit', 'DOUBLE NULL AFTER `ExtSatz`' );
    CALL us_add_to_table2('kopien', 'company_price_per_unit_ex', 'DOUBLE NOT NULL DEFAULT 1 AFTER `company_price_per_unit`' );
    CALL us_add_to_table2('kopien', 'company_price_per_unit_id', 'INT( 11 ) NULL AFTER `company_price_per_unit_ex`' );

    CALL us_add_to_table2('kopien', 'is_client_changed', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `Bemerkung`' );
    CALL us_add_to_table2('kopien', 'addi_val1', 'double NOT NULL DEFAULT 0' );
    CALL us_add_to_table2('kopien', 'addi_val2', 'double NOT NULL DEFAULT 0' );

    -- Add currencies support
    CALL us_add_to_table2('km', 'company_cost_per_unit', 'DOUBLE NULL AFTER `Ep`' );
    CALL us_add_to_table2('km', 'company_cost_per_unit_ex', 'DOUBLE NOT NULL DEFAULT 1 AFTER `company_cost_per_unit`' );
    CALL us_add_to_table2('km', 'company_cost_per_unit_id', 'INT( 11 ) NULL AFTER `company_cost_per_unit_ex`' );

    CALL us_add_to_table2('km', 'company_price_per_unit', 'DOUBLE NULL AFTER `ExtSatz`' );
    CALL us_add_to_table2('km', 'company_price_per_unit_ex', 'DOUBLE NOT NULL DEFAULT 1 AFTER `company_price_per_unit`' );
    CALL us_add_to_table2('km', 'company_price_per_unit_id', 'INT( 11 ) NULL AFTER `company_price_per_unit_ex`' );

    CALL us_add_to_table2('km', 'is_client_changed', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `Bemerkung`' );
    CALL us_add_to_table2('km', 'addi_val1', 'double NOT NULL DEFAULT 0' );
    CALL us_add_to_table2('km', 'addi_val2', 'double NOT NULL DEFAULT 0' );

    -- Add currencies support
    CALL us_add_to_table2('zeiten', 'company_gross_hourly_wage', 'DOUBLE NULL AFTER `BruttoLohn`' );
    CALL us_add_to_table2('zeiten', 'company_gross_hourly_wage_ex', 'DOUBLE NOT NULL DEFAULT 1 AFTER `company_gross_hourly_wage`' );
    CALL us_add_to_table2('zeiten', 'company_gross_hourly_wage_id', 'INT( 11 ) NULL AFTER `company_gross_hourly_wage_ex`' );
    
    -- Add currencies support
    CALL us_add_to_table2('posteingang', 'Arbeitsbereich', 'INT( 11 ) DEFAULT NULL' );
    CALL us_add_to_table2('posteingang', 'recipient_id', 'INT(11) DEFAULT NULL AFTER `Arbeitsbereich`' );
    UPDATE posteingang SET recipient_id=ErstVon WHERE recipient_id IS NULL;

    -- Add currencies support
    CALL us_add_to_table2('postausgang', 'Arbeitsbereich', 'INT( 11 ) DEFAULT NULL' );

    -- Add currencies support
    CALL us_add_to_table2('zeiten', 'company_value1', 'DOUBLE NULL AFTER `Betrag9`' );
    CALL us_add_to_table2('zeiten', 'company_value2', 'DOUBLE NULL AFTER `company_value1`' );
    CALL us_add_to_table2('zeiten', 'company_value3', 'DOUBLE NULL AFTER `company_value2`' );
    CALL us_add_to_table2('zeiten', 'company_value4', 'DOUBLE NULL AFTER `company_value3`' );
    CALL us_add_to_table2('zeiten', 'company_value5', 'DOUBLE NULL AFTER `company_value4`' );
    CALL us_add_to_table2('zeiten', 'company_value6', 'DOUBLE NULL AFTER `company_value5`' );
    CALL us_add_to_table2('zeiten', 'company_value7', 'DOUBLE NULL AFTER `company_value6`' );
    CALL us_add_to_table2('zeiten', 'company_value8', 'DOUBLE NULL AFTER `company_value7`' );
    CALL us_add_to_table2('zeiten', 'company_value9', 'DOUBLE NULL AFTER `company_value8`' );
    CALL us_add_to_table2('zeiten', 'company_value_ex', 'DOUBLE NOT NULL DEFAULT 1 AFTER `company_value9`' );
    CALL us_add_to_table2('zeiten', 'company_value_id', 'INT( 11 ) NULL AFTER `company_value_ex`' );

    CALL us_add_to_table2('zeiten', 'company_price_per_unit', 'DOUBLE NULL AFTER `ExtSatz`' );
    CALL us_add_to_table2('zeiten', 'company_price_per_unit_ex', 'DOUBLE NOT NULL DEFAULT 1 AFTER `company_price_per_unit`' );
    CALL us_add_to_table2('zeiten', 'company_price_per_unit_id', 'INT( 11 ) NULL AFTER `company_price_per_unit_ex`' );

    CALL us_add_to_table2('zeiten', 'company_price_external_staff', 'DOUBLE NULL AFTER `PersonenSatz`' );
    CALL us_add_to_table2('zeiten', 'company_price_external_staff_ex', 'DOUBLE NOT NULL DEFAULT 1 AFTER `company_price_external_staff`' );
    CALL us_add_to_table2('zeiten', 'company_price_external_staff_id', 'INT( 11 ) NULL AFTER `company_price_external_staff_ex`' );

    CALL us_add_to_table2('zeiten', 'company_price_area', 'DOUBLE NULL AFTER `ArbSatz`' );
    CALL us_add_to_table2('zeiten', 'company_price_area_ex', 'DOUBLE NOT NULL DEFAULT 1 AFTER `company_price_area`' );
    CALL us_add_to_table2('zeiten', 'company_price_area_id', 'INT( 11 ) NULL AFTER `company_price_area_ex`' );

    CALL us_add_to_table2('zeiten', 'is_client_changed', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `Bemerkung_Intern`' );
    CALL us_add_to_table2('zeiten', 'addi_val1', 'double NOT NULL DEFAULT 0' );
    CALL us_add_to_table2('zeiten', 'addi_val2', 'double NOT NULL DEFAULT 0' );
    CALL us_add_to_table2('zeiten', 'opta_id', 'INT( 11 ) DEFAULT 0 AFTER LfdNr_PersTermin' );
    CALL us_add_to_table2('stechuhr', 'is_client_changed', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `Bereich`' );
    CALL us_add_to_table2('stechuhr', 'skill_grp_id', 'INT( 11 ) DEFAULT NULL AFTER `Bereich`' );

    CALL us_add_to_table2('externekosten', 'UstSatz', 'double DEFAULT NULL' );
    CALL us_add_to_table2('geexternekosten', 'UstSatz', 'double DEFAULT NULL' );
    
    CALL us_add_to_table2('externekosten', 'surcharge_used', 'double DEFAULT NULL' );
    CALL us_add_to_table2('geexternekosten', 'surcharge_used', 'double DEFAULT NULL' );

    CALL change_column('kopien', 'zweck', 'VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('aenderungkopien', 'zweck', 'VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('km', 'zweck', 'VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('aenderungkm', 'zweck', 'VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');

    CALL change_column('honverw', 'Bemerkung', 'VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('gehonverw', 'Bemerkung', 'VARCHAR(255) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL us_add_to_table2('honverw', 'correspondence_id', 'INT( 11 ) NULL AFTER `Arbeitsbereich`' );
    CALL us_add_to_table2('gehonverw', 'correspondence_id', 'INT( 11 ) NULL AFTER `FaelligAmGeplant`' );
    CALL us_add_to_table2('honverw', 'is_calcexp', 'BOOLEAN DEFAULT 0 AFTER `correspondence_id`');
    CALL us_add_to_table2('gehonverw', 'is_calcexp', 'BOOLEAN DEFAULT 0 AFTER `correspondence_id`');
    CALL us_add_to_table2('honverw', 'discount', 'double DEFAULT 0' );
    CALL us_add_to_table2('gehonverw', 'discount', 'double DEFAULT 0' );
    CALL us_add_to_table2('honverw', 'discount_date', 'datetime DEFAULT NULL' );
    CALL us_add_to_table2('gehonverw', 'discount_date', 'datetime DEFAULT NULL' );
    CALL us_add_to_table2('honverw', 'finally_invoiced', 'BOOLEAN DEFAULT 0 ');
    CALL us_add_to_table2('gehonverw', 'finally_invoiced', 'BOOLEAN DEFAULT 0 ');

    CALL change_column('zeiten', 'Bemerkung', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('aenderungzeiten', 'Bemerkung', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('km', 'Bemerkung', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('aenderungkm', 'Bemerkung', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('kopien', 'Bemerkung', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('aenderungkopien', 'Bemerkung', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    
    CALL us_add_to_table2('kopien', 'KopienMwSt', 'double DEFAULT NULL AFTER `AnzahlExtern`' );
    CALL us_add_to_table2('km', 'KmMwSt', 'double DEFAULT NULL' );

    CALL change_column('planverz', 'CADFile', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('planverz', 'PlanNr', 'VARCHAR(255) NULL DEFAULT NULL');
    CALL change_column('planverz', 'PLOTFile', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
    CALL change_column('planeingang', 'PlanInhalt', 'VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');

    CALL us_add_to_table2('planeingang', 'description', 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL;' );

    -- VIEWS

    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `attendance_clock`
     AS SELECT LfdNr AS id, Projekt AS project_id, Person AS staff_id, Start AS start_date, Ende AS end_date, Aktuell AS current,
     Bemerkung AS purpose, BemerkungIntern AS purpose_internal, Bereich AS area_id, is_client_changed, editiert AS changed, Importiert AS imported, skill_grp_id FROM stechuhr;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `drawings` AS
     SELECT LfdNr AS id, Projekt AS project_id, PlanNr AS plan_no, PlanInhalt AS plan_content, `Index` AS `plan_index`, Art AS `type_id`, Flaeche as area,
     Gez AS signed_by, Datum AS `date`, ErsDurch AS replaced_by, ErsFuer AS re_destinations, Erledigt AS is_done, SubPlan AS sub_plan, 
     GroesseBez AS `size_id`, Aenderung AS redline_remarks, MassStab AS `scale`, CADFile AS cad_file, PLOTFile AS plot_file, ErstAm AS created_date, 
     ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id, Bemerkung AS purpose, Ablage AS clipboard, Freigabe AS approved, 
     FreiExt AS approved_external, FreiBis AS approval_deadline, Arbeitsbereich AS work_area_id, is_autopath
     FROM planverz;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `drawings_inbox` AS
     SELECT LfdNr AS id, Projekt AS project_id, PlanNr AS plan_no, PlanInhalt AS plan_content, EingangsDatum AS `date`, `Index` AS plan_index, Von AS address_id,
     Datei AS `file`, Arbeitsbereich AS work_area_id, ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id,
     description
     FROM planeingang;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `externalexpenses` AS
     SELECT  LfdNr AS id, Projekt AS project_id, Zweck as purpose, Beschreibung AS description, Betrag AS value, surcharge_used, Empfaenger AS address_id,
     Bereich AS area_id, UserNum AS creator_id, Datum AS date, ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date,
     AendVon AS changed_by_id, Kostenstelle AS cost_centre_id, isHonEff AS is_hon_decr, isANA AS is_calc_from_expenses, verrechnetAm AS charge_date,
     verrechnet AS charged, UstSatz AS turnover_rate,
     is_deducted, deducted_day, is_ignore
     FROM externekosten;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `externalexpenses_planned` AS
     SELECT  LfdNr AS id, Projekt AS project_id, Zweck as purpose, Beschreibung AS description, Betrag AS value, surcharge_used, Empfaenger AS address_id,
     Bereich AS area_id, UserNum AS creator_id, Datum AS date, ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date,
     AendVon AS changed_by_id, Kostenstelle AS cost_centre_id, isHonEff AS is_hon_decr, isANA AS is_calc_from_expenses, verrechnetAm AS charge_date,
     verrechnet AS charged, UEbernommen AS is_transfered, UstSatz AS turnover_rate
     FROM geexternekosten;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `extras`
     AS SELECT LfdNr AS id, Projekt AS project_id, Person AS staff_id, Zweck AS purpose, Anzahl AS number, Bemerkung AS description, is_client_changed,
     addi_val1, addi_val2,
     Datum AS DATE, ExtSatz AS price_per_unit, Art AS type_id, Bereich AS area_id, KostenStelle AS cost_centre_id, Ep AS cost_per_unit,
     deducted AS is_deducted, deductedDay AS deducted_day, AnzahlExtern AS number_external, 
     ErstAm AS change_date, EingabeVon AS changed_by_id, ignor AS is_ignore,
     company_id, company_cost_per_unit, company_cost_per_unit_ex, company_cost_per_unit_id, company_price_per_unit, company_price_per_unit_ex,
     company_price_per_unit_id, KopienMwSt as vat FROM kopien;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `extras_logging`
     AS SELECT  LfdNr AS id, Projekt AS project_id, Person AS staff_id, Zweck AS purpose, Anzahl AS number, Bemerkung AS description,
     Datum AS date, Art AS type_id, Bereich AS area_id, KostenStelle AS cost_centre_id,
     Ep AS cost_per_unit, deducted AS is_deducted, deductedDay AS deducted_day, 
     EingabeVon AS changed_by_id, ignor AS is_ignore, EditArt AS log_change_mode, EditDatum AS log_change_date, GeaendertVon  AS log_changed_by_id,
     company_id, is_client_changed,
     addi_val1, addi_val2
     FROM aenderungkopien;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `fees` AS
     SELECT  LfdNr AS id, Projekt AS project_id, Bemerkung as purpose, Betrag AS value, Betrag AS value_otcurr, Empfaenger AS address_id,
     Arbeitsbereich AS area_id, User AS user_id, ReDatum AS fee_date, ReNr AS fee_no, ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date,
     AendVon AS changed_by_id, Kostenstelle AS cost_centre_id, ErfDatum AS entry_date, Erledigt AS is_done, ErlDatum AS done_date,
     Konto AS fee_account_id, UstSatz AS turnover_rate, UEberwiesen AS remited, UEberweisungsbem AS remit_purpose, vorRechnung AS predecessor_id,
     nextRechnung AS successor_id, FaelligAm AS due_date, MahnStufe AS reminder_level, letzteMahn AS last_reminder_date, isProzent AS is_percentage,
     Abzug AS deduction, correspondence_id, is_calcexp, discount, discount_date, finally_invoiced
     FROM honverw;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `fees_planned` AS
     SELECT  LfdNr AS id, Projekt AS project_id, Bemerkung as purpose, Betrag AS value, Empfaenger AS address_id,
     Arbeitsbereich AS area_id, User AS user_id, ReDatum AS fee_date, ReNr AS fee_no, ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date,
     AendVon AS changed_by_id, Kostenstelle AS cost_centre_id, ErfDatum AS entry_date, Erledigt AS is_done, ErlDatum AS done_date,
     Konto AS fee_account_id, UstSatz AS turnover_rate, UEberwiesen AS remited, UEberweisungsbem AS remit_purpose, vorRechnung AS predecessor_id,
     nextRechnung AS successor_id, FaelligAm AS due_date, MahnStufe AS reminder_level, letzteMahn AS last_reminder_date, isProzent AS is_percentage,
     Abzug AS deduction, FaelligAmGeplant AS due_date_planned, correspondence_id, is_calcexp, discount, discount_date, finally_invoiced
     FROM gehonverw;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `lut_fee_areas_offer`
     AS SELECT LfdNr AS id, AngebotLfdNr AS offer_id, HonorarLfdNr AS fee_area_id, Kosten AS costs, KostenProzent AS costs_percent,
     KostenGesamt AS costs_sum, KostenGesamtEingabe AS is_costs_sum
     FROM lut_hon_angebot;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `times`
     AS SELECT LfdNr AS id, Projekt AS project_id, Person AS staff_id, Zeit01 AS number, Faktor AS factor, ZeitVon AS time_from, ZeitBis AS time_to, Bemerkung AS purpose,
     Bemerkung_Intern AS purpose_internal, is_client_changed, 
     addi_val1, addi_val2,
     Datum AS date, ArbBereich AS area_id, KostenStelle AS cost_centre_id, deducted AS is_deducted, deductedDay AS deducted_day, ignor AS is_ignore, 
     deducted_Pers AS is_deducted_staff, deductedDay_Pers AS deducted_day_staff, ignor_Pers AS is_ignore_staff, 
     deducted_Arb AS is_deducted_woa, deductedDay_Arb AS deducted_day_woa, ignor_Arb AS is_ignore_woa, 
     StundenExtern AS number_external, StundenExternChanged AS number_external_changed, 
     ErstAm AS change_date, EingabeVon AS changed_by_id, ExtSatz AS price_per_unit, PersonenSatz AS price_external_staff, ArbSatz AS price_area,
     Bruttolohn AS gross_hourly_wage, Satz1 AS percntg1, Satz2 AS percntg2, Satz3 AS percntg3, Satz4 AS percntg4, Satz5 AS percntg5, Satz6 AS percntg6,
     Satz7 AS percntg7, Satz8 AS percntg8, Satz9 AS percntg9, Betrag1 AS value1, Betrag2 AS value2, Betrag3 AS value3, Betrag4 AS value4, Betrag5 AS value5, Betrag6 AS value6,
     Betrag7 AS value7, Betrag8 AS value8, Betrag9 AS value9, company_id, company_value1, company_value2, company_value3, company_value4, company_value5,
     company_value6, company_value7, company_value8, company_value9, company_value_ex, company_value_id,
     LfdNr_Urlaub AS vacation_id, LfdNr_PersTermin AS persapp_id,
     company_gross_hourly_wage, company_gross_hourly_wage_ex, company_gross_hourly_wage_id, company_price_per_unit, company_price_per_unit_ex, company_price_per_unit_id,
     company_price_external_staff, company_price_external_staff_ex, company_price_external_staff_id, company_price_area, company_price_area_ex, company_price_area_id, opta_id, skill_id
     FROM zeiten;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `times_logging`
     AS SELECT LfdNr AS id, Projekt AS project_id, Person AS staff_id, Zeit01 AS number, Faktor AS factor, ZeitVon AS time_from, ZeitBis AS time_to,
     Bemerkung AS purpose, Datum AS date, ArbBereich AS area_id, KostenStelle AS cost_centre_id, deducted AS is_deducted, deductedDay AS deducted_day,
     EingabeVon AS changed_by_id, ignor AS is_ignore, Bruttolohn AS gross_hourly_wage, EditArt AS log_change_mode, EditDatum AS log_change_date, GeaendertVon  AS log_changed_by_id,
     company_id, Bemerkung_Intern AS purpose_internal, is_client_changed, skill_id,
     addi_val1, addi_val2
     FROM aenderungzeiten;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `transmittal_form` AS
     SELECT LfdNr AS id, Datum AS `date`, Projekt AS project_id, Empfaenger AS address_id, Von AS staff_id, Bemerkung AS purpose,
     ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id, LSNr AS transmittal_form_name,
     DOC_Datei AS doc_file, PDF_Datei AS pdf_file, BemerkungDruck AS print_purpose, ZusatzBemerkung AS comment, Anmerkungen AS remarks,
     Arbeitsbereich AS work_area_id
     FROM planlieferschein;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `travels`
     AS SELECT LfdNr AS id, Projekt AS project_id, Person AS staff_id, Ziel AS route, Zweck AS purpose, km AS number, BueroPKW AS is_company_car, 
     kmVon AS km_from, kmBis AS km_to, ZeitVon AS time_from, ZeitBis AS time_to, Bemerkung AS description, is_client_changed,
     addi_val1, addi_val2,
     Datum AS date, Ep AS cost_per_unit, ExtSatz AS price_per_unit, Art AS type_id, Bereich AS area_id, KostenStelle AS cost_centre_id, 
     deducted AS is_deducted, deductedDay AS deducted_day, KmExtern AS number_external, 
     ErstAm AS change_date, EingabeVon AS changed_by_id, ignor AS is_ignore,
     company_id, company_cost_per_unit, company_cost_per_unit_ex, company_cost_per_unit_id, company_price_per_unit, company_price_per_unit_ex,
     company_price_per_unit_id, KmMwSt as vat FROM km;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `travels_logging`
     AS SELECT LfdNr AS id, Projekt AS project_id, Person AS staff_id, Ziel AS route, Zweck AS purpose, km AS number, BueroPKW AS is_company_car, kmVon AS km_from, kmBis AS km_to,
     ZeitVon AS time_from, ZeitBis AS time_to, Bemerkung AS description, Datum AS date, Art AS type_id, Bereich AS area_id, KostenStelle AS cost_centre_id,
     Ep AS cost_per_unit, deducted AS is_deducted, deductedDay AS deducted_day, EingabeVon AS changed_by_id, ignor AS is_ignore,
     EditArt AS log_change_mode, EditDatum AS log_change_date, GeaendertVon  AS log_changed_by_id, company_id, is_client_changed,
     addi_val1, addi_val2
     FROM aenderungkm;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `work_areas_rates`
     AS SELECT LfdNr AS id, Satz AS rate_id, AB AS area_id, NeuerSatz AS rate, LfdNr_ProjektAbrAB_Zeit AS history_id FROM projektabrab;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `work_areas_rates_history`
     AS SELECT `LfdNr` AS id, `LfdNr_Satz` AS rate_id, `von` AS date_from, `bis` AS date_to
     FROM `projektabrab_zeit`;


    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `inbox` AS
     SELECT LfdNr AS id, Projekt AS project_id, Nr AS `no`, PostInhalt AS content, EingangsDatum AS `date`, `index` AS inbox_index, Von AS address_id,
     ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id, Art AS dispatch_mode_id, Mail_File AS msg_file,
     AddedFile AS `attached_file`, archiviert AS is_archived, MailAttAnz AS mail_attachments, MailGroesse AS mail_size, MailStufe AS security, Arbeitsbereich AS area_id, recipient_id
     FROM posteingang;

    CREATE OR REPLACE
     ALGORITHM = MERGE
     VIEW `outbox` AS
     SELECT LfdNr AS id, Projekt AS project_id, Nr AS `no`, PostInhalt AS content, AusgangsDatum AS `date`, `index` AS outbox_index, An AS address_id,
     Person AS staff_id, Art AS dispatch_mode_id, ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id,
     docNummer AS doc_id, AttAnzahl AS attachments, KopieAnAnz AS copies, PO_EM_OUT_Pfad AS mail_out_file, Pfad AS `path`, archiviert AS is_archived,
     AddedFile AS `attached_file`, MailID AS mail_id, ParentID AS parent_id, MailAttAnz AS mail_attachments, MailGroesse AS mail_size, LfdNr_Rechnung AS acc_after_exp_bill_id,
     LfdNr_Lieferschein AS transmittal_form_id, MailStufe AS security, Arbeitsbereich AS area_id
     FROM postausgang;


END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- USC Create prepare work package history for projects
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------


DROP PROCEDURE IF EXISTS us_migrate_11_12_projects_history_prepare;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_projects_history_prepare()
BEGIN

    -- --------------------------- Add date from previous rows ---------------------------
    UPDATE lut_fee_areas_proj_history AS bh
        JOIN (
            SELECT bh1.valid_to_date, bh1.project_id,
            (SELECT MAX(bh2.valid_to_date) FROM lut_fee_areas_proj_history AS bh2 WHERE bh2.project_id=bh1.project_id AND bh2.valid_to_date<bh1.valid_to_date) AS valid_from_date
            FROM lut_fee_areas_proj_history AS bh1
        ) bhsub
        ON bhsub.project_id=bh.project_id AND bhsub.valid_to_date=bh.valid_to_date
    SET bh.migrate_valid_from_date = bhsub.valid_from_date;


    UPDATE budget_plan_history AS bh
        JOIN (
            SELECT bh1.valid_to_date, bh1.project_id,
            (SELECT MAX(bh2.valid_to_date) FROM budget_plan_history AS bh2 WHERE bh2.project_id=bh1.project_id AND bh2.valid_to_date<bh1.valid_to_date) AS valid_from_date
            FROM budget_plan_history AS bh1
        ) bhsub
        ON bhsub.project_id=bh.project_id AND bhsub.valid_to_date=bh.valid_to_date
    SET bh.migrate_valid_from_date = bhsub.valid_from_date;


    UPDATE completion_level_history AS bh
        JOIN (
            SELECT bh1.valid_to_date, bh1.project_id,
            (SELECT MAX(bh2.valid_to_date) FROM completion_level_history AS bh2 WHERE bh2.project_id=bh1.project_id AND bh2.valid_to_date<bh1.valid_to_date) AS valid_from_date
            FROM completion_level_history AS bh1
        ) bhsub
        ON bhsub.project_id=bh.project_id AND bhsub.valid_to_date=bh.valid_to_date
    SET bh.migrate_valid_from_date = bhsub.valid_from_date;

    -- --------------------------- Add max date to NULL rows ---------------------------
    UPDATE lut_fee_areas_proj_history AS bh
        JOIN (
            SELECT bh1.project_id,
            (SELECT MAX(bh2.valid_to_date) FROM lut_fee_areas_proj_history AS bh2 WHERE bh2.project_id=bh1.project_id) AS valid_from_date
            FROM lut_fee_areas_proj_history AS bh1
        ) bhsub
        ON bhsub.project_id=bh.project_id
    SET bh.migrate_valid_from_date = bhsub.valid_from_date
    WHERE bh.valid_to_date IS NULL;


    UPDATE budget_plan_history AS bh
        JOIN (
            SELECT bh1.project_id,
            (SELECT MAX(bh2.valid_to_date) FROM budget_plan_history AS bh2 WHERE bh2.project_id=bh1.project_id) AS valid_from_date
            FROM budget_plan_history AS bh1
        ) bhsub
        ON bhsub.project_id=bh.project_id
    SET bh.migrate_valid_from_date = bhsub.valid_from_date
    WHERE bh.valid_to_date IS NULL;


    UPDATE completion_level_history AS bh
        JOIN (
            SELECT bh1.project_id,
            (SELECT MAX(bh2.valid_to_date) FROM completion_level_history AS bh2 WHERE bh2.project_id=bh1.project_id) AS valid_from_date
            FROM completion_level_history AS bh1
        ) bhsub
        ON bhsub.project_id=bh.project_id 
    SET bh.migrate_valid_from_date = bhsub.valid_from_date
    WHERE bh.valid_to_date IS NULL;

    -- --------------------------- Add date NULL values, for start entries ---------------------------
    UPDATE lut_fee_areas_proj_history SET migrate_valid_from_date = '1901-01-01 00:00:00' WHERE migrate_valid_from_date IS NULL;
    UPDATE budget_plan_history SET migrate_valid_from_date = '1901-01-01 00:00:00' WHERE migrate_valid_from_date IS NULL;
    UPDATE completion_level_history SET migrate_valid_from_date = '1901-01-01 00:00:00' WHERE migrate_valid_from_date IS NULL;

END $$
DELIMITER ;
-- EOC
-- USC Erstelle X2 Tabellen.

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Work package presets
CREATE TABLE IF NOT EXISTS `work_package_presets` (
    `id` BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL,
    `tag_id` INT(11) NULL,
    `lft` BIGINT NULL DEFAULT NULL, 
    `rgt` BIGINT NULL DEFAULT NULL, 
    `tree_id` INT(11) NULL DEFAULT NULL, 
    `is_purpose_required` BOOLEAN, 
    `is_fee_area` BOOLEAN,
    `is_special` INT(11) NULL DEFAULT 0,
    `is_special_phase` INT(11) NULL DEFAULT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('work_package_presets', 'threshold', 'DOUBLE DEFAULT 0' );
-- EOC

-- Work package - work_package lang-table
CREATE TABLE IF NOT EXISTS `langs_work_packages_presets` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`work_package_preset_id` INT(11) DEFAULT NULL,
`name` LONGTEXT NULL DEFAULT NULL ,
`language` VARCHAR( 100 ) NULL DEFAULT NULL 
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_work_packages_presets', 'langs_work_packages_presets_unique_1', 'UNIQUE', '`work_package_preset_id` , `language`' );
-- EOC

CALL change_column('work_package_presets', 'lft', 'BIGINT(20) NULL DEFAULT NULL');
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Work package

CREATE TABLE IF NOT EXISTS `work_packages` (
    `id` BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `name` VARCHAR( 203 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL,
    `tag_id` INT(11) NULL,
    `lft` BIGINT NULL DEFAULT NULL, 
    `rgt` BIGINT NULL DEFAULT NULL, 
    `project_id` INT(11) NULL DEFAULT NULL, 
    `is_purpose_required` BOOLEAN, 
    `is_fee_area` BOOLEAN,
    `is_special` INT(11) NULL DEFAULT 0,
    `is_special_phase` INT(11) NULL DEFAULT NULL,
    `completion_level` DOUBLE NULL,   
    `budget_proportion` DOUBLE NULL,
    `budget_planned_costs` DOUBLE NULL,
    `budget_planned_hours` DOUBLE NULL,
    `budget_start` DATETIME NULL,
    `budget_end` DATETIME NULL,
    `budget_threshold` DOUBLE DEFAULT 0,
    `fee_costs` DOUBLE NULL,   
    `fee_hours` DOUBLE NULL,   
    `fee_proportion` DOUBLE NULL,   
    `status` INT(11) NULL,
    `wpp_tree_id` INT(11) NULL COMMENT 'Work package preset tree id',
    `old_fee_area_id` INT(11) NULL,
    `old_work_area_id` INT(11) NULL,
    `category_rate` DOUBLE DEFAULT 0,
    INDEX `wp_lft_index` (`project_id`, `lft`),
    INDEX `wp_rgt_index` (`project_id`, `rgt`)
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Work package - work_package lang-table
CREATE TABLE IF NOT EXISTS `langs_work_packages_main` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`work_package_id` INT(11) DEFAULT NULL,
`name` LONGTEXT NULL DEFAULT NULL ,
`language` VARCHAR( 100 ) NULL DEFAULT NULL 
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_work_packages_main', 'langs_work_packages_main_unique_1', 'UNIQUE', '`work_package_id` , `language`' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Work package history

CREATE TABLE IF NOT EXISTS `work_packages_history` (
    `id` BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `project_id` INT(11) NULL DEFAULT NULL, 
    `completion_level` DOUBLE NULL,   
    `budget_proportion` DOUBLE NULL,
    `budget_planned_costs` DOUBLE NULL,
    `budget_planned_hours` DOUBLE NULL,
    `budget_threshold` DOUBLE DEFAULT 0,
    `fee_costs` DOUBLE NULL,   
    `fee_hours` DOUBLE NULL,   
    `fee_proportion` DOUBLE NULL,   
    `wpp_tree_id` INT(11) NULL,
    `date` DATETIME NULL ,
    `wp_id` BIGINT(20),
    INDEX `wp_id_index` ( `wp_id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Work package and skill group look up table
CREATE TABLE IF NOT EXISTS `lut_skill_wp` (
    `id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    `wp_id` INT(11) NULL,
    `skill_grp_id` INT(11) NULL,
    `proportion` DOUBLE NULL,
    `start` DATETIME NULL,
    `end` DATETIME NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Work package and staff look up table
CREATE TABLE IF NOT EXISTS `lut_staff_wp` (
    `id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    `rp_id` INT(11) NULL,
    `staff_id` INT(11) NULL,
    `proportion` DOUBLE NULL,
    `description` LONGTEXT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- work packages tags
CREATE TABLE IF NOT EXISTS `work_packages_tags` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `name` varchar(100) DEFAULT NULL,
    `wp_name` varchar(100) DEFAULT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- work packages tags lang-table
CREATE TABLE IF NOT EXISTS `langs_work_packages_tags` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `work_packages_tags_id` int(11) NOT NULL,
    `name` varchar(100) NOT NULL,
    `wp_name` varchar(100) NOT NULL,
    `language` varchar(50) NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_work_packages_tags', 'langs_work_packages_tags_unique_1', 'UNIQUE', '`work_packages_tags_id` , `language`' );
-- EOC

CREATE TABLE IF NOT EXISTS `work_packages_history_descriptions` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `description` varchar(255) DEFAULT NULL,
    `archive_date`  DATETIME NULL,
    `project_id` INT(11) NULL DEFAULT NULL, 
    `type` varchar(100) DEFAULT NULL,
    `staff_id` INT(11) NULL DEFAULT NULL, 
    `insert_date`  DATETIME NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Work packages
CALL change_column('work_packages', 'status', 'DOUBLE NULL DEFAULT 20');
-- EOC
CALL us_add_to_table2('work_packages_history', 'wp_id', 'BIGINT(20)' );
-- EOC
CALL us_add_to_table2('work_packages', 'is_special_phase', 'INT(11) NULL DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('work_packages', 'cost_centre_id', 'INT(11) NULL DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('work_packages', 'is_extern', 'TINYINT DEFAULT 0' );
-- EOC
CALL us_add_to_table2('work_packages', 'loc_estimation_date', 'DATETIME NULL' );
-- EOC
CALL us_add_to_table2('work_packages', 'min_start', 'DATETIME NULL' );
-- EOC
CALL us_add_to_table2('work_packages', 'max_end', 'DATETIME NULL' );
-- EOC
CALL us_add_to_table2('work_packages', 'category_rate', 'DOUBLE DEFAULT 0' );
-- EOC
CALL us_add_to_table2('work_packages_history', 'loc_estimation_date', 'DATETIME NULL' );
-- EOC
CALL us_add_to_table2('work_package_presets', 'is_special_phase', 'INT(11) NULL DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('work_package_presets', 'tag_id', 'INT(11) NULL' );
-- EOC
CALL us_add_to_table2('work_package_presets', 'is_extern', 'TINYINT DEFAULT 0' );
-- EOC
CALL us_add_to_table2('work_package_presets', 'old_fee_area_id', 'INT(11) NULL' );
-- EOC
CALL us_add_to_table2('work_package_presets', 'old_work_area_id', 'INT(11) NULL' );
-- EOC
CALL us_add_to_table2('work_package_presets', 'cost_centre_id', 'INT(11) NULL DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('work_packages', 'tag_id', 'INT(11) NULL' );
-- EOC


CALL us_add_constraint_to_table('work_package_presets', 'wpp_lft_unique', 'UNIQUE', '`tree_id` , `lft`' );
-- EOC
CALL us_add_constraint_to_table('work_package_presets', 'wpp_rgt_unique', 'UNIQUE', '`tree_id` , `rgt`' );
-- EOC
CALL us_add_constraint_to_table('work_packages_history', 'wp_history_unique', 'UNIQUE', '`project_id` , `wp_id`, `date`' );
-- EOC
CALL us_add_index_to_table('work_packages_history', 'wp_id_index', '`wp_id`' );
-- EOC


CREATE TABLE IF NOT EXISTS `langs_fee_areas` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	`fee_area_id` int(11) NOT NULL ,
	`name` VARCHAR( 100 ) NOT NULL ,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_fee_areas', 'langs_fee_areas_unique_1', 'UNIQUE', '`fee_area_id` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_work_areas` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
	`work_area_id` INT( 11 ) NOT NULL ,
	`name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_work_areas', 'langs_work_areas_unique_1', 'UNIQUE', '`work_area_id` , `language`' );
-- EOC

-- Add lut fee table
CREATE TABLE IF NOT EXISTS `lut_fee_wp` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `planned_fee_id` INT(11) NULL DEFAULT NULL, 
    `fee_id` INT(11) NULL DEFAULT NULL, 
    `project_id` INT(11) NULL DEFAULT NULL, 
    `wp_id` INT(11) NULL DEFAULT NULL, 
    `value`  DOUBLE NULL,
    `value_pre_tax`  DOUBLE NULL,
    `diff_pre_tax`  DOUBLE NULL,
    `done_pre_tax`  DOUBLE NULL,
    `value_otcurr`  DOUBLE NULL,
    `value_pre_tax_otcurr`  DOUBLE NULL,
    `done_pre_tax_otcurr`  DOUBLE NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Staff - cost_centre lut-table
CREATE TABLE IF NOT EXISTS `lut_staff_cost_centre` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `staff_id` INT(11) DEFAULT NULL,
    `cost_centre_id` INT(11) DEFAULT NULL,
    `from_date` DATETIME NULL,
    `to_date` DATETIME NULL,
    `description` LONGTEXT NULL DEFAULT NULL, 
    `unique_add_id` VARCHAR(140) NULL DEFAULT NULL 
    ) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Staff - gross_hourly_wage lut-table
CREATE TABLE IF NOT EXISTS `lut_staff_gross_hourly_wage` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `staff_id` INT(11) DEFAULT NULL,
    `gross_hourly_wage` double DEFAULT NULL,
    `from_date` DATETIME NULL,
    `to_date` DATETIME NULL,
    `description` LONGTEXT NULL DEFAULT NULL, 
    `unique_add_id` VARCHAR(140) NULL DEFAULT NULL 
    ) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- USC X2 Update...
SET @debug = 0;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Avoid the usage of old up0....sql scripts from pre X.2 (Just in case, someone thinks this is a good idea)
-- Otherwise all views -> tables would be deleted
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_add_to_table;
-- EOC

DROP TABLE IF EXISTS `us_add_to_table`;
-- EOC

CREATE TABLE `us_add_to_table`(
    do_not_use_x1_installer VARCHAR(255),
    PRIMARY KEY (`do_not_use_x1_installer`)
) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

INSERT INTO us_add_to_table VALUES('Versuchen Sie auf keinen Fall ein Downgrade von X.2 auf X.1 duchzuführen! Do not try to downgrade X.2 to X.1!');
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Cleanup, without any structure changes
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS us_migrate_11_12_cleanup;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_cleanup()
BEGIN

    UPDATE work_packages SET fee_costs=0 WHERE fee_costs IS NULL;
    UPDATE work_packages SET fee_hours=0 WHERE fee_hours IS NULL;

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate fee area tags to work package tags
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS us_migrate_fee_area_tags;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_fee_area_tags()
us_migrate_fee_area_tagslbl:BEGIN

    -- Prevent execution, if work_packages_tags has ids>10000000
    IF ( (SELECT COUNT(*) FROM work_packages_tags WHERE id>10000000)>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_fee_area_tags';
        LEAVE us_migrate_fee_area_tagslbl;
    END IF;

    -- Insert work_packages_tags for fee areas, which are not equal to work areas
    INSERT INTO work_packages_tags(id, name, wp_name) 
        SELECT 
            hb.LfdNr+10000000, 
            IF((SELECT count(*) FROM work_packages_tags wpt WHERE us_delete_spaces(TRIM(wpt.name))=us_delete_spaces(TRIM(hb.HonorarName)))>0, CONCAT('HB ',hb.HonorarName), hb.HonorarName), 
            IF((SELECT count(*) FROM work_packages_tags wpt WHERE us_delete_spaces(TRIM(wpt.name))=us_delete_spaces(TRIM(hb.HonorarName)))>0, CONCAT('HB ',hb.HonorarName), hb.HonorarName)
        FROM honorarbereiche hb 
            WHERE 
                (
                    SELECT count(*) FROM arbeitsbereiche AS ab 
                    WHERE
                        ab.HonBer=hb.LfdNr AND us_delete_spaces(TRIM(ab.ArbName))<>us_delete_spaces(TRIM(hb.HonorarName))
                        AND
                        (SELECT count(*) FROM work_packages_tags WHERE id = hb.LfdNr+10000000)=0
                ) > 0;

    -- Update work package with tag from above
    UPDATE work_packages wp 
    SET 
        tag_id = (
            SELECT 
                id
            FROM
                work_packages_tags wpt
            WHERE
                wp.old_fee_area_id = wpt.id - 10000000
        )
    WHERE
        tag_id IS NULL AND is_fee_area = 1 AND lft > 1;

    -- Update work package presets with tag from above
    UPDATE work_package_presets wpp
    SET 
        tag_id = (
            SELECT 
                id
            FROM
                work_packages_tags wpt
            WHERE
                wpp.old_fee_area_id = wpt.id - 10000000
        )
    WHERE
        tag_id IS NULL AND is_fee_area = 1 AND lft > 1;

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Update work packages status
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS update_wp_status;
-- EOC
DELIMITER $$
CREATE PROCEDURE update_wp_status()
update_wp_statuslbl:BEGIN

    DECLARE project_ids INTEGER;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE wpids CURSOR FOR SELECT project_id FROM work_packages WHERE lft=1 AND (status<>50 OR status IS NULL);
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    OPEN wpids;
    cursor_loop: LOOP -- -------------------------------------
        FETCH wpids INTO project_ids;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
            /*Do something with the row fetched*/
            INSERT INTO `01migrate_11_12` SET elem='project_id_status', info=project_ids;

            UPDATE work_packages AS wp1
                SET status=(
                    SELECT wp2.status FROM (
                        SELECT status, project_id, lft, rgt
                        FROM work_packages
                        WHERE project_id = project_ids
                    ) wp2
                    WHERE wp2.project_id = wp1.project_id AND wp2.lft < wp1.lft AND wp2.rgt > wp1.rgt
                    ORDER BY wp2.rgt - wp1.rgt ASC
                    LIMIT 1)
                WHERE wp1.project_id = project_ids AND wp1.rgt - wp1.lft = 1;

    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  wpids;

    -- For migrations where the status is acitve at every work package
    CALL repair_wp_status();
END $$
DELIMITER ;
-- EOC

DROP PROCEDURE IF EXISTS repair_wp_status;
-- EOC
DELIMITER $$
CREATE PROCEDURE repair_wp_status()
repair_wp_statuslbl:BEGIN
    UPDATE work_packages wp 
        SET status = IFNULL(
            (SELECT 
                MAX(HonStatus) -- avoid "subquery returns more than one row": get the max status if "clean lookup table" was not used before the update (multiple entries possible)
            FROM
                lut_hon_proj lhp
            WHERE
                lhp.History_LfdNr = (SELECT 
                    lhph.LfdNr
                FROM
                    lut_hon_proj_history lhph
                WHERE
                    lhph.Proj_LfdNr = wp.project_id
                        AND lhph.GueltigBis IS NULL
                GROUP BY lhph.LfdNr)
                    AND wp.old_fee_area_id = lhp.Hon_LfdNr),
            status)
        WHERE wp.old_fee_area_id IS NOT NULL;
END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Update work packages dates
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS update_wp_add_one_day;
-- EOC
DELIMITER $$
CREATE PROCEDURE update_wp_add_one_day()
BEGIN

    DECLARE project_ids INTEGER;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE project_vals CURSOR FOR SELECT id FROM projects;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    OPEN project_vals;
    cursor_loop: LOOP -- -------------------------------------
        FETCH project_vals INTO project_ids;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
            /*Do something with the row fetched*/
            INSERT INTO `01migrate_11_12` SET elem='update_wp_add_one_day', info=project_ids;
            CALL update_add_one_day_projects_history_dates(project_ids);
            
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  project_vals;

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Update work packages cost centres for 12.6
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS update_cost_centres;
-- EOC
DELIMITER $$
CREATE PROCEDURE update_cost_centres()
update_update_cost_centreslbl:BEGIN

    DECLARE project_ids INTEGER;
    DECLARE cost_centre_ids INTEGER;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE project_vals CURSOR FOR SELECT Kostenstelle, LfdNr FROM projekte WHERE Kostenstelle > 0;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    OPEN project_vals;
    cursor_loop: LOOP -- -------------------------------------
        FETCH project_vals INTO cost_centre_ids, project_ids;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;            
             /*Do something with the row fetched*/
            INSERT INTO `01migrate_11_12` SET elem='project_cost_centres', info=project_ids;
            UPDATE work_packages set cost_centre_id = cost_centre_ids WHERE project_id = project_ids AND cost_centre_id IS NULL;
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  project_vals;

END $$
DELIMITER ;
-- EOC

-- Remove old Kostenstelle field
DROP PROCEDURE IF EXISTS update_cost_centres_cleanup;
-- EOC
DELIMITER $$
CREATE PROCEDURE update_cost_centres_cleanup()
BEGIN

    -- RENAME staff_id to password_salt
    CALL check_rename_column_exists('projekte', 'Kostenstelle', 'Kostenstelle__old', 'INT(11) NULL DEFAULT NULL ');

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Move fee work package for 12.6 to the new look up table if user has used the work package dropdown field in the fee module
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS move_fee_work_package;
-- EOC
DELIMITER $$
CREATE PROCEDURE move_fee_work_package()
update_move_fee_work_packagelbl:BEGIN

     -- Insert the work package structure for the existing fee entries
    INSERT INTO lut_fee_wp (fee_id, project_id, wp_id, value, done_pre_tax, value_pre_tax) SELECT 
        fees.id AS fee_id,
        wp2.project_id,
        wp2.id AS work_package_id,
        CASE WHEN fees.work_package_id > 0 AND (fees.work_package_id = wp2.id OR wp2.id IN (SELECT id FROM work_packages WHERE lft < wp.lft AND rgt > wp.rgt AND project_id = wp.project_id)) THEN fees.value ELSE 0 END,
        CASE WHEN fees.work_package_id > 0 AND (fees.work_package_id = wp2.id OR wp2.id IN (SELECT id FROM work_packages WHERE lft < wp.lft AND rgt > wp.rgt AND project_id = wp.project_id)) THEN fees.remited ELSE 0 END,
	CASE WHEN fees.work_package_id > 0 AND (fees.work_package_id = wp2.id OR wp2.id IN (SELECT id FROM work_packages WHERE lft < wp.lft AND rgt > wp.rgt AND project_id = wp.project_id)) THEN (fees.value * fees.turnover_rate) + fees.value ELSE 0 END
    FROM
        work_packages wp
            INNER JOIN
        work_packages wp2 ON wp.lft >= wp2.lft AND wp.rgt <= wp2.rgt AND wp.project_id = wp2.project_id
            INNER JOIN
        fees on fees.project_id = wp.project_id
    WHERE
        wp.is_fee_area = 1 AND wp2.lft > 1 GROUP BY wp2.id, fee_id ORDER BY fee_id, wp.id;
        
    -- Insert the work package structure for the existing planned fee entries
    INSERT INTO lut_fee_wp (planned_fee_id, project_id, wp_id, value, done_pre_tax, value_pre_tax) SELECT 
        fees_planned.id AS planned_fee_id,
        wp2.project_id,
        wp2.id AS work_package_id,
        CASE WHEN fees_planned.work_package_id > 0 AND (fees_planned.work_package_id = wp2.id OR wp2.id IN (SELECT id FROM work_packages WHERE lft < wp.lft AND rgt > wp.rgt AND project_id = wp.project_id)) THEN fees_planned.value ELSE 0 END,
        CASE WHEN fees_planned.work_package_id > 0 AND (fees_planned.work_package_id = wp2.id OR wp2.id IN (SELECT id FROM work_packages WHERE lft < wp.lft AND rgt > wp.rgt AND project_id = wp.project_id)) THEN fees_planned.remited ELSE 0 END,
	CASE WHEN fees_planned.work_package_id > 0 AND (fees_planned.work_package_id = wp2.id OR wp2.id IN (SELECT id FROM work_packages WHERE lft < wp.lft AND rgt > wp.rgt AND project_id = wp.project_id)) THEN (fees_planned.value * fees_planned.turnover_rate) + fees_planned.value ELSE 0 END
    FROM
        work_packages wp
            INNER JOIN
        work_packages wp2 ON wp.lft >= wp2.lft AND wp.rgt <= wp2.rgt AND wp.project_id = wp2.project_id
            INNER JOIN
        fees_planned on fees_planned.project_id = wp.project_id
    WHERE
        wp.is_fee_area = 1 AND wp2.lft > 1 GROUP BY wp2.id, planned_fee_id ORDER BY planned_fee_id, wp.id;
    

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Move fee values from the payment entries to the new fee wp allocation table
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS update_alloc_with_payments_fees;
-- EOC
DELIMITER $$
CREATE PROCEDURE update_alloc_with_payments_fees()
update_alloc_with_payments_feeslbl:BEGIN

    DECLARE fee_ids INTEGER;
    DECLARE correspondence_ids INTEGER;
    DECLARE turnover_rates DOUBLE;
    DECLARE remited_perc DOUBLE;
    DECLARE fee_values DOUBLE;
    DECLARE fee_otcurr_values DOUBLE;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE fee_vals CURSOR FOR 
        SELECT id, correspondence_id, turnover_rate, remited*100/((value*(turnover_rate*100+100))/100), value, value_otcurr
        FROM fees 
        WHERE correspondence_id IN (select LfdNr from formular where related_id IS NOT NULL AND related_type = 'payment');
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    OPEN fee_vals;
    cursor_loop: LOOP -- -------------------------------------
        FETCH fee_vals INTO fee_ids, correspondence_ids, turnover_rates, remited_perc, fee_values, fee_otcurr_values;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;    
        
        /*Do something with the row fetched*/
        INSERT INTO `01migrate_11_12` SET elem='fee_work_package', info=fee_ids;

        /* SET by SELECT */
        (SELECT related_id 
            INTO @paymentsID
            FROM formular WHERE related_type = 'payment' AND LfdNr = correspondence_ids);

        (SELECT SUM(value), SUM(value_otcurr) 
            INTO @sumVal, @sumValOtcurr
            FROM payments_fees WHERE payments_id = @paymentsID);

        -- get the fee values from the payments table
        UPDATE lut_fee_wp lfw
                INNER JOIN 
        (SELECT fee_values * (pf.value/@sumVal) AS value, pf.value/@sumVal AS factor, IFNULL(fee_otcurr_values, fee_values) * (pf.value_otcurr/@sumValOtcurr) AS value_otcurr, lfao.work_package_id AS wp_id FROM payments_fees pf 
                INNER JOIN 
        lut_fee_areas_offer lfao ON pf.fee_areas_offer_id = lfao.id WHERE payments_id = @paymentsID) AS myTable
                ON myTable.wp_id = lfw.wp_id AND lfw.fee_id = fee_ids
        SET lfw.value = myTable.value, lfw.done_pre_tax = (((myTable.value*turnover_rates)+myTable.value)*remited_perc)/100, lfw.value_otcurr = myTable.value_otcurr, lfw.value_pre_tax=(myTable.value*turnover_rates)+myTable.value;
        
        -- UPDATE the parent values
        UPDATE lut_fee_wp
            INNER JOIN
        (SELECT 
            lfw.wp_id,
            SUM(lfw2.value) AS sum_val,
            SUM(lfw2.done_pre_tax) AS sum_done,
            SUM(lfw2.value_otcurr) AS sum_val_otcurr
        FROM
            lut_fee_wp lfw
                INNER JOIN
            lut_fee_wp lfw2 ON lfw2.wp_id IN (SELECT id FROM work_packages 
                WHERE
            lft > (SELECT lft FROM work_packages WHERE project_id = lfw.project_id AND id = lfw.wp_id)
                AND 
            rgt < (SELECT rgt FROM work_packages WHERE project_id = lfw.project_id AND id = lfw.wp_id) 
                AND 
            project_id = lfw.project_id AND lft > 1 AND is_fee_area = 1)
                AND
            lfw2.fee_id = fee_ids AND lfw.fee_id = fee_ids
                GROUP BY lfw.wp_id, lfw.fee_id) AS myTable
            SET value = myTable.sum_val, done_pre_tax=myTable.sum_done, value_otcurr = myTable.sum_val_otcurr, value_pre_tax=(myTable.sum_val*turnover_rates)+myTable.sum_val
                WHERE 
            lut_fee_wp.fee_id = fee_ids AND lut_fee_wp.wp_id = myTable.wp_id;
        
        END LOOP cursor_loop; -- -------------------------------------
    CLOSE  fee_vals;
END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate work package translation
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS us_migrate_translation;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_translation()
us_migrate_translationlbl:BEGIN

    -- Prevent execution, if times table has a work_package_id
    IF ( (SELECT COUNT(*) FROM langs_work_packages_main)>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_translation';
        LEAVE us_migrate_translationlbl;
    END IF;

    -- Langs for main table
    INSERT INTO langs_work_packages_main
        (work_package_id, `name`, `language`)
        SELECT wp.id, la.name, la.language FROM langs_work_areas AS la INNER JOIN work_packages AS wp on la.work_area_id=wp.old_work_area_id;

    INSERT INTO langs_work_packages_main
        (work_package_id, `name`, `language`)
        SELECT wp.id, la.name, la.language FROM langs_fee_areas AS la INNER JOIN work_packages AS wp on la.fee_area_id=wp.old_fee_area_id;

    -- Langs for tags table
    INSERT INTO langs_work_packages_tags
        (work_packages_tags_id, `name`, `wp_name`, `language`)
        SELECT wp.id, la.name, la.name, la.language FROM langs_work_areas AS la INNER JOIN work_packages_tags AS wp on la.work_area_id=wp.id;

    -- Langs for presets table
    INSERT INTO langs_work_packages_presets
        (work_package_preset_id, `name`, `language`)
        SELECT wp.id, la.name, la.language FROM langs_work_areas AS la INNER JOIN work_package_presets AS wp on la.work_area_id=wp.old_work_area_id;

    INSERT INTO langs_work_packages_presets
        (work_package_preset_id, `name`, `language`)
        SELECT wp.id, la.name, la.language FROM langs_fee_areas AS la INNER JOIN work_package_presets AS wp on la.fee_area_id=wp.old_fee_area_id;

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate work area values to work packages of the data tables
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS us_migrate_11_12_areas_to_packages;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_areas_to_packages()
us_migrate_11_12_areas_to_packageslbl:BEGIN

    -- Prevent execution, if times table has a work_package_id
    IF ( (SELECT COUNT(*) FROM information_schema.COLUMNS WHERE table_schema = DATABASE() AND table_name='times' AND column_name='work_package_id')>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_11_12_areas_to_packages';
        LEAVE us_migrate_11_12_areas_to_packageslbl;
    END IF;

    CALL us_migrate_11_12_migrate_wa2wp('attendance_clock', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('drawings', 'work_area_id');
    CALL us_migrate_11_12_migrate_wa2wp('drawings_inbox', 'work_area_id');
    CALL us_migrate_11_12_migrate_wa2wp('externalexpenses', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('externalexpenses_planned', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('extras', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('extras_logging', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('fees', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('fees_planned', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('inbox', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('outbox', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('times', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('times_logging', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('transmittal_form', 'work_area_id');
    CALL us_migrate_11_12_migrate_wa2wp('travels', 'area_id');
    CALL us_migrate_11_12_migrate_wa2wp('travels_logging', 'area_id');

    CALL us_add_to_table2('formular', 'work_package_id', 'INT (11) DEFAULT NULL' );
    UPDATE `formular` AS main SET work_package_id=(SELECT id FROM work_packages AS wp WHERE main.Projekt=wp.project_id AND main.Arbeitsbereich=wp.old_work_area_id);
    ALTER TABLE `formular` CHANGE COLUMN `Arbeitsbereich` `Arbeitsbereich__old` INT(11) NULL DEFAULT NULL ;

    UPDATE lut_fee_areas_offer AS main SET  work_package_id = (
        SELECT wp.id
        FROM project_offers AS po INNER JOIN work_packages AS wp ON po.project_id = wp.project_id
        WHERE main.offer_id = po.id AND main.fee_area_id = wp.old_fee_area_id
    );

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate views to tables
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS us_migrate_11_12_migrate_views_to_tables;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_migrate_views_to_tables()
us_migrate_11_12_migrate_views_to_tableslbl:BEGIN

    -- Prevent execution, if attendance_clock is not a VIEW
    IF ( (SELECT COUNT(*) FROM information_schema.TABLES WHERE table_schema = DATABASE() AND table_name='attendance_clock' AND table_type='VIEW')<=0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_11_12_migrate_views_to_tables';
        LEAVE us_migrate_11_12_migrate_views_to_tableslbl;
    END IF;

    CALL us_migrate_11_12_migrate_view('attendance_clock');
        ALTER TABLE `attendance_clock` ADD INDEX(`project_id`); 
        ALTER TABLE `attendance_clock` ADD INDEX(`staff_id`);
    CALL us_migrate_11_12_migrate_view('drawings');
        ALTER TABLE `drawings` ADD INDEX(`project_id`); 
        ALTER TABLE `drawings` ADD INDEX(`type_id`); 
        ALTER TABLE `drawings` ADD INDEX(`signed_by`);
        ALTER TABLE `drawings` ADD INDEX(`replaced_by`);
        ALTER TABLE `drawings` ADD INDEX(`re_destinations`);
        ALTER TABLE `drawings` ADD INDEX(`size_id`); 
        ALTER TABLE `drawings` ADD INDEX(`created_by_id`);
        ALTER TABLE `drawings` ADD INDEX(`changed_by_id`);
    CALL us_migrate_11_12_migrate_view('drawings_inbox');
        ALTER TABLE `drawings_inbox` ADD INDEX(`project_id`);
        ALTER TABLE `drawings_inbox` ADD INDEX(`address_id`);
        ALTER TABLE `drawings_inbox` ADD INDEX(`date`);
        ALTER TABLE `drawings_inbox` ADD INDEX(`created_by_id`);
        ALTER TABLE `drawings_inbox` ADD INDEX(`changed_by_id`);
    CALL us_migrate_11_12_migrate_view('externalexpenses');
        ALTER TABLE `externalexpenses` ADD INDEX(`is_hon_decr`);
        ALTER TABLE `externalexpenses` ADD INDEX(`is_calc_from_expenses`);
        ALTER TABLE `externalexpenses` ADD INDEX(`date`);
        ALTER TABLE `externalexpenses` ADD INDEX(`project_id`);
        ALTER TABLE `externalexpenses` ADD INDEX(`address_id`);
        ALTER TABLE `externalexpenses` ADD INDEX(`creator_id`);
        ALTER TABLE `externalexpenses` ADD INDEX(`cost_centre_id`);
        ALTER TABLE `externalexpenses` ADD INDEX(`created_by_id`);
        ALTER TABLE `externalexpenses` ADD INDEX(`changed_by_id`);
    CALL us_migrate_11_12_migrate_view('externalexpenses_planned');
        ALTER TABLE `externalexpenses_planned` ADD INDEX(`is_hon_decr`);
        ALTER TABLE `externalexpenses_planned` ADD INDEX(`is_calc_from_expenses`);
        ALTER TABLE `externalexpenses_planned` ADD INDEX(`date`);
        ALTER TABLE `externalexpenses_planned` ADD INDEX(`project_id`);
        ALTER TABLE `externalexpenses_planned` ADD INDEX(`address_id`);
        ALTER TABLE `externalexpenses_planned` ADD INDEX(`creator_id`);
        ALTER TABLE `externalexpenses_planned` ADD INDEX(`cost_centre_id`);
        ALTER TABLE `externalexpenses_planned` ADD INDEX(`created_by_id`);
        ALTER TABLE `externalexpenses_planned` ADD INDEX(`changed_by_id`);
    CALL us_migrate_11_12_migrate_view('extras');
        ALTER TABLE `extras` ADD INDEX(`project_id`);
        ALTER TABLE `extras` ADD INDEX(`date`);
        ALTER TABLE `extras` ADD INDEX(`staff_id`);
        ALTER TABLE `extras` ADD INDEX(`type_id`);
        ALTER TABLE `extras` ADD INDEX(`cost_centre_id`);
        ALTER TABLE `extras` ADD INDEX(`changed_by_id`);
        ALTER TABLE `extras` ADD INDEX(`company_id`);
        ALTER TABLE `extras` ADD INDEX(`is_deducted`);
    CALL us_migrate_11_12_migrate_view('extras_logging');
        ALTER TABLE `extras_logging` ADD INDEX(`project_id`);
        ALTER TABLE `extras_logging` ADD INDEX(`date`);
        ALTER TABLE `extras_logging` ADD INDEX(`staff_id`);
        ALTER TABLE `extras_logging` ADD INDEX(`type_id`);
        ALTER TABLE `extras_logging` ADD INDEX(`cost_centre_id`);
        ALTER TABLE `extras_logging` ADD INDEX(`changed_by_id`);
        ALTER TABLE `extras_logging` ADD INDEX(`log_change_date`);
        ALTER TABLE `extras_logging` ADD INDEX(`log_changed_by_id`);
        ALTER TABLE `extras_logging` ADD INDEX(`company_id`);
        ALTER TABLE `extras_logging` ADD INDEX(`is_deducted`);
    CALL us_migrate_11_12_migrate_view('fees');
        ALTER TABLE `fees` ADD INDEX(`fee_date`);
        ALTER TABLE `fees` ADD INDEX(`is_done`);
        ALTER TABLE `fees` ADD INDEX(`project_id`);
        ALTER TABLE `fees` ADD INDEX(`address_id`);
        ALTER TABLE `fees` ADD INDEX(`user_id`);
        ALTER TABLE `fees` ADD INDEX(`fee_account_id`);
        ALTER TABLE `fees` ADD INDEX(`predecessor_id`);
        ALTER TABLE `fees` ADD INDEX(`successor_id`);
        ALTER TABLE `fees` ADD INDEX(`cost_centre_id`);
        ALTER TABLE `fees` ADD INDEX(`created_by_id`);
        ALTER TABLE `fees` ADD INDEX(`changed_by_id`);
    CALL us_migrate_11_12_migrate_view('fees_planned');
        ALTER TABLE `fees_planned` ADD INDEX(`fee_date`);
        ALTER TABLE `fees_planned` ADD INDEX(`project_id`);
        ALTER TABLE `fees_planned` ADD INDEX(`address_id`);
        ALTER TABLE `fees_planned` ADD INDEX(`user_id`);
        ALTER TABLE `fees_planned` ADD INDEX(`fee_account_id`);
        ALTER TABLE `fees_planned` ADD INDEX(`predecessor_id`);
        ALTER TABLE `fees_planned` ADD INDEX(`successor_id`);
        ALTER TABLE `fees_planned` ADD INDEX(`cost_centre_id`);
        ALTER TABLE `fees_planned` ADD INDEX(`created_by_id`);
        ALTER TABLE `fees_planned` ADD INDEX(`changed_by_id`);
    CALL us_migrate_11_12_migrate_view('inbox');
        ALTER TABLE `inbox` ADD INDEX(`project_id`);
        ALTER TABLE `inbox` ADD INDEX(`date`);
        ALTER TABLE `inbox` ADD INDEX(`address_id`);
        ALTER TABLE `inbox` ADD INDEX(`dispatch_mode_id`);
        ALTER TABLE `inbox` ADD INDEX(`created_date`);
        ALTER TABLE `inbox` ADD INDEX(`created_by_id`);
        ALTER TABLE `inbox` ADD INDEX(`change_date`);
        ALTER TABLE `inbox` ADD INDEX(`changed_by_id`);
        ALTER TABLE `inbox` ADD INDEX(`recipient_id`);
    CALL us_migrate_11_12_migrate_view('lut_fee_areas_offer');
        ALTER TABLE lut_fee_areas_offer ADD COLUMN `work_package_id` INT NULL, ADD INDEX (`work_package_id`);
        ALTER TABLE `lut_fee_areas_offer` ADD INDEX(`offer_id`);
    CALL us_migrate_11_12_migrate_view('outbox');
        ALTER TABLE `outbox` ADD INDEX(`project_id`);
        ALTER TABLE `outbox` ADD INDEX(`date`);
        ALTER TABLE `outbox` ADD INDEX(`address_id`);
        ALTER TABLE `outbox` ADD INDEX(`staff_id`);
        ALTER TABLE `outbox` ADD INDEX(`dispatch_mode_id`);
        ALTER TABLE `outbox` ADD INDEX(`created_date`);
        ALTER TABLE `outbox` ADD INDEX(`created_by_id`);
        ALTER TABLE `outbox` ADD INDEX(`change_date`);
        ALTER TABLE `outbox` ADD INDEX(`changed_by_id`);
        ALTER TABLE `outbox` ADD INDEX(`doc_id`);
        ALTER TABLE `outbox` ADD INDEX(`mail_id`);
        ALTER TABLE `outbox` ADD INDEX(`parent_id`);
        ALTER TABLE `outbox` ADD INDEX(`acc_after_exp_bill_id`);
        ALTER TABLE `outbox` ADD INDEX(`transmittal_form_id`);
    CALL us_migrate_11_12_migrate_view('times');
        ALTER TABLE `times` ADD INDEX(`project_id`);
        ALTER TABLE `times` ADD INDEX(`date`);
        ALTER TABLE `times` ADD INDEX(`staff_id`);
        ALTER TABLE `times` ADD INDEX(`cost_centre_id`);
        ALTER TABLE `times` ADD INDEX(`changed_by_id`);
        ALTER TABLE `times` ADD INDEX(`company_id`);
        ALTER TABLE `times` ADD INDEX(`is_deducted`);
        ALTER TABLE `times` ADD INDEX(`is_deducted_staff`);
        ALTER TABLE `times` ADD INDEX(`is_deducted_woa`);
        ALTER TABLE `times` ADD INDEX(`vacation_id`);
        ALTER TABLE `times` ADD INDEX(`persapp_id`);
        ALTER TABLE `times` ADD INDEX(`opta_id`);
        ALTER TABLE `times` ADD INDEX(`skill_id`);
    CALL us_migrate_11_12_migrate_view('times_logging');
        ALTER TABLE `times_logging` ADD INDEX(`project_id`);
        ALTER TABLE `times_logging` ADD INDEX(`date`);
        ALTER TABLE `times_logging` ADD INDEX(`staff_id`);
        ALTER TABLE `times_logging` ADD INDEX(`cost_centre_id`);
        ALTER TABLE `times_logging` ADD INDEX(`changed_by_id`);
        ALTER TABLE `times_logging` ADD INDEX(`log_change_date`);
        ALTER TABLE `times_logging` ADD INDEX(`log_changed_by_id`);
        ALTER TABLE `times_logging` ADD INDEX(`company_id`);
        ALTER TABLE `times_logging` ADD INDEX(`is_deducted`);
        ALTER TABLE `times_logging` ADD INDEX(`skill_id`);
    CALL us_migrate_11_12_migrate_view('transmittal_form');
        ALTER TABLE `transmittal_form` ADD INDEX(`project_id`);
        ALTER TABLE `transmittal_form` ADD INDEX(`address_id`);
        ALTER TABLE `transmittal_form` ADD INDEX(`staff_id`);
        ALTER TABLE `transmittal_form` ADD INDEX(`date`);
        ALTER TABLE `transmittal_form` ADD INDEX(`created_by_id`);
        ALTER TABLE `transmittal_form` ADD INDEX(`changed_by_id`);
    CALL us_migrate_11_12_migrate_view('travels');
        ALTER TABLE `travels` ADD INDEX(`project_id`);
        ALTER TABLE `travels` ADD INDEX(`date`);
        ALTER TABLE `travels` ADD INDEX(`staff_id`);
        ALTER TABLE `travels` ADD INDEX(`type_id`);
        ALTER TABLE `travels` ADD INDEX(`cost_centre_id`);
        ALTER TABLE `travels` ADD INDEX(`changed_by_id`);
        ALTER TABLE `travels` ADD INDEX(`company_id`);
        ALTER TABLE `travels` ADD INDEX(`is_deducted`);
    CALL us_migrate_11_12_migrate_view('travels_logging');
        ALTER TABLE `travels_logging` ADD INDEX(`project_id`);
        ALTER TABLE `travels_logging` ADD INDEX(`date`);
        ALTER TABLE `travels_logging` ADD INDEX(`staff_id`);
        ALTER TABLE `travels_logging` ADD INDEX(`type_id`);
        ALTER TABLE `travels_logging` ADD INDEX(`cost_centre_id`);
        ALTER TABLE `travels_logging` ADD INDEX(`changed_by_id`);
        ALTER TABLE `travels_logging` ADD INDEX(`log_change_date`);
        ALTER TABLE `travels_logging` ADD INDEX(`log_changed_by_id`);
        ALTER TABLE `travels_logging` ADD INDEX(`company_id`);
        ALTER TABLE `travels_logging` ADD INDEX(`is_deducted`);
    CALL us_migrate_11_12_migrate_view('work_areas_rates');
        ALTER TABLE `work_areas_rates` ADD INDEX(`rate_id`);
        ALTER TABLE `work_areas_rates` ADD INDEX(`history_id`);
    CALL us_migrate_11_12_migrate_view('work_areas_rates_history');
        ALTER TABLE `work_areas_rates_history` ADD INDEX(`rate_id`);
END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Repair tables which are migrated from views
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_repair_table_structure;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_repair_table_structure()
BEGIN
    -- @todo Remove the UDPATE commands below, if the exchange rate is ever used.

    -- Times
    ALTER TABLE `times` CHANGE COLUMN `company_gross_hourly_wage_ex` `company_gross_hourly_wage_ex` DOUBLE NOT NULL DEFAULT 1;
    UPDATE `times` SET `company_gross_hourly_wage_ex` = 1 WHERE `company_gross_hourly_wage_ex` = 0;
    ALTER TABLE `times` CHANGE COLUMN `company_value_ex` `company_value_ex` DOUBLE NOT NULL DEFAULT 1;
    UPDATE `times` SET `company_value_ex` = 1 WHERE `company_value_ex` = 0;
    ALTER TABLE `times` CHANGE COLUMN `company_price_per_unit_ex` `company_price_per_unit_ex` DOUBLE NOT NULL DEFAULT 1;
    UPDATE `times` SET `company_price_per_unit_ex` = 1 WHERE `company_price_per_unit_ex` = 0;
    ALTER TABLE `times` CHANGE COLUMN `company_price_external_staff_ex` `company_price_external_staff_ex` DOUBLE NOT NULL DEFAULT 1;
    UPDATE `times` SET `company_price_external_staff_ex` = 1 WHERE `company_price_external_staff_ex` = 0;
    ALTER TABLE `times` CHANGE COLUMN `company_price_area_ex` `company_price_area_ex` DOUBLE NOT NULL DEFAULT 1;
    UPDATE `times` SET `company_price_area_ex` = 1 WHERE `company_price_area_ex` = 0;
    -- Extras
    ALTER TABLE `extras` CHANGE COLUMN `company_cost_per_unit_ex` `company_cost_per_unit_ex` DOUBLE NOT NULL DEFAULT 1;
    UPDATE `extras` SET `company_cost_per_unit_ex` = 1 WHERE `company_cost_per_unit_ex` = 0;
    ALTER TABLE `extras` CHANGE COLUMN `company_price_per_unit_ex` `company_price_per_unit_ex` DOUBLE NOT NULL DEFAULT 1;
    UPDATE `extras` SET `company_price_per_unit_ex` = 1 WHERE `company_price_per_unit_ex` = 0;
    -- Travels
    ALTER TABLE `travels` CHANGE COLUMN `company_cost_per_unit_ex` `company_cost_per_unit_ex` DOUBLE NOT NULL DEFAULT 1;
    UPDATE `travels` SET `company_cost_per_unit_ex` = 1 WHERE `company_cost_per_unit_ex` = 0;
    ALTER TABLE `travels` CHANGE COLUMN `company_price_per_unit_ex` `company_price_per_unit_ex` DOUBLE NOT NULL DEFAULT 1;
    UPDATE `travels` SET `company_price_per_unit_ex` = 1 WHERE `company_price_per_unit_ex` = 0;
END $$
DELIMITER ;
-- EOC


DROP PROCEDURE IF EXISTS us_migrate_11_12_migrate_view;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_migrate_view(IN tablename VARCHAR(100))
BEGIN
    DECLARE viewdef LONGTEXT CHARSET utf8 COLLATE utf8_general_ci;
    
    SELECT VIEW_DEFINITION INTO viewdef
    FROM INFORMATION_SCHEMA.VIEWS
    WHERE TABLE_SCHEMA = DATABASE() AND TABLE_NAME = tablename;

    -- Get position of FROM
    SET @from_pos = LOCATE(' from ', viewdef);
    IF @from_pos<10 THEN
        SET @error = CONCAT('View definition for ', tablename, ' misses FROM part.');
        SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = @error;
    END IF;

    -- Prepare variables
    SET @viewtable = REPLACE( SUBSTRING(viewdef, @from_pos+6), '`', '');
    SET @tablename_temp = CONCAT(tablename, '__view');
    SET @viewtable_oldname = CONCAT(@viewtable, '__old');

    INSERT INTO `01migrate_11_12` SET elem='use_new_names_for_table', info=CONCAT('We now use ', tablename, ' and save the old table ', @viewtable, ' as ', @viewtable_oldname);

    -- Save original view to ..._view
    SET @s = CONCAT('RENAME TABLE ', tablename, ' TO ', @tablename_temp);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
    DEALLOCATE PREPARE stmt1;

    -- Copy values from ..._view to tablename
    SET @s = CONCAT('CREATE TABLE ', tablename, ' AS SELECT * FROM ', @tablename_temp);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
    DEALLOCATE PREPARE stmt1;
    -- necessary to skip the logging tables here, because these throwing an error because of having multiple ids with 0
    IF (SUBSTRING(tablename, -7) NOT LIKE 'logging') THEN
            -- Add PRIMARY KEY
            SET @s = CONCAT('ALTER TABLE ', tablename, ' ADD PRIMARY KEY (`id`);');
            PREPARE stmt1 FROM @s;
            EXECUTE stmt1;
            DEALLOCATE PREPARE stmt1;

            -- Add AUTO_INCREMENT
            SET @s = CONCAT('ALTER TABLE ', tablename, ' CHANGE `id` `id` INT(11) NOT NULL AUTO_INCREMENT;');
            PREPARE stmt1 FROM @s;
            EXECUTE stmt1;
            DEALLOCATE PREPARE stmt1;
    END IF;   

    -- Remove original view to ..._view
    SET @s = CONCAT('DROP VIEW ', @tablename_temp);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
    DEALLOCATE PREPARE stmt1;

    -- Move now old table to new name
    SET @s = CONCAT('RENAME TABLE ', @viewtable, ' TO ', @viewtable_oldname);
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
    DEALLOCATE PREPARE stmt1;

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate work areas to work packages
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS us_migrate_11_12_migrate_wa2wp;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_migrate_wa2wp(IN tablename VARCHAR(100), IN area_id VARCHAR(100))
BEGIN

    INSERT INTO `01migrate_11_12` SET elem='wa2wp', info=CONCAT('For table ', tablename, ' we now use work_package_id insteas of ', area_id);

    -- Add work_package_id with INDEX
    SET @s = CONCAT('ALTER TABLE ', tablename, ' ADD COLUMN `work_package_id` INT NULL, ADD INDEX (`work_package_id`);');
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
    DEALLOCATE PREPARE stmt1;

    -- Add work_package_id with INDEX (LIMIT 1 for dirty databases)
    SET @s = CONCAT('UPDATE ', tablename, ' AS main SET work_package_id=(SELECT id FROM work_packages AS wp WHERE main.project_id=wp.project_id AND main.', area_id,'=wp.old_work_area_id  LIMIT 1)');
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
    DEALLOCATE PREPARE stmt1;


END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate work areas to work package tags
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS us_migrate_11_12_migrate_wa2wpt;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_migrate_wa2wpt()
us_migrate_11_12_migrate_wa2wptlbl:BEGIN

    -- Prevent execution, if work_packages_tags has content
    IF ( (SELECT COUNT(*) FROM work_packages_tags)>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_11_12_migrate_wa2wpt';
        LEAVE us_migrate_11_12_migrate_wa2wptlbl;
    END IF;

    INSERT INTO work_packages_tags (id, `name`, wp_name) SELECT LfdNr, ArbName, ArbName FROM arbeitsbereiche;

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Create presets for all work_areas_categories
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_work_areas_categories;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_work_areas_categories()
us_migrate_11_12_work_areas_categorieslbl:BEGIN

    DECLARE woac_id INTEGER;
    DECLARE woac_name VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE woac_special_id INTEGER;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE woac CURSOR FOR SELECT id, name, special_id FROM work_areas_categories WHERE one_project_only=0 AND is_active=1 ORDER BY id;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    -- Prevent execution, if work_package_presets has content
    IF ( (SELECT COUNT(*) FROM work_package_presets)>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_11_12_work_areas_categories';
        LEAVE us_migrate_11_12_work_areas_categorieslbl;
    END IF;

    /* Just for dev. In live it should be empty */
    TRUNCATE work_package_presets;

    OPEN woac;
    cursor_loop: LOOP -- -------------------------------------
        FETCH woac INTO woac_id, woac_name, woac_special_id;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
            /*Do something with the row fetched*/
            INSERT INTO `01migrate_11_12` SET elem='woac_id', info=woac_id;
            
            /* Create work_package_presets main entry */
            INSERT INTO work_package_presets (tree_id, name, lft, rgt, is_purpose_required, is_fee_area, is_special) VALUES(woac_id, woac_name, 1, 2, 0, 0, 0);
            SET @insert_id_main = LAST_INSERT_ID();
            
            /* Create work_package_presets from fee and work areas */
            CALL us_migrate_11_12_copy_fee_areas(woac_id, woac_special_id, @insert_id_main);
            
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  woac;
END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- SUB - Copy all fee and work areas for a certain work_areas_category
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_copy_fee_areas;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_copy_fee_areas(IN woac_id INTEGER, IN woac_special_id INTEGER, IN main_tree_id INTEGER)
BEGIN
    DECLARE wo_id INTEGER;
    DECLARE wo_name VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE wo_is_special INTEGER;
    DECLARE wo_is_purpose_required INTEGER;
    DECLARE fe_id INTEGER;
    DECLARE fe_name VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE fe_is_special INTEGER;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE fee_areas CURSOR FOR SELECT 
            wo.id,
            wo.name,
            wo.is_special,
            wo.is_purpose_required,
            fe.id,
            fe.name,
            fe.is_special
        FROM
            work_areas AS wo
                INNER JOIN
            fee_areas AS fe ON wo.fee_area_id = fe.id
                INNER JOIN
            lut_work_areas_work_areas_categories AS wc ON wc.work_area_id=wo.id
        WHERE 
                wc.work_area_categories_id=woac_id
        ORDER BY
                fe.sort ASC, wo.sort ASC;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    SET @last_fe_id = 0;

    OPEN fee_areas;
    cursor_loop: LOOP -- -------------------------------------
        FETCH fee_areas INTO wo_id, wo_name, wo_is_special, wo_is_purpose_required, fe_id, fe_name, fe_is_special;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
            /*Do something with the row fetched*/
            INSERT INTO `01migrate_11_12` SET elem='fee_woc', info=CONCAT_WS(', ', main_tree_id, wo_id, wo_name, wo_is_special, wo_is_purpose_required, fe_id, fe_name, fe_is_special);
            
            IF @last_fe_id<>fe_id THEN
                SET @have_work_area_for_fee_area = false;
                /* Create fee area as preset */
                CALL us_insert_work_package_presets(
                    main_tree_id, -- parent_id
                    fe_name, -- new_name
                    wo_is_purpose_required, -- is_purpose (Should only be set on LOWEST)
                    IF(fe_is_special, woac_special_id, 0), -- is_special
                    NULL, -- is_special_phase
                    true, -- is_fee_area
                    0, -- threshold
                    0, -- is_extern
                    null, -- cost centre
                    null, -- tag id
                    @insert_id_fe
                );
                INSERT INTO `01migrate_11_12` SET elem='us_insert_work_package_presets fee', info=COALESCE(@insert_id_fe, 'NULL');

                /* Set the fee values */
                UPDATE work_package_presets SET old_fee_area_id=fe_id WHERE ID=@insert_id_fe;
                
                /* Create preset if the work and fee area do not have the same name */
                IF us_delete_spaces(TRIM(wo_name))<>us_delete_spaces(TRIM(fe_name)) THEN
                    SET @have_work_area_for_fee_area = true;
                    CALL us_insert_work_package_presets(
                        @insert_id_fe, -- parent_id
                        wo_name, -- new_name
                        wo_is_purpose_required, -- is_purpose (Should only be set on LOWEST)
                        IF(wo_is_special, woac_special_id, 0), -- is_special
                        NULL, -- is_special_phase
                        false, -- is_fee_area
                        0, -- threshold
                        0, -- is_extern
                        null, -- cost centre
                        null, -- tag id
                        @insert_id_wo
                    );
                
                    /* Set the work area values */
                    UPDATE work_package_presets SET old_work_area_id=wo_id, tag_id=wo_id WHERE ID=@insert_id_wo;
                ELSE
                    /* Set the work area values */
                    UPDATE work_package_presets SET old_work_area_id=wo_id, tag_id=wo_id WHERE ID=@insert_id_fe;
                END IF;
            ELSE
                /* Create preset of first work area, if not  @have_work_area_for_fee_area */
                IF @have_work_area_for_fee_area<>true THEN
                    SET @have_work_area_for_fee_area = true;
                    CALL us_insert_work_package_presets(
                        @insert_id_fe, -- parent_id
                        fe_name, -- new_name (New name of wo was same)
                        @last_wo_is_purpose_required, -- is_purpose (Should only be set on LOWEST)
                        IF(@last_wo_is_special, woac_special_id, 0), -- is_special
                        NULL, -- is_special_phase
                        false, -- is_fee_area
                        0, -- threshold
                        0, -- is_extern
                        null, -- cost centre
                        null, -- tag id
                        @insert_id_wo
                    );
                    
                    /* Set the work area values */
                    UPDATE work_package_presets SET old_work_area_id=wo_id, tag_id=wo_id WHERE ID=@insert_id_wo;
                END IF;

                /* Create preset of next work area */
                CALL us_insert_work_package_presets(
                    @insert_id_fe, -- parent_id
                    wo_name, -- new_name
                    wo_is_purpose_required, -- is_purpose (Should only be set on LOWEST)
                    IF(wo_is_special, woac_special_id, 0), -- is_special
                    NULL, -- is_special_phase
                    false, -- is_fee_area
                    0, -- threshold
                    0, -- is_extern
                    null, -- cost centre
                    null, -- tag id
                    @insert_id_wo
                );
                
                /* Set the work area values */
                UPDATE work_package_presets SET old_work_area_id=wo_id, tag_id=wo_id WHERE ID=@insert_id_wo;
            END IF;

            SET @last_wo_is_purpose_required = wo_is_purpose_required;
            SET @last_wo_is_special = wo_is_special;
            SET @last_fe_id = fe_id;
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  fee_areas;
END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Create work_packages for project and change all work areas to work packages
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS us_migrate_11_12_projects;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_projects()
us_migrate_11_12_projectslbl:BEGIN
    
    DECLARE projects_id INTEGER;
    DECLARE projects_project_name VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE woac_id INTEGER;
    DECLARE woac_name VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE woac_special_id INTEGER;
    DECLARE woac_one_project_only INTEGER;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE projects CURSOR FOR SELECT 
            p.id, p.project_name, 
            woac.id, woac.name, IFNULL(woac.special_id, woac.was_special_id), one_project_only
        FROM
            projects AS p
                INNER JOIN
            work_areas_categories AS woac ON p.work_area_category_id=woac.id
        ORDER BY p.id;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    -- Prevent execution, if work_packages has content
    IF ( (SELECT COUNT(*) FROM work_packages)>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_11_12_projects';
        LEAVE us_migrate_11_12_projectslbl;
    END IF;

    IF @debug THEN -- ==========================================================
        select now() AS 'THE_START_TIME_____', 'us_migrate_11_12_projects <--' AS 'COMMENT';
    END IF;

    OPEN projects;
    cursor_loop: LOOP -- -------------------------------------
        FETCH projects INTO projects_id, projects_project_name, woac_id, woac_name, woac_special_id, woac_one_project_only;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
            /*Do something with the row fetched*/
            INSERT INTO `01migrate_11_12` SET elem='projects_id', info=projects_id;

            /* Create work_package main entry */
            INSERT INTO work_packages (project_id, name, lft, rgt, is_purpose_required, is_fee_area, is_special, wpp_tree_id) 
                VALUES(projects_id, CONCAT(projects_project_name, ' - ', woac_name), 1, 2, 0, 0, 0, IF(woac_one_project_only, null, woac_id));
            SET @insert_id_main = LAST_INSERT_ID();
            
            UPDATE projects SET work_package_preset_id = (SELECT wpp_tree_id FROM work_packages WHERE project_id = projects_id AND wpp_tree_id IS NOT NULL GROUP BY project_id) WHERE id = projects_id;

            CALL us_migrate_11_12_copy_projects_fee_areas(projects_id, woac_id, woac_special_id, @insert_id_main);
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  projects;

    IF @debug THEN -- ==========================================================
        select now() AS 'THE_START_TIME_____', 'Change INDEX to UNIQUE' AS 'COMMENT';
    END IF;

    -- Rem INDEX key for work_packages
    CALL us_drop_index_if_exists('work_packages', 'wp_lft_index');
    CALL us_drop_index_if_exists('work_packages', 'wp_rgt_index');

    -- Add UNIQUE key for work_packages
    CALL us_add_constraint_to_table('work_packages', 'wp_lft_unique', 'UNIQUE', '`project_id` , `lft`' );
    CALL us_add_constraint_to_table('work_packages', 'wp_rgt_unique', 'UNIQUE', '`project_id` , `rgt`' );


    IF @debug THEN -- ==========================================================
        select now() AS 'THE_START_TIME_____', 'Done <--' AS 'COMMENT';
    END IF;

END $$
DELIMITER ;
-- EOC


-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- SUB - Copy all fee and work areas for a certain work_areas_category
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_copy_projects_fee_areas;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_copy_projects_fee_areas(IN projects_id INTEGER, IN woac_id INTEGER, IN woac_special_id INTEGER, IN main_tree_id INTEGER)
BEGIN
    DECLARE wo_id INTEGER;
    DECLARE wo_name VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE wo_is_special INTEGER;
    DECLARE wo_is_purpose_required INTEGER;
    DECLARE fe_id INTEGER;
    DECLARE fe_name VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE fe_is_special INTEGER;
    DECLARE fep_status INTEGER;
    DECLARE fep_costs DOUBLE;
    DECLARE fep_hours DOUBLE;
    DECLARE bp_proportion DOUBLE;
    DECLARE bp_costs DOUBLE;
    DECLARE bp_hours DOUBLE;
    DECLARE bp_start_date DATETIME;
    DECLARE bp_end_date DATETIME;
    DECLARE bp_threshold DOUBLE;
    DECLARE cl_completion_level DOUBLE;
    DECLARE cl_date DATE;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE fee_areas CURSOR FOR SELECT 
        wo.id,
        wo.name,
        wo.is_special,
        wo.is_purpose_required,
        fe.id,
        fe.name,
        fe.is_special,
        fep.status,
        fep.costs,
        fep.hours,
        bp.proportion,
        bp.planned_costs,
        bp.planned_hours,
        bp.start_date,
        bp.end_date,
        bp.threshold,
        cl.completion_level,
        cl.date
    FROM
        work_areas AS wo
            INNER JOIN
        fee_areas AS fe ON wo.fee_area_id = fe.id
            INNER JOIN
        lut_work_areas_work_areas_categories AS wc ON wc.work_area_id = wo.id
            LEFT JOIN
        -- Fee areas for project
        lut_fee_areas_proj as fep ON fe.id = fep.fee_area_id AND fep.project_id = projects_id
            LEFT JOIN
        lut_fee_areas_proj_history AS lfaph ON lfaph.id = fep.lut_fee_areas_proj_history_id
            LEFT JOIN
        -- Budget plan for project
        budget_plan AS bp ON wo.id = bp.area_id AND bp.project_id = projects_id
            LEFT JOIN
        budget_plan_history AS bph ON bp.budget_plan_history_id = bph.id
            LEFT JOIN
        -- Completion level for project
        completion_level AS cl ON wo.id = cl.work_area_id AND cl.project_id = projects_id
            LEFT JOIN
        completion_level_history AS clh ON cl.completion_level_history_id = clh.id
        WHERE
        wc.work_area_categories_id = woac_id
            AND
        lfaph.valid_to_date IS NULL
            AND
        bph.valid_to_date IS NULL
            AND
        clh.valid_to_date IS NULL
        GROUP BY wo.id
        ORDER BY fe.sort ASC , wo.sort ASC;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    SET @last_fe_id = 0;

    OPEN fee_areas;
    cursor_loop: LOOP -- -------------------------------------
        FETCH fee_areas INTO wo_id, wo_name, wo_is_special, wo_is_purpose_required, fe_id, fe_name, fe_is_special, fep_status, fep_costs, fep_hours, 
            bp_proportion, bp_costs, bp_hours, bp_start_date, bp_end_date, bp_threshold, cl_completion_level, cl_date;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
            /*Do something with the row fetched*/
            INSERT INTO `01migrate_11_12` SET elem='fee_woc', info=CONCAT_WS(', ', projects_id, main_tree_id, wo_id, wo_name, wo_is_special, wo_is_purpose_required, fe_id, fe_name, fe_is_special);
            
            IF @last_fe_id<>fe_id THEN
                SET @have_work_area_for_fee_area = false;
                /* Create fee area as preset */
                CALL us_migrate_11_12_copy_projects_fee_areas_insert(
                    main_tree_id, -- parent_id
                    fe_name, -- new_name
                    NULL, -- TAG ID
                    wo_is_purpose_required, -- is_purpose (Should only be set on LOWEST)
                    IF(fe_is_special, woac_special_id, 0), -- is_special
                    true, -- is_fee_area
                    0, -- threshold
                    @insert_id_fe
                );
                INSERT INTO `01migrate_11_12` SET elem='us_insert_work_package fee', info=@insert_id_fe;
                
                /* Set the fee values */
                UPDATE work_packages SET fee_costs=fep_costs, fee_hours=fep_hours, status=IFNULL(fep_status,20),
                    old_fee_area_id=fe_id WHERE ID=@insert_id_fe;
                INSERT INTO `01migrate_11_12` SET elem='work_packages fee values', info=CONCAT_WS(', ', projects_id, @insert_id_fe, fep_costs, fep_hours, fep_status);

                /* Create preset if the work and fee area do not have the same name */
                IF us_delete_spaces(TRIM(wo_name))<>us_delete_spaces(TRIM(fe_name)) THEN
                    SET @have_work_area_for_fee_area = true;
                    CALL us_migrate_11_12_copy_projects_fee_areas_insert(
                        @insert_id_fe, -- parent_id
                        wo_name, -- new_name
                        NULL, -- TAG ID
                        wo_is_purpose_required, -- is_purpose (Should only be set on LOWEST)
                        IF(wo_is_special, woac_special_id, 0), -- is_special
                        false, -- is_fee_area
                        0, -- threshold
                        @insert_id_wo
                    );
                    INSERT INTO `01migrate_11_12` SET elem='us_insert_work_package wo1', info=@insert_id_wo;
                    
                    /* Set the budget plan values */
                    UPDATE work_packages SET budget_proportion=IFNULL(bp_proportion,0), budget_planned_costs=bp_costs, budget_planned_hours=bp_hours, 
                        budget_start=bp_start_date, budget_end=bp_end_date, budget_threshold=bp_threshold, completion_level=cl_completion_level, loc_estimation_date=cl_date,
                        old_work_area_id=wo_id, tag_id=wo_id WHERE ID=@insert_id_wo;
                ELSE
                    /* Set the budget plan values */
                    UPDATE work_packages SET budget_proportion=IFNULL(bp_proportion,0), budget_planned_costs=bp_costs, budget_planned_hours=bp_hours, 
                        budget_start=bp_start_date, budget_end=bp_end_date, budget_threshold=bp_threshold, completion_level=cl_completion_level, loc_estimation_date=cl_date,
                        old_work_area_id=wo_id, tag_id=wo_id WHERE ID=@insert_id_fe;
                END IF;
            ELSE
                /* Create preset of first work area, if not  @have_work_area_for_fee_area */
                IF @have_work_area_for_fee_area<>true THEN
                    SET @have_work_area_for_fee_area = true;

                    /* Remove work area values from now fee area only */
                    UPDATE work_packages SET budget_proportion=null, budget_planned_costs=null, budget_planned_hours=null, 
                        budget_start=null, budget_end=null, budget_threshold=null, completion_level=null, loc_estimation_date=null,
                        old_work_area_id=null WHERE ID=@insert_id_fe;
                    
                    /* Create the new sub work area */
                    CALL us_migrate_11_12_copy_projects_fee_areas_insert(
                        @insert_id_fe, -- parent_id
                        fe_name, -- new_name (New name of wo was same)
                        NULL, -- TAG ID
                        @last_wo_is_purpose_required, -- is_purpose (Should only be set on LOWEST)
                        IF(@last_wo_is_special, woac_special_id, 0), -- is_special
                        false, -- is_fee_area
                        0, -- threshold
                        @insert_id_wo
                    );
                    INSERT INTO `01migrate_11_12` SET elem='us_insert_work_package wo2', info=@insert_id_wo;
                    
                    /* Set the budget plan values */
                    UPDATE work_packages SET budget_proportion=@last_bp_proportion, budget_planned_costs=@last_bp_costs, budget_planned_hours=@last_bp_hours, 
                        budget_start=@last_bp_start_date, budget_end=@last_bp_end_date, budget_threshold=@last_bp_threshold, completion_level=@last_cl_completion_level, loc_estimation_date=@last_cl_completion_level_date,
                        old_work_area_id=@last_wo_id, tag_id=@last_wo_id WHERE ID=@insert_id_wo;
                END IF;

                /* Create preset of next work area */
                CALL us_migrate_11_12_copy_projects_fee_areas_insert(
                    @insert_id_fe, -- parent_id
                    wo_name, -- new_name
                    NULL, -- TAG ID
                    wo_is_purpose_required, -- is_purpose (Should only be set on LOWEST)
                    IF(wo_is_special, woac_special_id, 0), -- is_special
                    false, -- is_fee_area
                    0, -- threshold
                    @insert_id_wo
                );
                INSERT INTO `01migrate_11_12` SET elem='us_insert_work_package wo3', info=@insert_id_wo;
                
                /* Set the budget plan values */
                UPDATE work_packages SET budget_proportion=IFNULL(bp_proportion,0), budget_planned_costs=bp_costs, budget_planned_hours=bp_hours, 
                    budget_start=bp_start_date, budget_end=bp_end_date, budget_threshold=bp_threshold, completion_level=cl_completion_level,
                    old_work_area_id=wo_id, tag_id=wo_id WHERE ID=@insert_id_wo;
            END IF;

            SET @last_wo_is_purpose_required = wo_is_purpose_required;
            SET @last_wo_is_special = wo_is_special;
            SET @last_fe_id = fe_id;
            SET @last_bp_proportion = IFNULL(bp_proportion,0);
            SET @last_bp_costs = bp_costs;
            SET @last_bp_hours = bp_hours;
            SET @last_bp_start_date = bp_start_date;
            SET @last_bp_end_date = bp_end_date;
            SET @last_bp_threshold = bp_threshold;
            SET @last_cl_completion_level = cl_completion_level;
            SET @last_cl_completion_level_date = cl_date;
            SET @last_wo_id = wo_id;
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  fee_areas;
    
CALL calc_sum(projects_id);
CALL get_date(projects_id);
CALL calc_completion_level(projects_id);
END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- SUB - Write work packages
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_copy_projects_fee_areas_insert;
-- EOC
DELIMITER $$
-- If you update this PROCEDURE also update us_insert_work_package
CREATE PROCEDURE us_migrate_11_12_copy_projects_fee_areas_insert(IN parent_id VARCHAR(100), IN new_name VARCHAR(100), IN tag_id INTEGER, IN is_purpose BOOLEAN, IN is_special BOOLEAN, IN is_fee_area BOOLEAN, IN threshold DOUBLE, OUT insert_id VARCHAR(100))
BEGIN
    /* SET by SELECT */
    (SELECT rgt, project_id 
        INTO @myRight, @myProjectId
        FROM work_packages WHERE id = parent_id);

    UPDATE work_packages SET rgt = rgt + 2 WHERE rgt >= @myRight AND project_id = @myProjectId;
    UPDATE work_packages SET lft = lft + 2 WHERE lft >= @myRight AND project_id = @myProjectId;

    INSERT INTO work_packages(name, tag_id, lft, rgt, project_id, is_purpose_required, is_special, is_fee_area, budget_threshold) VALUES(new_name, tag_id, @myRight, @myRight + 1, @myProjectId, is_purpose, is_special, is_fee_area, threshold);

    SET insert_id = LAST_INSERT_ID();

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Create work package history for projects
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS us_migrate_11_12_projects_history;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_projects_history()
us_migrate_11_12_projects_historylbl:BEGIN
    
    DECLARE projects_id INTEGER;
    DECLARE projects_project_name VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE projects CURSOR FOR SELECT 
            p.id, p.project_name
        FROM
            projects AS p
        ORDER BY p.id;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    -- Prevent execution, if work_packages_history has content
    IF ( (SELECT COUNT(*) FROM work_packages_history)>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_11_12_projects_history';
        LEAVE us_migrate_11_12_projects_historylbl;
    END IF;

    IF @debug THEN -- ==========================================================
        select now() AS 'THE_START_TIME_____', 'us_migrate_11_12_projects_history <--' AS 'COMMENT';
    END IF;
    -- --------------------------- Add dates ---------------------------
    OPEN projects;
    cursor_loop: LOOP -- -------------------------------------
        FETCH projects INTO projects_id, projects_project_name;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
            /*Do something with the row fetched*/
            INSERT INTO `01migrate_11_12` SET elem='projects_history', info=projects_id;

            CALL us_migrate_11_12_projects_history_dates(projects_id);

    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  projects;

    IF @debug THEN -- ==========================================================
        select now() AS 'THE_START_TIME_____', 'Add description values' AS 'COMMENT';
    END IF;
    -- --------------------------- Update history values ---------------------------
    INSERT INTO work_packages_history_descriptions (description, archive_date, project_id, type) 
        SELECT comment, migrate_valid_from_date, project_id, 'fee' FROM lut_fee_areas_proj_history
        WHERE comment IS NOT NULL AND COMMENT <> '';
    INSERT INTO work_packages_history_descriptions (description, archive_date, project_id, type) 
        SELECT comment, migrate_valid_from_date, project_id, 'budget' FROM budget_plan_history
        WHERE comment IS NOT NULL AND COMMENT <> '';
    INSERT INTO work_packages_history_descriptions (description, archive_date, project_id, type) 
        SELECT comment, migrate_valid_from_date, project_id, 'loc' FROM completion_level_history
        WHERE comment IS NOT NULL AND COMMENT <> '';

    IF @debug THEN -- ==========================================================
        select now() AS 'THE_START_TIME_____', 'Update history values' AS 'COMMENT';
    END IF;
    -- --------------------------- Update history values ---------------------------
    CALL us_migrate_11_12_projects_update_history();

    IF @debug THEN -- ==========================================================
        select now() AS 'THE_START_TIME_____', 'SUM child values' AS 'COMMENT';
    END IF;
    -- --------------------------- SUM child values ----------------------------
    CALL us_migrate_11_12_projects_sum_history();

    IF @debug THEN -- ==========================================================
        select now() AS 'THE_START_TIME_____', 'Calculate loc' AS 'COMMENT';
    END IF;
    -- --------------------------- Calculate loc -------------------------------
    CALL us_migrate_11_12_projects_calc_loc();

    IF @debug THEN -- ==========================================================
        select now() AS 'THE_START_TIME_____', 'Done <--' AS 'COMMENT';
    END IF;

END $$
DELIMITER ;
-- EOC

-- --------------------------- --------------------------- --------------------------- ---------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_projects_update_history;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_projects_update_history()
BEGIN

    DECLARE dates datetime;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE get_dates CURSOR FOR SELECT 
            DISTINCT date 
        FROM work_packages_history
        ORDER BY date DESC;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    -- --------------------------- Update history values ---------------------------
    OPEN get_dates;
    cursor_loop: LOOP -- -------------------------------------
        FETCH get_dates INTO dates;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
            /*Do something with the row fetched*/
            INSERT INTO `01migrate_11_12` SET elem='projects_update_history', info=dates;
            
            -- Update lut_fee_areas_proj values
            UPDATE work_packages_history AS wh 
                INNER JOIN (
                    SELECT w.id, l.costs, l.hours
                    FROM work_packages AS w
                    INNER JOIN lut_fee_areas_proj AS l ON w.old_fee_area_id=l.fee_area_id AND w.project_id=l.project_id
                    INNER JOIN lut_fee_areas_proj_history AS lh ON l.lut_fee_areas_proj_history_id=lh.id
                    WHERE lh.migrate_valid_from_date=dates
                ) whsub
                ON wh.wp_id=whsub.id
            SET wh.fee_costs=whsub.costs, wh.fee_hours=whsub.hours
            WHERE wh.fee_costs IS NULL AND wh.fee_hours IS NULL AND wh.date>=dates;
            
            -- Update basic budget_plan values
            UPDATE work_packages_history AS wh 
                INNER JOIN (
                    SELECT w.id, p.proportion, p.planned_costs, p.planned_hours, p.threshold
                    FROM work_packages AS w
                    INNER JOIN budget_plan AS p ON w.old_work_area_id=p.area_id AND w.project_id=p.project_id
                    INNER JOIN budget_plan_history AS ph ON p.budget_plan_history_id=ph.id
                    WHERE ph.migrate_valid_from_date=dates
                ) whsub
                ON wh.wp_id=whsub.id
            SET wh.budget_proportion=whsub.proportion, wh.budget_planned_costs=whsub.planned_costs, 
                wh.budget_planned_hours=whsub.planned_hours, wh.budget_threshold=whsub.threshold
            WHERE wh.budget_proportion IS NULL AND wh.budget_planned_costs IS NULL AND 
                wh.budget_planned_hours IS NULL AND wh.budget_threshold IS NULL AND wh.date>=dates;
            
            -- Update basic completion_level values
            UPDATE work_packages_history AS wh 
                INNER JOIN (
                    SELECT w.id, c.completion_level
                    FROM work_packages AS w
                    INNER JOIN completion_level AS c ON w.old_work_area_id=c.work_area_id AND w.project_id=c.project_id
                    INNER JOIN completion_level_history AS ch ON c.completion_level_history_id=ch.id
                    WHERE ch.migrate_valid_from_date=dates
                ) whsub
                ON wh.wp_id=whsub.id
            SET wh.completion_level=whsub.completion_level
            WHERE wh.completion_level IS NULL AND wh.date>=dates;
            
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  get_dates;

    UPDATE work_packages_history SET fee_costs=0 WHERE fee_costs IS NULL;
    UPDATE work_packages_history SET fee_hours=0 WHERE fee_hours IS NULL;
    UPDATE work_packages_history SET budget_proportion=0 WHERE budget_proportion IS NULL;
    UPDATE work_packages_history SET budget_planned_costs=0 WHERE budget_planned_costs IS NULL;
    UPDATE work_packages_history SET budget_planned_hours=0 WHERE budget_planned_hours IS NULL;
    UPDATE work_packages_history SET budget_threshold=0 WHERE budget_threshold IS NULL;
    UPDATE work_packages_history SET completion_level=0 WHERE completion_level IS NULL;

END $$
DELIMITER ;
-- EOC

-- --------------------------- --------------------------- --------------------------- ---------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_projects_sum_history;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_projects_sum_history()
BEGIN

    -- --------------------------- Sum budget child values ---------------------------
    UPDATE
        work_packages_history AS wph
        JOIN (
                SELECT 
                    wph.date, wp.id,
                    (
                        SELECT 
                            SUM(wph1.budget_planned_costs)
                        FROM
                            work_packages_history AS wph1
                            INNER JOIN
                            work_packages AS wp1 ON wp1.id = wph1.wp_id
                        WHERE
                            wph1.project_id=wph.project_id
                            AND
                            wph1.date=wph.date
                            AND
                            wp1.lft>=wp.lft
                            AND
                            wp1.rgt<=wp.rgt
                    ) AS cost,
                    (
                        SELECT 
                            SUM(wph1.budget_planned_hours)
                        FROM
                            work_packages_history AS wph1
                            INNER JOIN
                            work_packages AS wp1 ON wp1.id = wph1.wp_id
                        WHERE
                            wph1.project_id=wph.project_id
                            AND
                            wph1.date=wph.date
                            AND
                            wp1.lft>=wp.lft
                            AND
                            wp1.rgt<=wp.rgt
                    ) AS hours,
                    (
                        SELECT 
                            SUM(wph1.budget_proportion)
                        FROM
                            work_packages_history AS wph1
                            INNER JOIN
                            work_packages AS wp1 ON wp1.id = wph1.wp_id
                        WHERE
                            wph1.project_id=wph.project_id
                            AND
                            wph1.date=wph.date
                            AND
                            wp1.lft>=wp.lft
                            AND
                            wp1.rgt<=wp.rgt
                    ) AS proportion,
                    (
                        SELECT 
                            SUM(wph1.fee_costs)
                        FROM
                            work_packages_history AS wph1
                            INNER JOIN
                            work_packages AS wp1 ON wp1.id = wph1.wp_id
                        WHERE
                            wph1.project_id=wph.project_id
                            AND
                            wph1.date=wph.date
                            AND
                            wp1.lft>=wp.lft
                            AND
                            wp1.rgt<=wp.rgt
                    ) AS fee_costs,
                    (
                        SELECT 
                            SUM(wph1.fee_hours)
                        FROM
                            work_packages_history AS wph1
                            INNER JOIN
                            work_packages AS wp1 ON wp1.id = wph1.wp_id
                        WHERE
                            wph1.project_id=wph.project_id
                            AND
                            wph1.date=wph.date
                            AND
                            wp1.lft>=wp.lft
                            AND
                            wp1.rgt<=wp.rgt
                    ) AS fee_hours
                FROM
                    work_packages_history AS wph
                    INNER JOIN
                    work_packages AS wp ON wp.id = wph.wp_id
        ) bhsub
        ON 
            bhsub.id=wph.wp_id
            AND
            bhsub.date=wph.date
    SET wph.budget_planned_costs = bhsub.cost, wph.budget_planned_hours = bhsub.hours, wph.budget_proportion = bhsub.proportion,
        wph.fee_costs = bhsub.fee_costs, wph.fee_hours = bhsub.fee_hours;

END $$
DELIMITER ;
-- EOC

-- --------------------------- Avoid division by zero (first sum all values together (especially budget_proportion) and then calculate loc --------------------------- ---------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_projects_calc_loc;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_projects_calc_loc()
BEGIN

    -- --------------------------- Sum child values ---------------------------
    DECLARE wps_id INTEGER;
    DECLARE project_ids INTEGER;
    DECLARE lft_vals INTEGER;
    DECLARE rgt_vals INTEGER;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE work_packages CURSOR FOR SELECT 
            wp.id, wp.project_id, wp.lft, wp.rgt
        FROM
            work_packages AS wp WHERE rgt-lft > 1
        ORDER BY wp.id;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    OPEN work_packages;
    cursor_loop: LOOP -- -------------------------------------
        FETCH work_packages INTO wps_id, project_ids, lft_vals, rgt_vals;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;

        CALL us_migrate_11_12_projects_calc_loc_sub(wps_id, project_ids, lft_vals, rgt_vals);
            
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  work_packages;
END $$
DELIMITER ;
-- EOC

-- --------------------------- --------------------------- --------------------------- ---------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_projects_calc_loc_sub;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_projects_calc_loc_sub(IN my_wp_id integer, IN my_project integer, lft_val integer, rgt_val integer)
BEGIN

    -- --------------------------- Sum child values ---------------------------
    DECLARE dates datetime;
    DECLARE ids integer;
    DECLARE proportions integer;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE history_dates CURSOR FOR SELECT 
            wph.date, wph.id, wph.budget_proportion
        FROM
            work_packages_history AS wph WHERE wp_id = my_wp_id
        ORDER BY wph.date;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    OPEN history_dates;
    cursor_loop: LOOP -- -------------------------------------
        FETCH history_dates INTO dates, ids, proportions;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF; 

        IF (SELECT budget_proportion from work_packages_history where wp_id = my_wp_id AND date = dates) > 0 THEN
            /* SET by SELECT */
            (SELECT SUM((completion_level * budget_proportion) / (SELECT budget_proportion from work_packages_history where wp_id = my_wp_id AND date = dates)) 
                INTO @loc
                FROM work_packages_history WHERE wp_id IN 
                (SELECT id from work_packages where project_id = my_project AND lft BETWEEN lft_val AND rgt_val AND rgt-lft = 1) AND date = dates);
            SET @s = CONCAT('UPDATE work_packages_history SET completion_level = ', @loc, '  WHERE id = ', ids);
            PREPARE stmt1 FROM @s;
            EXECUTE stmt1;
        END IF;
            
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  history_dates;
END $$
DELIMITER ;
-- EOC

-- --------------------------- --------------------------- --------------------------- ---------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_projects_clone_to_history;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_projects_clone_to_history(IN projects_id INTEGER, IN valid_from_datess DATE)
BEGIN
    REPLACE INTO work_packages_history (project_id, completion_level, budget_proportion, budget_planned_costs, budget_planned_hours, budget_threshold, 
        fee_costs, fee_hours, fee_proportion, wpp_tree_id, date, wp_id, loc_estimation_date)
    SELECT project_id, NULL AS completion_level, NULL AS budget_proportion, NULL AS budget_planned_costs, NULL AS budget_planned_hours, NULL AS budget_threshold, 
        NULL AS fee_costs, NULL AS fee_hours, NULL AS fee_proportion, wpp_tree_id, valid_from_datess AS date, id AS wp_id, 
        NULL AS loc_estimation_date from work_packages
        WHERE project_id=projects_id;
END $$
DELIMITER ;
-- EOC

-- --------------------------- --------------------------- --------------------------- ---------------------------
DROP PROCEDURE IF EXISTS us_migrate_11_12_projects_history_dates;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12_projects_history_dates(IN projects_id INTEGER)
BEGIN
    
    DECLARE valid_from_dates datetime;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    -- --------------------------- Get current dates ---------------------------
    DECLARE project_dates CURSOR FOR 	SELECT migrate_valid_from_date
            FROM lut_fee_areas_proj_history AS lh WHERE lh.project_id=projects_id
        UNION DISTINCT
            SELECT migrate_valid_from_date
            FROM budget_plan_history AS bh WHERE bh.project_id=projects_id
        UNION DISTINCT
            SELECT migrate_valid_from_date
            FROM completion_level_history AS cl WHERE cl.project_id=projects_id
        ORDER BY migrate_valid_from_date;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    -- --------------------------- Add dates ---------------------------
    OPEN project_dates;
    cursor_loop: LOOP -- -------------------------------------
        FETCH project_dates INTO valid_from_dates;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
        
        CALL us_migrate_11_12_projects_clone_to_history(projects_id, valid_from_dates);
        
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  project_dates;

END $$
DELIMITER ;
-- EOC


-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Improve todo - In procedure, so that change_column does not always trigger remove of fulltext index
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

DROP PROCEDURE IF EXISTS update_improve_todo;
-- EOC
DELIMITER $$
CREATE PROCEDURE update_improve_todo()
BEGIN

    -- Open tasks
    CALL check_rename_column_exists('todo', 'Modultyp', 'IsPostAusgang', 'integer');

    CALL check_rename_column_exists('todo', 'Modul_id', 'lfdnr_post', 'integer');

    -- Open tasks
    CALL us_add_to_table2('todo', 'active_from', 'DATETIME DEFAULT NULL AFTER `Bemerkung`' );

    CALL us_add_to_table2('todo', 'period_value', 'INT( 4 ) DEFAULT 1 AFTER `Bemerkung`' );

    CALL us_add_to_table2('todo', 'unit', 'INT( 4 )  DEFAULT 1 AFTER `period_value`' );

    CALL us_add_to_table2('todo', 'working_hours', 'DOUBLE  DEFAULT NULL AFTER `unit`' );

    CALL change_column('todo', 'ErlVermerk', 'LONGTEXT NULL DEFAULT NULL AFTER `AendVon`');

    CALL us_add_to_table2('todo', 'lfdnr_post', 'INT( 11 ) NULL AFTER `IsPostAusgang`' );

    CALL change_column('todo', 'lfdnr_post', 'VARCHAR( 50 ) NULL DEFAULT NULL AFTER `IsPostAusgang`');

END $$
DELIMITER ;
-- EOC-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Update offers to new format with more features for X3, especially for HOAI
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS update_lut_fee_areas_offer_add_fields;
-- EOC
DELIMITER $$
CREATE PROCEDURE update_lut_fee_areas_offer_add_fields()
BEGIN
    CALL us_add_to_table2('lut_fee_areas_offer', 'costs_view', 'DOUBLE NOT NULL DEFAULT 0 AFTER `costs`' );
    CALL us_add_to_table2('lut_fee_areas_offer', 'extras', 'DOUBLE NOT NULL DEFAULT 0 AFTER `costs_view`' );
    CALL us_add_to_table2('lut_fee_areas_offer', 'rebate', 'DOUBLE NOT NULL DEFAULT 0 AFTER `extras`' );

    CALL us_add_to_table2('lut_fee_areas_offer', 'percent_sum', 'INT( 11 ) DEFAULT 0' );

    -- Original currency values
    CALL us_add_to_table2('lut_fee_areas_offer', 'costs_otcurr', 'double DEFAULT NULL');
    CALL us_add_to_table2('lut_fee_areas_offer', 'costs_sum_otcurr', 'double DEFAULT NULL');

END $$
DELIMITER ;
-- EOC


DROP PROCEDURE IF EXISTS update_lut_fee_areas_offer;
-- EOC
DELIMITER $$
CREATE PROCEDURE update_lut_fee_areas_offer()
update_lut_fee_areas_offerlbl:BEGIN

    DECLARE prof_ids INTEGER;
    DECLARE fees_arr_values TEXT;
    DECLARE version_values varchar(10);
    DECLARE type_values varchar(10);
    DECLARE supplement_values DOUBLE;
    DECLARE specserv_values DOUBLE;
    DECLARE extras_values DOUBLE;
    DECLARE rebate_values DOUBLE;
    DECLARE fees_value DOUBLE DEFAULT NULL;
    DECLARE is_special_index INTEGER DEFAULT NULL;
    DECLARE is_special_value INTEGER DEFAULT NULL;
    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE prof CURSOR FOR SELECT 
        id,
        us_json_extract_arr(portlet_data, 'fees') AS fees_arr_value, 
        us_json_extract(portlet_data, 'version') AS version_value, 
        us_json_extract(portlet_data, 'type') AS type_value, 
        us_json_extract(portlet_data, 'supplement') AS supplement_value, 
        us_json_extract(portlet_data, 'specserv') AS specserv_value, 
        us_json_extract(portlet_data, 'extras') AS extras_value, 
        us_json_extract(portlet_data, 'rebate') AS rebate_value
        FROM project_offers;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    CALL update_lut_fee_areas_offer_add_fields();

    -- Prevent execution, if work_packages_tags has ids>10000000
    IF ( (SELECT SUM(costs_view) FROM lut_fee_areas_offer)>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - update_lut_fee_areas_offer';
        LEAVE update_lut_fee_areas_offerlbl;
    END IF;

    -- Use those as sets to get the HOAI is_special value from Version-Type
    SET @planungsleistungen_keys = '2013-1,2013-2,2013-3,2013-4,2013-5,2013-6,2013-7,2009-1,2009-12,2009-13,2009-14,2009-15,2009-16,2009-17';
    SET @planungsleistungen_values = '-10,-11,-12,-13,-14,-15,-16,-2,-3,-4,-5,-6,-7,-8';

    OPEN prof;
    cursor_loop: LOOP -- -------------------------------------
        FETCH prof INTO prof_ids, fees_arr_values, version_values, type_values, supplement_values, specserv_values, extras_values, rebate_values;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;

        -- Simple update now
        UPDATE lut_fee_areas_offer SET 
            costs_view = costs,
            extras = IFNULL(extras_values,0), 
            rebate = IFNULL(rebate_values,0) 
            WHERE offer_id = prof_ids;

        -- Get HOAI is_special value
        SET is_special_index = FIND_IN_SET(CONCAT(version_values,'-',type_values), @planungsleistungen_keys);
        IF is_special_index = 0 THEN
            SET is_special_value = 0;
        ELSE
            SET is_special_value = CONVERT(us_get_from_set(is_special_index, @planungsleistungen_values), SIGNED);
        END IF;

        -- Update fee area costs_view value
        IF is_special_value IS NOT NULL THEN
            -- For each fee area
            SET @x = 0;
            REPEAT
                SET @x = @x + 1;
                
                SET fees_value = us_get_from_set(@x, fees_arr_values)+0.0;
                IF fees_value IS NOT NULL THEN
                    UPDATE lut_fee_areas_offer AS lof INNER JOIN work_packages AS wps ON lof.work_package_id=wps.id SET 
                        lof.costs_view = fees_value
                        WHERE lof.offer_id = prof_ids
                        AND wps.is_special = is_special_value
                        AND wps.is_special_phase = @x;
                END IF;

            UNTIL @x > 9 END REPEAT;

        END IF;

    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  prof;
END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrates/Inserts the table drawings_inbox into table drawings
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS migrate_drawings_inbox_to_drawings;
-- EOC
DELIMITER $$
CREATE PROCEDURE migrate_drawings_inbox_to_drawings()
migrate_drawings_inbox_to_drawingslbl:BEGIN
    DECLARE drawings_type INT;
    DECLARE plot_file_types VARCHAR(255);
    
    -- If table doesn't exist, migration is not possible
    IF NOT EXISTS( (SELECT * FROM information_schema.TABLES WHERE TABLE_SCHEMA=DATABASE() AND TABLE_NAME='drawings_inbox') ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='No migration necessary - migrate_drawings_inbox_to_drawings';
        LEAVE migrate_drawings_inbox_to_drawingslbl;
    END IF;
    
    -- If old-table already exists, migration must not be executed
    IF EXISTS( (SELECT * FROM information_schema.TABLES WHERE TABLE_SCHEMA=DATABASE() AND TABLE_NAME='drawings_inbox_old') ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='No migration necessary - migrate_drawings_inbox_to_drawings';
        LEAVE migrate_drawings_inbox_to_drawingslbl;
    END IF;

    -- Add new field received_from
    CALL us_add_to_table2('drawings', 'received_from', 'INT( 11 ) DEFAULT NULL AFTER date' );

    -- Insert new drawing type for the migrated drawings
    INSERT INTO planart (ArtText, Bemerkung) VALUES('us_Planeingang', '');
    SET drawings_type = LAST_INSERT_ID();

    -- Get plot file types as SET
    SET plot_file_types = (SELECT SettingValue FROM settingssystem WHERE SettingType LIKE 'plot_file_types');
    IF plot_file_types IS NULL OR trim(plot_file_types)='' THEN
        SET plot_file_types = 'PLT,PDF';
    ELSE
        SET plot_file_types = REPLACE(plot_file_types, '|', ',');
    END IF;

    -- Insert data from the drawings_inbox table into the drawings table
    INSERT INTO
        drawings(
            type_id,
            project_id,
            plan_no,
            plan_content,
            `date`,
            plan_index,
            received_from,
            cad_file,
            plot_file,
            work_area_id,
            created_date,
            created_by_id,
            change_date,
            changed_by_id,
            purpose,
            work_package_id
        )
    SELECT
        drawings_type as type_id,
        inbox.project_id as project_id,
        inbox.plan_no as plan_no,
        inbox.plan_content as plan_content,
        inbox.`date` as `date`,
        inbox.plan_index as plan_index,
        inbox.address_id as received_from,
        (
            CASE WHEN 
                FIND_IN_SET(
                    SUBSTRING_INDEX(inbox.`file`, '.', -1),
                    plot_file_types
                ) <= 0
            THEN 
                inbox.`file` 
            ELSE 
                NULL 
            END
        ) as cad_file,
        (
            CASE WHEN 
                FIND_IN_SET(
                    SUBSTRING_INDEX(inbox.`file`, '.', -1), 
                    plot_file_types
                ) >= 1
            THEN
                inbox.`file` 
            ELSE 
                NULL 
            END
        ) as plot_file,
        inbox.work_area_id as work_area_id,
        inbox.created_date as created_date,
        inbox.created_by_id as created_by_id,
        inbox.change_date as change_date,
        inbox.changed_by_id as changed_by_id,
        inbox.description as purpose,
        inbox.work_package_id as work_package_id
    FROM
        drawings_inbox AS inbox;

    -- Finally rename table
    RENAME TABLE drawings_inbox TO drawings_inbox_old;
END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate Improve correspondence
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_12_13_improve_correspondence;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_12_13_improve_correspondence()
BEGIN
    
    UPDATE formular SET Stufe=3 WHERE Stufe IS NULL;

    UPDATE formart SET Sicherheit=3 WHERE Sicherheit IS NULL;

    ALTER TABLE `formular` CHANGE COLUMN `Stufe` `Stufe` SMALLINT(6) NULL DEFAULT 3;
    -- NOT NULL would be better, but takes ages

    ALTER TABLE `formart` CHANGE COLUMN `Sicherheit` `Sicherheit` SMALLINT(6) NOT NULL DEFAULT 3;

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate time protocol to user option for X3
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_12_13_migrate_timelog_to_user;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_12_13_migrate_timelog_to_user()
BEGIN
    
    -- Shall we start project, derive from timelogging_start_view and then set to old ProTodo
    INSERT INTO settingsuser(SettingType, SettingValue, LfdNr_User)
        SELECT 'timelogging_start_project_id', (SELECT LfdNr FROM projekte WHERE ProTodo=1), LfdNr_User 
        FROM settingsuser 
        WHERE SettingType='timelogging_start_view' AND SettingValue='TRUE';

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate to lut_company_proj
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_12_13_lut_company_proj;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_12_13_lut_company_proj()
BEGIN
    
    DROP VIEW IF EXISTS `lut_company_proj`;

    CREATE TABLE `lut_company_proj` (
      `company_id` int(11) NOT NULL,
      `project_id` int(11) NOT NULL,
      PRIMARY KEY (`company_id`, `project_id`),
      KEY `lut_firm_proj_firmen_id` (`company_id`),
      KEY `lut_firm_proj_projekt_id` (`project_id`)
    ) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;

    INSERT INTO `lut_company_proj`
        SELECT DISTINCT Firmen_ID, Projekt_ID FROM lut_firm_proj;

    RENAME TABLE lut_firm_proj TO lut_firm_proj__bak;

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate to lut_correspondence_addresses
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_12_13_lut_correspondence_addresses;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_12_13_lut_correspondence_addresses()
BEGIN
    
    DROP VIEW IF EXISTS `lut_correspondence_addresses`;

    CREATE TABLE `lut_correspondence_addresses` (
      `id` int(11) NOT NULL AUTO_INCREMENT,
      `correspondence_id` int(11) DEFAULT NULL,
      `address_id` int(11) DEFAULT NULL,
      `address_group_id` int(11) DEFAULT NULL,
      PRIMARY KEY (`id`),
      UNIQUE KEY `verteiler_unique` (`correspondence_id`,`address_id`),
      KEY `verteiler_empfaengernr` (`address_id`),
      KEY `verteiler_dokunr` (`correspondence_id`)
    ) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;

    INSERT INTO `lut_correspondence_addresses`
        SELECT MIN(LfdNr), DokuNr, EmpfaengerNr, MIN(AdrGrpNr) FROM verteiler GROUP BY DokuNr, EmpfaengerNr;

    RENAME TABLE verteiler TO verteiler__bak;

END $$
DELIMITER ;
-- EOC


-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migrate fee_multipliers from system table to fee_multipliers table
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_12_13_fee_multipliers_from_system;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_12_13_fee_multipliers_from_system()
us_migrate_12_13_fee_multipliers_from_systemlbl:BEGIN

    DECLARE fee_mul_x  INT;

    -- Prevent execution, if system fee_multipliers are already in fee_multipliers table
    IF ( (SELECT count(*) FROM fee_multipliers WHERE is_standard <> FALSE OR is_r2 <> FALSE OR is_r3 <> FALSE OR is_r4 <> FALSE)>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_12_13_fee_multipliers_from_system';
        LEAVE us_migrate_12_13_fee_multipliers_from_systemlbl;
    END IF;


    SET fee_mul_x = 1;
    WHILE fee_mul_x  <= 9 DO
        IF ( (SELECT count(*) FROM settingssystem WHERE SettingType=CONCAT('bueroausas_',fee_mul_x) AND SettingValue='TRUE')>0 ) THEN 
            UPDATE fee_multipliers SET is_standard=TRUE, is_r2=TRUE, is_r3=TRUE, is_r4=TRUE WHERE rate_id=fee_mul_x;
        END IF;
        
        SET  fee_mul_x = fee_mul_x + 1; 
    END WHILE;

END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- ADD staff_cost_centre to times/travels/extras
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_12_13_staff_cost_centre_times_travles_extras;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_12_13_staff_cost_centre_times_travles_extras()
us_migrate_12_13_staff_cost_centre_times_travles_extraslbl:BEGIN

    DECLARE staff_ids INT;
    DECLARE staff_cost_centres INT;
    DECLARE l_last_row_fetched INTEGER;

    DECLARE curs CURSOR FOR SELECT LfdNr, PersKostenStelle FROM personen;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;
    SET l_last_row_fetched=0;
    OPEN curs;

        -- ADD staff_cost_centre to times/travels/extras
        CALL us_add_to_table2('times', 'staff_cost_centre', 'INT( 11 ) DEFAULT 0' );
        CALL us_add_to_table2('travels', 'staff_cost_centre', 'INT( 11 ) DEFAULT 0' );
        CALL us_add_to_table2('extras', 'staff_cost_centre', 'INT( 11 ) DEFAULT 0' );

	cursor_loop: LOOP -- -------------------------------------
            FETCH curs INTO staff_ids, staff_cost_centres;
            IF l_last_row_fetched=1 THEN
                LEAVE cursor_loop;
            END IF;

            UPDATE times set staff_cost_centre=staff_cost_centres WHERE staff_id = staff_ids AND staff_cost_centre = 0;
            UPDATE travels set staff_cost_centre=staff_cost_centres WHERE staff_id = staff_ids AND staff_cost_centre = 0;
            UPDATE extras set staff_cost_centre=staff_cost_centres WHERE staff_id = staff_ids AND staff_cost_centre = 0;
	END LOOP cursor_loop; -- -------------------------------------
    CLOSE curs;
	SET l_last_row_fetched=0;	
END $$
DELIMITER ;
-- EOC


-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- ADD values to lut_staff_cost_centre
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_12_13_lut_staff_cost_centre;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_12_13_lut_staff_cost_centre()
us_migrate_12_13_lut_staff_cost_centrelbl:BEGIN

    DECLARE staff_ids INT;
    DECLARE staff_cost_centres INT;
    DECLARE staff_start DateTime;
    DECLARE l_last_row_fetched INTEGER;

    DECLARE curs CURSOR FOR SELECT LfdNr, PersKostenStelle, `Start` FROM personen;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;


    -- Prevent execution, if system fee_multipliers are already in fee_multipliers table
    IF ( (SELECT count(*) FROM lut_staff_cost_centre)>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_12_13_lut_staff_cost_centre';
        LEAVE us_migrate_12_13_lut_staff_cost_centrelbl;
    END IF;


    SET l_last_row_fetched=0;
    OPEN curs;

	cursor_loop: LOOP -- -------------------------------------
            FETCH curs INTO staff_ids, staff_cost_centres, staff_start;
            IF l_last_row_fetched=1 THEN
                LEAVE cursor_loop;
            END IF;

            IF staff_start IS NULL THEN
                SET staff_start = '1990-01-01 00:00:00';
            END IF;

            INSERT INTO lut_staff_cost_centre (staff_id, cost_centre_id, from_date, description) 
                VALUES (staff_ids, staff_cost_centres, staff_start, 'Start');

	END LOOP cursor_loop; -- -------------------------------------
    CLOSE curs;
	SET l_last_row_fetched=0;	
END $$
DELIMITER ;
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- ADD values to lut_staff_gross_hourly_wage
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
DROP PROCEDURE IF EXISTS us_migrate_12_13_lut_staff_gross_hourly_wage;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_12_13_lut_staff_gross_hourly_wage()
us_migrate_12_13_lut_staff_gross_hourly_wagelbl:BEGIN

    DECLARE staff_ids INT;
    DECLARE staff_gross_hourly_wage DOUBLE;
    DECLARE staff_staff_start DateTime;
    DECLARE l_last_row_fetched INTEGER;

    DECLARE curs CURSOR FOR SELECT LfdNr, Bruttolohn, `Start` FROM personen;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;


    -- Prevent execution, if system fee_multipliers are already in fee_multipliers table
    IF ( (SELECT count(*) FROM lut_staff_gross_hourly_wage)>0 ) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info='ERROR No migration necessary - us_migrate_12_13_lut_staff_gross_hourly_wage';
        LEAVE us_migrate_12_13_lut_staff_gross_hourly_wagelbl;
    END IF;

    -- Create the temp table to work from
    CREATE TEMPORARY TABLE IF NOT EXISTS migrate_staff_gross_hourly_wage_changed AS 
        SELECT staff_id, gross_hourly_wage, `date` FROM times GROUP BY staff_id, gross_hourly_wage, `date`;

    SET l_last_row_fetched=0;
    OPEN curs;

	cursor_loop: LOOP -- -------------------------------------
            FETCH curs INTO staff_ids, staff_gross_hourly_wage, staff_staff_start;
            IF l_last_row_fetched=1 THEN
                LEAVE cursor_loop;
            END IF;
 
            IF staff_staff_start IS NULL THEN
                SET staff_staff_start = '1990-01-01';
            END IF;

            -- Set values for staff
            CALL _us_migrate_12_13_lut_staff_gross_hourly_wage_staff(staff_ids, staff_gross_hourly_wage, staff_staff_start);

	END LOOP cursor_loop; -- -------------------------------------
    CLOSE curs;
	SET l_last_row_fetched=0;	
END $$
DELIMITER ;
-- EOC


DROP PROCEDURE IF EXISTS _us_migrate_12_13_lut_staff_gross_hourly_wage_staff;
-- EOC
DELIMITER $$
CREATE PROCEDURE _us_migrate_12_13_lut_staff_gross_hourly_wage_staff(IN xstaff_id INTEGER, IN xstaff_gross_hourly_wage DOUBLE, IN xstaff_staff_start DATE)
BEGIN

    DECLARE time_gross_hourly_wage DOUBLE;
    DECLARE time_date DATE;
    DECLARE ls_last_row_fetched INTEGER;

    DECLARE curs CURSOR FOR SELECT gross_hourly_wage, `date` FROM migrate_staff_gross_hourly_wage_changed WHERE staff_id=xstaff_id ORDER BY `date` DESC;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET ls_last_row_fetched=1;

    SET @last_gross_hourly_wage = xstaff_gross_hourly_wage;
    SET @last_to_date = NULL;
    SET @rows_count = 0;

    SET ls_last_row_fetched=0;
    OPEN curs;

	cursor_loop: LOOP -- -------------------------------------
            FETCH curs INTO time_gross_hourly_wage, time_date;
            IF ls_last_row_fetched=1 THEN
                LEAVE cursor_loop;
            END IF;
            
            -- When the value changes and not still the same date
            IF round(time_gross_hourly_wage, 2) != @last_gross_hourly_wage AND (@last_to_date IS NULL OR @last_to_date != time_date) THEN
                
                INSERT INTO lut_staff_gross_hourly_wage (staff_id, gross_hourly_wage, from_date, to_date) 
                    VALUES (xstaff_id, @last_gross_hourly_wage, DATE_ADD(time_date, INTERVAL 1 DAY), @last_to_date);

                SET @last_gross_hourly_wage = round(time_gross_hourly_wage, 2);
                SET @last_to_date = time_date;
                SET @rows_count = @rows_count + 1;
            END IF;

            -- Max X rows +1
            IF @rows_count>20 THEN
                LEAVE cursor_loop;
            END IF;
            
	END LOOP cursor_loop; -- -------------------------------------
    CLOSE curs;

    IF @rows_count<=0 THEN
        INSERT INTO lut_staff_gross_hourly_wage (staff_id, gross_hourly_wage, from_date, description) 
            VALUES (xstaff_id, xstaff_gross_hourly_wage, xstaff_staff_start, 'Start');
    ELSEIF @rows_count<20 THEN
        IF xstaff_staff_start>=time_date THEN
            SET xstaff_staff_start = time_date;
        END IF;
        IF xstaff_staff_start>=@last_to_date THEN
            SET xstaff_staff_start = DATE_SUB(@last_to_date, INTERVAL 1 DAY);
        END IF;

        INSERT INTO lut_staff_gross_hourly_wage (staff_id, gross_hourly_wage, from_date, to_date, description) 
            VALUES (xstaff_id, @last_gross_hourly_wage, xstaff_staff_start, @last_to_date, 'Start');
    END IF;

    SET ls_last_row_fetched=0;	
END $$
DELIMITER ;
-- EOC
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Migration router
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

CREATE TABLE IF NOT EXISTS `01migrate_11_12_steps` (
  `stepname` varchar(100) NOT NULL,
  `date` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

DROP PROCEDURE IF EXISTS us_migrate_11_12;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_11_12(IN step VARCHAR(100))
us_migrate_11_12lbl:BEGIN
    IF ( SELECT COUNT(*) FROM `01migrate_11_12_steps` WHERE stepname=step) THEN 
        -- INSERT info
        INSERT INTO `01migrate_11_12` SET elem='start', info=CONCAT('No migration necessary - ', step);
        LEAVE us_migrate_11_12lbl;
    END IF;

    -- INSERT info
    INSERT INTO `01migrate_11_12` SET elem='start', info=CONCAT('Starting migration - ', step);
    INSERT INTO `01migrate_11_12_steps` SET stepname=step; 

    IF @debug THEN
        select now() AS 'THE_START_TIME_____', CONCAT('Start ', step) AS 'COMMENT';
    END IF;
    
    CASE step
        WHEN 'prepare_old_views' THEN
            -- CALL Prepare old views
            IF @debug THEN
                select now() AS 'THE_START_TIME_____', 'Start migration <==========' AS 'COMMENT';
            END IF;

            CALL us_migrate_11_12_prepare_old_views();

        WHEN 'improve_todo' THEN
            -- CALL Improve todo
            CALL update_improve_todo();

        WHEN 'migrate_wa2wpt' THEN
            -- CALL INSERT work area to work packages tags
            CALL us_migrate_11_12_migrate_wa2wpt();

        WHEN 'tables' THEN
            -- CALL Migrate views to tables
            CALL us_migrate_11_12_migrate_views_to_tables();

        WHEN 'repair_table_structure' THEN
            -- CALL Migrate views to tables
            CALL us_migrate_11_12_repair_table_structure();

        WHEN 'work_areas_categories' THEN
            -- CALL Create presets for all work_areas_categories
            CALL us_migrate_11_12_work_areas_categories();

        WHEN 'projects' THEN
            -- CALL Create work_packages for project and change all work areas to work packages
            CALL us_migrate_11_12_projects();

        WHEN 'projects_history_prepare' THEN
            -- CALL Create prepare work package history for projects
            CALL us_migrate_11_12_projects_history_prepare();

        WHEN 'projects_history' THEN
            -- CALL Create work package history for projects
            CALL us_migrate_11_12_projects_history();

        WHEN 'areas_to_packages' THEN
            -- CALL Migrate work area values to work packages of the data tables
            CALL us_migrate_11_12_areas_to_packages();

        WHEN 'migrate_fee_area_tags' THEN
            -- CALL Migrate fee area tags to work package tags
            CALL us_migrate_fee_area_tags();

        WHEN 'migrate_translation' THEN
            -- CALL Migrate work package translation
            CALL us_migrate_translation();

        WHEN 'update_wp_status' THEN
            -- CALL Update work packages status
            CALL update_wp_status();

        WHEN 'update_wp_add_one_day' THEN
            -- CALL Update work packages dates
            CALL update_wp_add_one_day();

        WHEN 'update_cost_centres' THEN
            -- CALL Update work packages cost centres for 12.6
            CALL update_cost_centres();

        WHEN 'move_fee_work_package' THEN
            -- CALL Update move fee work package for 12.6
            CALL move_fee_work_package();

        WHEN 'update_alloc_with_payments_fees' THEN
            -- CALL Update move fee work package from the payments table for 12.6
            CALL update_alloc_with_payments_fees();

        WHEN 'update_lut_fee_areas_offer' THEN
            -- CALL Update offers to new format with more features for X3
            CALL update_lut_fee_areas_offer();

        WHEN 'migrate_drawings_inbox_to_drawings' THEN
            -- CALL Migrate drawings inbox to drawings X3
            CALL migrate_drawings_inbox_to_drawings();

        WHEN 'increase_rightblockwidth' THEN
            -- Increase rightblockwidth by 4 for X3
            UPDATE settingsuser SET SettingValue=SettingValue+4 WHERE SettingType='webs_rightblockwidth';

        WHEN 'set_new_default_font' THEN
            -- Set new default font
            UPDATE settingsuser SET SettingValue='"Open Sans", sans-serif' WHERE SettingType='webs_font' AND SettingValue='Arial, Helvetica, sans-serif';

        WHEN 'table_lut_company_proj' THEN
            -- CALL Migrate to lut_company_proj
            CALL us_migrate_12_13_lut_company_proj();

        WHEN 'table_lut_correspondence_addresses' THEN
            -- CALL Migrate to lut_correspondence_addresses
            CALL us_migrate_12_13_lut_correspondence_addresses();

        WHEN 'improve_correspondence' THEN
            -- CALL Improve correspondence
            CALL us_migrate_12_13_improve_correspondence();

        WHEN 'migrate_timelog_to_user' THEN
            -- CALL Migrate time protocol to user option for X3
            CALL us_migrate_12_13_migrate_timelog_to_user();

        WHEN 'migrate_fee_multipliers_from_system' THEN
            -- CALL Migrate fee_multipliers from system table to fee_multipliers table for X3
            CALL us_migrate_12_13_fee_multipliers_from_system();

        WHEN 'update_cost_centres_cleanup' THEN
            -- CALL Update work packages cost centres cleanup for 12.6
            CALL update_cost_centres_cleanup();

        WHEN 'add_staff_coste_centre_times_travles_extras' THEN
            -- CALL ADD staff_cost_centre for X3
            CALL us_migrate_12_13_staff_cost_centre_times_travles_extras();

        WHEN 'update_lut_staff_cost_centre' THEN
            -- CALL Update lut_staff_cost_centre for X3
            CALL us_migrate_12_13_lut_staff_cost_centre();

        WHEN 'update_lut_staff_gross_hourly_wage' THEN
            -- CALL Update lut_staff_gross_hourly_wage for X3
            CALL us_migrate_12_13_lut_staff_gross_hourly_wage();

        WHEN 'cleanup' THEN
            -- CALL Cleanup, without any structure changes
            CALL us_migrate_11_12_cleanup();

            IF @debug THEN
                select now() AS 'THE_START_TIME_____', 'End migration <==========' AS 'COMMENT';
            END IF;

    END CASE;

    IF @debug THEN
        select now() AS 'THE_START_TIME_____', CONCAT('End ', step) AS 'COMMENT';
    END IF;

END $$
DELIMITER ;
-- EOC




-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Call migration
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------

-- USC X2 Update - Erstelle Log-Tabelle.
CALL us_create_01migrate_11_12();
-- EOC

-- USC X2 Update - Erstelle Views.
CALL us_migrate_11_12('prepare_old_views');
-- EOC

-- USC X1 Improve todo
CALL us_migrate_11_12('improve_todo');
-- EOC

-- USC X2 Update - Migrate work areas to work package tags.
CALL us_migrate_11_12('migrate_wa2wpt');
-- EOC

-- USC X2 Update - Migrate views to tables.
CALL us_migrate_11_12('tables');
-- EOC

-- USC X2 Update - Repair table structure for tables created from views.
CALL us_migrate_11_12('repair_table_structure');
-- EOC

-- USC X2 Update - Create presets for all work_areas_categories.
CALL us_migrate_11_12('work_areas_categories');
-- EOC

-- USC X2 Update - Create work packages for projects and change all work areas to work packages.
CALL us_migrate_11_12('projects');
-- EOC

-- USC X2 Update - Create work package history for projects - prepare
CALL us_migrate_11_12('projects_history_prepare');
-- EOC

-- USC X2 Update - Create work package history for projects
CALL us_migrate_11_12('projects_history');
-- EOC

-- USC X2 Update - Migrate work area values to work packages for all data tables.
CALL us_migrate_11_12('areas_to_packages');
-- EOC

-- USC X2 Update - Migrate fee area tags to work package tags
CALL us_migrate_11_12('migrate_fee_area_tags');
-- EOC

-- USC X2 Update - Migrate work package translation.
CALL us_migrate_11_12('migrate_translation');
-- EOC

-- USC X2 Update - Update work packages status
CALL us_migrate_11_12('update_wp_status');
-- EOC

-- USC X2 Update - Update work packages dates
CALL us_migrate_11_12('update_wp_add_one_day');
-- EOC

-- USC X2 Update - Update work packages cost centres for 12.6
CALL us_migrate_11_12('update_cost_centres');
-- EOC

-- USC X2 Update - Update work packages cost centres for 12.6
CALL us_migrate_11_12('move_fee_work_package');
-- EOC

-- USC X2 Update - Update work packages cost centres for 12.6
CALL us_migrate_11_12('update_alloc_with_payments_fees');
-- EOC

-- USC X3 Update - Update offers to new format
CALL us_migrate_11_12('update_lut_fee_areas_offer');
-- EOC

-- USC X3 Update - Move drawings inbox to drawings
CALL us_migrate_11_12('migrate_drawings_inbox_to_drawings');
-- EOC

-- USC X3 Increase rightblockwidth by 4
CALL us_migrate_11_12('increase_rightblockwidth');
-- EOC

-- USC X3 Set new default font
CALL us_migrate_11_12('set_new_default_font');
-- EOC

-- USC X3 Migrate to lut_company_proj
CALL us_migrate_11_12('table_lut_company_proj');
-- EOC

-- USC X3 Migrate to lut_correspondence_addresses
CALL us_migrate_11_12('table_lut_correspondence_addresses');
-- EOC

-- USC X3 Improve correspondence
CALL us_migrate_11_12('improve_correspondence');
-- EOC

-- USC X3 Migrate time protocol to user option
CALL us_migrate_11_12('migrate_timelog_to_user');
-- EOC

-- USC X3 Migrate fee_multipliers from system table to fee_multipliers table
CALL us_migrate_11_12('migrate_fee_multipliers_from_system');
-- EOC

-- USC X2 Remove old Kostenstelle field
CALL us_migrate_11_12('update_cost_centres_cleanup');
-- EOC

-- USC X2 Add staff_cost_centre for X3
CALL us_migrate_11_12('add_staff_coste_centre_times_travles_extras');
-- EOC

-- USC X3 Update lut_staff_cost_centre
CALL us_migrate_11_12('update_lut_staff_cost_centre');
-- EOC

-- USC X3 Update lut_staff_gross_hourly_wage
CALL us_migrate_11_12('update_lut_staff_gross_hourly_wage');
-- EOC

-- USC X2 Update - Cleanup.
CALL us_migrate_11_12('cleanup');
-- EOC
-- USC Entferne alte Views.

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Remove old views

DROP VIEW IF EXISTS `fee_areas`;
-- EOC
DROP VIEW IF EXISTS `work_areas`;
-- EOC
DROP VIEW IF EXISTS `lut_work_areas_work_areas_categories`;
-- EOC
DROP VIEW IF EXISTS `lut_fee_areas_proj`;
-- EOC
DROP VIEW IF EXISTS `lut_fee_areas_proj_history`;
-- EOC
DROP VIEW IF EXISTS `budget_plan`;
-- EOC
DROP VIEW IF EXISTS `budget_plan_history`;
-- EOC
DROP VIEW IF EXISTS `completion_level`;
-- EOC
DROP VIEW IF EXISTS `completion_level_history`;
-- EOC

-- Java clients - OLD
DROP VIEW IF EXISTS `staff_hashes`;
-- EOC

DROP VIEW IF EXISTS `report_times_from_to`;
-- EOC

DROP VIEW IF EXISTS `cost_centres_report_info`;
-- EOC

-- Old views
DROP VIEW IF EXISTS `budget_plan_extended_data`;
-- EOC
DROP VIEW IF EXISTS `report_budget_plan_history`;
-- EOC
DROP VIEW IF EXISTS `report_completion_level_fee`;
-- EOC
DROP VIEW IF EXISTS `report_fee_areas_project_allocation`;
-- EOC
DROP VIEW IF EXISTS `report_fee_areas_project_allocation_history`;
-- EOC
DROP VIEW IF EXISTS `report_projects_fee_areas`;
-- EOC

DROP VIEW IF EXISTS `completion_level_extended_data`;
-- EOC
DROP VIEW IF EXISTS `report_completion_level`;
-- EOC
DROP VIEW IF EXISTS `report_completion_level_bbl1`;
-- EOC
DROP VIEW IF EXISTS `report_completion_level_bbl2`;
-- EOC
DROP VIEW IF EXISTS `report_completion_level_current`;
-- EOC
DROP VIEW IF EXISTS `report_completion_level_fee`;
-- EOC
DROP VIEW IF EXISTS `report_completion_level_only`;
-- EOC

DROP VIEW IF EXISTS `report_cost_centres_info`;
-- EOC

DROP VIEW IF EXISTS `report_externalexpenses`;
-- EOC
DROP VIEW IF EXISTS `report_externalexpenses_planned`;
-- EOC

DROP VIEW IF EXISTS `report_extras`;
-- EOC
DROP VIEW IF EXISTS `report_times`;
-- EOC
DROP VIEW IF EXISTS `report_travels`;
-- EOC

-- Very old views
DROP VIEW IF EXISTS `report_completion_level_history`;
-- EOC
-- USC Erstelle weitere X2 Tabellen.

-- --------------------------------------------------------------------------------------------------
-- Add session information table
DROP PROCEDURE IF EXISTS ensure_u10_ci_sessions;
-- EOC
DELIMITER $$
CREATE PROCEDURE ensure_u10_ci_sessions()
BEGIN
    DROP TABLE IF EXISTS `u10_ci_sessions`;

    CREATE TABLE `u10_ci_sessions` (
      `uniqid` varchar(40) COLLATE utf8_general_ci NULL DEFAULT NULL,
      `session_id` varchar(40) COLLATE utf8_general_ci NOT NULL DEFAULT '0',
      `old1_session_id` varchar(40) COLLATE utf8_general_ci NULL DEFAULT NULL,
      `old2_session_id` varchar(40) COLLATE utf8_general_ci NULL DEFAULT NULL,
      `ip_address` varchar(16) COLLATE utf8_general_ci NOT NULL DEFAULT '0',
      `user_agent` varchar(150) COLLATE utf8_general_ci NOT NULL,
      `first_activity` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
      `last_activity` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
      `login_time` DATETIME NULL DEFAULT NULL,
      `login_type` BOOLEAN DEFAULT 0, -- Was webversion
      `login_staff_id` int(11) DEFAULT NULL, -- Was UserNummer
      `user_data` VARBINARY(20000) NULL,
      PRIMARY KEY (`session_id`)
    ) ENGINE=MEMORY DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;

    -- FLASHDATA
    DROP TABLE IF EXISTS `u10_ci_session_flashdata`;

    CREATE TABLE `u10_ci_session_flashdata` (
      `uniqid` VARCHAR(40) NOT NULL,
      `name` VARCHAR(55) NOT NULL,
      `value` LONGBLOB NULL,
      PRIMARY KEY (`name`, `uniqid`)
    ) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;

END $$
DELIMITER ;
-- EOC
CALL ensure_u10_ci_sessions();
-- EOC

-- Clean flashdata table
DROP PROCEDURE IF EXISTS clean_u10_ci_session_flashdata;
-- EOC
DELIMITER $$
CREATE PROCEDURE clean_u10_ci_session_flashdata()
BEGIN
    DELETE b FROM u10_ci_session_flashdata b LEFT JOIN u10_ci_sessions ses 
        ON ses.uniqid = b.uniqid 
        WHERE ses.uniqid IS NULL;

END $$
DELIMITER ;
-- EOC

-- To update tables - counter
CREATE TABLE IF NOT EXISTS `update_counters` (
    `type` int(11) NOT NULL,
    `count` bigint(20) NOT NULL,
    UNIQUE KEY `type` (`type`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

-- Insert first
INSERT IGNORE INTO `update_counters` (`type`, `count`) VALUES (1, 1);
-- EOC

-- To save config.php values
CREATE TABLE IF NOT EXISTS `u10_ci_config` (
    `id` int(11) NOT NULL AUTO_INCREMENT ,
    `name` varchar(255) NOT NULL ,
    `value` TEXT NOT NULL ,
    PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Create mutex table (Keep as MyISAM to prevent problems with transactions)
DROP TABLE IF EXISTS `u10_mutex`;
-- EOC
CREATE TABLE IF NOT EXISTS `u10_mutex`(
    i int(1) NOT NULL PRIMARY KEY
) ENGINE=MyISAM CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

INSERT INTO u10_mutex(i) VALUES (0), (1);
-- EOC

-- Create calendar table
DROP TABLE IF EXISTS `u10_calendar`;
-- EOC
CREATE TABLE IF NOT EXISTS `u10_calendar`(
    date DATE,
    PRIMARY KEY ( `date` ),
    datemonth varchar(10) NOT NULL
) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC
CALL us_add_to_calendar(SUBDATE(now(), INTERVAL 1 MONTH), ADDDATE(now(), INTERVAL 1 MONTH));
-- EOC

-- Add modifications table
CREATE TABLE IF NOT EXISTS `modifications` (
  `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY,
  `date` datetime DEFAULT NULL,
  `staff` varchar(255) DEFAULT NULL,
  `staff_id` int(11) DEFAULT NULL,
  `title` longtext,
  `info` LONGBLOB DEFAULT NULL,
  `isCompressed` BOOLEAN DEFAULT 0,
  `item_model` varchar(100) DEFAULT NULL,
  `item_title` text,
  `item_id` int(11) DEFAULT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC
ALTER TABLE `modifications` ENGINE = InnoDB ;
-- EOC
CALL us_add_to_table2('modifications', 'staff_id', 'int(11) DEFAULT NULL AFTER `staff`');
-- EOC
CALL change_column('modifications', 'item_model', 'varchar(100) DEFAULT NULL');
-- EOC
CALL change_column('modifications', 'info', 'LONGBLOB DEFAULT NULL');
-- EOC
CALL us_add_to_table2('modifications', 'isCompressed', 'BOOLEAN DEFAULT 0' );
-- EOC

-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- -------------------------------------  -- ------------------------------------- -- ------------------------------------- -- -------------------------------------
-- Add ebInterface columns to fees
CALL us_add_to_table2('fees', 'ebinterface_json', 'LONGTEXT' );
-- EOC
CALL us_add_to_table2('fees_planned', 'ebinterface_json', 'LONGTEXT' );
-- EOC
CALL us_add_to_table2('fees', 'zugferd_json', 'LONGTEXT' );
-- EOC
CALL us_add_to_table2('fees_planned', 'zugferd_json', 'LONGTEXT' );
-- EOC
CALL us_add_to_table2('fees', 'xrechnung_json', 'LONGTEXT' );
-- EOC
CALL us_add_to_table2('fees_planned', 'xrechnung_json', 'LONGTEXT' );
-- EOC
CALL us_add_to_table2('fees', 'fatturapa_json', 'LONGTEXT' );
-- EOC
CALL us_add_to_table2('fees_planned', 'fatturapa_json', 'LONGTEXT' );
-- EOC
CALL us_add_to_table2('fees', 'e_invoice_used', 'int(11) DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('fees_planned', 'e_invoice_used', 'int(11) DEFAULT NULL' );
-- EOC

-- Increase character limit of existing ebInterface columns
CALL check_rename_column_exists('fees', 'ebinterface_json', 'ebinterface_json', 'LONGTEXT');
-- EOC
CALL check_rename_column_exists('fees_planned', 'ebinterface_json', 'ebinterface_json', 'LONGTEXT');
-- EOC
CALL check_rename_column_exists('fees', 'zugferd_json', 'zugferd_json', 'LONGTEXT');
-- EOC
CALL check_rename_column_exists('fees_planned', 'zugferd_json', 'zugferd_json', 'LONGTEXT');
-- EOC
CALL check_rename_column_exists('fees', 'xrechnung_json', 'xrechnung_json', 'LONGTEXT');
-- EOC
CALL check_rename_column_exists('fees_planned', 'xrechnung_json', 'xrechnung_json', 'LONGTEXT');
-- EOC
CALL check_rename_column_exists('fees', 'fatturapa_json', 'fatturapa_json', 'LONGTEXT');
-- EOC
CALL check_rename_column_exists('fees_planned', 'fatturapa_json', 'fatturapa_json', 'LONGTEXT');
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Improve erased
CALL us_add_to_table2('erased', 'staff_id', 'int(11) DEFAULT NULL AFTER `Wer`');
-- EOC
CALL us_add_to_table2('erased', 'item_model', 'varchar(100) COLLATE utf8_general_ci DEFAULT NULL AFTER `Was`' );
-- EOC
CALL change_column('erased', 'item_model', 'varchar(100) DEFAULT NULL');
-- EOC
CALL us_add_to_table2('erased', 'item_title', 'text DEFAULT NULL AFTER `item_model`' );
-- EOC
CALL change_column('erased', 'item_title', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC
CALL change_column('erased', 'Was', 'LONGBLOB DEFAULT NULL');
-- EOC
CALL us_add_to_table2('erased', 'item_id', 'INT( 11 ) NULL AFTER `item_title`' );
-- EOC
CALL us_add_to_table2('erased', 'isCompressed', 'BOOLEAN DEFAULT 0' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add web settings and user
CALL us_add_to_table2('zusatz_personen', 'web', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL' );
-- EOC

CALL us_add_to_table2('personen', 'username', 'varchar(80) DEFAULT NULL' );
-- EOC

CALL us_add_to_table2('personen', 'staff_location_id', 'INT( 11 ) NOT NULL DEFAULT 1' );
-- EOC

CALL change_column('personen', 'passwort', 'VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC

CALL us_add_to_table2('personen', 'passwort_mode', 'INT( 11 ) DEFAULT NULL AFTER `passwort`' );
-- EOC

CALL us_add_to_table2('personen', 'secdetail_2', 'SMALLINT(6) DEFAULT 0 AFTER ModSecurity02' );
-- EOC

CALL us_add_to_table2('personen', 'secdetail_3', 'SMALLINT(6) DEFAULT 0 AFTER ModSecurity03' );
-- EOC

CALL us_add_to_table2('personen', 'secdetail_5', 'SMALLINT(6) DEFAULT 0 AFTER ModSecurity05' );
-- EOC

CALL us_add_to_table2('personen', 'secdetail_16', 'SMALLINT(6) DEFAULT 0 AFTER ModSecurity16' );
-- EOC

CALL us_add_to_table2('personen', 'days_warning_no_workinput', 'INT(11) NOT NULL DEFAULT 7 AFTER EditHowManyDays' );
-- EOC
CALL us_add_to_table2('personen', 'days_warning_no_mail_allocation', 'INT(11) NOT NULL DEFAULT 7 AFTER days_warning_no_workinput' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add company_id to tables
CALL us_add_to_table2('kostenstellen', 'company_id', 'INT( 11 ) NULL AFTER `Bemerkung`' );
-- EOC
CALL us_add_to_table2('kopiekat', 'company_id', 'INT( 11 ) NULL AFTER `KostenEinheit`' );
-- EOC
CALL us_add_to_table2('gruppen', 'company_id', 'INT( 11 ) NULL FIRST' );
-- EOC
CALL us_add_to_table2('uestd', 'company_id', 'INT( 11 ) NULL AFTER `WerHatAusb`' );
-- EOC
CALL us_add_to_table2('uesatz', 'company_id', 'INT( 11 ) NULL AFTER `Drucken`' );
-- EOC
CALL us_add_to_table2('orte', 'company_id', 'INT( 11 ) NULL AFTER `Km`' );
-- EOC
CALL us_add_to_table2('zusatz_personen', 'company_id', 'INT( 11 ) NULL FIRST' );
-- EOC
CALL us_add_to_table2('zeiten_beginnende', 'company_id', 'INT( 11 ) NULL AFTER `ZeitAbzug`' );
-- EOC
CALL us_add_to_table2('settingssystem', 'company_id', 'INT( 11 ) NULL AFTER `SettingValue`' );
-- EOC
CALL us_add_to_table2('arbbeschtxte', 'company_id', 'INT( 11 ) NULL AFTER `OrdnNr`' );
-- EOC
CALL us_add_to_table2('reisekosten', 'company_id', 'INT( 11 ) NULL AFTER `ZeitVonBis`' );
-- EOC
CALL us_add_to_table2('projektabrsaetze', 'company_id', 'INT( 11 ) NULL AFTER `ABSatz`' );
-- EOC
CALL us_add_to_table2('adrkategorien', 'company_id', 'INT( 11 ) NULL AFTER `EditDatum`' );
-- EOC
CALL us_add_to_table2('adrgruppendef', 'company_id', 'INT( 11 ) NULL AFTER `LfdNr_Projekt`' );
-- EOC
CALL us_add_to_table2('planart', 'company_id', 'INT( 11 ) NULL AFTER `LfdNr`' );
-- EOC
CALL us_add_to_table2('plangroessen', 'company_id', 'INT( 11 ) NULL AFTER `LfdNr`' );
-- EOC
CALL us_add_to_table2('postart', 'company_id', 'INT( 11 ) NULL AFTER `LfdNr`' );
-- EOC

-- ADD flag to the cost centre table
CALL us_add_to_table2('kostenstellen', 'use_in_reports', 'tinyint(1) unsigned NOT NULL DEFAULT 0' );
-- EOC

-- Installation information for every company
CREATE TABLE IF NOT EXISTS `company_installation` (
        `id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
        `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
        `address` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
        `telephone` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
        `company_id` INT( 11 ) NOT NULL ,
	PRIMARY KEY ( `id` ) ,
	UNIQUE KEY ( `company_id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC


-- --------------------------------------------------------------------------------------------------
-- Prepare drawings for read from dir

CREATE TABLE IF NOT EXISTS `drawings_autopath_log` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
	`project_id` INT( 11 ) NOT NULL ,
	`hash` VARCHAR( 100 ) NOT NULL ,
	`change_date` DATETIME NOT NULL ,
	INDEX ( `project_id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CREATE TABLE IF NOT EXISTS `drawings_deleted_log` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
	`project_id` INT( 11 ) NOT NULL ,
	`filename` LONGTEXT NOT NULL ,
	`deleted_date` DATETIME NOT NULL ,
	INDEX `project_filename` ( `project_id` , `filename` ( 255 ) )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Change filename from varchar to longtext
CALL us_drop_index_if_exists('drawings_deleted_log', 'project_id');
-- EOC
CALL change_column('drawings_deleted_log', 'filename', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL');
-- EOC


-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add workinput, HonZus to certain tables

CALL us_add_to_table2('formular', 'Arbeitsbereich', 'INT( 11 ) DEFAULT NULL' );
-- EOC

CALL us_add_to_table2('formular', 'HonZus', 'longtext' );
-- EOC

CALL us_add_to_table2('formular', 'is_valid', 'tinyint(1) unsigned NOT NULL DEFAULT 1 AFTER `HonZus`' );
-- EOC

CALL us_add_to_table2('formular', 'timestamp_file', 'datetime DEFAULT NULL COMMENT \'Timestamp of modification of file\'' );
-- EOC

CALL us_add_to_table2('formular', 'last_autofulltext', 'datetime DEFAULT NULL COMMENT \'Last check for file changes\'' );
-- EOC
CALL us_add_to_table2('formular', 'last_autofulltext_staff_id', 'INT( 11 ) DEFAULT NULL COMMENT \'Last check for file changes started by\'' );
-- EOC

CALL us_add_to_table2('formular', 'last_fulltext_update', 'datetime DEFAULT NULL COMMENT \'Last actual update of fulltext\'' );
-- EOC

CALL us_add_to_table2('formular', 'discount', 'double DEFAULT 0' );
-- EOC
CALL us_add_to_table2('formular', 'discount_date', 'datetime DEFAULT NULL' );
-- EOC

CALL us_add_to_table2('formular', 'fee_account_id', 'INT( 11 ) DEFAULT NULL' );
-- EOC

CALL us_add_to_table2('formart', 'special_type', 'VARCHAR(30) DEFAULT \'\' COMMENT \'Special Type\'' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_cost_centres;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_cost_centres` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	`cost_centre_id` INT( 11 ) NOT NULL ,
	`name` VARCHAR( 60 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	`remark` varchar(100) NULL,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_cost_centres', 'langs_cost_centres_unique_1', 'UNIQUE', '`cost_centre_id` , `language`' );
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_fee_areas;
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_countries;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_countries` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	`country_name_id` VARCHAR( 50 ) NOT NULL ,
	`name` VARCHAR( 50 ) NOT NULL ,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_countries', 'langs_countries_unique_1', 'UNIQUE', '`country_name_id` , `language`' );
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_extras_types;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_extras_types` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	`extras_type_id` INT( 11 ) NOT NULL ,
	`type` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	`unit` VARCHAR( 10 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_extras_types', 'langs_extras_types_unique_1', 'UNIQUE', '`extras_type_id` , `language`' );
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_occupation_groups;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_occupation_groups` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	`occupation_groups_id` INT( 11 ) NOT NULL ,
	`name` VARCHAR( 30 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	`description` LONGTEXT NULL,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_occupation_groups', 'langs_occupation_groups_unique_1', 'UNIQUE', '`occupation_groups_id` , `language`' );
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_travels_types;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_travels_types` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
	`travels_type_id` INT( 11 ) NOT NULL ,
	`type` VARCHAR( 60 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	`unit` VARCHAR( 15 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_travels_types', 'langs_travels_types_unique_1', 'UNIQUE', '`travels_type_id` , `language`' );
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_work_areas;
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_overtime_rates;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_overtime_rates` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
    `rate_id` INT( 11 ) NOT NULL ,
    `name` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_overtime_rates', 'langs_overtime_rates_unique_1', 'UNIQUE', '`rate_id` , `language`' );
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_task_descriptions;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_task_descriptions` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
    `descr_name_id` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `name` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_task_descriptions', 'langs_task_descriptions_unique_1', 'UNIQUE', '`descr_name_id` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_fee_multipliers` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `fee_multiplier_id` int(11) NOT NULL,
    `name` VARCHAR( 30 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `purpose` LONGTEXT NULL,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_fee_multipliers', 'langs_fee_multipliers_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_address_categories;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_address_categories` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `address_category_id` int(11) NOT NULL,
    `name` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `purpose` LONGTEXT NULL,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_address_categories', 'langs_address_categories_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_address_groups` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `address_groups_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_address_groups', 'lang_address_groups_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_addresses_addi_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_addresses_addi_def', 'lang_addresses_addi_def_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_opentasks_addi_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_opentasks_addi_def', 'lang_opentasks_addi_def_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC
CALL us_add_to_table2('todo', 'work_package_id', 'INT( 11 ) NULL' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_externalexpenses_addi_merged_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_externalexpenses_addi_merged_def', 'lang_externalexpenses_addi_merged_def_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_fees_addi_merged_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_fees_addi_merged_def', 'lang_fees_addi_merged_def_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_project_phases;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_project_phases` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	`project_phase_id` VARCHAR( 50 ) NOT NULL ,
	`name` VARCHAR( 50 ) NOT NULL ,
        `purpose` varchar(100) NULL,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC


CALL us_add_constraint_to_table('langs_project_phases', 'langs_project_phases_unique_1', 'UNIQUE', '`project_phase_id` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_projects_addi_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_projects_addi_def', 'lang_projects_addi_def_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- Add translation tables

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_project_categories;
-- EOC

CREATE TABLE IF NOT EXISTS `langs_project_categories` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	`project_categories_id` int(11) NOT NULL ,
	`name` VARCHAR( 50 ) NOT NULL ,
	`purpose` LONGTEXT NULL,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_project_categories', 'langs_project_categories_unique_1', 'UNIQUE', '`project_categories_id` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_staff_additional_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_staff_additional_def', 'lang_staff_additional_def_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
DROP TABLE IF EXISTS lang_measurements;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_measurements` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT,
	`measurements_id` int(11) NOT NULL,
	`name` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_measurements', 'langs_measurements_unique_1', 'UNIQUE', '`measurements_id` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_project_participants_addi_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_project_participants_addi_def', 'lang_project_participants_addi_def_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_whatiswhere_addi_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC


-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_drawings_addi_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_drawing_types` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	`type_id` VARCHAR( 50 ) NOT NULL ,
	`name` VARCHAR( 50 ) NOT NULL ,
        `purpose` varchar(100) NULL,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC


CALL us_add_constraint_to_table('langs_drawing_types', 'langs_drawing_types_unique_1', 'UNIQUE', '`type_id` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_dispatch_modes` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	`mode_id` VARCHAR( 50 ) NOT NULL ,
	`name` VARCHAR( 50 ) NOT NULL ,
        `purpose` varchar(100) NULL,
	`language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC


CALL us_add_constraint_to_table('langs_dispatch_modes', 'langs_dispatch_modes_unique_1', 'UNIQUE', '`mode_id` , `language`' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add Zip to name table
CREATE TABLE IF NOT EXISTS `zip_to_name` (
	`tzip` varchar(50) NOT NULL,
	`tname` varchar(255) NOT NULL,
	`tcountry` varchar(50) NOT NULL,
        PRIMARY KEY ( 
	`tzip` ( 50 ),
	`tname` ( 200 ) ,
	`tcountry` ( 50 )
        )
) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC
ALTER TABLE `zip_to_name` ENGINE = InnoDB ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add meetingmanager tables
CREATE TABLE IF NOT EXISTS `meetingmanager_subjects` (
        `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY,
        `number` INTEGER,
        `name` VARCHAR( 60 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
        `project_id` INTEGER
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('meetingmanager_subjects', 'created_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('meetingmanager_subjects', 'changed_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('meetingmanager_subjects', 'created_date', 'DATETIME NULL' );
-- EOC
CALL us_add_to_table2('meetingmanager_subjects', 'change_date', 'DATETIME NULL' );
-- EOC

CREATE TABLE IF NOT EXISTS `meetingmanager_appointments` (
        `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY,
        `number` INTEGER,
        `name` VARCHAR( 60 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
        `description` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci,
        `date` DATE NOT NULL,
        `time` TIME NOT NULL,
        `location` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
        `is_published` BOOLEAN NOT NULL,
        `is_transfered` BOOLEAN NOT NULL,
        `subject_id` INTEGER
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('meetingmanager_appointments', 'created_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('meetingmanager_appointments', 'changed_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('meetingmanager_appointments', 'created_date', 'DATETIME NULL' );
-- EOC
CALL us_add_to_table2('meetingmanager_appointments', 'change_date', 'DATETIME NULL' );
-- EOC

CREATE TABLE IF NOT EXISTS `meetingmanager_tasks` (
        `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY,
        `number` INTEGER,
        `name` VARCHAR( 60 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
        `description` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci,
        `due_date` DATE
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('meetingmanager_tasks', 'created_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('meetingmanager_tasks', 'changed_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('meetingmanager_tasks', 'created_date', 'DATETIME NULL' );
-- EOC
CALL us_add_to_table2('meetingmanager_tasks', 'change_date', 'DATETIME NULL' );
-- EOC

CREATE TABLE IF NOT EXISTS `lut_meetingmanager_tasks_status` (
        `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY,
        `task_id` INTEGER NOT NULL,
        `status` INTEGER NOT NULL,
        `description` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('lut_meetingmanager_tasks_status', 'created_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('lut_meetingmanager_tasks_status', 'created_date', 'DATETIME NULL' );
-- EOC

CREATE TABLE IF NOT EXISTS `lut_meetingmanager_tasks_appointments` (
        `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY,
        `task_id` INTEGER NOT NULL,
        `appointment_id` INTEGER NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('lut_meetingmanager_tasks_appointments', 'created_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('lut_meetingmanager_tasks_appointments', 'created_date', 'DATETIME NULL' );
-- EOC

CREATE TABLE IF NOT EXISTS `lut_meetingmanager_appointments_addresses` (
        `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY,
        `appointment_id` INTEGER NOT NULL,
        `address_id` INTEGER NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('lut_meetingmanager_appointments_addresses', 'created_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('lut_meetingmanager_appointments_addresses', 'created_date', 'DATETIME NULL' );
-- EOC

CREATE TABLE IF NOT EXISTS `lut_meetingmanager_tasks_addresses` (
        `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY,
        `task_id` INTEGER NOT NULL,
        `address_id` INTEGER NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('lut_meetingmanager_tasks_addresses', 'created_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('lut_meetingmanager_tasks_addresses', 'created_date', 'DATETIME NULL' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add address to companies
CALL us_add_to_table2('firmen', 'address_id', 'INT( 11 ) NULL AFTER `name`' );
-- EOC

-- Add country code
CALL us_add_to_table2('land', 'Kennzeichen', 'VARCHAR(2) AFTER `Bez`');
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add currencies table
CREATE TABLE IF NOT EXISTS `currencies` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
	`name` VARCHAR( 60 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
	`abbreviation` VARCHAR( 10 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL,
	UNIQUE KEY ( `abbreviation` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Add currencies_translation table
CREATE TABLE IF NOT EXISTS `currencies_translation` (
	`currency_id_from` INT( 11 ) NOT NULL ,
	`currency_id_to` INT( 11 ) NOT NULL ,
	`exchange_rate` DOUBLE NOT NULL,
        `online` TEXT DEFAULT NULL,
        PRIMARY KEY (`currency_id_from`,`currency_id_to`)
) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('currencies_translation', 'online', 'TEXT DEFAULT NULL' );
-- EOC

-- Add currencies support
CALL us_add_to_table2('firmen', 'currency_id', 'INT( 11 ) NULL AFTER `aktiv`' );
-- EOC



-- Add currency code
CALL us_add_to_table2('currencies', 'currency_code', 'VARCHAR(3) AFTER `abbreviation`');
-- EOC

-- Add currencies support
CALL us_add_to_table2('uestd', 'company_price_per_unit', 'DOUBLE NULL AFTER `BruttoSatz`' );
-- EOC
CALL us_add_to_table2('uestd', 'company_price_per_unit_ex', 'DOUBLE NOT NULL DEFAULT 1 AFTER `company_price_per_unit`' );
-- EOC
CALL us_add_to_table2('uestd', 'company_price_per_unit_id', 'INT( 11 ) NULL AFTER `company_price_per_unit_ex`' );
-- EOC

-- Add currencies support - Stammdaten
CALL us_add_to_table2('kopiekat', 'currency_id', 'INT( 11 ) NULL AFTER `company_id`' );
-- EOC

CALL us_add_to_table2('reisekosten', 'currency_id', 'INT( 11 ) NULL AFTER `company_id`' );
-- EOC

CALL us_add_to_table2('personen', 'currency_id', 'INT( 11 ) NULL AFTER `Bruttolohn`' );
-- EOC

CALL us_add_to_table2('gruppen_zeit', 'currency_id', 'INT( 11 ) NULL AFTER `Betrag9`' );
-- EOC

CALL us_add_to_table2('projektabrsaetze', 'currency_id', 'INT( 11 ) NULL AFTER `ABSatz`' );
-- EOC

CALL us_add_to_table2('projektabrsaetze', 'aktiv', 'BOOLEAN DEFAULT 1 AFTER `currency_id`' );
-- EOC

CALL us_add_to_table2('adressen', 'Master_Titel', 'VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `TITEL`' );
-- EOC

CALL us_add_to_table2('adressen', 'Strasse2', 'VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `Strasse`' );
-- EOC

CALL us_add_to_table2('adressen', 'bMaster_Titel', 'BOOLEAN DEFAULT 0 AFTER `bTitel`' );
-- EOC

CALL us_add_to_table2('adressen', 'bStrasse2', 'BOOLEAN DEFAULT 0 AFTER `bStrasse`' );
-- EOC

CALL us_add_to_table2('adressen', 'longitude', 'double DEFAULT NULL AFTER `Bilddatei`' );
-- EOC

CALL us_add_to_table2('adressen', 'longitude_deri', 'BOOLEAN DEFAULT 0 AFTER `longitude`' );
-- EOC

CALL us_add_to_table2('adressen', 'latitude', 'double DEFAULT NULL AFTER `longitude`' );
-- EOC

CALL us_add_to_table2('adressen', 'latitude_deri', 'BOOLEAN DEFAULT 0 AFTER `latitude`' );
-- EOC

CALL us_add_to_table2('adressen', 'comment_main', 'VARCHAR(200) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT \'\'' );
-- EOC
CALL us_add_to_table2('adressen', 'comment_main_staff', 'INT( 11 ) NULL AFTER comment_main' );
-- EOC
CALL us_add_to_table2('adressen', 'comment_main_date', 'datetime DEFAULT NULL AFTER comment_main_staff' );
-- EOC

-- Creditor number / Debtor number fields (DO NOT MAKE THEM DERIVABLE.)
CALL us_add_to_table2('adressen', 'creditor_no', 'VARCHAR(50) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT \'\' AFTER `Bilddatei`;' );
-- EOC

CALL us_add_to_table2('adressen', 'debtor_no', 'VARCHAR(50) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT \'\' AFTER `Bilddatei`;' );
-- EOC

CALL us_add_to_table2('adressen', 'dsgvo_delete_date', 'datetime DEFAULT NULL AFTER debtor_no' );
-- EOC
CALL us_add_to_table2('adressen', 'dsgvo_data_from', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci AFTER dsgvo_delete_date' );
-- EOC

-- ADD auto_assigned_wp flag to times/travels/extras
CALL us_add_to_table2('times', 'auto_assigned_wp', 'tinyint(1) unsigned NOT NULL DEFAULT 0' );
-- EOC
CALL us_add_to_table2('travels', 'auto_assigned_wp', 'tinyint(1) unsigned NOT NULL DEFAULT 0' );
-- EOC
CALL us_add_to_table2('extras', 'auto_assigned_wp', 'tinyint(1) unsigned NOT NULL DEFAULT 0' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- BIC / IBAN
CALL change_column('honkonten', 'Blz', 'VARCHAR( 11 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC
CALL change_column('honkonten', 'KNR', 'VARCHAR( 35 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC
CALL change_column('personen', 'KontoNr', 'VARCHAR( 150 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add search tables
DROP TABLE IF EXISTS `search_times`;
-- EOC

-- Add search tables
DROP TABLE IF EXISTS `search_adresses`;
-- EOC

-- Add search tables
DROP TABLE IF EXISTS `search_projects`;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Translate folders for different OS
CREATE TABLE IF NOT EXISTS `folder_translation` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `target` enum('WIN','UNIX','MAC') COLLATE utf8_general_ci NOT NULL,
    `path` varchar(254) COLLATE utf8_general_ci NOT NULL,
    `trans_path` varchar(254) COLLATE utf8_general_ci NOT NULL,
  PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci ;
-- EOC

CALL us_add_pk_auto_increment('folder_translation', 'id');
-- EOC

CALL us_add_to_table2('folder_translation', 'auto_translate_project_paths', 'BOOLEAN DEFAULT 0 AFTER `trans_path`' );
-- EOC

-- Java clients - OLD
DROP TABLE IF EXISTS `staff_javaclients`;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add instinfo
CALL us_add_to_table2('installation', 'instinfo', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL AFTER `Mod60`' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS `aktuser`;
-- EOC
DROP TABLE IF EXISTS `aktuser_log`;
-- EOC
DROP VIEW IF EXISTS `aktuser`;
-- EOC

-- Add never active users view
CREATE VIEW `aktuser` AS SELECT 
  NULL AS `LfdNr_AktUser`,
  NULL AS `UserNummer`,
  NULL AS `UserName`,
  NULL AS `LoginTime`,
  NULL AS `WorkstationName`,
  NULL AS `WorkstationLogin`,
  NULL AS `LfdNr_Alt`,
  NULL AS `webversion`
FROM dual WHERE FALSE;
-- EOC

DROP TABLE IF EXISTS `aktuser2`;
-- EOC
DROP VIEW IF EXISTS `aktuser2`;
-- EOC

-- Add never active users 2 view
CREATE VIEW `aktuser2` AS SELECT 
  NULL AS `LfdNr_AktUser`,
  NULL AS `UserNummer`,
  NULL AS `UserName`,
  NULL AS `LoginTime`,
  NULL AS `WorkstationName`,
  NULL AS `WorkstationLogin`,
  NULL AS `LfdNr_Alt`,
  NULL AS `webversion`
FROM dual WHERE FALSE;
-- EOC

CALL change_column('extra', 'zweck', 'VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC

CALL change_column('reise', 'zweck', 'VARCHAR(100) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC


-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- VAT
CALL us_add_to_table2('kopiekat', 'MwStKopie', 'double DEFAULT NULL' );
-- EOC

CALL us_add_to_table2('reise', 'KmMwSt', 'double DEFAULT NULL' );
-- EOC

CALL us_add_to_table2('reisekosten', 'MwStReise', 'double DEFAULT NULL' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Allow workinput related types to be set inactive
CALL us_add_to_table2('kopiekat', 'active', 'INT( 11 ) DEFAULT 1 AFTER `MwStKopie`' );
-- EOC
CALL us_add_to_table2('reisekosten', 'active', 'INT( 11 ) DEFAULT 1 AFTER `MwStReise`' );
-- EOC
CALL us_add_to_table2('uesatz', 'active', 'INT( 11 ) DEFAULT 1 AFTER `Satz`' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- UNIQUE ID FOR NEW ENTRIES
CALL us_add_to_table2('adrbeschr', 'unique_add_id', 'VARCHAR( 140 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `LfdNr_Adr`' );
-- EOC

CALL us_add_to_table2('adrbeschr', 'call_id', 'INT( 11 ) NULL DEFAULT NULL AFTER `unique_add_id`' );
-- EOC

CALL us_add_to_table2('tagessollstd', 'unique_add_id', 'VARCHAR( 140 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `Pers_LfdNr`' );
-- EOC

CALL us_add_to_table2('tagessollstdtemp', 'unique_add_id', 'VARCHAR( 140 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `Pers_LfdNr`' );
-- EOC

CALL us_add_to_table2('lut_pers_grp', 'unique_add_id', 'VARCHAR( 140 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `LfdNr_Pers`' );
-- EOC

CALL us_add_to_table2('lut_pers_grp_temp', 'unique_add_id', 'VARCHAR( 140 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `LfdNr_Pers`' );
-- EOC

CALL us_add_to_table2('lut_plan_adr', 'unique_add_id', 'VARCHAR( 140 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `LfdNr_Plan`' );
-- EOC


-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add tableconfig table
CREATE TABLE IF NOT EXISTS `tableconfig` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`table_id` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL ,
`column` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL ,
`staff_id` INT( 11 ) NULL DEFAULT NULL ,
`visible` BOOLEAN DEFAULT NULL,
`visiblemob` BOOLEAN DEFAULT NULL,
`width` double DEFAULT NULL,
`maxchars` double DEFAULT NULL,
`widthmob` double DEFAULT NULL,
`maxcharsmob` double DEFAULT NULL,
`order` SMALLINT( 4 ) DEFAULT NULL,
INDEX ( `staff_id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('tableconfig', 'maxchars', 'double DEFAULT NULL AFTER `width`' );
-- EOC
CALL us_add_to_table2('tableconfig', 'maxcharsmob', 'double DEFAULT NULL AFTER `widthmob`' );
-- EOC

-- Add tableconfig table
CREATE TABLE IF NOT EXISTS `tableconfig_save` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `table_id` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL ,
 `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL ,
 `config` LONGTEXT NOT NULL,
 INDEX ( `table_id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('tableconfig_save', 'staff_id', 'INT( 11 ) NULL DEFAULT NULL AFTER `config`' );
-- EOC
CALL us_add_to_table2('tableconfig_save', 'security', 'SMALLINT( 6 ) DEFAULT NULL AFTER `staff_id`' );
-- EOC
CALL us_add_to_table2('tableconfig_save', 'company_id', 'INT( 11 ) NULL DEFAULT NULL AFTER `security`' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add portlets table
CREATE TABLE IF NOT EXISTS `portlets` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`position_id` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL ,
`portlet_id` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL ,
`staff_id` INT( 11 ) NULL DEFAULT NULL ,
`col` INT( 11 ) UNSIGNED DEFAULT NULL,
`row` INT( 11 ) UNSIGNED DEFAULT NULL,
`open` BOOLEAN DEFAULT 0,
INDEX ( `staff_id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('portlets', 'tab', 'INT( 11 ) NULL DEFAULT NULL AFTER `row`' );
-- EOC

CALL us_add_to_table2('portlets', 'page_id', 'INT( 11 ) DEFAULT 1 AFTER `tab`' );
-- EOC

CALL us_add_to_table2('portlets', 'unique_id', 'VARCHAR( 140 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `portlet_id`' );
-- EOC

CALL us_add_to_table2('portlets', 'data', ' TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `open`' );
-- EOC

CALL us_add_to_table2('portlets', 'title', ' VARCHAR( 140 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `data`' );
-- EOC

CALL us_add_to_table2('portlets', 'auto_refresh', 'INT( 11 ) NULL DEFAULT NULL AFTER `title`' );
-- EOC

-- Add portlets_tabs
CREATE TABLE IF NOT EXISTS `portlets_tabs` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`staff_id` INT( 11 ) NOT NULL ,
`name` VARCHAR( 100 ) NOT NULL ,
`sort` SMALLINT( 4 ) NOT NULL DEFAULT 0
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('portlets_tabs', 'columns', 'INT( 2 ) NULL DEFAULT 3 AFTER `name`' );
-- EOC

CALL us_add_to_table2('portlets_tabs', 'color', 'INT( 2 ) NULL DEFAULT 1 AFTER `columns`' );
-- EOC

CALL us_add_to_table2('portlets_tabs', 'page_id', 'INT( 11 ) NOT NULL DEFAULT 1 AFTER `staff_id`' );
-- EOC

-- Add portlets_pages
CREATE TABLE IF NOT EXISTS `portlets_pages` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`staff_id` INT( 11 ) NOT NULL DEFAULT 0 ,
`name` VARCHAR( 100 ) NOT NULL ,
`is_deleteable` BOOLEAN DEFAULT 1
) auto_increment=100 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add address relations
CREATE TABLE IF NOT EXISTS `address_relations` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `address_id` INT( 11 ) NOT NULL ,
 `from_staff_id` INT( 11 ) DEFAULT NULL ,
 `from_address_id` INT( 11 ) DEFAULT NULL ,
 `from_personalapp_id` INT( 11 ) DEFAULT NULL ,
 `description` TEXT NOT NULL,
  `date` datetime DEFAULT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- UNIQUE ID FOR NEW ENTRIES
CALL us_add_to_table2('address_relations', 'unique_add_id', 'VARCHAR( 140 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `address_id`' );
-- EOC

CALL us_add_to_table2('address_relations', 'created_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('address_relations', 'changed_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('address_relations', 'created_date', 'DATETIME NULL' );
-- EOC
CALL us_add_to_table2('address_relations', 'change_date', 'DATETIME NULL' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Messaging system
CREATE TABLE IF NOT EXISTS `messaging_message` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `text` TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
 `url` TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
 `sticky` BOOLEAN NOT NULL,
 `staff_id` INT( 11 ) NULL ,
 `min_right` SMALLINT( 6 ) NULL ,
 `from_date` DATETIME NULL ,
 `to_date` DATETIME NULL ,
 `active` BOOLEAN NOT NULL,
 `created_by_id` INT( 11 ) NOT NULL ,
 `changed_by_id` INT( 11 ) NOT NULL ,
 `created_date` DATETIME NOT NULL ,
 `change_date` DATETIME NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CREATE TABLE IF NOT EXISTS `messaging_received` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `message_id` INT( 11 ) NOT NULL ,
 `staff_id` INT( 11 ) NOT NULL,
 `show_date` DATETIME NOT NULL,
 `active` BOOLEAN NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CREATE TABLE IF NOT EXISTS `reminder_appoi_received` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `appoi_id` INT( 11 ) NOT NULL ,
 `staff_id` INT( 11 ) NOT NULL,
 `show_date` DATETIME NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CREATE TABLE IF NOT EXISTS `reminder_opta_received` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `opta_id` INT( 11 ) NOT NULL ,
 `staff_id` INT( 11 ) NOT NULL,
 `show_date` DATETIME NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add project_directory_structures table
CREATE TABLE IF NOT EXISTS `project_directory_structures` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`name` VARCHAR( 100 ) NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('project_directory_structures', 'exec_script', 'varchar(255) DEFAULT NULL AFTER `name`' );
-- EOC

-- Add project_directory_structure_trees table
CREATE TABLE IF NOT EXISTS `project_directory_structure_trees` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`parent_id` INT( 11 ) NOT NULL,
`structure_id` INT( 11 ) NOT NULL,
`name` VARCHAR( 100 ) NOT NULL,
`is_path_correspondence` BOOLEAN DEFAULT NULL,
`is_path_inbox` BOOLEAN DEFAULT NULL,
`is_path_outbox` BOOLEAN DEFAULT NULL,
`is_path_drawings` BOOLEAN DEFAULT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('project_directory_structure_trees', 'file', 'VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `is_path_drawings`' );
-- EOC
CALL change_column('project_directory_structure_trees', 'file', 'VARCHAR( 255 ) NULL DEFAULT NULL AFTER `is_path_drawings`');
-- EOC
CALL change_column('project_directory_structure_trees', 'name', 'VARCHAR( 200 ) NOT NULL AFTER `structure_id`');
-- EOC
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------

-- Personalappointments
CALL us_add_to_table2('perstermin', 'state_color_id', 'INT( 11 ) NULL AFTER `bOrt`' );
-- EOC
UPDATE perstermin SET TerminStatus=10 WHERE TerminStatus IS NULL;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Report settings
CREATE TABLE IF NOT EXISTS `report_designs` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `name` VARCHAR( 100 ) NOT NULL,
 `company_id` INT( 11 ) NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci COMMENT = 'Report design settings';
-- EOC

CREATE TABLE IF NOT EXISTS `report_designs_vars` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `design_id` INT( 11 ) NOT NULL ,
 `name` VARCHAR( 100 ) NOT NULL ,
 `param` VARCHAR( 255 ) NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL change_column('report_designs_vars', 'param', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL');
-- EOC

DROP TABLE IF EXISTS `report_output`;
-- EOC

CREATE TABLE IF NOT EXISTS `report_output_templates` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `report_id` VARCHAR( 255 ) NOT NULL ,
 `company_id` INT( 11 ) NULL ,
 `result` LONGTEXT NOT NULL,
 `output` LONGTEXT NOT NULL
) auto_increment=1 ENGINE = InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci COMMENT = 'Report column settings';
-- EOC

CREATE TABLE IF NOT EXISTS `report_output_templates_sort` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `report_id` VARCHAR( 255 ) NOT NULL ,
 `output_template_id` VARCHAR( 100 ) NOT NULL ,
 `sort` SMALLINT( 4 ) NOT NULL DEFAULT 0
) auto_increment=1 ENGINE = InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci COMMENT = 'Report output sorting';
-- EOC

CALL us_add_to_table2('report_output_templates_sort', 'inactive', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `sort`' );
-- EOC

CREATE TABLE IF NOT EXISTS `report_output_portlet` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `xml_id` VARCHAR( 255 ) NOT NULL ,
 `name` VARCHAR( 100 ) NOT NULL,
 `company_id` INT( 11 ) NULL DEFAULT NULL ,
 `output` LONGTEXT NOT NULL
) auto_increment=1 ENGINE = InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci COMMENT = 'Report column settings for portlet based reports';
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Filter settings

DROP TABLE IF EXISTS `filter_settings`;
-- EOC

CREATE TABLE IF NOT EXISTS `filter_settings_main` (
 `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
 `staff_id` INT( 11 ) NOT NULL ,
 `xml_id` VARCHAR( 255 ) NOT NULL ,
 `name` VARCHAR( 100 ) NOT NULL,
 `security` SMALLINT( 6 ) NOT NULL DEFAULT 0 ,
 `request_settings` LONGTEXT NOT NULL,
 `request_settings_addable` LONGTEXT NOT NULL
) auto_increment=1 ENGINE = InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci COMMENT = 'Filter settings';
-- EOC

CALL us_add_to_table2('filter_settings_main', 'security', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `name`' );
-- EOC

CALL us_add_to_table2('progruppendef', 'filter_id', 'int(11) DEFAULT NULL' );
-- EOC

CALL us_add_to_table2('progruppendef', 'filter_last_count', 'bigint(20) DEFAULT NULL' );
-- EOC

CALL us_add_to_table2('progruppendef', 'filter_hash', 'VARCHAR( 40 ) DEFAULT NULL' );
-- EOC

CALL us_add_to_table2('progruppendef', 'filter_creator_info', 'longtext DEFAULT NULL' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Add staff_group table
CREATE TABLE IF NOT EXISTS `staff_groups` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`name` VARCHAR( 100 ) NOT NULL ,
`created_by_id` INT( 11 ) NOT NULL ,
`security` SMALLINT( 6 ) DEFAULT NULL,
`company_id` INT( 11 ) NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Add staff - staff_group lut-table
CREATE TABLE IF NOT EXISTS `lut_staff_staff_group` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`staff_id` INT( 11 ) NOT NULL ,
`staff_group_id` INT( 11 ) NOT NULL 
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC


CALL change_column('dokattachments', 'Att_Pfad', 'LONGTEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('dokattachments', 'orig_correspondence_id', 'INT( 11 ) NULL AFTER `Betreff`' );
-- EOC
CALL us_add_to_table2('dokattachments', 'drawing_id', 'INT( 11 ) NULL AFTER `orig_correspondence_id`' );
-- EOC
CALL us_add_to_table2('dokattachments', 'addressfile_id', 'INT( 11 ) NULL AFTER `drawing_id`' );
-- EOC
CALL us_add_to_table2('dokattachments', 'created_by_id', 'INT( 11 ) NULL AFTER `orig_correspondence_id`' );
-- EOC
CALL us_add_to_table2('dokattachments', 'created_date', 'DATETIME NULL AFTER `created_by_id`' );
-- EOC

CREATE TABLE IF NOT EXISTS `zusatz_formular` (
  `formular_lfdnr` int(11) DEFAULT NULL,
  `LfdNr_Alt` int(11) DEFAULT NULL,
  `LfdNr` int(11) NOT NULL AUTO_INCREMENT,
  PRIMARY KEY (`LfdNr`),
  KEY `zusatz_formular_formular_lfdnr` (`formular_lfdnr`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
-- EOC

-- This very important index INDEX was not created on MSSQL2MySQL migration 
CALL us_add_index_to_table('zusatz_formular', 'zusatz_formular_formular_lfdnr', '`formular_lfdnr`');
-- EOC

CREATE TABLE IF NOT EXISTS `zusatz_formular_def` (
  `OrdNr` smallint(6) DEFAULT NULL,
  `FeldName` varchar(30) DEFAULT NULL,
  `Typ` smallint(6) DEFAULT NULL,
  `Name` varchar(100) DEFAULT NULL,
  `Laenge` smallint(6) DEFAULT NULL,
  `Sicherheit` smallint(6) DEFAULT NULL,
  `LfdNr_Alt` int(11) DEFAULT NULL,
  `LfdNr` int(11) NOT NULL AUTO_INCREMENT,
  PRIMARY KEY (`LfdNr`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_correspondence_addi_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_correspondence_addi_def', 'lang_correspondence_addi_def_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_outbox_addi_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_outbox_addi_def', 'lang_outbox_addi_def_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `lang_inbox_addi_def` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `def_id` int(11) NOT NULL,
    `name` VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('lang_inbox_addi_def', 'lang_inbox_addi_def_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC

CREATE TABLE IF NOT EXISTS `staff_last_login` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `staff_id` INT( 11 ) NOT NULL ,
    `date` DATETIME NOT NULL ,
    INDEX ( `staff_id` )
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CREATE TABLE IF NOT EXISTS `staff_last_login_fail` (
  `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
  `staff_id` INT( 11 ) DEFAULT NULL ,
  `staff_name` varchar(50) DEFAULT NULL,
  `ip_address` VARCHAR( 16 ) NOT NULL ,
  `user_agent` VARCHAR( 150 ) NOT NULL ,
  `date` DATETIME NOT NULL
) auto_increment=1 ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci ;
-- EOC

CALL change_column('staff_last_login_fail', 'user_agent', 'VARCHAR(150) NOT NULL');
-- EOC

CREATE TABLE IF NOT EXISTS `fee_number_log` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `last_number` VARCHAR( 20 ) NOT NULL,
     UNIQUE KEY (`last_number`)
) ENGINE = InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_to_table2('zusatz_adressen_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_adressen_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_adressen_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_adressen_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_personen_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_personen_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_personen_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_personen_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_projekt_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_projekt_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_projekt_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_projekt_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_propers_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_propers_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_propers_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_propers_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_todo_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_todo_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_todo_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_todo_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_extkosten_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_extkosten_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_extkosten_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_extkosten_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_honverw_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_honverw_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_honverw_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_honverw_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_wasistwo_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_wasistwo_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_wasistwo_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_wasistwo_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_planverz_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_planverz_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_planverz_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_planverz_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_formular_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_formular_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_formular_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_formular_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_postaus_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_postaus_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_postaus_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_postaus_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC

CALL us_add_to_table2('zusatz_postein_def', 'subtype', 'SMALLINT( 6 ) NOT NULL DEFAULT 0 AFTER `Typ`' );
-- EOC

CALL us_add_to_table2('zusatz_postein_def', 'fieldvalues', 'text DEFAULT NULL AFTER `subtype`' );
-- EOC

CALL us_add_to_table2('zusatz_postein_def', 'isrequired', 'SMALLINT(1) DEFAULT 0 AFTER `fieldvalues`' );
-- EOC

CALL us_add_to_table2('zusatz_postein_def', 'color', 'VARCHAR(30) DEFAULT NULL AFTER `isrequired`' );
-- EOC
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
CALL us_add_to_table2('lut_projektabr', 'warnschwelle', 'DOUBLE NULL AFTER `SatzAB`' );
-- EOC

CALL us_add_to_table2('outsync', 'A10', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `A9`' );
-- EOC

CALL us_add_to_table2('outsync', 'ADR_Kategorie', 'INT( 11 ) DEFAULT NULL AFTER `A10`' );
-- EOC

CALL change_column('projektabrsaetze', 'Beschreibung', 'VARCHAR( 100 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC

CALL change_column('projektabrsaetze', 'RKSatz', 'VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC

CALL change_column('projektabrsaetze', 'ExtraSatz', 'VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC

CALL change_column('projektabrsaetze', 'MASatz', 'VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC

CALL change_column('projektabrsaetze', 'ABSatz', 'VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC

CALL us_add_to_table2('telefonate', 'LfdNr_Adr_Caller', 'INT( 11 ) DEFAULT NULL AFTER `LfdNr_Adr`' );
-- EOC

CALL us_add_to_table2('telefonate', 'No_Adr_Caller', 'BOOLEAN DEFAULT FALSE AFTER `LfdNr_Adr_Caller`' );
-- EOC

CALL us_add_to_table2('telefonate', 'disconnected', 'BOOLEAN DEFAULT FALSE AFTER `beendet`' );
-- EOC

CALL us_add_to_table2('telefonate', 'last_update', 'DATETIME NULL' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Oauth accounts
CREATE TABLE IF NOT EXISTS `oauth_clients` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `name` VARCHAR( 255 ) DEFAULT NULL,
    `client_type` VARCHAR( 20 ) DEFAULT NULL,
    `client_authority` VARCHAR( 255 ) DEFAULT NULL,
    `client_id` VARCHAR( 255 ) DEFAULT NULL,
    `client_secret` VARCHAR( 255 ) DEFAULT NULL,
    `client_config` LONGTEXT NOT NULL
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_to_table2('oauth_clients', 'client_authority', 'VARCHAR( 255 ) DEFAULT NULL AFTER `client_type`' );
-- EOC

-- Oauth tokens
CREATE TABLE IF NOT EXISTS `oauth_tokens` (
    `staff_id` INT NOT NULL,
    `account_id` INT NOT NULL,
    `access_token` LONGBLOB NOT NULL,
    `refresh_token` LONGBLOB NOT NULL,
    `last_update` DATETIME NOT NULL,
    `expires_date` DATETIME DEFAULT NULL,
  PRIMARY KEY (`staff_id`, `account_id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC


CALL check_rename_column_exists('oauth_tokens', 'client_id', 'staff_id', 'INT(11) NOT NULL ');
-- EOC

-- E-mail accounts
-- E-mail 256 http://www.rfc-editor.org/errata_search.php?rfc=3696
-- DNS name max 255 octets http://blogs.msdn.com/b/oldnewthing/archive/2012/04/12/10292868.aspx
-- Port number max 65535
CREATE TABLE IF NOT EXISTS `email_accounts` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `staff_id` INT( 11 ) NOT NULL ,
    `auth_mode` enum('basic', 'oauth') COLLATE utf8_general_ci DEFAULT 'basic',
    `oauth_account` INT( 11 ) DEFAULT NULL,
    `password_mode` INT( 11 ) NOT NULL  DEFAULT 1,
    `password_mode_info` VARCHAR( 255 ) DEFAULT NULL,
    `email` VARCHAR( 256 ) NOT NULL,
    `smtp_host` VARCHAR( 255 ) NOT NULL ,
    `smtp_user` VARCHAR( 100 ) NOT NULL ,
    `smtp_pass` VARCHAR( 255 ) NOT NULL ,
    `smtp_port` INT( 5 ) NOT NULL ,
    `smtp_service_flags` VARCHAR( 254 ) NOT NULL ,
    `imap_host` VARCHAR( 255 ) NOT NULL ,
    `imap_user` VARCHAR( 100 ) NOT NULL ,
    `imap_pass` VARCHAR( 255 ) NOT NULL ,
    `imap_port` INT( 5 ) NOT NULL ,
    `imap_mailbox` VARCHAR( 100 ) NOT NULL ,
    `imap_service_flags` VARCHAR( 254 ) NOT NULL
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_to_table2('email_accounts', 'auth_mode', 'enum(\'basic\', \'oauth\') COLLATE utf8_general_ci DEFAULT \'basic\' AFTER `id`'  );
-- EOC
UPDATE email_accounts SET auth_mode='basic' WHERE auth_mode IS NULL; 
-- EOC
CALL us_add_to_table2('email_accounts', 'oauth_account', 'INT( 11 ) DEFAULT NULL AFTER `auth_mode`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'password_mode', 'INT( 11 ) DEFAULT 1 AFTER `id`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'password_mode_info', 'VARCHAR( 255 ) DEFAULT NULL AFTER `password_mode`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'smtp_mode', 'enum(\'plain\', \'ssl\', \'tls\') COLLATE utf8_general_ci DEFAULT \'plain\' AFTER `smtp_port`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'imap_mode', 'enum(\'plain\', \'ssl\', \'tls\') COLLATE utf8_general_ci DEFAULT \'plain\' AFTER `imap_port`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'imap_mailbox_out', 'VARCHAR( 100 ) NOT NULL AFTER `imap_mailbox`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'imap_mailbox_trash', 'VARCHAR( 100 ) DEFAULT NULL AFTER `imap_mailbox_out`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'imap_save_sent', 'BOOLEAN DEFAULT TRUE AFTER `imap_mailbox_out`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'imap_mbox_delimiter', 'VARCHAR( 255 ) DEFAULT NULL AFTER `imap_mailbox_out`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'owner_id', 'INT ( 11 ) DEFAULT NULL AFTER `email`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'smtp_ldap_auth_staff_id', 'INT ( 11 ) DEFAULT NULL AFTER `owner_id`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'imap_ldap_auth_staff_id', 'INT ( 11 ) DEFAULT NULL AFTER `smtp_ldap_auth_staff_id`' );
-- EOC

CALL us_add_to_table2('email_accounts', 'auto_expunge', 'BOOLEAN DEFAULT FALSE' );
-- EOC

CALL us_add_to_table2('email_accounts', 'import_eml_later', 'BOOLEAN DEFAULT TRUE' );
-- EOC

CALL us_add_to_table2('email_accounts', 'address_signature_id', 'INT ( 11 ) DEFAULT 1 AFTER `owner_id`' );
-- EOC

CREATE TABLE IF NOT EXISTS `email_favourite_mboxes` (
    `id` int(11) NOT NULL AUTO_INCREMENT ,
    `email_account_id` INT( 11 ) NOT NULL ,
    `name` varchar(255) NOT NULL ,
    `is_out` BOOLEAN DEFAULT 0,
    PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_drop_index_if_exists('mail_id', 'mail_id_Mail_ID');
-- EOC

CALL us_add_to_table2('mail_id', 'Mail_ID_long', 'VARCHAR( 255 ) DEFAULT NULL AFTER `Mail_ID`' );
-- EOC

CALL us_add_to_table2('mail_id', 'inbox_id', 'VARCHAR( 11 ) DEFAULT NULL AFTER `Mail_ID_long`' );
-- EOC

CALL us_add_to_table2('mail_id', 'outbox_id', 'VARCHAR( 11 ) DEFAULT NULL AFTER `inbox_id`' );
-- EOC

CALL us_add_to_table2('dokattachments', 'real_name', 'VARCHAR( 255 ) DEFAULT \'\' AFTER `Att_Pfad`' );
-- EOC

CALL change_column('mail_id', 'Mail_ID', 'VARCHAR( 500 ) DEFAULT NULL');
-- EOC

CALL change_column('mail_id', 'Mail_ID_long', 'VARCHAR( 500 ) DEFAULT NULL');
-- EOC

-- Logging for moving form types
CREATE TABLE IF NOT EXISTS `move_form_types_log` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `moving_session_id` VARCHAR( 50 ) NOT NULL,
    `project_id` INT( 11 ) NOT NULL ,
    `correspondence_id` INT( 11 ) NOT NULL ,
    `correspondence_type_id` INT( 11 ) NOT NULL ,
    `project_path_correspondence` longtext NOT NULL,
    `correspondence_structure_path` VARCHAR( 254 ) NOT NULL,
    `filename` VARCHAR( 254 ) NOT NULL,
    `path_from` longtext NOT NULL,
    `path_to` longtext NOT NULL,
    `move_failed` BOOLEAN DEFAULT 0,
    `created_date` DATETIME NOT NULL,
    `created_by_id` INT( 11 ) NOT NULL,
    `change_date` DATETIME NOT NULL,
    `changed_by_id` INT( 11 ) NOT NULL
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

-- Logging for moving form types (archive table)
CALL us_create_move_form_types_log_archive();
-- EOC

-- Create system_modifications_log, if not exists
CALL us_create_system_modifications_log();
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Send email

CREATE TABLE IF NOT EXISTS `sendmail` (
    `id` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY, 
    `send_account` INT(11) DEFAULT NULL,
    `recipients_to` LONGTEXT NOT NULL,
    `recipients_cc` LONGTEXT NOT NULL,
    `recipients_bcc` LONGTEXT NOT NULL,
    `subject` TEXT NOT NULL, 
    `message` LONGTEXT NOT NULL, 
    `related_id` INT( 11 ) NULL,
    `related_type` VARCHAR(30) NULL, 
    `status` ENUM('new', 'send_do', 'send_done') NOT NULL, 
    `created_date` DATETIME NOT NULL,
    `created_by_id` INT(11) NOT NULL
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_to_table2('sendmail', 'recipients', 'MEDIUMTEXT NOT NULL AFTER `recipients_bcc`' );
-- EOC
CALL us_add_to_table2('sendmail', 'flags_mode', 'VARCHAR( 255 ) DEFAULT \'\' AFTER `status`' );
-- EOC
CALL us_add_to_table2('sendmail', 'flags', 'VARCHAR( 255 ) DEFAULT \'\' AFTER `status`' );
-- EOC
CALL us_add_to_table2('sendmail', 'outbox', 'VARCHAR( 255 ) DEFAULT \'\' AFTER `flags`' );
-- EOC
CALL us_add_to_table2('sendmail', 'attachments', 'LONGTEXT AFTER `outbox`' );
-- EOC
CALL us_add_to_table2('sendmail', 'zip_attachments', 'BOOLEAN DEFAULT FALSE AFTER `attachments`' );
-- EOC
CALL us_add_to_table2('sendmail', 'external', 'LONGTEXT DEFAULT NULL AFTER `zip_attachments`' );
-- EOC
CALL change_column('sendmail', 'related_id', 'VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL');
-- EOC

CALL us_add_to_table2('sendmail', 'recipients_to_ids', 'MEDIUMTEXT NULL AFTER `recipients_to`' );
-- EOC
CALL us_add_to_table2('sendmail', 'recipients_cc_ids', 'MEDIUMTEXT NULL AFTER `recipients_cc`' );
-- EOC
CALL us_add_to_table2('sendmail', 'recipients_bcc_ids', 'MEDIUMTEXT NULL AFTER `recipients_bcc`' );
-- EOC

CREATE TABLE IF NOT EXISTS `staff_status` (
    `staff_id` INT(11) DEFAULT NULL,
    `status` INT(1) NOT NULL,
    `comment` VARCHAR( 100 ) NOT NULL,
    `max_age` datetime
) ENGINE=MEMORY  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_to_table2('arbbeschtxte', 'active', 'tinyint(1) unsigned NOT NULL DEFAULT 1 AFTER `OrdnNr`' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Measurements

CREATE TABLE IF NOT EXISTS `lut_measurements` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `project_id` int(11) NOT NULL,
  `measurement_id` int(11) NOT NULL,
  `value` double NOT NULL,
  PRIMARY KEY (`id`),
  UNIQUE `proj_meas` (`project_id`, `measurement_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci AUTO_INCREMENT=1 ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Orte km as double

CALL change_column('orte', 'Km', 'DOUBLE NULL DEFAULT NULL');
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- More description fields

CALL us_add_to_table2('tagessollstd', 'description', 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL;' );
-- EOC

CALL us_add_to_table2('lut_pers_grp', 'description', 'TEXT CHARACTER SET utf8 COLLATE utf8_general_ci NULL;' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Calcexp payments

CREATE TABLE IF NOT EXISTS `payments` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `offer_id` int(11) NOT NULL,
  `correspondence_type_id` int(11) NOT NULL,
  `from_date` datetime DEFAULT NULL COMMENT 'Optional',
  `to_date` datetime DEFAULT NULL COMMENT 'Optional',
  `fee_type` int(2) NOT NULL COMMENT 'instalment or final',
  `fee_no` varchar(20) DEFAULT NULL COMMENT 'set when invoiced',
  `fee_date` datetime DEFAULT NULL COMMENT 'set when invoiced',
  `due_days` int(5) NOT NULL,
  `status` int(2) NOT NULL COMMENT 'PLANNED 10, INVOICED 20, INVALID 30',
  `discount` double NOT NULL,
  `discount_days` int(5) NOT NULL,
  `approved_by` int(11) DEFAULT NULL COMMENT 'set when approved',
  `approved_date` datetime DEFAULT NULL COMMENT 'set when approved',
  `created_date` DATETIME NOT NULL,
  `created_by_id` INT( 11 ) NOT NULL,
  `change_date` DATETIME NOT NULL,
  `changed_by_id` INT( 11 ) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci AUTO_INCREMENT=1 ;
-- EOC

CALL us_add_to_table2('payments', 'vat', 'double DEFAULT 0 AFTER `fee_type`' );
-- EOC

CALL us_add_to_table2('payments', 'round', 'int(5) DEFAULT 0 AFTER `discount_days`' );
-- EOC
CALL us_add_to_table2('payments', 'round_gross', 'BOOLEAN DEFAULT FALSE AFTER `round`' );
-- EOC

CALL us_add_to_table2('payments', 'value_supplement', 'double DEFAULT 0 COMMENT \'set when approved. all values not beeing part of payments_fees.\' AFTER `approved_date`' );
-- EOC
CALL us_add_to_table2('payments', 'value_specserv', 'double DEFAULT 0 COMMENT \'set when approved. all values not beeing part of payments_fees.\' AFTER `value_supplement`' );
-- EOC
CALL us_add_to_table2('payments', 'value_extras', 'double DEFAULT 0 COMMENT \'set when approved. all values not beeing part of payments_fees.\' AFTER `value_specserv`' );
-- EOC
CALL us_add_to_table2('payments', 'value_rebate', 'double DEFAULT 0 COMMENT \'set when approved. all values not beeing part of payments_fees.\' AFTER `value_extras`' );
-- EOC
CALL us_add_to_table2('payments', 'value_ontop', 'double DEFAULT 0 COMMENT \'set when approved. all values not beeing part of payments_fees.\' AFTER `value_rebate`' );
-- EOC

CALL us_add_to_table2('payments', 'invoice_date', 'DATETIME NULL AFTER `status`' );
-- EOC

CALL us_add_index_to_table('payments', 'offer_id', '`offer_id`');
-- EOC

CREATE TABLE IF NOT EXISTS `payments_fees` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `payments_id` int(11) NOT NULL,
  `fee_areas_offer_id` int(11) NOT NULL,
  `percentage` double NOT NULL,
  `value` double DEFAULT 0 COMMENT 'set when invoiced',
  `fixed_value` double DEFAULT 0 COMMENT 'set when invoiced',
  `value_otcurr` double DEFAULT 0 COMMENT 'set when invoiced',
  `extras_value` double DEFAULT 0 COMMENT 'set when invoiced',
  `rebate_value` double DEFAULT 0 COMMENT 'set when invoiced',
  `extras_otcurr` double DEFAULT 0 COMMENT 'set when invoiced',
  `rebate_otcurr` double DEFAULT 0 COMMENT 'set when invoiced',
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci AUTO_INCREMENT=1 ;
-- EOC

CALL us_add_to_table2('payments_fees', 'fixed_value', 'double DEFAULT 0' );
-- EOC

CALL us_add_to_table2('payments_fees', 'value_otcurr', 'double DEFAULT 0');
-- EOC

CALL us_add_to_table2('payments_fees', 'extras_value', 'double DEFAULT 0');
-- EOC
CALL us_add_to_table2('payments_fees', 'rebate_value', 'double DEFAULT 0');
-- EOC
CALL us_add_to_table2('payments_fees', 'extras_otcurr', 'double DEFAULT 0');
-- EOC
CALL us_add_to_table2('payments_fees', 'rebate_otcurr', 'double DEFAULT 0');
-- EOC

CREATE TABLE IF NOT EXISTS `payments_change_project_offer` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `payment_id` int(11) NOT NULL,
  `old_offer_id` int(11) NOT NULL,
  `new_offer_id` int(11) NOT NULL,
  `change_date` datetime NOT NULL,
  `staff_id` int(11) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci AUTO_INCREMENT=1 ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Addresses e-mail

CREATE TABLE IF NOT EXISTS `address_emails` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `address_id` int(11) NOT NULL,
  `email` VARCHAR( 255 ) NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci AUTO_INCREMENT=1 ;
-- EOC

CALL us_add_to_table2('address_emails', 'created_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('address_emails', 'changed_by_id', 'INT( 11 ) NULL' );
-- EOC
CALL us_add_to_table2('address_emails', 'created_date', 'DATETIME NULL' );
-- EOC
CALL us_add_to_table2('address_emails', 'change_date', 'DATETIME NULL' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Cache for measurement

CREATE TABLE IF NOT EXISTS `cache_project_measurement` (
  `project_id` int(11) NOT NULL,
  `name_german` varchar(500) NOT NULL,
  `name_germanarch` varchar(500) NOT NULL,
  `name_germanch` varchar(500) NOT NULL,
  `name_english` varchar(500) NOT NULL,
  `name_french` varchar(500) NOT NULL,
  UNIQUE KEY `project_id` (`project_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- New security_zip

CALL us_add_to_table2('formular', 'security_zip', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `Stufe`' );
-- EOC
CALL us_add_to_table2('dokattachments', 'security_zip', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `Stufe`' );
-- EOC

CALL us_add_to_table2('formart', 'security_zip', 'tinyint(1) unsigned NOT NULL DEFAULT 0 AFTER `Sicherheit`' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- For external systems. A key value table with type for external type
CREATE TABLE IF NOT EXISTS `external` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `type` varchar(100) NOT NULL,
  `keyid` varchar(100) NOT NULL,
  `value` longtext NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci COMMENT='For external systems';
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS `node_chat_messages`;
-- EOC

CREATE TABLE IF NOT EXISTS `node_chat_messages` (
    `id` int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
    `staff_id` INT(11) DEFAULT NULL,
    `room` varchar(100) DEFAULT NULL,
    `message` varchar(255) NOT NULL,
    `created_date` datetime
) ENGINE=MEMORY  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
DROP TABLE IF EXISTS `sse_event`;
-- EOC

CREATE TABLE IF NOT EXISTS `sse_event` (
    `uniqid` varchar(40) COLLATE utf8_general_ci NOT NULL PRIMARY KEY,
    `staff_id` int(11) NOT NULL 
) ENGINE=MEMORY  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CREATE TABLE IF NOT EXISTS `sse_event_data` (
	`id` BIGINT(20) NOT NULL AUTO_INCREMENT,
	`staff_id` INT(11) NOT NULL,
	`data_object` LONGTEXT NULL,
	`lifetime` DATETIME NULL DEFAULT NULL,
	PRIMARY KEY (`id`)
) ENGINE=InnoDB COLLATE=utf8_general_ci;
-- EOC

CALL us_add_to_table2('sse_event_data', 'uniqid', 'VARCHAR(40) COLLATE utf8_general_ci NULL DEFAULT NULL' );
-- EOC

CALL us_add_to_table2('sse_event_data', 'ensure_execution_lock', 'DATETIME DEFAULT NULL' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Staff or office location
CREATE TABLE IF NOT EXISTS `staff_locations` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `name` varchar(255) NOT NULL,
  `address` text NOT NULL,
  `tapi` LONGTEXT NULL DEFAULT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_to_table2('staff_locations', 'tapi', 'LONGTEXT NULL DEFAULT NULL AFTER `address`' );
-- EOC

-- Ensure we have always have a staff_location
INSERT INTO staff_locations (`id`, `name`, `address`) 
SELECT 1, 'Zentrale', '' FROM DUAL
WHERE NOT EXISTS (SELECT * FROM staff_locations);
-- EOC

UPDATE personen SET staff_location_id=1 WHERE staff_location_id=0;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Skill Group

CREATE TABLE IF NOT EXISTS `skill_group` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`name` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL ,
`description` LONGTEXT NULL DEFAULT NULL ,
`sort` SMALLINT( 4 ) NULL DEFAULT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL change_column('skill_group', 'sort', 'SMALLINT( 4 ) NULL DEFAULT NULL');
-- EOC

-- Skill Group - skill_group lut-table

CREATE TABLE IF NOT EXISTS `lut_staff_skill_group` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`staff_id` INT(11) DEFAULT NULL,
`skill_id` INT(11) DEFAULT NULL,
`from_date` DATETIME NULL,
`to_date` DATETIME NULL,
`description` LONGTEXT NULL DEFAULT NULL, 
`unique_add_id` VARCHAR(140) NULL DEFAULT NULL 
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('lut_skill_wp', 'description', 'LONGTEXT' );
-- EOC

-- Skill Group - skill_group lang-table

CREATE TABLE IF NOT EXISTS `langs_skill_groups` (
`id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`skill_id` INT(11) DEFAULT NULL,
`name` LONGTEXT NULL DEFAULT NULL ,
`description` LONGTEXT NULL DEFAULT NULL ,
`language` VARCHAR( 50 ) NULL DEFAULT NULL 
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Report working hours

CREATE TABLE IF NOT EXISTS `report_working_hours` (
`staff_id` INT(11),
`skill_id` INT(11) DEFAULT NULL,
`date` DATETIME,
`hours` FLOAT(11) DEFAULT NULL,
PRIMARY KEY(`staff_id`, `date`)
) ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- CALL change_column('report_working_hours', 'date', 'DATETIME DEFAULT `1900-01-01 00:00:00`');
ALTER TABLE `report_working_hours` CHANGE `date` `date` DATETIME DEFAULT '1900-01-01 00:00:00';
-- EOC

ALTER TABLE `report_working_hours` CHANGE `hours` `hours` FLOAT(11) DEFAULT NULL;
-- EOC

call remove_column('report_working_hours', 'id');
-- EOC

CALL us_add_constraint_to_table('report_working_hours', 'US_PRIMARY', 'PRIMARY', '`staff_id` , `date`' );
-- EOC

CALL us_add_to_table2('report_working_hours', 'is_vac', 'TINYINT DEFAULT 0' );
-- EOC

CREATE TABLE IF NOT EXISTS `report_working_hours_dirty` (
`id` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`staff_id` INT(11) DEFAULT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CREATE TABLE IF NOT EXISTS `email_cache_mboxes` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `imap_host` VARCHAR( 255 ) NOT NULL ,
    `imap_user` VARCHAR( 100 ) NOT NULL ,
    `imap_mbox` VARCHAR( 255 ) NOT NULL ,
    `last_check_start` DATETIME NOT NULL,
    `last_check_stop` DATETIME DEFAULT NULL,
    UNIQUE `unique_imap` (`imap_host`, `imap_user`, `imap_mbox`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_to_table2('email_cache_mboxes', 'pid', 'VARCHAR( 25 ) DEFAULT NULL AFTER `last_check_stop`' );
-- EOC

ALTER TABLE `email_cache_mboxes` CHANGE COLUMN `pid` `pid` VARCHAR(50) NULL DEFAULT NULL AFTER `last_check_stop`;
-- EOC

CREATE TABLE IF NOT EXISTS `email_cache_data` (
    `id` BIGINT( 20 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `email_cache_mbox_id` INT( 11 ) NOT NULL ,
    `uid` INT( 11 ) NOT NULL ,
    `message_id` VARCHAR( 500 ) NOT NULL ,
    `subject` VARCHAR( 255 ) DEFAULT NULL ,
    `from` VARCHAR( 500 ) DEFAULT NULL ,
    `to` LONGTEXT DEFAULT NULL,
    `to_count` INT( 11 ) DEFAULT 0 ,
    `cc` LONGTEXT DEFAULT NULL,
    `cc_count` INT( 11 ) DEFAULT 0 ,
    `bcc` LONGTEXT DEFAULT NULL,
    `bcc_count` INT( 11 ) DEFAULT 0 ,
    `in_reply_to` VARCHAR( 500 ) DEFAULT NULL ,
    `attachments` LONGTEXT DEFAULT NULL,
    `attachments_count` INT( 11 ) DEFAULT 0 ,
    `date` DATETIME NOT NULL,
    `size` double DEFAULT NULL,
    `delete` BOOLEAN DEFAULT 0,
    `text` LONGTEXT DEFAULT NULL,
    `ignored` BOOLEAN DEFAULT 0,
    UNIQUE `unique_mbox_uid` (`email_cache_mbox_id`, `uid`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_to_table2('email_cache_data', 'email_cache_hash', 'VARCHAR( 500 ) DEFAULT NULL AFTER `uid`' );
-- EOC

CALL change_column('email_cache_data', 'message_id', 'VARCHAR(500) NULL DEFAULT NULL');
-- EOC
CALL change_column('email_cache_data', 'from', 'VARCHAR(500) NULL DEFAULT NULL');
-- EOC
CALL change_column('email_cache_data', 'in_reply_to', 'VARCHAR(500) NULL DEFAULT NULL');
-- EOC

CALL us_add_index_to_table('mail_id', 'mail_id_Mail_ID', '`Mail_ID`(255)');
-- EOC

CALL us_add_to_table2('inbox', 'email_cache_id', 'BIGINT(20) DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('inbox', 'email_cache_hash', 'VARCHAR( 500 ) DEFAULT NULL AFTER `email_cache_id`' );
-- EOC
CALL us_add_to_table2('inbox', 'email_account_id', 'INT(11) DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('inbox', 'pid', 'VARCHAR( 25 ) DEFAULT NULL' );
-- EOC
ALTER TABLE `inbox` CHANGE COLUMN `pid` `pid` VARCHAR(50) DEFAULT NULL;
-- EOC
CALL us_add_to_table2('inbox', 'email_import_start', 'DATETIME DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('inbox', 'email_import_attempts', 'INT(11) DEFAULT 0' );
-- EOC
CALL us_add_to_table2('inbox', 'projectmail_account_id', 'INT(11) DEFAULT 0' );
-- EOC

CALL us_add_to_table2('outbox', 'email_cache_id', 'BIGINT(20) DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('outbox', 'email_cache_hash', 'VARCHAR( 500 ) DEFAULT NULL AFTER `email_cache_id`' );
-- EOC
CALL us_add_to_table2('outbox', 'email_account_id', 'INT(11) DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('outbox', 'pid', 'VARCHAR( 25 ) DEFAULT NULL' );
-- EOC
ALTER TABLE `outbox` CHANGE COLUMN `pid` `pid` VARCHAR(50) DEFAULT NULL;
-- EOC
CALL us_add_to_table2('outbox', 'email_import_start', 'DATETIME DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('outbox', 'email_import_attempts', 'INT(11) DEFAULT 0' );
-- EOC
CALL us_add_to_table2('outbox', 'projectmail_account_id', 'INT(11) DEFAULT 0' );
-- EOC

CREATE TABLE IF NOT EXISTS `email_projects` (
	`id` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
	`project_id` INT(11) NOT NULL,
	`email_account_id` INT(11) NOT NULL,
	`inbox` VARCHAR(255) NULL DEFAULT NULL,
	`outbox` VARCHAR(255) NULL DEFAULT NULL,
	`security` INT(11) NOT NULL DEFAULT '3',
	`delete_after_save` BOOLEAN NOT NULL DEFAULT '0'
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------
-- Outlook-Sync
CALL us_add_to_table2('outsync', 'ProfilName', 'VARCHAR(30) NULL' );
-- EOC
CALL us_add_to_table2('outsync', 'ProfilKennwort', 'VARCHAR(30) NULL' );
-- EOC
CALL us_add_to_table2('outsync', 'AutoSync', 'BOOLEAN NOT NULL DEFAULT 0' );
-- EOC

CREATE TABLE IF NOT EXISTS `lut_working_user` (
    `id` INT( 11 ) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
    `user_id` int(11) NOT NULL,
    `date` DATETIME NOT NULL,
    `type` VARCHAR(100) NOT NULL,
    `item_id` int(11) NOT NULL
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Update Fees
UPDATE fees SET remited=0 WHERE remited IS NULL;
-- EOC

-- UPDATE work packages NULL fields to 0
UPDATE work_packages SET budget_planned_costs = 0 WHERE budget_planned_costs IS NULL;
-- EOC
UPDATE work_packages SET budget_planned_hours = 0 WHERE budget_planned_hours IS NULL;
-- EOC
UPDATE work_packages SET budget_proportion = 0 WHERE budget_proportion IS NULL;
-- EOC
UPDATE work_packages SET fee_costs = 0 WHERE fee_costs IS NULL;
-- EOC
UPDATE work_packages SET fee_hours = 0 WHERE fee_hours IS NULL;
-- EOC
UPDATE work_packages SET fee_proportion = 0 WHERE fee_proportion IS NULL;
-- EOC
UPDATE work_packages SET completion_level = 0 WHERE completion_level IS NULL;
-- EOC

CALL change_column('work_packages', 'fee_costs', 'DOUBLE NULL DEFAULT 0');
-- EOC
CALL change_column('work_packages', 'fee_hours', 'DOUBLE NULL DEFAULT 0');
-- EOC
CALL change_column('work_packages', 'budget_planned_costs', 'DOUBLE NULL DEFAULT 0');
-- EOC
CALL change_column('work_packages', 'budget_planned_hours', 'DOUBLE NULL DEFAULT 0');
-- EOC
CALL change_column('work_packages', 'fee_proportion', 'DOUBLE NULL DEFAULT 0');
-- EOC
CALL change_column('work_packages', 'budget_proportion', 'DOUBLE NULL DEFAULT 0');
-- EOC

CREATE TABLE IF NOT EXISTS `staff_token` (
  `id` INT NOT NULL AUTO_INCREMENT,
  `staff_id` INT NOT NULL,
  `date` DATETIME NOT NULL,
  `token` VARCHAR(255) NOT NULL,
  PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CALL us_add_to_table2('externalexpenses_planned', 'valid_from', 'datetime DEFAULT NULL' );
-- EOC

UPDATE externalexpenses_planned SET `valid_from` = `created_date` WHERE `valid_from` IS NULL;
-- EOC

-- Add fee_categories
CREATE TABLE IF NOT EXISTS `fee_categories` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `name` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- Add translation tables
CREATE TABLE IF NOT EXISTS `langs_fee_categories` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `fee_category_id` int(11) NOT NULL,
    `name` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    `language` VARCHAR( 50 ) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ,
    PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CALL us_add_constraint_to_table('langs_fee_categories', 'langs_fee_categories_unique_1', 'UNIQUE', '`name` , `language`' );
-- EOC


CALL us_add_to_table2('externalexpenses', 'fee_category_id', 'INT(11) DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('externalexpenses_planned', 'fee_category_id', 'INT(11) DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('fees', 'fee_category_id', 'INT(11) DEFAULT NULL' );
-- EOC
CALL us_add_to_table2('fees_planned', 'fee_category_id', 'INT(11) DEFAULT NULL' );
-- EOC


CALL us_add_to_table2('externalexpenses', 'currency_id', 'INT( 11 ) NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('externalexpenses', 'value_otcurr', 'double DEFAULT NULL');
-- EOC
CALL us_add_to_table2('externalexpenses_planned', 'currency_id', 'INT( 11 ) NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('externalexpenses_planned', 'value_otcurr', 'double DEFAULT NULL');
-- EOC


CALL us_add_to_table2('fees', 'currency_id', 'INT( 11 ) NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('fees', 'value_otcurr', 'double DEFAULT NULL');
-- EOC
CALL us_add_to_table2('fees', 'remited_otcurr', 'double DEFAULT NULL');
-- EOC
CALL us_add_to_table2('fees', 'deduction_otcurr', 'double DEFAULT NULL');
-- EOC
CALL us_add_to_table2('fees', 'retention_type', 'INT( 11 ) NOT NULL DEFAULT 0');
-- EOC
CALL us_add_to_table2('fees', 'retention_date', 'DATETIME NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('fees', 'retention', 'DOUBLE NOT NULL DEFAULT 0');
-- EOC
CALL us_add_to_table2('fees_planned', 'currency_id', 'INT( 11 ) NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('fees_planned', 'value_otcurr', 'double DEFAULT NULL');
-- EOC
CALL us_add_to_table2('fees_planned', 'remited_otcurr', 'double DEFAULT NULL');
-- EOC
CALL us_add_to_table2('fees_planned', 'deduction_otcurr', 'double DEFAULT NULL');
-- EOC
CALL us_add_to_table2('fees_planned', 'retention_type', 'INT( 11 ) NOT NULL DEFAULT 0');
-- EOC
CALL us_add_to_table2('fees_planned', 'retention_date', 'DATETIME NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('fees_planned', 'retention', 'DOUBLE NOT NULL DEFAULT 0');
-- EOC


CALL us_add_to_table2('formular', 'fee_currency_id', 'INT( 11 ) NULL DEFAULT NULL');
-- EOC
CALL us_add_to_table2('formular', 'fee_value_otcurr', 'double DEFAULT NULL');
-- EOC

CALL us_add_to_table2('uv_buchungen', 'is_absent_social', 'BOOLEAN DEFAULT FALSE AFTER `UV_ZA`' );
-- EOC

DROP TABLE IF EXISTS portlets_settings;
-- EOC

CREATE TABLE IF NOT EXISTS `portlets_settings` (
	`id` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
	`portlet_id` VARCHAR(100) NOT NULL,
	`allow_right` BOOLEAN NOT NULL DEFAULT '0',
	UNIQUE INDEX `portlet_id` (`portlet_id`)
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Funpic_portlet', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Sysinfo_portlet', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Updateinfo_view', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Webinar_view', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Active_users', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Call_phone', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Send_sms_at', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Iframe', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Rss_view', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Daily_hint', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Exchange_rate', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Numbergeneration', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Timelogging_active_staff', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Timelogging_select', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Oauth_login', 1);
-- EOC
INSERT INTO `portlets_settings` (`id`, `portlet_id`, `allow_right`) VALUES (null, 'Com_untermstrich_us_download', 1);
-- EOC

-- Remove Portlet_admin from db
DELETE FROM portlets WHERE portlet_id='Portlet_admin';
-- EOC

-- -------------------------------------------------------------
-- Move the prohibited portlets from the rightblock to a new tab
-- -------------------------------------------------------------
DROP PROCEDURE IF EXISTS move_prohibited_portlets_from_rightblock;
-- EOC

DELIMITER $$
CREATE PROCEDURE move_prohibited_portlets_from_rightblock()
BEGIN
    DECLARE cur_id INTEGER;
    DECLARE cur_portlet_id VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE cur_staff_id INTEGER;
    DECLARE cur_page_id INTEGER;
    DECLARE cur_tab_id INTEGER;
    DECLARE last_staff_id INTEGER;
    DECLARE last_page_id INTEGER;	
    DECLARE last_row_fetched INTEGER;

    DECLARE portlets_to_move CURSOR FOR SELECT id, portlet_id, staff_id, page_id FROM portlets where 
        position_id = 'rightblockcontent' AND
        portlet_id NOT IN(SELECT portlet_id from portlets_settings where allow_right = 1) ORDER BY staff_id, page_id, col;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET last_row_fetched=1;	

    SET last_row_fetched = 0;	
    SET last_staff_id = -1;
    SET last_page_id = -1;	
    OPEN portlets_to_move;
        cursor_loop: LOOP -- -------------------------------------
            FETCH portlets_to_move INTO cur_id, cur_portlet_id, cur_staff_id, cur_page_id;
            IF last_row_fetched = 1 THEN
                LEAVE cursor_loop;
            END IF;
            /*Have to reset page_id cause page 1 is available for all users*/
            IF (cur_staff_id != last_staff_id) THEN 
                SET last_page_id = -1;
            END IF;

            /*Create a new tab*/
            IF (cur_staff_id != last_staff_id OR cur_page_id != last_page_id) THEN 
                /* SET by SELECT */
                (SELECT (IFNULL(MAX(portlets_tabs.sort), 0)+1)
                    INTO @nextsort
                    FROM portlets_tabs WHERE staff_id= cur_staff_id AND page_id=cur_page_id);
                SET @s = CONCAT('INSERT INTO portlets_tabs (staff_id, page_id, name, `columns`, `sort`) VALUES (', cur_staff_id, ', ', cur_page_id,', "von rechts", 1, ', @nextsort, ')');
                PREPARE stmt11 FROM @s;
                EXECUTE stmt11;
                SET cur_tab_id = LAST_INSERT_ID();
            END IF;

            /*Move*/
            SET @nextrow = COALESCE((SELECT IFNULL(MAX(portlets.row), 0)+1 FROM portlets WHERE tab= cur_tab_id), 0);
            SET @s = CONCAT('UPDATE portlets SET tab=',cur_tab_id,', position_id="midblockcontent", `col`=0, `row`=',@nextrow,' WHERE id=',cur_id);
            PREPARE stmt11 FROM @s;
            EXECUTE stmt11;					

            /*Set last_..._id*/
            SET last_staff_id = cur_staff_id;
            SET last_page_id = cur_page_id;
        END LOOP cursor_loop; -- -------------------------------------
    CLOSE  portlets_to_move;		
    SET last_row_fetched = 0;	
END $$
DELIMITER ;
-- EOC

CALL move_prohibited_portlets_from_rightblock();
-- EOC

CREATE TABLE IF NOT EXISTS `fees_collective_bill` (
    `id` INT NOT NULL AUTO_INCREMENT,
    `collective_id` INT NOT NULL COMMENT 'correspondence ID of collective bill',
    `fees_id` INT NOT NULL,
    `correspondence_id` INT NULL,
    PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CREATE TABLE IF NOT EXISTS `project_collective_offer` (
    `id` INT NOT NULL AUTO_INCREMENT,
    `collective_id` INT NOT NULL COMMENT 'correspondence ID of collective offer',
    `offer_id` INT NOT NULL,
    `correspondence_id` INT NULL,
    PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

CREATE TABLE IF NOT EXISTS `email_projects` (
	`id` INT(11) NOT NULL AUTO_INCREMENT,
	`project_id` INT(11) NOT NULL,
	`email_account_id` INT(11) NOT NULL,
	`inbox` VARCHAR(255) NULL DEFAULT NULL,
	`outbox` VARCHAR(255) NULL DEFAULT NULL,
	`security` INT(11) NOT NULL DEFAULT '3',
	`delete_after_save` BOOLEAN NOT NULL DEFAULT '0',
	PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

DROP TABLE IF EXISTS `watchdog`;
-- EOC

CREATE TABLE IF NOT EXISTS `watchdog` (
	`keyname` VARCHAR(255) NOT NULL,
	`created` DATETIME NOT NULL,
	`updated` DATETIME NOT NULL,
	PRIMARY KEY (`keyname`)
) ENGINE=MEMORY  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
-- EOC

CREATE TABLE IF NOT EXISTS `address_signatures` (
	`id` INT(11) NOT NULL AUTO_INCREMENT,
        `name` VARCHAR(255) NULL DEFAULT NULL,
        `main` longtext NULL DEFAULT NULL,
        `main_plain` longtext NULL DEFAULT NULL,
        `main_subject` VARCHAR(255) NULL DEFAULT NULL,
        `appoi` longtext NULL DEFAULT NULL,
        `appoi_plain` longtext NULL DEFAULT NULL,
        `appoi_subject` VARCHAR(255) NULL DEFAULT NULL,
        `opta` longtext NULL DEFAULT NULL,
        `opta_plain` longtext NULL DEFAULT NULL,
        `opta_subject` VARCHAR(255) NULL DEFAULT NULL,
	PRIMARY KEY (`id`)
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

INSERT INTO address_signatures SELECT 1 as id, 'Standard' as `name`, 
null as main, null as main_plain, null as main_subject, null as appoi, null as appoi_plain, null as appoi_subject, null as opta, null as opta_plain, null as opta_subject
FROM address_signatures
WHERE (id=1)
HAVING COUNT(*) = 0;
-- EOC

CALL update_lut_fee_areas_offer_add_fields();
-- EOC

-- us_create_u10_session_log ---------------------------------------------------
DROP PROCEDURE IF EXISTS us_create_u10_session_log;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_create_u10_session_log()
BEGIN

    DECLARE sql1 TEXT DEFAULT 'CREATE TABLE IF NOT EXISTS `u10_session_log` (
        `info` varchar(500) NOT NULL,

        `session_id` varchar(40) COLLATE utf8_general_ci NULL DEFAULT NULL,
        `old1_session_id` varchar(40) COLLATE utf8_general_ci NULL DEFAULT NULL,
        `old2_session_id` varchar(40) COLLATE utf8_general_ci NULL DEFAULT NULL,
        `session_uniqid` varchar(40) COLLATE utf8_general_ci NULL DEFAULT NULL,
        `ip_address` varchar(16) COLLATE utf8_general_ci NULL DEFAULT NULL,
        `user_agent` varchar(50) COLLATE utf8_general_ci NULL DEFAULT NULL,
        `last_activity` int(10) unsigned NULL DEFAULT NULL,

        `aktuser_id` int(11) NULL DEFAULT NULL COMMENT "LfdNr_AktUser",
        `staff_id` int(11) NULL DEFAULT NULL COMMENT "UserNummer",
        `staff_name` varchar(50) NULL DEFAULT NULL COMMENT "UserName",
        `login_update_time` datetime NULL DEFAULT NULL COMMENT "LoginTime",
        `browser_info` varchar(50) NULL DEFAULT NULL COMMENT "WorkstationName",
        `aktuser_uniqid` varchar(50) NULL DEFAULT NULL COMMENT "WorkstationLogin",

        `date` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
    ) ENGINE=';

    DECLARE sql2 TEXT DEFAULT '  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;';

    IF EXISTS( (SELECT * FROM information_schema.ENGINES WHERE ENGINE='ARCHIVE' AND SUPPORT='YES') ) THEN
        SET @s = CONCAT(sql1, 'ARCHIVE', sql2);
    ELSE
        SET @s = CONCAT(sql1, 'MyISAM', sql2);
    END IF;
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

END $$
DELIMITER ;
-- EOC

CALL us_create_u10_session_log();
-- EOC

-- Add new field received_from -------------------------------------------------
CALL us_add_to_table2('drawings', 'received_from', 'INT( 11 ) DEFAULT NULL AFTER date' );
-- EOC

CALL us_add_to_table2('opendocs', 'session_uniqid', 'varchar(40) COLLATE utf8_general_ci NULL DEFAULT NULL AFTER `User_LfdNr`' );
-- EOC

-- Add new field lock_time - Will fail before 5.6.5
CALL us_add_to_table2('opendocs', 'lock_time', 'DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP AFTER `session_uniqid`' );
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Allow holiday_rates to be set inactive
CALL us_add_to_table2('feiertagsatz', 'is_active', 'INT( 11 ) DEFAULT 1' );
-- EOC

-- us_create_reports_history ---------------------------------------------------
DROP PROCEDURE IF EXISTS us_create_reports_history;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_create_reports_history()
BEGIN

    DECLARE sql1 TEXT DEFAULT 'CREATE TABLE IF NOT EXISTS `reports_history` (
        `report_id` VARCHAR(255) NOT NULL,
        `created_date` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
        `data` LONGTEXT NOT NULL,
        `for_date` DATE NOT NULL
    ) ENGINE=';

    DECLARE sql2 TEXT DEFAULT '  DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;';

    IF EXISTS( (SELECT * FROM information_schema.ENGINES WHERE ENGINE='ARCHIVE' AND SUPPORT='YES') ) THEN
        SET @s = CONCAT(sql1, 'ARCHIVE', sql2);
    ELSE
        SET @s = CONCAT(sql1, 'MyISAM', sql2);
    END IF;
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;

END $$
DELIMITER ;
-- EOC

CALL us_create_reports_history();
-- EOC

CALL remove_pk_column('reports_history', 'id');
-- EOC

CREATE TABLE IF NOT EXISTS `home_office` (
    `id` INT NOT NULL AUTO_INCREMENT,
    `staff_id` INT NOT NULL,
    `date` DATE NOT NULL,
    `in_home_office` TINYINT NOT NULL,
    `change_date` DATETIME NOT NULL,
    `changed_by_id` INT( 11 ) NOT NULL,
    PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC

-- More space for URLs
ALTER TABLE `perstermin` CHANGE COLUMN `Ort` `Ort` TEXT NULL DEFAULT NULL ;
-- EOC

ALTER TABLE `adressen` CHANGE COLUMN `URL` `URL` TEXT NULL DEFAULT NULL ;
-- EOC

-- End of table creation

-- USC Views werden angelegt.

-- --------------------------------------------------------------------------------------------------
CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `addresses`
 AS SELECT LfdNr AS id, name, bName AS name_deri, firma AS comp_supp, bFirma AS comp_supp_deri, anrede AS title, bAnrede AS title_deri, Titel AS academic_title,
 bTitel AS academic_title_deri, Master_Titel AS master_title, bMaster_Titel AS master_title_deri, Vorname AS first_name, bVorname AS first_name_deri,
 Ansprechpartner AS contact_person, bAnsprech AS contact_person_deri, Strasse AS street, bStrasse AS street_deri, Strasse2 AS street2, bStrasse2 AS street2_deri, Pltz AS zip, Ort AS town, bPltzOrt as zip_town_deri,
 Land AS country, bLand AS country_deri, Telefon1 AS phone1, bTelefon1 AS phone1_deri, Telefon2 AS phone2, bTelefon2 AS phone2_deri,
 Mobil AS mobile_phone, bMobil AS mobile_phone_deri, fax, bFax AS fax_deri, modem, bModem AS modem_deri, email, bEMail AS email_deri, UID_Nr AS vat, bUID_Nr AS vat_deri,
 URL AS url, bURL AS url_deri, ErstVon AS created_by_id, AendVon AS changed_by_id, ErstAm AS created_date,
 AendAm AS change_date, Aktiv AS active, bAktiv AS active_deri, Bilddatei AS image, creditor_no, debtor_no, dsgvo_delete_date, dsgvo_data_from,
 longitude, longitude_deri, latitude, latitude_deri, 
 bKategorien AS categories_deri, comment_main, comment_main_staff, comment_main_date, parent FROM adressen;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `addresses_fullname` AS
 SELECT id,
    CONCAT_WS( ', ',
        CASE WHEN NULLIF(contact_person, '') IS NULL
            THEN CONCAT_WS( ' ' , NULLIF(academic_title, ''), NULLIF(first_name, ''), name, NULLIF(master_title, '') )
            ELSE CONCAT_WS( ', ', name, CONCAT_WS( ' ', NULLIF(academic_title, '') , NULLIF(first_name, '') , NULLIF(contact_person, ''), NULLIF(master_title, '') ) )
        END,
        NULLIF(REPLACE(comp_supp, CHAR(13, 10), ' '), ''),
        NULLIF(town, '')
    ) AS fullname,
    CONCAT_WS( ', ',
        NULLIF(NULLIF(creditor_no, '0'), ''),
        NULLIF(NULLIF(debtor_no, '0'), ''),
        CASE WHEN NULLIF(contact_person, '') IS NULL
            THEN CONCAT_WS( ' ' , NULLIF(academic_title, ''), NULLIF(first_name, ''), name, NULLIF(master_title, '') )
            ELSE CONCAT_WS( ', ', name, CONCAT_WS( ' ', NULLIF(academic_title, '') , NULLIF(first_name, '') , NULLIF(contact_person, ''), NULLIF(master_title, '') ) )
        END,
        NULLIF(REPLACE(comp_supp, CHAR(13, 10), ' '), ''),
        NULLIF(street, ''),
        NULLIF(street2, ''),
        CONCAT_WS( ' ', NULLIF(zip, '') , NULLIF(town, '') )
    ) AS fullnamelong,
    CASE WHEN NULLIF(contact_person, '') IS NULL
            THEN CONCAT_WS( ' ' , NULLIF(academic_title, ''), NULLIF(first_name, ''), name, NULLIF(master_title, '') )
        ELSE CONCAT_WS( ', ', CONCAT_WS( ' ', NULLIF(academic_title, '') , NULLIF(first_name, '') , NULLIF(contact_person, ''), NULLIF(master_title, '') ) )
    END AS fullnameshort,
    CASE WHEN NULLIF(contact_person, '') IS NULL
        THEN CONCAT_WS('; ', name, first_name, '', CONCAT_WS(',', NULLIF(title, ''), NULLIF(academic_title, '')), master_title )
        ELSE CONCAT_WS('; ', contact_person, first_name, '', CONCAT_WS(',', NULLIF(title, ''), NULLIF(academic_title, '')), master_title)
    END as fullnamevcardn,
    CASE WHEN NULLIF(contact_person, '') IS NULL
        THEN ''
        ELSE CONCAT_WS( ', ',
            name,
            NULLIF(REPLACE(comp_supp, CHAR(13, 10), ' '), '')
        )
    END AS fullnameorg,
 name,
 active,
 email,
 comment_main,
 comment_main_staff,
 comment_main_date,
 parent
 FROM addresses;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `addresses_addi_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color, Laenge AS length, Sicherheit AS security FROM zusatz_adressen_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `staff_additional_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color, Laenge AS length, Sicherheit AS security FROM zusatz_personen_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `address_categories`
 AS SELECT LfdNr AS id, Bezeichnung AS name, Beschreibung as purpose, company_id FROM adrkategorien;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `address_descriptions`
 AS SELECT LfdNr AS id, LfdNr_Adr AS address_id, unique_add_id, call_id, beschreibung AS description, wer AS staff_id, wann AS created_date,
 prioritaet AS security, LfdNr_Proj AS project_id FROM adrbeschr;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `address_files`
 AS SELECT LfdNr AS id, LfdNr_Adr AS address_id, Datei AS file, DateiBeschreibung AS description,
 DateiPosition AS sort,
 ErstVon AS created_by_id, AendVon AS changed_by_id, ErstAm AS created_date,
 AendAm AS change_date FROM lut_adr_dateien;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `address_groups`
 AS SELECT LfdNr AS id, GruppenName AS name, company_id, LfdNr_Projekt AS project_id FROM adrgruppendef;
-- EOC

CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `zip_to_name_merged` AS
 SELECT zip, town, country, COUNT( * ) AS count_hits FROM addresses GROUP BY zip, town, country
 UNION
 SELECT tzip AS zip, tname AS town, tcountry AS country, 0 FROM zip_to_name;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `companies`
 AS SELECT LfdNr AS id, name, address_id, canDel as can_del, aktiv as active, currency_id FROM firmen;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `cost_centres`
 AS SELECT LfdNr AS id, Kostenstelle AS name, Bemerkung AS remark, company_id, use_in_reports FROM kostenstellen;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `countries`
 AS SELECT LfdNr as id, Bez as name, Kennzeichen as country_code FROM land;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `expenses_rates`
 AS SELECT LfdNr AS id, Beschreibung AS description, RKSatz AS travel_name, ExtraSatz AS extras_name, MASatz AS times_name,
 ABSatz AS work_areas_name, company_id, currency_id, aktiv AS is_active FROM projektabrsaetze;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `extras_types`
 AS SELECT LfdNr AS id, OrdNr AS sort, Bezeichnung AS type, Einheit AS unit, KostenEinheit AS rate, company_id, currency_id, 
 MwStKopie AS vat, active FROM kopiekat;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `extras_rates`
 AS SELECT LfdNr AS id, Satz AS rate_id, Extra AS extra_id, NeuerSatz AS rate, LfdNr_ProjektAbrExtra_Zeit AS history_id FROM projektabrextra;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `extras_rates_history`
 AS SELECT `LfdNr` AS id, `LfdNr_Satz` AS rate_id, `von` AS date_from, `bis` AS date_to
 FROM `projektabrextra_zeit`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `locking`
 AS SELECT Modul AS module, LfdNr_Locking AS id_to_lock, LfdNr_User AS staff_id, LockingTime AS locking_time FROM tbllocking;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_addresses_address_categories`
 AS SELECT LfdNr_Adr AS address_id, LfdNr_Kat AS address_category_id FROM adrkat_lut;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_addresses_address_group`
 AS SELECT AdressNr AS address_id, GruppenNr AS address_group_id FROM adrgruppenmitglieder;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_addresses_projects`
 AS SELECT LfdNr AS id, Person AS address_id, Projekt AS project_id, LfdNr_Kat AS address_category_id, isFreigeber AS auto_plan_approv, Bemerkung as purpose,
 Reihenfolge AS sort FROM propersonen;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_proj_category_rates`
 AS SELECT LfdNr AS id, Projekt AS project_id,
 SatzExtra AS extras_rate_id, SatzRK AS travels_rate_id, SatzMA AS times_rate_id, SatzAB AS areas_rate_id, warnschwelle AS threshold FROM lut_projektabr;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_proj_proj_grps`
 AS SELECT LfdNr AS id, Projekt AS project_id, ProGrupNr AS project_grp_id FROM progrp_lut;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_route_proj`
 AS SELECT LfdNr AS id, Proj_LfdNr AS project_id, Ort_LfdNr AS route_id FROM lut_orte_proj;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `occupation_groups`
 AS SELECT LfdNr AS id, LfdNr_Mitarb AS staff_id, KurzBesch AS name, Beschreibung AS description, company_id FROM gruppen;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_staff_occ_group`
 AS SELECT LfdNr AS id, LfdNr_Pers AS staff_id, LfdNr_Grp AS group_id, von AS fr_om_date, bis AS to_date, unique_add_id, description FROM lut_pers_grp;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_task_desc_proj`
 AS SELECT LfdNr AS id, Proj_LfdNr AS project_id, ArbBesch_LfdNr AS tdesc_id FROM lut_arbbeschtxte_proj;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `occupation_groups_fee_multipliers`
 AS SELECT  LfdNr AS id, LfdNr_Grp AS group_id, von AS fr_om_date, bis AS to_date,
 Satz1 AS percntg1, Satz2 AS percntg2, Satz3 AS percntg3, Satz4 AS percntg4, Satz5 AS percntg5, Satz6 AS percntg6, Satz7 AS percntg7, Satz8 AS percntg8, Satz9 AS percntg9,
 Betrag1 AS value1, Betrag2 AS value2, Betrag3 AS value3, Betrag4 AS value4, Betrag5 AS value5, Betrag6 AS value6, Betrag7 AS value7, Betrag8 AS value8, Betrag9 AS value9,
 currency_id FROM gruppen_zeit;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `overtime`
 AS SELECT LfdNr AS id, Projekt AS project_id, Person AS staff_id, Datum AS date, Anzahl AS number, UeStdSatz AS overtime_rate,
 SatzArt AS overtime_rate_id, Lagerung AS storage, Ausbezahlt AS paid, WerHatAusb AS paid_by_staff_id, company_id FROM uestd;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `overtime_rates`
 AS SELECT LfdNr AS id, OrdNr AS sort, SatzArt AS name, Satz AS rate, active, Drucken AS print, company_id AS company_id FROM uesatz;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `projects_addi_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color,  Laenge AS length, Sicherheit AS security FROM zusatz_projekt_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `project_participants_addi_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color,  Laenge AS length, Sicherheit AS security FROM zusatz_propers_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `projects_groups`
 AS SELECT LfdNr AS id, ProGrupName AS name, filter_id, filter_hash, filter_last_count, filter_creator_info FROM progruppendef;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `project_categories`
 AS SELECT LfdNr as id, OrdNr AS sort, Kategorie AS name, Bemerkung as purpose
 FROM prokategorie;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `routes`
 AS SELECT LfdNr AS id, Ort AS name, Km AS value, company_id FROM orte;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `staff`
 AS SELECT LfdNr AS id, Name AS staff_name, Passwort AS password, passwort_mode AS password_mode, Strasse AS street, Ort AS town, Pltz AS zip_code, Telefon1 AS phone_1, Telefon2 AS phone_2,
 Fax AS fax, Start AS start_date, Ende AS end_date, SVNummer AS insurance_no, Geschlecht AS gender, Bemerkung AS remark,
 ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id, BLZ AS bank_code, Bank AS bank_name, KontoNr AS account_no,
 Kurzzeichen AS abbreviation, DW AS direct_dial, EMail AS email, Zusatz AS annex, FeiertagSatz AS holiday_rate_id, GebDatum AS birthday,
 ModSecurity01 AS security_1, ModSecurity02 AS security_2, ModSecurity03 AS security_3, ModSecurity04 AS security_4, ModSecurity05 AS security_5,
 ModSecurity06 AS security_6, ModSecurity07 AS security_7, ModSecurity08 AS security_8, ModSecurity09 AS security_9, ModSecurity10 AS security_10, ModSecurity11 AS security_11,
 ModSecurity12 AS security_12, ModSecurity13 AS security_13, ModSecurity14 AS security_14, ModSecurity15 AS security_15, ModSecurity16 AS security_16, ModSecurity17 AS security_17,
 ModSecurity18 AS security_18, ModSecurity19 AS security_19, ModSecurity20 AS security_20, ModSecurity21 AS security_21, ModSecurity22 AS security_22, ModSecurity23 AS security_23,
 ModSecurity24 AS security_24, ModSecurity25 AS security_25, ModSecurity26 AS security_26, ModSecurity27 AS security_27, ModSecurity28 AS security_28, ModSecurity29 AS security_29,
 ModSecurity30 AS security_30, ModSecurity31 AS security_31, ModSecurity32 AS security_32, ModSecurity33 AS security_33, ModSecurity34 AS security_34, ModSecurity35 AS security_35,
 ModSecurity36 AS security_36, ModSecurity37 AS security_37, ModSecurity38 AS security_38, ModSecurity39 AS security_39, ModSecurity40 AS security_40, ModSecurity41 AS security_41,
 ModSecurity42 AS security_42, ModSecurity43 AS security_43, ModSecurity44 AS security_44, ModSecurity45 AS security_45, ModSecurity46 AS security_46, ModSecurity47 AS security_47,
 ModSecurity48 AS security_48, ModSecurity49 AS security_49, ModSecurity50 AS security_50,
 secdetail_2 AS secd_2, secdetail_3 AS secd_3, secdetail_5 AS secd_5, secdetail_16 AS secd_16, 
 Firmen_ID AS company_id, staff_location_id, UseLeiterInfo AS respect_project_manager, IstProjektLeiter AS is_project_manager, DientUnter AS project_manager_id, IsSU AS is_superuser,
 KstPrioritaet AS cost_centre_present, EditHowManyDays AS change_days_past, days_warning_no_workinput, days_warning_no_mail_allocation, Lehrling AS exclude_from_turnover,
 Urlaub_Stichtag AS holiday_deadline, Urlaub_Anspruch AS add_holiday_balance, Bilddatei AS photo_file, currency_id, username FROM personen;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `staff_favourite_proj`
 AS SELECT LfdNr AS id, Person AS staff_id, Projekt AS project_id FROM stechprojekte;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `start_end_times`
 AS SELECT LfdNr AS id, Person AS staff_id, Datum AS date, ZeitBeginn AS start_time, ZeitEnde AS end_time, ZeitAbzug AS time_deduct, company_id FROM zeiten_beginnende;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `system_settings`
 AS SELECT LfdNr AS id, SettingType AS name, SettingValue AS value, company_id FROM settingssystem;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `sys_installation`
 AS SELECT LfdNr AS id, Seriennummer AS serial,
 mod01, mod02, mod03, mod04, mod05, mod06, mod07, mod08, mod09, mod10, mod11, mod12, mod13, mod14, mod15, mod16, mod17, mod18,
 mod19, mod20, mod21, mod22, mod23, mod24, mod25, mod26, mod27, mod28, mod29, mod30, mod31, mod32, mod33, mod34, mod35, mod36,
 mod37, mod38, mod39, mod40, mod41, mod42, mod43, mod44, mod45, mod46, mod47, mod48, mod49, mod50, mod51, mod52, mod53, mod54,
 mod55, mod56, mod57, mod58, mod59, mod60,
 Ablauf AS end_date, DemoVersion AS is_demo,
 r1, r2, r3,
 instname AS name, instadresse AS address, insttel AS telephone, instinfo FROM installation;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `target_hours_daily`
 AS SELECT LfdNr AS id, Pers_LfdNr AS staff_id, Nz01 AS norm_work_mon, Nz02 AS norm_work_tue, Nz03 AS norm_work_wed, Nz04 AS norm_work_thu,
 Nz05 AS norm_work_fri, Nz06 AS norm_work_sat, Nz07 AS norm_work_sun, TagSollvon AS fr_om_date, TagSollbis AS to_date, ErstAm AS created_date,
 ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id, unique_add_id, description
 FROM tagessollstd;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `task_descriptions`
 AS SELECT LfdNr AS id, active, OrdnNr AS sort, ArbBeschTxt AS name, company_id FROM arbbeschtxte;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `times_rates`
 AS SELECT LfdNr AS id, Satz AS rate_id, Mitarbeiter AS times_id, NeuerSatz AS rate,
 PersonenSatz AS external_staff_rate, LfdNr_ProjektAbrMA_Zeit AS history_id FROM projektabrma;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `times_rates_history`
 AS SELECT `LfdNr` AS id, `LfdNr_Satz` AS rate_id, `von` AS date_from, `bis` AS date_to
 FROM `projektabrma_zeit`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `travels_rates`
 AS SELECT LfdNr AS id, Satz AS rate_id, RK AS travel_id, NeuerSatz AS rate, LfdNr_ProjektAbrRK_Zeit AS history_id FROM projektabrrk;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `travels_rates_history`
 AS SELECT `LfdNr` AS id, `LfdNr_Satz` AS rate_id, `von` AS date_from, `bis` AS date_to
 FROM `projektabrrk_zeit`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `travels_types`
 AS SELECT LfdNr AS id, OrdNr AS sort, KostenArt AS type, Einheit AS unit, Satz AS rate, KmVonBis AS km_from_to,
 ZeitVonBis AS time_from_to, company_id, currency_id, MwStReise AS vat, active FROM reisekosten;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `user_settings`
 AS SELECT LfdNr AS id, LfdNr_User AS staff_id, SettingType AS name, SettingValue AS value FROM settingsuser;
-- EOC

-- Create  view for the address_descriptions including the firmen_id
CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `address_descriptions_comp` AS
 SELECT address_descriptions . * , staff.company_id
 FROM address_descriptions
 LEFT JOIN staff ON ( address_descriptions.staff_id = staff.id );
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `company_installation_merged`  AS
 SELECT name, address, telephone, company_id
 FROM company_installation
 UNION
 SELECT name, address, telephone, 1+0 AS company_id
 FROM sys_installation;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `holiday_rates`
 AS SELECT LfdNr AS id, Beschreibung AS title, is_active
 FROM `feiertagsatz`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `holidays`
 AS SELECT LfdNr AS id, Satz AS holiday_rate_id, Datum AS date, Beschreibung AS description, Stunden AS hour_deduction, Faktor AS rate
 FROM `feiertage`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `deletions` AS
 SELECT LfdNr AS id, Wann AS date, Wer AS staff, staff_id, Wo AS title, Was AS info,
 item_model, item_title, item_id, isCompressed,
 ModNr AS moduleid
 FROM `erased`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `personalappointments` AS
 SELECT LfdNr as id, Termin as date, Zeit as time_from, BisZeit as time_to, Projekt as project_id, Von as staff_from_id, An as staff_to_id,
 Ort as location,
 Bemerkung as comment, ErstAm as created_date, ErstVon as created_by_id, AendAm as change_date, AendVon as changed_by_id, Neu as is_new, Privat as is_private, FirstLfdNr multistaff_first_id, SeveralDays as several_days, SeveralNumber as several_days_first_id, TerminBis as several_days_last_date, Tag as several_days_day, Periodical as periodical, TerminArt as app_type, PeriodeBis as periodical_last_date, PeriodNumber as periodical_first_id, IsEr as is_notify, ErDate as notify_date, TerminStatus as status, LfdNr_Urlaub as vacation_id, bZeit as changed_time_from,
 bOrt as changed_location,
 bBemerkung as changed_comment, bBisZeit as changed_time_to, bProjekt as changed_project_id, bIsEr as changed_is_notify, bErDate as changed_notify_date, bTerminStatus as changed_status, state_color_id
 FROM perstermin;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_personalapp_address` AS
 SELECT LfdNr AS id, LfdNr_Termin AS personalapp_id, LfdNr_Adr AS address_id, OrdNr as sort
 FROM lut_termin_teilnehmer;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `opentasks` AS
 SELECT LfdNr AS id, Projekt AS project_id, Beschreibung AS purpose, bis AS todo_date, Erledigt AS is_done, ErlVon AS done_staff_id,
 ErlAm AS done_date, ErlVermerk AS done_annotation, ZuTunVon AS todo_id, Person AS creator_id, Datum AS creation_date, ErstAm AS created_date,
 ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id, Neu AS is_new, Privat AS is_private, prioritaet AS need_mgmt_right,
 dringend AS is_urgent, Mail_File AS msg_file, IsPostAusgang AS module_type, lfdnr_post AS module_id, IsEr AS is_notify, ErDate AS notify_date,
 LfdNr_Adr AS address_id, Bemerkung AS description, active_from, ToDoParent AS parent_id, period_value, unit, working_hours, work_package_id
 FROM todo;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `opentasks_addi_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color,  Laenge AS length, Sicherheit AS security FROM zusatz_todo_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `vacations`
 AS SELECT LfdNr AS id, LfdNr_Mitarbeiter AS staff_id, LfdNr_Projekt AS project_id, UV_DatumVon AS start_date, UV_DatumBis AS end_date,
 UV_Tage AS days, UV_Bemerkung AS comment , UV_OK AS holiday_is_ok, UV_Beantragt AS is_submitted, UV_NO AS is_refused, UV_ZA AS is_absent, is_absent_social,
 ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id, EingabeSaved AS imported_workinput,
 PersTerminSaved AS imported_persapp, aufgestockt AS increased, Saldo AS balance
 FROM uv_buchungen;
-- EOC










CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `externalexpenses_calc` AS
 SELECT  id, project_id, purpose, description, value, 
 (value * (1+turnover_rate)) AS value_pre_tax, surcharge_used,
 address_id,
 area_id, creator_id, date, created_date, created_by_id, change_date,
 changed_by_id, cost_centre_id, is_hon_decr, is_calc_from_expenses, charge_date,
 charged, 0 AS is_transfered, 1+0 AS is_invoiced, turnover_rate, NULL AS valid_from, work_package_id, fee_category_id,
 currency_id, value_otcurr, (value_otcurr * (1+turnover_rate)) AS value_otcurr_pre_tax
 FROM externalexpenses;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `externalexpenses_planned_calc` AS
 SELECT  id, project_id, purpose, description, value, 
 (value * (1+turnover_rate)) AS value_pre_tax, surcharge_used,
 address_id,
 area_id, creator_id, date, created_date, created_by_id, change_date,
 changed_by_id, cost_centre_id, is_hon_decr, is_calc_from_expenses, charge_date,
 charged, is_transfered, 0 AS is_invoiced, turnover_rate, valid_from, work_package_id, fee_category_id,
 currency_id, value_otcurr, (value_otcurr * (1+turnover_rate)) AS value_otcurr_pre_tax
 FROM externalexpenses_planned;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `externalexpenses_merged`  AS
 SELECT id, project_id, purpose, description, value, value_pre_tax, surcharge_used, address_id, area_id, creator_id, date, created_date, created_by_id, change_date,
 changed_by_id, cost_centre_id, is_hon_decr, is_calc_from_expenses, charge_date, charged, is_transfered, is_invoiced, turnover_rate, valid_from, work_package_id,
 fee_category_id, currency_id, value_otcurr, value_otcurr_pre_tax
 FROM externalexpenses_calc
 UNION
 SELECT id, project_id, purpose, description, value, value_pre_tax, surcharge_used, address_id, area_id, creator_id, date, created_date, created_by_id, change_date,
 changed_by_id, cost_centre_id, is_hon_decr, is_calc_from_expenses, charge_date, charged, is_transfered, is_invoiced, turnover_rate, valid_from, work_package_id,
 fee_category_id, currency_id, value_otcurr, value_otcurr_pre_tax
 FROM externalexpenses_planned_calc;
-- EOC





CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `externalexpenses_addi_merged_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color, Laenge AS length, Sicherheit AS security FROM zusatz_extkosten_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `externalexpenses_addi_invoiced_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color, Laenge AS length, Sicherheit AS security FROM zusatz_extkosten_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `externalexpenses_addi_planned_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color, Laenge AS length, Sicherheit AS security FROM zusatz_extkosten_def;
-- EOC





CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `fees_calc` AS
 SELECT  id, project_id, purpose, value, address_id,
 area_id, user_id, fee_date, fee_no, created_date, created_by_id, change_date,
 changed_by_id, cost_centre_id, ebinterface_json, zugferd_json, xrechnung_json, fatturapa_json, e_invoice_used, entry_date, is_done, done_date,
 fee_account_id, turnover_rate, remited, remit_purpose, predecessor_id,
 successor_id, due_date, reminder_level, last_reminder_date, is_percentage,
 deduction,
 (value * (1+turnover_rate)) AS value_pre_tax, 
 (value * (1+turnover_rate) - remited) AS diff_pre_tax, 
 null AS due_date_planned, 1+0 AS is_invoiced, 0 AS is_transfered, (remited/(turnover_rate*100+100)*100) AS remited_after_tax, correspondence_id,
 is_calcexp, discount, discount_date, work_package_id, finally_invoiced, fee_category_id,
 currency_id, value_otcurr, remited_otcurr, deduction_otcurr,
 (value_otcurr * (1+turnover_rate)) AS value_otcurr_pre_tax, 
 (value_otcurr * (1+turnover_rate) - remited_otcurr) AS diff_otcurr_pre_tax,
 (remited_otcurr/(turnover_rate*100+100)*100) AS remited_otcurr_after_tax,
 retention_type, retention_date, retention
 FROM fees;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `fees_planned_calc` AS
 SELECT  id, project_id, purpose, value, address_id,
 area_id, user_id, fee_date, fee_no, created_date, created_by_id, change_date,
 changed_by_id, cost_centre_id, ebinterface_json, zugferd_json, xrechnung_json, fatturapa_json, e_invoice_used, entry_date, is_done, done_date,
 fee_account_id, turnover_rate, remited, remit_purpose, predecessor_id,
 successor_id, due_date, reminder_level, last_reminder_date, is_percentage,
 deduction, 
 (value * (1+turnover_rate)) AS value_pre_tax, 
 (value * (1+turnover_rate) - remited) AS diff_pre_tax, 
 due_date_planned, 0 AS is_invoiced, is_done AS is_transfered, (remited/(turnover_rate*100+100)*100) AS remited_after_tax, correspondence_id,
 is_calcexp, discount, discount_date, work_package_id, finally_invoiced, fee_category_id,
 currency_id, value_otcurr, remited_otcurr, deduction_otcurr,
 (value_otcurr * (1+turnover_rate)) AS value_otcurr_pre_tax, 
 (value_otcurr * (1+turnover_rate) - remited_otcurr) AS diff_otcurr_pre_tax,
 (remited_otcurr/(turnover_rate*100+100)*100) AS remited_otcurr_after_tax,
 retention_type, retention_date, retention
 FROM fees_planned;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `fees_merged`  AS
 SELECT id, project_id, purpose, value, address_id, area_id, user_id, fee_date, fee_no, created_date, created_by_id, change_date, changed_by_id, cost_centre_id, ebinterface_json, zugferd_json, e_invoice_used,
 xrechnung_json, fatturapa_json, entry_date, is_done, done_date, fee_account_id, turnover_rate, remited, remit_purpose, predecessor_id, successor_id, due_date, reminder_level,
 last_reminder_date, is_percentage, deduction,
 value_pre_tax, diff_pre_tax, due_date_planned, is_invoiced, is_transfered, remited_after_tax, correspondence_id, is_calcexp, discount, discount_date, 
 work_package_id, finally_invoiced, fee_category_id, currency_id, value_otcurr, remited_otcurr, deduction_otcurr,value_otcurr_pre_tax, diff_otcurr_pre_tax, remited_otcurr_after_tax,
 retention_type, retention_date, retention
 FROM fees_calc
 UNION
 SELECT id, project_id, purpose, value, address_id, area_id, user_id, fee_date, fee_no, created_date, created_by_id, change_date, changed_by_id, cost_centre_id, ebinterface_json, zugferd_json, e_invoice_used,
 xrechnung_json, fatturapa_json, entry_date, is_done, done_date, fee_account_id, turnover_rate, remited, remit_purpose, predecessor_id, successor_id, due_date, reminder_level,
 last_reminder_date, is_percentage, deduction,
 value_pre_tax, diff_pre_tax, due_date_planned, is_invoiced, is_transfered, remited_after_tax, correspondence_id, is_calcexp, discount, discount_date, 
 work_package_id, finally_invoiced, fee_category_id, currency_id, value_otcurr, remited_otcurr, deduction_otcurr, value_otcurr_pre_tax, diff_otcurr_pre_tax, remited_otcurr_after_tax,
 retention_type, retention_date, retention
 FROM fees_planned_calc;
-- EOC




CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `fees_addi_merged_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color, Laenge AS length, Sicherheit AS security FROM zusatz_honverw_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `fees_addi_invoiced_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color, Laenge AS length, Sicherheit AS security FROM zusatz_honverw_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `fees_addi_planned_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color, Laenge AS length, Sicherheit AS security FROM zusatz_honverw_def;
-- EOC




CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `fees_reminder_level_logging` AS
 SELECT  LfdNr AS id, Hon_LfdNr AS fee_id, MahnStufe AS reminder_level, MahnDatum as reminder_date
 FROM honverwmahnung;
-- EOC

-- Update extras views
CALL update_extras_views_do();
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `project_phases`
 AS SELECT LfdNr as id, PhasenBez as name, Bemerkung AS purpose, OrdNr AS sort FROM phasen;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `fee_accounts`
 AS SELECT LfdNr as id, Blz as bank_code, Bank AS bank, KNR AS account_number, KontoBez AS account_designation, Firmen_ID AS company_id,
 inaktiv AS inactive FROM honkonten;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `project_archives`
 AS SELECT LfdNr as id, ArchivBezeichnung as name FROM projektarchive;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `workinput` AS
	(SELECT id, 1+0 AS time, 0 AS travel, 0 AS extra, date, staff_id FROM times)
	UNION ALL
	(SELECT id, 0 AS time, 1+0 AS travel, 0 AS extra, date, staff_id FROM travels)
	UNION ALL
	(SELECT id, 0 AS time, 0 AS travel, 1+0 AS extra, date, staff_id FROM extras);
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `measurements` AS
 SELECT LfdNr as id, MessSize_Name as name FROM messsize;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `address_relations_comp`
 AS SELECT adre.*,
 adrf.fullname AS address_name,
 from_adrf.fullname AS from_address_name,
 staff.staff_name AS from_staff_name,
 CONCAT_WS(' - ', peap.comment, peap.location) AS from_personalapp_comment,
 CONCAT_WS(' | ', NULLIF(from_adrf.fullname, ''), NULLIF(adre.description, ''), NULLIF(peap.comment, ''), NULLIF(staff.staff_name, '')) AS title
 FROM `address_relations` AS adre
 LEFT JOIN `staff` ON adre.from_staff_id=staff.id
 LEFT JOIN `addresses_fullname` AS adrf ON adre.address_id=adrf.id
 LEFT JOIN `addresses_fullname` AS from_adrf ON adre.from_address_id=from_adrf.id
 LEFT JOIN `personalappointments` AS peap ON adre.from_personalapp_id=peap.id;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `address_relations_both`
 AS
  SELECT adrecomp.id,
  adrecomp.address_id, adrecomp.address_name, 1+0 AS 'direct',
   adrecomp.description, adrecomp.date,
   adrecomp.from_staff_id, adrecomp.from_staff_name,
   adrecomp.from_personalapp_id, adrecomp.from_personalapp_comment,
   adrecomp.from_address_id, adrecomp.from_address_name,
  adrecomp.address_id AS first_address_id,
  adrecomp.from_address_id as snd_address_id,
  adrecomp.title
  FROM `address_relations_comp` AS adrecomp
 UNION ALL
  SELECT adrecomp.id,
  adrecomp.address_id, adrecomp.address_name, 0 AS 'direct',
   adrecomp.description, adrecomp.date,
   adrecomp.from_staff_id, adrecomp.from_staff_name,
   adrecomp.from_personalapp_id, adrecomp.from_personalapp_comment,
   adrecomp.from_address_id, adrecomp.from_address_name,
  adrecomp.from_address_id as first_address_id,
  adrecomp.address_id AS snd_address_id,
  CONCAT_WS(' | ', NULLIF(adrecomp.address_name, ''), NULLIF(adrecomp.description, ''), NULLIF(adrecomp.from_personalapp_comment, ''), NULLIF(adrecomp.from_staff_name, '')) AS title
  FROM `address_relations_comp` AS adrecomp;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `locking_info` AS SELECT `module`, staff_id FROM locking
UNION
 SELECT `module`, id_to_lock AS staff_id FROM locking
 WHERE `module` = 'UNTERMSTRICH.EINGABE.CXARBEITSEINGABE';
-- EOC



CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `wiw_items` AS
 SELECT LfdNr AS id, Artikel AS title, autor AS author, buchtitel AS article_id, seite AS page,
 memofeld AS note, adressat AS address_id,
 file
 FROM `artikeltab`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `wiw_articles` AS
 SELECT LfdNr AS article_id, buchtitel AS article_title, ausgabe AS version, eingelangtAm AS in_date, autor AS article_author,
 BTRaum AS room_id, BTRregal AS rack_id, BTFach AS tray_id, BTAblage AS filing_id, BTadressat AS address_id, BTadressatJN AS addressee_yn
 FROM `buchtiteltab`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `wiw_rooms` AS
 SELECT LfdNr AS id, raum AS title
 FROM `raumtab`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `wiw_racks` AS
 SELECT LfdNr AS id, regal AS title, lfdNrRegal AS room_id
 FROM `regaltab`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `wiw_trays` AS
 SELECT LfdNr AS id, fach AS title, lfdnrFach AS rack_id
 FROM `fachtab`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `wiw_filings` AS
 SELECT LfdNr AS id, ablage AS title, lfdnrAblage AS tray_id
 FROM `ablagetab`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `lut_wiw_items_projects` AS
 SELECT LfdNr_Artikel AS item_id, LfdNr_Proj AS project_id
 FROM `lut_artikel_proj`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `lut_wiw_items_themes` AS
 SELECT ArtikellfdNr AS item_id, ThemenlfdNr AS theme_id, StandThLfdnr as stdtheme_id
 FROM `zwischtab`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = UNDEFINED
 VIEW `wiw_themes` AS
 SELECT LfdNr AS id, Thema AS theme_name
 FROM `thementab`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `wiw_merged` AS
 SELECT wiw_items.id, wiw_items.title, wiw_items.author, wiw_items.page, wiw_items.note, wiw_items.file,
 CONCAT_WS('/', wiw_articles.article_title, wiw_articles.version) AS article_fulltitle,
 wiw_articles.article_id, wiw_articles.article_title, wiw_articles.version,
 wiw_rooms.title AS room_title, wiw_racks.title AS rack_title,
 wiw_trays.title AS tray_title, wiw_filings.title AS filing_title, wiw_items.address_id
 FROM wiw_items
 INNER JOIN wiw_articles ON wiw_items.article_id=wiw_articles.article_id
 LEFT JOIN wiw_rooms ON wiw_rooms.id=wiw_articles.room_id 
 LEFT JOIN wiw_racks ON wiw_racks.id=wiw_articles.rack_id 
 LEFT JOIN wiw_trays ON wiw_trays.id=wiw_articles.tray_id
 LEFT JOIN wiw_filings ON wiw_filings.id=wiw_articles.filing_id;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `whatiswhere_addi_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color, Laenge AS length, Sicherheit AS security FROM zusatz_wasistwo_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `calls` AS 
 SELECT LfdNr AS id, `Datum` AS date_from, beendet AS date_to, disconnected, `RufNummer` AS phone, `ausgehend` AS active_call, `angenommen` AS received,
 `LfdNr_Adr` AS address_id, `LfdNr_Adr_Caller` AS caller_address_id, `No_Adr_Caller` AS caller_no_address, `LfdNr_User` AS staff_id, last_update FROM `telefonate`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `drawings_addi_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color, Laenge AS length, Sicherheit AS security FROM zusatz_planverz_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_drawings_addresses`
 AS SELECT LfdNr AS id, LfdNr_Adr AS address_id, LfdNr_Plan AS drawing_id, unique_add_id,
 isFrei AS is_released, FreiDate AS released_on, FreiBis AS released_until FROM lut_plan_adr;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `drawing_sizes`
 AS SELECT LfdNr AS id, Bez AS name, Flaeche AS area, company_id  FROM plangroessen;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `drawing_types`
 AS SELECT LfdNr AS id, ArtText AS name, Bemerkung AS purpose, company_id FROM planart;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `drawings_outbox` AS
 SELECT LfdNr AS id, An AS address_id, plan AS drawing_id, Anzahl AS amount, Datum AS `date`,  Bemerkung AS purpose, LieferScheinNr AS transmittal_form_id,
 ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id
 FROM planausgabe;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_transmittal_form_addresses` AS
 SELECT `LfdNr` AS id, `LfdNrLieferschein` AS transmittal_form_id, `LfdNrAdresse` AS address_id, `OrdNr` AS sort 
 FROM `lut_lieferscheinadresse`;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `outbox_path` AS
 SELECT outb.id, outb.project_id, p.path_outbox
 FROM outbox AS outb
 LEFT JOIN projects AS p ON outb.project_id=p.id;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_outbox_bcc_addresses`
 AS SELECT LfdNr AS id, LfdNr_PoA AS outbox_id, LfdNr_AdrBCC AS address_id  FROM poausbcc;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_outbox_attachments`
 AS SELECT LfdNr AS id, LfdNr_PoA AS outbox_id, PoAtt_Pfad AS `path`, PoBeschreibung AS description  FROM poausattachments;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `outbox_addi_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color,  Laenge AS length, Sicherheit AS security FROM zusatz_postaus_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `inbox_addi_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color,  Laenge AS length, Sicherheit AS security FROM zusatz_postein_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `inbox_path` AS
 SELECT inb.id, inb.project_id, p.path_inbox
 FROM inbox AS inb
 LEFT JOIN projects AS p ON inb.project_id=p.id;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `dispatch_modes`
 AS SELECT LfdNr AS id, ArtText AS name, Bemerkung AS purpose, company_id  FROM postart;
-- EOC

-- ALSO CHANGE FOR VIEW correspondence_readonly
CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `correspondence` AS
 SELECT LfdNr AS id, Person AS staff_id, Projekt AS project_id, ArtForm AS `correspondence_type_id`, Betrifft AS subject, Empfaenger AS address_id, 
 Text AS text, Datum AS `date`, nurText AS search_text, Serie AS is_serial, Datei AS `file`, Stufe AS security, security_zip, Seiten AS pages, Att_Anz AS attachments, 
 Kopien_Anz AS copies, ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id, PoAusWieOft AS outbox_times,
 archiviert AS is_archived, InHonverwExp AS is_exported_to_fees, RechnungsBetrag AS fee_value, RechnungsNummer AS fee_no, FaelligAm AS due_date, 
 ust AS vat, Rechnungsdatum AS fee_date, Arbeitsbereich AS area_id, HonZus AS fee_addi, is_valid, timestamp_file, last_autofulltext, last_autofulltext_staff_id,
 last_fulltext_update, related_id, related_type, discount, discount_date, work_package_id, fee_currency_id, fee_value_otcurr, fee_account_id
 FROM formular;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `correspondence_readonly` AS
 SELECT CONCAT_WS( '__', corr.id, lca.address_id) AS id, corr.id AS correspondence_id, lca.address_id AS address_id, corr.address_id AS address_or_group_id,
 corr.staff_id, corr.project_id, corr.correspondence_type_id, corr.subject, corr.text, corr.`date`, corr.search_text, corr.is_serial, corr.`file`, corr.security, corr.security_zip, corr.pages, corr.attachments,
 corr.copies, corr.created_date, corr.created_by_id, corr.change_date, corr.changed_by_id, corr.outbox_times, corr.is_archived, corr.is_exported_to_fees, corr.fee_value,
 corr.fee_no, corr.due_date, corr.vat, corr.fee_date, corr.area_id, corr.fee_addi, corr.is_valid, corr.timestamp_file, corr.last_autofulltext, corr.last_autofulltext_staff_id,
 corr.last_fulltext_update, corr.related_id, corr.related_type, discount, discount_date, work_package_id, fee_currency_id, fee_value_otcurr, fee_account_id
 FROM lut_correspondence_addresses AS lca
 INNER JOIN correspondence AS corr ON corr.id = lca.correspondence_id;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `correspondence_addi_def`
 AS SELECT LfdNr AS id, name, OrdNr AS sort, FeldName AS field, Typ AS type, subtype, fieldvalues, isrequired, color,  Laenge AS length, Sicherheit AS security FROM zusatz_formular_def;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `correspondence_types` AS
 SELECT LfdNr AS id, Art AS `name`, Text1 AS text1, Text2 AS text2, rtfDefault AS rtf_default, VorlagenDatei AS template_file, gueltig AS is_valid,
 ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id, Sicherheit AS security, security_zip, special_type
 FROM formart;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `correspondence_types_structure` AS
 SELECT formart.LfdNr AS type_id, 
    struct.LfdNr AS struct_id, struct.LfdNr_Parent AS parent_id, struct.OrdnerBezeichnung AS name, struct.VorlagenPfad AS path
 FROM formart
 LEFT JOIN lut_formart_struktur AS lut ON lut.LfdNr_FormArt = formart.LfdNr
 LEFT JOIN formart_struktur AS struct ON lut.LfdNr_Struktur = struct.LfdNr;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `correspondence_structure` AS
 SELECT LfdNr AS id, LfdNr_Parent AS parent_id, OrdnerBezeichnung AS name, VorlagenPfad AS path
 FROM formart_struktur;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_correspondence_structure_type` AS
 SELECT LfdNr AS id, LfdNr_FormArt AS type_id, LfdNr_Struktur AS struct_id
 FROM lut_formart_struktur;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `correspondence_locking`
 AS SELECT LfdNr AS id, OpenDoc_Nr AS correspondence_id, User_LfdNr AS staff_id, session_uniqid, lock_time FROM opendocs;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `lut_correspondence_copies`
 AS SELECT LfdNr AS id, LfdNr_Dok AS correspondence_id, Lfdnr_Empf AS address_id, OrdNr AS sort FROM kopienan;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `correspondence_attachments`
 AS SELECT LfdNr AS id, LfdNr_Dok AS correspondence_id, Att_Pfad AS `path`, real_name, Beschreibung AS description, 
 ustrDok AS is_us_document, Stufe AS security, security_zip,
 Betreff AS subject, orig_correspondence_id, drawing_id, addressfile_id, created_by_id, created_date FROM dokattachments;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `files`
 AS SELECT LfdNr AS id, Pfad AS `path`, Beschreibung AS purpose, Projekt AS project_id, Ebene AS `level`, Titel AS title, Root AS root,
 ErstAm AS created_date, ErstVon AS created_by_id, AendAm AS change_date, AendVon AS changed_by_id FROM dateien;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `numbergeneration`
 AS SELECT `Nummer` AS number, `Text` AS subject, `User` AS staff_id, `Projekt` AS project_id, `Datum` AS gdate,
 `Jahr` AS gyear, `Monat` AS gmonth FROM nummern_generieren;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `numbergeneration_log`
 AS SELECT `Nummer` AS gnumber, `User` AS staff_id, `Datum` AS gdate, `Bemerkung` AS subject FROM nummern_protokoll;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `payments_fees_values`
 AS SELECT pf.payments_id AS id, pf.fee_areas_offer_id AS fee_areas_offer_id, pf.percentage AS percentage, pf.`value` AS `value`, pp.invoice_date
 FROM `payments_fees` `pf` 
 INNER JOIN `payments` `pp` 
 ON `pf`.`payments_id` = `pp`.`id`
 WHERE status=20;
-- EOC


CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `address_groups_fullname` AS
 SELECT a.id, a.name, a.company_id, a.project_id, 
    IFNULL(CONCAT(p.project_name, ' ', p.project_number), '') AS projectname1,
    IFNULL(CONCAT(p.project_number, ' ', p.project_name), '') AS projectname2,
    IFNULL(CONCAT(a.name, ' [', p.project_name, ' ', p.project_number, ']'), a.name) AS fullname1,
    IFNULL(CONCAT(a.name, ' [', p.project_number, ' ', p.project_name, ']'), a.name) AS fullname2
FROM `address_groups` AS a
    LEFT JOIN `projects` AS p
    ON a.project_id = p.id;
-- EOC

-- Add translations main tables
CREATE TABLE IF NOT EXISTS `translations` (
	`id` INT( 11 ) NOT NULL AUTO_INCREMENT ,
	`name` VARCHAR( 50 ) NOT NULL ,
	`language` VARCHAR( 50 ) NOT NULL ,
	`value` TEXT NOT NULL ,
	PRIMARY KEY ( `id` )
) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
-- EOC-- USC Entferne alte Stored-Procedures.

-- --------------------------------------------------------------------------------------------------
-- Create report procedure for completion level - start
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_report_completion_level_start;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Create report procedure for completion level - create data (Fill with projects before)
-- --------------------------------------------------------------------------------------------------
DROP PROCEDURE IF EXISTS us_report_completion_level_create;
-- EOC
-- USC Erstelle Trigger und Indizes.

-- --------------------------------------------------------------------------------------------------
-- @todo Remove the DROP commands below, after some time
DROP TRIGGER IF EXISTS update_zusatz_extkosten;
-- EOC
DROP TRIGGER IF EXISTS rem_zusatz_extkosten;
-- EOC
DROP TRIGGER IF EXISTS update_zusatz_honverw;
-- EOC
DROP TRIGGER IF EXISTS rem_zusatz_honverw;
-- EOC
DROP TABLE IF EXISTS `update_views`;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- Create Trigger for INSERT
DROP TRIGGER IF EXISTS add_projekte_update_views;
-- EOC

DELIMITER $$
	CREATE TRIGGER add_projekte_update_views AFTER INSERT on projekte
		FOR EACH ROW BEGIN
                        UPDATE `update_counters` SET `count` = `count` + 1 WHERE `type` = 1;
		END;
	$$
DELIMITER ;
-- EOC

-- Create Trigger for UPDATE
DROP TRIGGER IF EXISTS update_projekte_update_views;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_projekte_update_views AFTER UPDATE on projekte
		FOR EACH ROW BEGIN
                        UPDATE `update_counters` SET `count` = `count` + 1 WHERE `type` = 1;
		END;
	$$
DELIMITER ;
-- EOC

-- Create Trigger for DELETE
DROP TRIGGER IF EXISTS rem_projekte_to_search;
-- EOC

DELIMITER $$
	CREATE TRIGGER rem_projekte_to_search AFTER DELETE on projekte
		FOR EACH ROW BEGIN
                        UPDATE `update_counters` SET `count` = `count` + 1 WHERE `type` = 1;
		END;
	$$
DELIMITER ;
-- EOC

-- Create Triggers for the resource planner portlet

-- when staff start, end or holiday rate has changed AND - after staff gets set inactive
DROP TRIGGER IF EXISTS update_report_wh_staff_change;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_staff_change AFTER UPDATE on personen
            FOR EACH ROW BEGIN
                if (NEW.`Start` <=> OLD.`Start`)=0 || (NEW.`Ende` <=> OLD.`Ende`)=0 || (NEW.`FeiertagSatz` <=> OLD.`FeiertagSatz`)=0 THEN
                    begin
                            INSERT INTO report_working_hours_dirty(staff_id) VALUES(NEW.LfdNr);
                    end;
                end if;

                DELETE FROM report_working_hours WHERE staff_id = NEW.LfdNr AND date >= NEW.Ende;
            END;
	$$
DELIMITER ;
-- EOC

-- after staff has been deleted
DROP TRIGGER IF EXISTS delete_staff_from_report_working_hours;
-- EOC

DELIMITER $$
	CREATE TRIGGER delete_staff_from_report_working_hours BEFORE DELETE on personen
		FOR EACH ROW BEGIN
			DELETE FROM report_working_hours WHERE staff_id = OLD.LfdNr;
			DELETE FROM report_working_hours_dirty WHERE staff_id = OLD.LfdNr;
		END;
	$$
DELIMITER ;
-- EOC

-- Remove old after staff gets set inactive
DROP TRIGGER IF EXISTS after_set_staff_incative;
-- EOC

-- when target hours change
DROP TRIGGER IF EXISTS update_report_wh_target_hours_change;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_target_hours_change AFTER UPDATE on tagessollstd
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(NEW.Pers_LfdNr);
		END;
	$$
DELIMITER ;
-- EOC

-- when new target hours added
DROP TRIGGER IF EXISTS update_report_wh_target_hours_insert;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_target_hours_insert AFTER INSERT on tagessollstd
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(NEW.Pers_LfdNr);
		END;
	$$
DELIMITER ;
-- EOC

-- when target hour gets removed
DROP TRIGGER IF EXISTS update_report_wh_target_hours_delete;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_target_hours_delete BEFORE DELETE on tagessollstd
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(OLD.Pers_LfdNr);
		END;
	$$
DELIMITER ;
-- EOC

-- when skill grp gets updated
DROP TRIGGER IF EXISTS update_report_wh_skill_grp_update;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_skill_grp_update AFTER UPDATE on lut_staff_skill_group
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(NEW.staff_id);
		END;
	$$
DELIMITER ;
-- EOC

-- when user gets a new skill grp
DROP TRIGGER IF EXISTS update_report_wh_skill_grp_insert;
-- EOC
 
DELIMITER $$
	CREATE TRIGGER update_report_wh_skill_grp_insert AFTER INSERT on lut_staff_skill_group
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(NEW.staff_id);
		END;
	$$
DELIMITER ;
-- EOC

-- before skill grps gets removed
DROP TRIGGER IF EXISTS update_report_wh_skill_grp_delete;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_skill_grp_delete BEFORE DELETE on lut_staff_skill_group
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(OLD.staff_id);
		END;
	$$
DELIMITER ;
-- EOC

-- when vacation added
DROP TRIGGER IF EXISTS update_report_wh_vacation_insert;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_vacation_insert AFTER INSERT on uv_buchungen
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(NEW.LfdNr_Mitarbeiter);
		END;
	$$
DELIMITER ;
-- EOC

-- when vacation has changed
DROP TRIGGER IF EXISTS update_report_wh_vacation_change;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_vacation_change AFTER UPDATE on uv_buchungen
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(NEW.LfdNr_Mitarbeiter);
		END;
	$$
DELIMITER ;
-- EOC

-- before vacation has been deleted
DROP TRIGGER IF EXISTS update_report_wh_vacation_delete;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_vacation_delete BEFORE DELETE on uv_buchungen
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(OLD.LfdNr_Mitarbeiter);
		END;
	$$
DELIMITER ;
-- EOC

-- when holiday changed
DROP TRIGGER IF EXISTS update_report_wh_holiday_change;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_holiday_change AFTER UPDATE on feiertage
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(-1);
		END;
	$$
DELIMITER ;
-- EOC

-- before holiday has been removed
DROP TRIGGER IF EXISTS update_report_wh_holiday_delete;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_holiday_delete BEFORE DELETE on feiertage
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(-1);
		END;
	$$
DELIMITER ;
-- EOC

-- after holiday has been insert
DROP TRIGGER IF EXISTS update_report_wh_holiday_insert;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_report_wh_holiday_insert AFTER INSERT on feiertage
		FOR EACH ROW BEGIN
			INSERT INTO report_working_hours_dirty(staff_id) VALUES(-1);
		END;
	$$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- update the work package table when the resource planner changes the start/end date
DROP TRIGGER IF EXISTS update_work_package_date;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_work_package_date AFTER UPDATE on lut_skill_wp
		FOR EACH ROW BEGIN
                    SET @new_start = (SELECT min(`start`) FROM lut_skill_wp WHERE wp_id = NEW.wp_id);
                    SET @new_end = (SELECT max(`end`) FROM lut_skill_wp WHERE wp_id = NEW.wp_id);
                    UPDATE `work_packages` SET `budget_start` = @new_start, `budget_end` = @new_end WHERE `id` = NEW.wp_id; 
		END;
	$$
DELIMITER ;
-- EOC

-- after insert to the lut_skill_wp table
DROP TRIGGER IF EXISTS update_work_package_date_after_insert;
-- EOC

DELIMITER $$
	CREATE TRIGGER update_work_package_date_after_insert AFTER INSERT on lut_skill_wp
		FOR EACH ROW BEGIN
                    SET @new_start = (SELECT min(`start`) FROM lut_skill_wp WHERE wp_id = NEW.wp_id);
                    SET @new_end = (SELECT max(`end`) FROM lut_skill_wp WHERE wp_id = NEW.wp_id);
                    UPDATE `work_packages` SET `budget_start` = @new_start, `budget_end` = @new_end WHERE `id` = NEW.wp_id; 
		END;
	$$
DELIMITER ;
-- EOC

DROP TRIGGER IF EXISTS update_work_package_date_after_delete;
-- EOC

-- after delete at the lut_skill_wp table
DELIMITER $$
	CREATE TRIGGER update_work_package_date_after_delete AFTER DELETE on lut_skill_wp
		FOR EACH ROW BEGIN
                    SET @new_start = (SELECT min(`start`) FROM lut_skill_wp WHERE wp_id = OLD.wp_id);
                    SET @new_end = (SELECT max(`end`) FROM lut_skill_wp WHERE wp_id = OLD.wp_id);
                    UPDATE `work_packages` SET `budget_start` = @new_start, `budget_end` = @new_end WHERE `id` = OLD.wp_id; 
		END;
	$$
DELIMITER ;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- check if left value is not bigger than right value

-- before insert check if rgt is greater than the lft value
DROP trigger IF EXISTS CHK_rgtGreaterlft_on_insert;
-- EOC
DELIMITER $$ 
	CREATE TRIGGER CHK_rgtGreaterlft_on_insert BEFORE
		INSERT
			ON work_packages FOR EACH ROW BEGIN IF (NEW.rgt < NEW.lft) THEN SIGNAL SQLSTATE '45000'
		SET
			MESSAGE_TEXT = 'uStrich: left value has to be less than the right value';
		END IF;
	END;
	$$
DELIMITER ;
-- EOC

-- before update check if rgt is greater than the lft value
DROP trigger IF EXISTS CHK_rgtGreaterlft_on_update;
-- EOC
DELIMITER $$
	CREATE TRIGGER CHK_rgtGreaterlft_on_update BEFORE
		UPDATE
  			ON work_packages FOR EACH ROW BEGIN IF (NEW.rgt < NEW.lft) THEN SIGNAL SQLSTATE '45000'
		SET
  			MESSAGE_TEXT = 'uStrich: left value has to be less than the right value';
		END IF;
	END;
	$$
DELIMITER ;
-- EOC

CALL us_add_index_to_table('email_cache_data', 'date', '`date`');
-- EOC

CALL us_add_index_to_table('fees', 'fee_no', '`fee_no`');
-- EOC
CALL us_add_index_to_table('fees_planned', 'fee_no', '`fee_no`');
-- EOC
CALL us_add_index_to_table('fees', 'correspondence_id', '`correspondence_id`');
-- EOC
CALL us_add_index_to_table('fees_planned', 'correspondence_id', '`correspondence_id`');
-- EOC

CALL us_add_index_to_table('formular', 'date', '`Datum`');
-- EOC
CALL us_add_index_to_table('formular', 'is_exported_to_fees', '`InHonverwExp`');
-- EOC

CALL us_add_index_to_table('work_packages', 'is_fee_area', '`is_fee_area`');
-- EOC
CALL us_add_index_to_table('work_packages', 'tag_id', '`tag_id`');
-- EOC
CALL us_add_index_to_table('work_packages', 'cost_centre_id', '`cost_centre_id`');
-- EOC
CALL us_add_index_to_table('work_packages_history', 'date', '`date`' );
-- EOC

CALL us_add_index_to_table('inbox', 'emailcount_test', 'created_by_id, email_cache_id, email_account_id, pid, email_import_start, email_import_attempts' );
-- EOC
CALL us_add_index_to_table('outbox', 'emailcount_test', 'created_by_id, email_cache_id, email_account_id, pid, email_import_start, email_import_attempts' );
-- EOC

CALL us_add_index_to_table('formular', 'formular_stufe', '`Stufe`');
-- EOC

CALL us_add_index_to_table('lut_fee_wp', 'planned_fee_id', '`planned_fee_id`');
-- EOC
CALL us_add_index_to_table('lut_fee_wp', 'fee_id', '`fee_id`');
-- EOC
CALL us_add_index_to_table('lut_fee_wp', 'project_id', '`project_id`');
-- EOC
CALL us_add_index_to_table('lut_fee_wp', 'wp_id', '`wp_id`');
-- EOC

CALL us_add_constraint_to_table('lut_measurements', 'proj_meas', 'UNIQUE', '`project_id` , `measurement_id`' );
-- EOC-- USC Messgrößen.

-- Migrate projekte to lut_measurements
CALL us_migrate_projekte_to_lut_measurements();
-- EOC

-- Prepare cache_project_measurement
CALL us_create_cache_project_measurement(1, ~0 >> 33, @validate_outinfo);
-- EOC
-- USC Views fuer Auswertungen werden angelegt.

-- --------------------------------------------------------------------------------------------------
-- For reports
-- --------------------------------------------------------------------------------------------------

-- #x2#
-- CREATE OR REPLACE
--  ALGORITHM = MERGE
--  VIEW `report_times`
--  AS SELECT times.*, woa.fee_area_id
--  FROM times
--  LEFT JOIN work_areas AS woa ON times.area_id=woa.id;
-- -- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `report_times_datemonth`
 AS SELECT project_id, staff_id, number, date, CONCAT(MONTH(date),'_',YEAR(date)) AS datemonth
 FROM times;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `report_overtime_datemonth`
 AS SELECT overtime_rate_id, staff_id, number, date, CONCAT(MONTH(date),'_',YEAR(date)) AS datemonth, paid
 FROM overtime;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `report_start_end_times`
 AS SELECT Person AS staff_id, Datum AS date,
 CONCAT_WS(' - ', DATE_FORMAT(ZeitBeginn, '%H:%i'), DATE_FORMAT(ZeitEnde, '%H:%i')) AS set_info,
 TIMESTAMPDIFF(SECOND, ZeitBeginn, ZeitEnde)-(ZeitAbzug*3600) AS timediff,
 DATE_FORMAT(ZeitBeginn, '%H:%i') AS start_time, DATE_FORMAT(ZeitEnde, '%H:%i') AS end_time,
 ZeitAbzug AS time_deduct
 FROM zeiten_beginnende;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `report_measurements`
 AS SELECT mess.id, mess.name,
 ( SELECT SUM(lm1.value) FROM lut_measurements AS lm1 WHERE lm1.measurement_id=mess.id ) AS mess_sum,
 ( SELECT SUM(t1.number) FROM times AS t1 WHERE t1.project_id IN (SELECT lm2.project_id from lut_measurements AS lm2 WHERE lm2.measurement_id=mess.id) ) AS mess_hours
 FROM measurements AS mess;
-- EOC

-- --------------------------------------------------------------------------------------------------
-- For logging reports
-- --------------------------------------------------------------------------------------------------
CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `report_times_logging`
 AS SELECT tilo.id AS old_id, tilo.project_id AS old_project_id, tilo.staff_id AS old_staff_id, tilo.number AS old_number,
 tilo.factor AS old_factor, tilo.time_from AS old_time_from, tilo.time_to AS old_time_to,
 tilo.purpose AS old_purpose, tilo.date AS old_date, tilo.area_id AS old_area_id, tilo.cost_centre_id AS old_cost_centre_id,
 tilo.is_deducted AS old_is_deducted, tilo.deducted_day AS old_deducted_day,
 tilo.changed_by_id AS old_changed_by_id, tilo.is_ignore AS old_is_ignore, tilo.gross_hourly_wage AS old_gross_hourly_wage,
 tilo.log_change_mode AS log_change_mode, tilo.log_change_date AS log_change_date, tilo.log_changed_by_id  AS log_changed_by_id,
 tilo.company_id AS old_company_id, tilo.purpose_internal AS old_purpose_internal,
 tilo.is_client_changed AS old_is_client_changed, 
 tilo.addi_val1 AS old_addi_val1, tilo.addi_val2 AS old_addi_val2, 
 tilo.work_package_id AS old_work_package_id,
 1+0 AS one, times.*
 FROM times_logging AS tilo
 LEFT OUTER JOIN times ON tilo.id = times.id;
-- EOC

CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `report_extras_logging`
 AS SELECT 
 exlo.id AS old_id, exlo.project_id AS old_project_id, exlo.staff_id AS old_staff_id, 
 exlo.purpose AS old_purpose, exlo.number AS old_number, exlo.description AS old_description,
 exlo.date AS old_date, exlo.type_id AS old_type_id,
 exlo.area_id AS old_area_id, exlo.cost_centre_id AS old_cost_centre_id,
 exlo.changed_by_id AS old_changed_by_id, exlo.is_ignore AS old_is_ignore,
 exlo.log_change_mode AS log_change_mode, exlo.log_change_date AS log_change_date, exlo.log_changed_by_id  AS log_changed_by_id,
 exlo.company_id AS old_company_id, 
 exlo.is_client_changed AS old_is_client_changed, 
 exlo.addi_val1 AS old_addi_val1, exlo.addi_val2 AS old_addi_val2, 
 exlo.work_package_id AS old_work_package_id,
 1+0 AS one, extras.*
 FROM extras_logging AS exlo
 LEFT OUTER JOIN extras ON exlo.id = extras.id;
-- EOC


CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW `report_travels_logging`
 AS SELECT 
 trlo.id AS old_id, trlo.project_id AS old_project_id, trlo.staff_id AS old_staff_id, 
 trlo.number AS old_number, trlo.route AS old_route, trlo.is_company_car AS old_is_company_car,
 trlo.km_from AS old_km_from, trlo.km_to AS old_km_to, 
 trlo.time_from AS old_time_from, trlo.time_to AS old_time_to, 
 trlo.purpose AS old_purpose, trlo.description AS old_description,
 trlo.date AS old_date, trlo.type_id AS old_type_id,
 trlo.area_id AS old_area_id, trlo.cost_centre_id AS old_cost_centre_id,
 trlo.changed_by_id AS old_changed_by_id, trlo.is_ignore AS old_is_ignore,
 trlo.log_change_mode AS log_change_mode, trlo.log_change_date AS log_change_date, trlo.log_changed_by_id  AS log_changed_by_id,
 trlo.company_id AS old_company_id, 
 trlo.is_client_changed AS old_is_client_changed,
 trlo.addi_val1 AS old_addi_val1, trlo.addi_val2 AS old_addi_val2,
 trlo.work_package_id AS old_work_package_id,
 1+0 AS one, travels.*
 FROM travels_logging AS trlo
 LEFT OUTER JOIN travels ON trlo.id = travels.id;
-- EOC

-- #x2#
-- CREATE OR REPLACE
--  ALGORITHM = MERGE
--  VIEW `report_fee_areas_project_allocation`
--  AS SELECT sum(l1.costs) as costs, l1.project_id, lh1.valid_to_date,
-- lh1.id,
-- 	CASE lh1.valid_to_date IS NULL WHEN true THEN 
-- 		(SELECT max(lh3.valid_to_date) FROM lut_fee_areas_proj_history AS lh3 WHERE lh1.project_id=lh3.project_id)
-- 	ELSE
-- 		(SELECT max(lh2.valid_to_date) FROM lut_fee_areas_proj_history AS lh2 WHERE lh1.project_id=lh2.project_id AND lh2.valid_to_date<lh1.valid_to_date)
-- 	END AS before_date
-- FROM lut_fee_areas_proj AS l1
-- LEFT JOIN lut_fee_areas_proj_history AS lh1 ON l1.lut_fee_areas_proj_history_id=lh1.id
-- 
-- GROUP BY l1.project_id, lh1.valid_to_date
-- 
-- ORDER BY before_date, lh1.valid_to_date;
-- -- EOC

-- #x2#
-- CREATE OR REPLACE
--  ALGORITHM = MERGE
--  VIEW `report_completion_level`
--  AS SELECT cl.project_id, cl.work_area_id,
--  cl.completion_level AS completion_level,
--  cl.completion_level_history_id AS completion_level_history_id,
--  clh.valid_to_date AS valid_to_date,
--  (
--     SELECT MAX(bbl1.planned_costs)
--     FROM budget_plan AS bbl1 
--     LEFT JOIN budget_plan_history AS lh1 ON bbl1.budget_plan_history_id=lh1.id 
--     WHERE lh1.valid_to_date IS NULL AND bbl1.project_id=cl.project_id AND bbl1.area_id=cl.work_area_id
--  ) * cl.completion_level AS costscompleted,
--  (
--     SELECT MAX(bbl1.proportion)
--     FROM budget_plan AS bbl1 
--     LEFT JOIN budget_plan_history AS lh1 ON bbl1.budget_plan_history_id=lh1.id 
--     WHERE lh1.valid_to_date IS NULL AND bbl1.project_id=cl.project_id AND bbl1.area_id=cl.work_area_id
--  ) AS proportion,
--  (
--     SELECT SUM(bbl1.proportion)
--     FROM budget_plan AS bbl1 
--     LEFT JOIN budget_plan_history AS lh1 ON bbl1.budget_plan_history_id=lh1.id 
--     WHERE lh1.valid_to_date IS NULL AND bbl1.project_id=cl.project_id
--  ) AS sumproportion,
--  CASE clh.valid_to_date IS NULL WHEN true THEN 
--     (SELECT max(clh3.valid_to_date) FROM completion_level_history AS clh3 WHERE clh.project_id=clh3.project_id)
--  ELSE
--     (SELECT max(clh2.valid_to_date) FROM completion_level_history AS clh2 WHERE clh.project_id=clh2.project_id AND clh2.valid_to_date<clh.valid_to_date)
--  END AS before_date
--  FROM `completion_level` AS cl
--  LEFT JOIN `completion_level_history` AS clh ON cl.completion_level_history_id = clh.id;
-- -- EOC

-- #x2#
-- CREATE OR REPLACE
--  VIEW `report_completion_level_only`
--  AS SELECT cl.project_id, cl.work_area_id,
--  cl.completion_level AS completion_level,
--  cl.completion_level_history_id AS completion_level_history_id,
--  clh.valid_to_date AS valid_to_date,
--  (
--     SELECT bbl1.planned_costs
--     FROM budget_plan AS bbl1 
--     LEFT JOIN budget_plan_history AS lh1 ON bbl1.budget_plan_history_id=lh1.id 
--     WHERE lh1.valid_to_date IS NULL AND bbl1.project_id=cl.project_id AND bbl1.area_id=cl.work_area_id
--     LIMIT 1
--  ) * cl.completion_level AS costscompleted,
--  CASE clh.valid_to_date IS NULL WHEN true THEN 
--     (SELECT max(clh3.valid_to_date) FROM completion_level_history AS clh3 WHERE clh.project_id=clh3.project_id)
--  ELSE
--     (SELECT max(clh2.valid_to_date) FROM completion_level_history AS clh2 WHERE clh.project_id=clh2.project_id AND clh2.valid_to_date<clh.valid_to_date)
--  END AS before_date
--  FROM `completion_level` AS cl
--  LEFT JOIN `completion_level_history` AS clh ON cl.completion_level_history_id = clh.id;
-- -- EOC

-- #x2#
-- CREATE OR REPLACE
--  ALGORITHM = MERGE
--  VIEW `report_completion_level_bbl1`
--  AS SELECT bbl.planned_costs, bbl.proportion, bbl.project_id, bbl.area_id
--     FROM budget_plan AS bbl 
--     LEFT JOIN budget_plan_history AS lh1 ON bbl.budget_plan_history_id=lh1.id 
--     WHERE lh1.valid_to_date IS NULL;
-- -- EOC
    
-- #x2#
-- CREATE OR REPLACE
--  ALGORITHM = MERGE
--  VIEW `report_completion_level_bbl2`
--  AS SELECT SUM(bbl.proportion) AS proportion, bbl.project_id
--     FROM budget_plan AS bbl 
--     LEFT JOIN budget_plan_history AS lh1 ON bbl.budget_plan_history_id=lh1.id 
--     WHERE lh1.valid_to_date IS NULL
--     GROUP BY bbl.project_id;
-- -- EOC

-- #x2#
-- CREATE OR REPLACE
--  ALGORITHM = MERGE
--  VIEW `report_completion_level_current`
--  AS SELECT cl.project_id, cl.work_area_id,
--  cl.completion_level AS completion_level,
--  cl.completion_level_history_id AS completion_level_history_id,
--  clh.valid_to_date AS valid_to_date,
--  bbl1.planned_costs * cl.completion_level AS costscompleted,
--  bbl1.proportion,
--  bbl2.proportion AS sumproportion
--  FROM `completion_level` AS cl
--  LEFT JOIN `completion_level_history` AS clh ON cl.completion_level_history_id = clh.id
--  LEFT JOIN report_completion_level_bbl1 AS bbl1 ON bbl1.project_id=cl.project_id AND bbl1.area_id=cl.work_area_id
--  LEFT JOIN report_completion_level_bbl2 AS bbl2 ON bbl2.project_id=cl.project_id
--  WHERE clh.valid_to_date IS NULL;
-- -- EOC

-- #x2#
-- CREATE OR REPLACE
--  ALGORITHM = MERGE
--  VIEW `report_completion_level_fee`
--  AS SELECT cl.*, woas.fee_area_id
--  FROM `report_completion_level_current` AS cl
--  LEFT JOIN work_areas AS woas ON cl.work_area_id=woas.id
--  WHERE cl.valid_to_date IS NULL;
-- -- EOC


CREATE OR REPLACE
 VIEW `report_times_changed_gross`
 AS SELECT staff_id, gross_hourly_wage, `date`
 FROM times
 GROUP BY staff_id, gross_hourly_wage, `date`;
-- EOC

-- #x2#
-- CREATE OR REPLACE
--  VIEW `report_projects_fee_areas`
--  AS SELECT
--  p.id as project_id,
--  p.project_name as project_name,
--  p.work_area_category_id,
--  lutwoa.work_area_id,
--  woa.sort AS work_area_sort,
--  woa.fee_area_id
--  FROM projects AS p
--  INNER JOIN lut_work_areas_work_areas_categories AS lutwoa ON lutwoa.work_area_categories_id= p.work_area_category_id
--  INNER JOIN work_areas AS woa ON lutwoa.work_area_id=woa.id;
-- -- EOC

-- #x2#
-- CREATE OR REPLACE
--  VIEW `report_fee_areas_project_allocation_history`
--  AS 
--     SELECT 
--     sum(l1.costs) as costs,
--     l1.project_id,
--     CASE
--         WHEN valid_to_date IS NULL THEN '2099-01-01'
--         ELSE valid_to_date
--     END AS valid_to_date
--     FROM lut_fee_areas_proj AS l1
--     LEFT JOIN lut_fee_areas_proj_history AS lh1 ON l1.lut_fee_areas_proj_history_id=lh1.id
--     GROUP BY l1.project_id, lh1.valid_to_date;
-- -- EOC

-- #x2#
-- CREATE OR REPLACE
--  VIEW `report_budget_plan_history`
--  AS 
--     SELECT 
--     sum(l1.planned_costs) as planned_costs,
--     sum(l1.planned_hours) as planned_hours,
--     l1.project_id, 
--     CASE
--         WHEN valid_to_date IS NULL THEN '2099-01-01'
--         ELSE valid_to_date
--     END AS valid_to_date,
--     lh1.id
--     FROM budget_plan AS l1
--     LEFT JOIN budget_plan_history AS lh1 ON l1.budget_plan_history_id=lh1.id
--     GROUP BY l1.project_id, lh1.valid_to_date;
-- -- EOC
-- USC Standardwerte.

INSERT INTO `currencies` (`id`, `name`, `abbreviation`) 
VALUES (1, 'Euro', '€') 
ON DUPLICATE KEY UPDATE id=id;
-- EOC

-- ONE TIME BUG FIX - REMOVE!!
-- Set invoice date in payments tabe, if missing
UPDATE payments 
SET invoice_date=change_date 
WHERE invoice_date IS NULL 
    AND status=20;
-- EOC

-- Get passwort mode from system settings
UPDATE `personen` 
SET passwort_mode=( SELECT IF(SettingValue='TRUE', 1, 0) FROM settingssystem WHERE SettingType='userpasswordencrypted' )
WHERE passwort_mode IS NULL;
-- EOC

-- Increase passwort mode to 1 (SHA1 with fixed salts)
UPDATE `personen` 
SET `passwort` = SHA1( CONCAT('477518110', `passwort`, '13749', '48492') ),
    passwort_mode = 1
WHERE passwort_mode=0 OR passwort_mode IS NULL;
-- EOC

-- Increase passwort mode to 2 (Only adds random salt)
UPDATE `personen` 
SET `passwort` = CONCAT(SHA1(RAND()), `passwort`),
    passwort_mode = 2
WHERE passwort_mode=1;
-- EOC

-- Increase passwort mode to 3 (Create SHA1 with random salt)
UPDATE `personen` 
SET `passwort` = CONCAT(LEFT(`passwort`, 40), SHA1(`passwort`)),
    passwort_mode = 3
WHERE passwort_mode=2;
-- EOC

-- Write is_special_phase based on the name for work_package_presets
UPDATE 
    work_package_presets 
SET 
    is_special_phase=CONVERT(SUBSTRING(TRIM(name), 1, 1), UNSIGNED) 
WHERE 
        is_special<0 
    AND
        is_special_phase IS NULL
    AND
        LENGTH(name)>1;
-- EOC

-- Write is_special_phase based on the name for work_packages
UPDATE 
    work_packages 
SET 
    is_special_phase=CONVERT(SUBSTRING(TRIM(name), 1, 1), UNSIGNED) 
WHERE 
        is_special<0 
    AND
        is_special_phase IS NULL
    AND
        LENGTH(name)>1;
-- EOC

-- Create is_absent_social flag, based on project

DROP PROCEDURE IF EXISTS us_migrate_12_5__12_6;
-- EOC
DELIMITER $$
CREATE PROCEDURE us_migrate_12_5__12_6()
us_migrate_12_5__12_6lbl:BEGIN

    DECLARE xdate DATETIME DEFAULT curdate() - INTERVAL 1 YEAR;
    DECLARE step VARCHAR(30) DEFAULT 'us_migrate_12_5__12_6';
    
    -- Prevent double execution
    IF ( SELECT COUNT(*) FROM `01migrate_11_12_steps` WHERE stepname=step) THEN 
        LEAVE us_migrate_12_5__12_6lbl;
    END IF;
    INSERT INTO `01migrate_11_12_steps` SET stepname=step; 

    -- Prevent execution, if we already have is_absent_social vacations
    IF ( (SELECT COUNT(*) FROM vacations WHERE is_absent_social=true)>0 ) THEN 
        -- INSERT info
        LEAVE us_migrate_12_5__12_6lbl;
    END IF;
    
    UPDATE vacations 
    SET is_absent_social = true
    WHERE 
        is_absent=1 
    AND
        imported_workinput=0
    AND 
        start_date>xdate 
    AND 
        project_id IN (
            SELECT id FROM projects 
            WHERE is_included_in_time_sheet=true 
            AND ID<>(SELECT MAX(value) FROM system_settings WHERE NAME='projekturlaub')
        );
    
END $$
DELIMITER ;
-- EOC

CALL us_migrate_12_5__12_6();
-- EOC

DROP PROCEDURE IF EXISTS migrate_to_global_email_accounts;
-- EOC

DELIMITER $$
CREATE PROCEDURE migrate_to_global_email_accounts()
update_global_email_accountslbl:BEGIN

    DECLARE l_last_row_fetched INTEGER DEFAULT 0;
    DECLARE email_account_id INTEGER;
    DECLARE email_account_staff_id INTEGER;
    DECLARE email_account_email VARCHAR(256) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE email_account_smtp_host VARCHAR(255);
    DECLARE email_account_smtp_user VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE email_account_imap_host VARCHAR(255);
    DECLARE email_account_imap_user VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE email_account_imap_mailbox VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE email_account_imap_mailbox_out VARCHAR(100) CHARSET utf8 COLLATE utf8_general_ci;
    DECLARE email_account_default_account BOOLEAN;
    DECLARE account_vals CURSOR FOR SELECT id, staff_id, email, smtp_host, smtp_user, imap_host, imap_user, imap_mailbox, imap_mailbox_out, default_account 
        FROM email_accounts
        ORDER BY email, smtp_host, smtp_user, imap_host, imap_user, imap_mailbox, imap_mailbox_out, id;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET l_last_row_fetched=1;

    -- Prevent double execution
    IF ( SELECT COUNT(*) FROM `01migrate_11_12_steps` WHERE stepname='migrate_to_global_email_accounts') THEN 
        LEAVE update_global_email_accountslbl;
    END IF;
    INSERT INTO `01migrate_11_12_steps` SET stepname='migrate_to_global_email_accounts'; 
    
    -- Only for migration
    CALL us_add_to_table2('email_accounts', 'default_account', 'BOOLEAN DEFAULT FALSE AFTER `email`' );

    -- CREATE TABLE
    SET @s = '
        CREATE TABLE IF NOT EXISTS `lut_email_accounts_staff` (
            `id` BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
            `email_account_id` INT(11) NOT NULL,
            `staff_id` INT(11) NOT NULL,
            `right` INT(11) NOT NULL, 
            `is_default_account` BOOLEAN NOT NULL DEFAULT "0"
        ) auto_increment=1 ENGINE=InnoDB CHARACTER SET utf8 COLLATE utf8_general_ci;
    ';
    PREPARE stmt1 FROM @s;
    EXECUTE stmt1;
    DEALLOCATE PREPARE stmt1;

    -- MIGRATE
    SET @last_hash = '';
    SET @current_email_account_id = 0;
    OPEN account_vals;
    cursor_loop: LOOP -- -------------------------------------
        FETCH account_vals INTO email_account_id, email_account_staff_id, email_account_email, email_account_smtp_host, email_account_smtp_user, email_account_imap_host, email_account_imap_user, email_account_imap_mailbox, email_account_imap_mailbox_out, email_account_default_account;
        IF l_last_row_fetched = 1 THEN
            LEAVE cursor_loop;
        END IF;
            /*Create hash value*/
            SET @current_hash = SHA1(CONCAT(email_account_email, email_account_smtp_host, email_account_smtp_user, email_account_imap_host, email_account_imap_user, email_account_imap_mailbox, email_account_imap_mailbox_out));

            /*Cleanup*/
            IF @last_hash = @current_hash THEN
                -- update inbox/outbox for eml download
                UPDATE inbox SET email_account_id = @current_email_account_id WHERE inbox.email_account_id = email_account_id;
                UPDATE outbox SET email_account_id = @current_email_account_id WHERE outbox.email_account_id = email_account_id;
                -- delete from accounts
                DELETE FROM email_accounts WHERE id = email_account_id;
            ELSE
                SET @current_email_account_id = email_account_id;
            END IF;

            /*Update is_default_account*/    
            IF email_account_default_account = 1 THEN
                UPDATE lut_email_accounts_staff SET is_default_account = 0 WHERE lut_email_accounts_staff.staff_id = email_account_staff_id;
            END IF;

            /*Add to lut with full rights*/
            INSERT INTO lut_email_accounts_staff(`email_account_id`, `staff_id`, `right`, `is_default_account`) VALUES(@current_email_account_id, email_account_staff_id, 7, email_account_default_account);

            /*Update last hash value*/
            SET @last_hash = @current_hash;
    END LOOP cursor_loop; -- -------------------------------------
    CLOSE  account_vals;

    -- RENAME staff_id to password_salt
    CALL check_rename_column_exists('email_accounts', 'staff_id', 'password_salt', 'INT(11)');

END $$
DELIMITER ;
-- EOC

CALL migrate_to_global_email_accounts();
-- EOC

-- REMOVE default_account cause it has moved to lut table
CALL remove_column('email_accounts', 'default_account');
-- EOC
-- USC Weitere Standardwerte.

-- #x2#
-- -- FIX LUT FEE AREA PROJECT WITHOUT HISTORY ID
-- UPDATE lut_fee_areas_proj SET lut_fee_areas_proj_history_id = (
-- 	SELECT id FROM lut_fee_areas_proj_history WHERE project_id = lut_fee_areas_proj.project_id AND lut_fee_areas_proj_history.valid_to_date IS NULL LIMIT 1
-- ) WHERE lut_fee_areas_proj_history_id IS NULL;
-- -- EOC


-- FIX NEW RIGHT R_DATACOLLECTION - DEFAULT GRANT RIGHT
UPDATE staff SET security_49=0 WHERE security_49<>99 OR security_49 IS NULL;
-- EOC

-- corr_mailout_to_pdf - This is not neccessary for X.1. Mail-In is deprecated and mail allocation is done in mailinboxandoutbox
UPDATE system_settings SET value='FALSE' WHERE name='em_out';
-- EOC

-- Ensure DataBaseVersion is 99, to avoid X.1 Connector installation.
UPDATE system_settings SET value='99' WHERE name='DataBaseVersion' AND value<99;
-- EOC

-- Get rid of address_signature_subject_project
UPDATE system_settings SET value = '<D_BETREFF> (<PROJEKT>)'
WHERE 
name = 'signatur_subject' 
AND value = ''
AND (SELECT count(*) FROM (SELECT t2.name FROM system_settings AS t2 WHERE t2.name = 'BetreffMitProjekt'  AND t2.value = 'TRUE') AS t1)  = 1;
-- EOC

UPDATE system_settings SET value = 'FALSE' WHERE name = 'BetreffMitProjekt' OR name = 'BetreffMitProjektLS';
-- EOC

-- X3 Set project office_in_charge show
INSERT INTO settingssystem (`SettingType`, `SettingValue`) 
SELECT 'project_office_in_charge_show', IF( (SELECT count(*) FROM projekte WHERE Planer2 IS NOT NULL AND Planer2 > 0)>1, 'TRUE', 'FALSE') FROM DUAL
WHERE NOT EXISTS (SELECT * FROM settingssystem WHERE SETTINGTYPE='project_office_in_charge_show');
-- EOC

-- X3 Set project archive show
INSERT INTO settingssystem (`SettingType`, `SettingValue`) 
SELECT 'project_archive_show', IF( (SELECT count(*) FROM projektarchive)>=1, 'TRUE', 'FALSE') FROM DUAL
WHERE NOT EXISTS (SELECT * FROM settingssystem WHERE SETTINGTYPE='project_archive_show');
-- EOC

-- X3 Add country_code for default countries
UPDATE `countries` SET `country_code`='AT' WHERE `name`='Österreich' AND `country_code` IS NULL;
-- EOC
UPDATE `countries` SET `country_code`='DE' WHERE `name`='Deutschland' AND `country_code` IS NULL;
-- EOC
UPDATE `countries` SET `country_code`='CH' WHERE `name`='Schweiz' AND `country_code` IS NULL;
-- EOC

-- X3 Add currency_code for default currencies
UPDATE `currencies` SET `currency_code`='EUR' WHERE `name`='Euro' AND `currency_code` IS NULL;
-- EOC

-- Ensure fee_multipliers flags are set to ignore, when no name
UPDATE `fee_multipliers` 
SET is_standard=false, is_r2=false, is_r3=false, is_r4=false, is_work=false, is_ignore_mgmt=true
WHERE `name`='' AND purpose='';
-- EOC
-- USC Entferne alte Trigger.

-- --------------------------------------------------------------------------------------------------
-- --------------------------------------------------------------------------------------------------

-- Drop Trigger for INSERT
DROP TRIGGER IF EXISTS add_adressen_to_search;
-- EOC

-- Drop Trigger for UPDATE
DROP TRIGGER IF EXISTS update_adressen_to_search;
-- EOC

-- Drop Trigger for DELETE
DROP TRIGGER IF EXISTS rem_adressen_to_search;
-- EOC

-- Drop Trigger for INSERT
DROP TRIGGER IF EXISTS add_projekte_to_search;
-- EOC

-- Drop Trigger for UPDATE
DROP TRIGGER IF EXISTS update_projekte_to_search;
-- EOC

-- Drop Trigger for DELETE
DROP TRIGGER IF EXISTS rem_projekte_to_search;
-- EOC

-- ---------------------------------------------------
-- TRIGGER FOR UPDATE
-- ---------------------------------------------------

-- Drop UPDATE inbox
DROP TRIGGER IF EXISTS update_crm_table_inbox;
-- EOC
			

-- Drop UPDATE outbox
DROP TRIGGER IF EXISTS update_crm_table_outbox;
-- EOC

-- Drop UPDATE addr_desc
DROP TRIGGER IF EXISTS update_crm_table_addr;
-- EOC
			
-- ---------------------------------------------------
-- TRIGGER FOR INSERET
-- ---------------------------------------------------

-- Drop INSERT INTO INBOX
DROP TRIGGER IF EXISTS insert_crm_table_inbox;
-- EOC

-- Drop INSERT INTO OUTBOX
DROP TRIGGER IF EXISTS insert_crm_table_outbox;
-- EOC

-- Drop INSERT into addr_desc
DROP TRIGGER IF EXISTS insert_crm_table_addr;
-- EOC
			
-- ---------------------------------------------------
-- TRIGGER FOR DELETE
-- ---------------------------------------------------

-- Drop DELETE FROM INBOX
DROP TRIGGER IF EXISTS delete_crm_table_inbox;
-- EOC

-- Drop DELETE FROM OUTBOX
DROP TRIGGER IF EXISTS delete_crm_table_outbox;
-- EOC

-- Drop DELETE FROM addr_desc
DROP TRIGGER IF EXISTS delete_crm_table_addr;
-- EOC

-- USC Alle Indizes fuer die Volltextsuche werden angelegt, falls aktiv.

CALL add_all_fulltext_indexes_if_fulltext_search_is_active;
-- EOC-- USC Abschluss.

-- --------------------------------------------------------------------------------------------------
DELETE FROM system_settings WHERE name='u10_last_update';
-- EOC

INSERT INTO system_settings(`name`, `value`) VALUES ('u10_last_update', '2018-10-01');
-- EOC






